/* BIO: A bioacoustical analyzer */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>
#include <malloc.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include "../config.h"
#else
#define SGI
#include <audiofile.h>
#include <audio.h>
#endif

#ifdef SGI
#include <sys/fpu.h>
#endif

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Display.h>
#include <Xm/RowColumn.h>
#include <Xm/Frame.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/CascadeB.h>
#include <Xm/MessageB.h>
#include <Xm/FileSB.h>
#include <Xm/DrawingA.h>
#include <Xm/Scale.h>
#include <Xm/SelectioB.h>
#include <Xm/BulletinB.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/Label.h>
#include <Xm/ToggleB.h>
#include <Xm/ScrollBar.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>

#include "pixmaps.h"

#define FORWARD 1
#define INVERSE 0
#define MAX_FFT_SIZE 4096
#define fvec(name, size)\
if ((name=(double *)calloc(size, sizeof(double)))==NULL) {\
  printf("Insufficient memory\n");\
}
#define svec(name, size)\
if ((name=(short int *)calloc(size, sizeof(short int)))==NULL) {\
  printf("Insufficient memory\n");\
}
#define bvec(name, size)\
if ((name=(signed char *)calloc(size, sizeof(signed char)))==NULL) {\
  printf("Insufficient memory\n");\
}
#define LSIZE 512
#define FLUSH_LIST(list, count, theGC)        \
          XDrawPoints(theDisplay,bitmap,theGC,list,count,     \
              CoordModeOrigin);       \
        count=0

#define ADD_POINT(xx, yy, list, count, theGC) \
      list[count].x = xx;     \
      list[count].y = yy;     \
      if (++(count)==LSIZE) { \
              FLUSH_LIST(list,count, theGC);    \
      }       

#define FLUSH_LINES(list, count, GC)        \
          XDrawSegments(theDisplay,bitmap,GC,list,count);   \
        count=0

#define ADD_LINE(xx1, yy1, xx2, yy2, list, count, GC) \
      list[count].x1 = xx1;     \
      list[count].y1 = yy1;     \
      list[count].x2 = xx2;     \
      list[count++].y2 = yy2;     \
      if (count==LSIZE) { \
              FLUSH_LINES(list,count,GC);    \
      }       



/* extern char *malloc(),  *calloc(); */
Widget loadFileBox, playFileBox, toamplitudeFileBox, tocentroidFileBox,
  topitchFileBox, tonoiseFileBox, tomarkersFileBox, loadMarkersFileBox;
Widget sketchpad, topLevel, dw;
Widget fileWarning, sizeWarning, centroidWarning, amplitudeWarning,
  pitchWarning, noiseWarning, markersWarning, printWarning, specprintWarning,
  specexportWarning, corrprintWarning, correxportWarning;
Widget displayForm, displayToggleGrey, displayToggleHot, displayToggleCold,
  displayToggleGrid, displayToggleCentro;
Widget analysisForm;
Widget specForm, specOkButton, specSketchpad, specPrintButton, specExportButton;
Widget corrForm, corrOkButton, corrSketchpad, corrPrintButton, corrExportButton;

XtAppContext app_context;
Widget mainWindow, menuBar, frame, largeBB;
Widget fileButton, fileMenu, quit, load, loadMarkers, play,
  help, helpButton, helpMenu, helpBox, temp;
Widget ltas;
Widget exportButton, exportMenu, tocentroid, toamplitude, topitch, tonoise,
  tomarkers;
Widget printButton, printMenu, printpage, printpageFileBox, printspecFileBox,
  exportspecFileBox, printcorrFileBox, exportcorrFileBox;
Widget settingsButton, settingsMenu, displayButton, specButton, corrButton;
Widget analyButton, analyOkButton, analyCancelButton, analyLabel1, analyLabel2,
  analyRadio, analyForm, analyTextfield, analyTextfield2,
  analyToggle1, analyToggle2;
Widget displayOkButton, displayCancelButton, colorRadio;
Widget horiscroll, zoomBB, zoominB, zoomoutB, posL, playBB, playB, stopB,
  loopToggle;
Widget leftBB, leftLabSon, leftLabAmp, leftLabPitch, leftLabNoise;
Widget markersButton, markersMenu, addMarker, removeMarker;

Boolean fileDirty = FALSE;
double PI, TWOPI;

AFfilehandle infile,  outfile;
AFfilesetup in_AFsetup,  out_AFsetup;

int samps_per_frame=1;  /* Mono/stereo */
long compression,  filefmt,  bits_per_samp,  samp_type,  vers;
long framecnt;

signed char *megosc=NULL;
short int *megfreq=NULL, *megpitch=NULL, *megcentro=NULL, *megnoise=NULL,
      *megamp=NULL, *megnoise2=NULL;         /* Enormous arrays */
long horiz_num=0;	    /* Number of analysis frames */
double duration;		    /* Duration in secs */
int numchannels;	    /* Number of phase vocoder channels */
double binfreq;		    /* Frequency pr. bin */
int theheight, defdepth;
int si_valid=-1, marked=0, marking=0, cursoring=0;
double gain=1., afreq=440., threshold=0.05;
int smoothing=0, octave=3, realtiming=0, xold, cxold;

short int *framebuff, *mixbuff;
int *outbuff, *auxbuff;

int R=44100,  D,  N,  Nw,  N2,  I;
int Nw2, obank=0, noisest=0;
double P,  synt,  *Wanal,  *Wsyn,  *input,  *buffer,  *channel,  *output;
double *tempamp, *tempfreq, *last, *this;
double leftsec=0., sec_pr_scr=10., sec_pr_bin, mstart=0., mstop=960., curstid=5.;
double markertid=4., markers[9999];
int nummark=0, markering=-1, markerxold, removemark=0;

XGCValues values;
Window theRoot, theWindow;
Colormap theCmap;
Display *theDisplay;
int screen;
Pixmap bitmap, specbitmap, corrbitmap;
XColor gray0, gray25, gray50, gray75, gray100;
XColor red, orange, yellow, green, brown;
XColor darkblue, blue, lightblue;

GC theGC, showGC, funtheGC, funshowGC;
GC theGC0, theGC25, theGC50, theGC75, theGC100;
GC showGC0, showGC25, showGC50, showGC75, showGC100;
GC funcGC, gridGC, markerGC;

int xpos[51], ypos[51], numsquare=2, selected=0, funcpt;
int farge=0;
Pixmap pixelmap, playpixmap, noplaypixmap, stoppixmap, nostoppixmap;
char infil[200];

struct {
  int showgrid, showcentro, scroll, fastdraw;
  double buffsize;
} settings;


/* LPC routine. Only residual energy is used. */

double lpa(x, N, M)
double x[];
int N, M;
{
  int i, j;
  double s, at, a0, *rx, *rc, *b, rms=0.;

  rx = (double *)malloc((M+2)*sizeof(double));
  rc = (double *)malloc((M+2)*sizeof(double));
  b = (double *)malloc((M+2)*sizeof(double));

  for (i=0; i<N; i++) rms+=x[i]*x[i];
  rms=sqrt(rms/N);

  for (i=0; i<=M+1; i++)
    for (rx[i]=j=0; j<N-i; j++)
      rx[i]+=x[j]*x[i+j];
  b[0]=1.;
  b[1]=rc[0]=rx[0] != 0. ? -rx[1]/rx[0] : 0.;
  for (i=2; i<=M; i++) b[i]=0.;
  a0=rx[0]+rx[1]*rc[0];

  for (i=1; i<M; i++) {
    for (s=j=0; j<=i; j++)
      s+=rx[i-j+1]*b[j];
    rc[i]=a0!=0.?-s/a0:0.;
    for (j=1; j<=(i+1)>>1; j++) {
      at=b[j]+rc[i]*b[i-j+1];
      b[i-j+1]+=rc[i]*b[j];
      b[j]=at;
    }
    b[i+1]=rc[i];
    a0+=rc[i]*s;
  }
    
  free(rx);
  free(rc);
  free(b);
  return(rms==0.?0.:a0/rms);
}


/* FFT ROUTINES */

/* If forward is true, rfft replaces 2*N real data points in x with
   N complex values representing the positive frequency half of their
   Fourier spectrum, with x[1] replaced with the real part of the Nyquist
   frequency value.  If forward is false, rfft expects x to contain a
   positive frequency spectrum arranged as before, and replaces it with
   2*N real values.  N MUST be a power of 2. */

rfft( x, N, forward )
double x[]; int N, forward;
{
  double 	c1,c2,
  		h1r,h1i,
		h2r,h2i,
		wr,wi,
		wpr,wpi,
  		temp,
		theta;
  double 	xr,xi;
  int 		i,
		i1,i2,i3,i4,
		N2p1;
  static int 	first = 1;

    if ( first ) {
	PI = 4.*atan( 1. );
	TWOPI = 8.*atan( 1. );
	first = 0;
    }
    theta = PI/N;
    wr = 1.;
    wi = 0.;
    c1 = 0.5;
    if ( forward ) {
	c2 = -0.5;
	cfft( x, N, forward );
	xr = x[0];
	xi = x[1];
    } else {
	c2 = 0.5;
	theta = -theta;
	xr = x[1];
	xi = 0.;
	x[1] = 0.;
    }
    wpr = -2.*powf( sinf( 0.5*theta ), 2. );
    wpi = sinf( theta );
    N2p1 = (N<<1) + 1;
    for ( i = 0; i <= N>>1; i++ ) {
	i1 = i<<1;
	i2 = i1 + 1;
	i3 = N2p1 - i2;
	i4 = i3 + 1;
	if ( i == 0 ) {
	    h1r =  c1*(x[i1] + xr );
	    h1i =  c1*(x[i2] - xi );
	    h2r = -c2*(x[i2] + xi );
	    h2i =  c2*(x[i1] - xr );
	    x[i1] =  h1r + wr*h2r - wi*h2i;
	    x[i2] =  h1i + wr*h2i + wi*h2r;
	    xr =  h1r - wr*h2r + wi*h2i;
	    xi = -h1i + wr*h2i + wi*h2r;
	} else {
	    h1r =  c1*(x[i1] + x[i3] );
	    h1i =  c1*(x[i2] - x[i4] );
	    h2r = -c2*(x[i2] + x[i4] );
	    h2i =  c2*(x[i1] - x[i3] );
	    x[i1] =  h1r + wr*h2r - wi*h2i;
	    x[i2] =  h1i + wr*h2i + wi*h2r;
	    x[i3] =  h1r - wr*h2r + wi*h2i;
	    x[i4] = -h1i + wr*h2i + wi*h2r;
	}
	wr = (temp = wr)*wpr - wi*wpi + wr;
	wi = wi*wpr + temp*wpi + wi;
    }
    if ( forward )
	x[1] = xr;
    else
	cfft( x, N, forward );
}

/* cfft replaces double array x containing NC complex values
   (2*NC double values alternating real, imagininary, etc.)
   by its Fourier transform if forward is true, or by its
   inverse Fourier transform if forward is false, using a
   recursive Fast Fourier transform method due to Danielson
   and Lanczos.  NC MUST be a power of 2. */

cfft( x, NC, forward )
double x[]; int NC, forward;
{
  double 	wr,wi,
		wpr,wpi,
		theta,
		scale;
  int 		mmax,
		ND,
		m,
		i,j,
		delta;

    ND = NC<<1;
    bitreverse( x, ND );
    for ( mmax = 2; mmax < ND; mmax = delta ) {
	delta = mmax<<1;
	theta = TWOPI/( forward? mmax : -mmax );
	wpr = -2.*powf( sinf( 0.5*theta ), 2. );
	wpi = sinf( theta );
	wr = 1.;
	wi = 0.;
	for ( m = 0; m < mmax; m += 2 ) {
	 register double rtemp, itemp;
	    for ( i = m; i < ND; i += delta ) {
		j = i + mmax;
		rtemp = wr*x[j] - wi*x[j+1];
		itemp = wr*x[j+1] + wi*x[j];
		x[j] = x[i] - rtemp;
		x[j+1] = x[i+1] - itemp;
		x[i] += rtemp;
		x[i+1] += itemp;
	    }
	    wr = (rtemp = wr)*wpr - wi*wpi + wr;
	    wi = wi*wpr + rtemp*wpi + wi;
	}
    }

/* scale output */

    scale = forward ? 1./ND : 2.;
    { register double *xi=x, *xe=x+ND;
	while ( xi < xe )
	    *xi++ *= scale;
    }
}

/* bitreverse places double array x containing N/2 complex values
   into bit-reversed order */

bitreverse( x, N )
double x[]; int N;
{
  double 	rtemp,itemp;
  int 		i,j,
		m;

    for ( i = j = 0; i < N; i += 2, j += m ) {
	if ( j > i ) {
	    rtemp = x[j]; itemp = x[j+1]; /* complex exchange */
	    x[j] = x[i]; x[j+1] = x[i+1];
	    x[i] = rtemp; x[i+1] = itemp;
	}
	for ( m = N>>1; m >= 2 && j >= m; m >>= 1 )
	    j -= m;
    }
}


/* PHASE VOCODER INTERNALS */

/*
 * make balanced pair of analysis (A) and synthesis (S) windows;
 * window lengths are Nw, FFT length is N, synthesis interpolation
 * factor is I
 */
makewindows(A, S, Nw, N, I)
    double A[], S[] ; int Nw, N, I;
{
 int i ;
 double sum ;
/*
 * basic Hamming windows
 */
    for ( i = 0 ; i < Nw ; i++ )
	A[i] = S[i] = 0.54 - 0.46*cos( TWOPI*i/(Nw - 1) ) ;
/*
 * when Nw > N, also apply interpolating (sinc) windows to
 * ensure that window are 0 at increments of N (the FFT length)
 * away from the center of the analysis window and of I away
 * from the center of the synthesis window
 */
    if ( Nw > N ) {
     double x ;
     double PI = 4.*atan(1.) ;
/*
 * take care to create symmetrical windows
 */
	x = -(Nw - 1)/2. ;
	for ( i = 0 ; i < Nw ; i++, x += 1. )
	    if ( x != 0. ) {
		A[i] *= N*sinf( PI*x/N )/(PI*x) ;
		if ( I )
		    S[i] *= I*sinf( PI*x/I )/(PI*x) ;
	    }
    }
/*
 * normalize windows for unity gain across unmodified
 * analysis-synthesis procedure
 */
    for ( sum = i = 0 ; i < Nw ; i++ )
	sum += A[i] ;

    for ( i = 0 ; i < Nw ; i++ ) {
     double afac = 2./sum ;
     double sfac = Nw > N ? 1./afac : afac ;
	A[i] *= afac ;
	S[i] *= sfac ;
    }

    if ( Nw <= N && I ) {
	for ( sum = i = 0 ; i < Nw ; i += I )
	    sum += S[i]*S[i] ;
	for ( sum = 1./sum, i = 0 ; i < Nw ; i++ )
	    S[i] *= sum ;
    }
}

int shiftin(A,  N,  D, max, ch)
double A[];
int N, D;
double *max;
int ch;
{
    int i;
    double *Aptr = A, mmax=0.;
    
    if (si_valid<0) si_valid=N;
    i = N-D;
    while (i--) {
      *Aptr = *(Aptr+D);
      Aptr++;
    }
    if (si_valid==N) {
      short int *frames = framebuff;
      int sampsread=AFreadframes(infile, AF_DEFAULT_TRACK, frames, D);
      if (samps_per_frame==1) {
        si_valid = sampsread+N-D;
        while(sampsread--) { 
          *(Aptr) = (double) *(frames++) / 32768.0;
          if (fabs(*Aptr)>mmax) mmax=fabs(*Aptr); Aptr++;
        }
      } else {
        si_valid = sampsread+N-D;
        while(sampsread--) {
          *(Aptr) = (double) (*(frames+ch)) / 32768.0;
          frames+=samps_per_frame;
          if (fabs(*Aptr)>mmax) mmax=fabs(*Aptr); Aptr++;
        }
      }
    }
    if (si_valid<N) {
        for (i=si_valid; i<N; i++) A[i]=0.;
        si_valid-=D;
    }
    *max=mmax;
    return (si_valid<=0);
}

int readosc(int ch)
{
  int sampsread, i, base;
  short int frames[32768];

  base=ch*framecnt;
  AFseekframe(infile, AF_DEFAULT_TRACK, 0);
  do {
    sampsread=AFreadframes(infile, AF_DEFAULT_TRACK, frames, 8192);
    if (samps_per_frame==1) for (i=0; i<sampsread; i++)
      megosc[base+i]=(signed char)(frames[i]>>8);
    else for (i=0; i<sampsread; i++)
      megosc[base+i]=(signed char)(frames[i*samps_per_frame+ch]>>8);
    base+=8192;
  } while (sampsread>0);
}

/*
 * multiply current input I by window W (both of length Nw);
 * using modulus arithmetic, fold and rotate windowed input
 * into output array O of (FFT) length N according to current
 * input time n
 */
fold( I, W, Nw, O, N, n )
    double I[], W[], O[] ; int Nw, N, n ;
{
 
    int i;

    for ( i = 0; i < N; i++ )
	O[i] = 0.;

    while ( n < 0 )
	n += N;
    n %= N;
    for ( i = 0; i < Nw; i++ ) {
	O[n] += I[i]*W[i];
	if ( ++n == N )
	    n = 0;
    }
}


/* S is a spectrum in rfft format, i.e., it contains N real values
   arranged as real followed by imaginary values, except for first
   two values, which are real parts of 0 and Nyquist frequencies;
   convert first changes these into N/2+1 PAIRS of magnitude and
   phase values to be stored in output array C; the phases are then
   unwrapped and successive phase differences are used to compute
   estimates of the instantaneous frequencies for each phase vocoder
   analysis channel; decimation rate D and sampling rate R are used
   to render these frequency values directly in Hz. */

convert( S, C, N2, D, R )
double S[], C[]; int N2, D, R;
{
  static int 	first = 1;
  static double 	*lastphase,
		fundamental,
		factor;
  double 	phase,
		phasediff;
  int 		real,
		imag,
		amp,
		freq;
  double 	a,
		b;
  int 		i;

/* first pass: allocate zeroed space for previous phase
   values for each channel and compute constants */

    if ( first ) {
      first = 0;
      fvec(lastphase, N2+1);
      fundamental = (double) R/(N2<<1);
      factor = R/(D*TWOPI);
    } 

/* unravel rfft-format spectrum: note that N2+1 pairs of
   values are produced */

    for ( i = 0; i <= N2; i++ ) {
      imag = freq = ( real = amp = i<<1 ) + 1;
      a = ( i == N2 ? S[1] : S[real] );
      b = ( i == 0 || i == N2 ? 0. : S[imag] );

/* compute magnitude value from real and imaginary parts */

      C[amp] = hypotf( a, b );

/* compute phase value from real and imaginary parts and take
   difference between this and previous value for each channel */

      if ( C[amp] == 0. )
	phasediff = 0.;
      else {
	phasediff = ( phase = -atan2f( b, a ) ) - lastphase[i];
	lastphase[i] = phase;
	
/* unwrap phase differences */

	while ( phasediff > PI )
	  phasediff -= TWOPI;
	while ( phasediff < -PI )
	  phasediff += TWOPI;
      }

/* convert each phase difference to Hz */

      C[freq] = phasediff*factor + i*fundamental;
    }
}


/* unconvert essentially undoes what convert does, i.e., it
  turns N2+1 PAIRS of amplitude and frequency values in
  C into N2 PAIR of complex spectrum data (in rfft format)
  in output array S; sampling rate R and interpolation factor
  I are used to recompute phase values from frequencies */

unconvert( C, S, N2, I, R )
double C[], S[]; int N2, I, R;
{
  static int 	first = 1;
  static double 	*lastphase,
		fundamental,
		factor;
  int 		i,
		real,
		imag,
		amp,
		freq;
  double 	mag,
		phase;

/* first pass: allocate memory and compute constants */

    if ( first ) {
	first = 0;
	fvec(lastphase,N2+1);
	fundamental = (double) R/(N2<<1);
        factor = TWOPI*I/R;
    } 

/* subtract out frequencies associated with each channel,
   compute phases in terms of radians per I samples, and
   convert to complex form */

    for ( i = 0; i <= N2; i++ ) {
	imag = freq = ( real = amp = i<<1 ) + 1;
	if ( i == N2 )
	    real = 1;
	mag = C[amp];
	lastphase[i] += C[freq] - i*fundamental;
	phase = lastphase[i]*factor;
	S[real] = mag*cos( phase );
	if ( i != N2 )
	    S[imag] = -mag*sinf( phase );
    }
}


/*
 * input I is a folded spectrum of length N; output O and
 * synthesis window W are of length Nw--overlap-add windowed,
 * unrotated, unfolded input data into output O
 */
overlapadd( I, N, W, O, Nw, n )
    double I[], W[], O[] ; int N, Nw, n ; 
{
 int i ;
    while ( n < 0 )
	n += N ;
    n %= N ;
    for ( i = 0 ; i < Nw ; i++ ) {
	O[i] += I[n]*W[i] ;
	if ( ++n == N )
	    n = 0 ;
    }
}

void entropy(int ch)
{
  int i, j, amp;
  long point;
  double sum, scale;

  point=horiz_num*ch;
  for (i=0; i<horiz_num; i++) {
    scale=0.; sum=0.;
    for (j=0; j<numchannels; j++) {
      amp=megamp[point]*megamp[point];
      scale+=amp;
      sum+=(amp==0?0.:amp*log(amp));
      point++;
    }
    sum=sum/scale;
    megnoise2[i+ch*horiz_num]=100-(short int)(sum*5.);
  }
}

void SetWatchCursor(w)
Widget w;
{
  static Cursor watch = NULL;

  if(!watch)
          watch = XCreateFontCursor(XtDisplay(w),XC_watch);

  XDefineCursor(XtDisplay(w),XtWindow(w),watch);
  XmUpdateDisplay(w);
}

void ResetCursor(w)
Widget w;
{
  XUndefineCursor(XtDisplay(w),XtWindow(w));
  XmUpdateDisplay(w);
}

void drawscale()
{
  int i, j, x, y, sec, ch, dy;
  static char dashes[2]={1,4};
  char tall[10];

  for (ch=0; ch<samps_per_frame; ch++) {
    dy=ch*800;
    theGC=theGC0;
    XDrawLine(theDisplay,bitmap,theGC,40,theheight+20+dy,950,theheight+20+dy);
    XDrawLine(theDisplay,bitmap,theGC,40,theheight+20+dy,40,10+dy);
    for (i=0; i<=(int)(sec_pr_scr); i++) {
      x=(i-leftsec+floor(leftsec))*900./sec_pr_scr+50; sec=i+floor(leftsec);
      if (sec%10==0) XDrawLine(theDisplay,bitmap,theGC,x,theheight+10+dy,
        x,theheight+20+dy);
      if (sec%60==0) XDrawLine(theDisplay,bitmap,theGC,x,theheight+5+dy,
        x,theheight+20+dy);
      if (sec_pr_scr<960.) XDrawLine(theDisplay,bitmap,theGC,x,theheight+15+dy,
        x,theheight+20+dy);
      if ((sec_pr_scr<240.) && ((sec%10==0) || (sec_pr_scr<15.))) {
        sprintf(tall, "%2d:%02d", sec/60,sec%60);
        XDrawString(theDisplay,bitmap,theGC,x-29,theheight+33+dy,
          tall,strlen(tall));
      }
      else if (sec%60==0) {
        sprintf(tall, "%2d", sec/60);
        XDrawString(theDisplay,bitmap,theGC,x-15,theheight+33+dy,
          tall,strlen(tall));
      }
      if ((settings.showgrid) && ((sec_pr_scr<120.) || (sec%10==0))) {
        XSetLineAttributes(theDisplay, theGC, 0, LineOnOffDash, CapRound,
          JoinMiter); 
        XSetDashes(theDisplay, theGC, 0, dashes, 2);
        XDrawLine(theDisplay,bitmap,theGC,x,10+dy,x,800+dy);
        XSetLineAttributes(theDisplay, theGC, 0, LineSolid, CapRound,
          JoinMiter); 
      }
    }
    for (i=1; i<=(int)(numchannels*binfreq/1000.); i++) {
      y=theheight-(int)(i*1000./binfreq)+10+dy;
      XDrawLine(theDisplay,bitmap,theGC,35,y,45,y);
      sprintf(tall, "%2d", i);
      XDrawString(theDisplay,bitmap,theGC, 2,y+5,tall,strlen(tall));
    }
    XSetLineAttributes(theDisplay, theGC, 0, LineOnOffDash, CapRound,
      JoinMiter); 
    XSetDashes(theDisplay, theGC, 0, dashes, 2);
    for (i=0; i<4; i++)
      XDrawLine(theDisplay,bitmap,theGC,50,700-i*48+dy,950,700-i*48+dy);
    XSetLineAttributes(theDisplay, theGC, 0, LineSolid, CapRound,
      JoinMiter);
  }
}

void drawspecscale()
{
  int i, x, y, ch, dy;
  static char dashes[2]={1,4};
  char tall[10];

  for (ch=0; ch<samps_per_frame; ch++) {
    dy=(ch+1)*300;
    theGC=theGC0;
    XDrawLine(theDisplay,specbitmap,theGC,39,dy-40,39,dy-290);
    XDrawLine(theDisplay,specbitmap,theGC,39,dy-40,numchannels+39,dy-40);

    for (i=1; i<=(int)(numchannels*binfreq/1000.); i++) {
      x=39+(int)(i*1000./binfreq);
      XDrawLine(theDisplay,specbitmap,theGC,x,dy-35,x,dy-40);
      if (i%2==0) {
        sprintf(tall, "%2d", i);
        XDrawString(theDisplay,specbitmap,theGC,x-14,dy-20,tall,strlen(tall));
      }
    }
    XSetLineAttributes(theDisplay, theGC, 0, LineOnOffDash, CapRound,
      JoinMiter); 
    XSetDashes(theDisplay, theGC, 0, dashes, 2);
    for (i=1; i<11; i++)
      XDrawLine(theDisplay,specbitmap,theGC,39,dy-40-i*25,numchannels+39,
        dy-40-i*25);
    XSetLineAttributes(theDisplay, theGC, 0, LineSolid, CapRound,
      JoinMiter);
  }
}

void drawcorrscale()
{
  int i, j, x, y, sec, ch, dy;
  static char dashes[2]={1,4};
  char tall[10];
  double maxlagtime=0.1;

  XSetLineAttributes(theDisplay, theGC, 0, LineOnOffDash, CapRound,
    JoinMiter); 
  XSetDashes(theDisplay, theGC, 0, dashes, 2);
  for (i=-9; i<=9; i++)
    XDrawLine(theDisplay,corrbitmap,theGC,i*50+500,0,i*50+500,600);
  XSetLineAttributes(theDisplay, theGC, 0, LineSolid, CapRound,
    JoinMiter);
}



/* CALLBACK ROUTINES */

void RestoreSpec(w, client, call)
Widget w;
XtPointer client, call;
{
  XCopyArea(theDisplay, specbitmap, XtWindow(specSketchpad),
    DefaultGCOfScreen(XtScreen(sketchpad)), 0, 0, 500, 300, 0, 0);
}

void RedrawSpec(void)
{
  int i, j, lasty, val;
  long point;

  if (curstid>=duration) return;
  XFillRectangle(theDisplay, specbitmap, theGC100, 0, 0, 500, 300);
  SetWatchCursor(topLevel);

  drawspecscale();

  if (horiz_num) {
    point=numchannels*(long)(curstid/sec_pr_bin);
    for (i=0; i<numchannels; i++) {
/*      val=(int)(log(megamp[point+i]+1)*20.); */
      val=(int)(megamp[point+i]/50.)*gain;
      if (i>0) XDrawLine(theDisplay, specbitmap, theGC0, i+39, 260-lasty, i+40,
        260-val);
      lasty=val;
    }
  }
  RestoreSpec(NULL, NULL, NULL);
  ResetCursor(topLevel);
}

void RestoreCorr(w, client, call)
Widget w;
XtPointer client, call;
{
  XCopyArea(theDisplay, corrbitmap, XtWindow(corrSketchpad),
    DefaultGCOfScreen(XtScreen(sketchpad)), 0, 0, 1000, 600, 0, 0);
}

void DrawOneCorr(c1, c2, ypos)
int c1, c2, ypos;
{
  int lag, maxlag, i, grafx, grafold, s1, s2, oldy, c0=0;
  long start, stop, p1, p2, offset1, offset2;
  double sum, maxlagtime, maxval, y, peaksec;
  XSegment list0[LSIZE];
  double ytab[2001];
  char tall[20];

  for (i=0; i<=2000; i++) ytab[i]=0.; 
  maxlagtime=0.1; maxlag=R*maxlagtime;
  start=mstart*R; stop=mstop*R;
  grafold=0; offset1=c1*framecnt; offset2=c2*framecnt;
  for (lag=-maxlag; lag<maxlag; lag++) {
    sum=0.;
    for (p1=start; p1<stop; p1++) {
      s1=megosc[offset1+p1];
      p2=p1-lag;
      if ((p2<0) || (p2>=framecnt)) s2=0;
        else s2=megosc[offset2+p2];
      sum+=s1*s2;
    }
    grafx = (int)(1000+lag*(double)1000./maxlag);
    y=sum/maxlag;
    if (fabs(y)>fabs(ytab[grafx])) ytab[grafx]=y;
  }
  maxval=0.;
  for (i=0; i<=2000; i++) if (fabs(ytab[i])>maxval) maxval=fabs(ytab[i]);
  for (i=0; i<=2000; i++) ytab[i]/=maxval;
  for (i=0; i<2000; i++) XDrawLine(theDisplay, corrbitmap, theGC0,
      i/2, 90-90.*ytab[i]+ypos*200,(i+1)/2,90-90.*ytab[i+1]+ypos*200);
  maxval=0.;
  for (i=0; i<=2000; i++) if (ytab[i]>maxval) {
    maxval=ytab[i]; peaksec=maxlagtime*(i-1000)/1000.;
  }
  sprintf(tall, "Ch %d+%d: Max @ %.1f ms", c1+1, c2+1, peaksec*1000.);
  XDrawString(theDisplay,corrbitmap,theGC,10,ypos*200+15,
     tall,strlen(tall));
}

void RedrawCorr(void)
{
  int i, j, lasty, val;
  long point;

  if (curstid>=duration) return;
  XFillRectangle(theDisplay, corrbitmap, theGC100, 0, 0, 1000, 600);
  SetWatchCursor(topLevel);

  drawcorrscale();

  if ((horiz_num) && (samps_per_frame>1) && (mstop>mstart)) {
    if (samps_per_frame==2) DrawOneCorr(0,1,0);
    else if (samps_per_frame==4) {
      DrawOneCorr(0,1,0);
      DrawOneCorr(0,2,1);
      DrawOneCorr(0,3,2);
      DrawOneCorr(1,2,3);
      DrawOneCorr(1,3,4);
      DrawOneCorr(2,3,5);
    }
  }
  RestoreCorr(NULL, NULL, NULL);
  ResetCursor(topLevel);
}

void RestoreWin(w, client, call)
Widget w;
XtPointer client, call;
{
  int xs, xe, i, th;

  th=800*samps_per_frame;
  XCopyArea(theDisplay, bitmap, XtWindow(sketchpad),
    DefaultGCOfScreen(XtScreen(sketchpad)), 0, 0, 1000, th, 0, 0);
  if (marked) {
    XSetForeground(theDisplay, markerGC, red.pixel);
    xs=(int)(900.*(mstart-leftsec)/sec_pr_scr)+50;
    xe=(int)(900.*(mstop-leftsec)/sec_pr_scr)+50;
    for (i=0; i<samps_per_frame; i++)
      XFillRectangle(theDisplay, XtWindow(sketchpad), markerGC, xs, 450+i*800,
        xe-xs, 350);
  }
  xs=(int)(900.*(curstid-leftsec)/sec_pr_scr)+50;
  XSetFunction(theDisplay,markerGC,GXxor);
  XSetForeground(theDisplay, markerGC, green.pixel);
  XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, xs, 30, xs, th);
  cxold=xs;

  XSetForeground(theDisplay, markerGC, red.pixel);
  for (i=0; i<nummark; i++) {
    xs=(int)(900.*(markers[i]-leftsec)/sec_pr_scr)+50;
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, xs, 30, xs, th);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, xs-5, 25, xs+5, 25);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, xs-5, 25, xs, 30);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, xs+5, 25, xs, 30);
  }

  if (realtiming)
    XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,xold,30,xold,th);
}

void RedrawWin(void)
{
  int i, j, grafx, grafold=-1, max=-999, min=999, br, oldy=0, th;
  long point;
  XSegment list0[LSIZE],list1[LSIZE],list2[LSIZE],list3[LSIZE];
  int c0=0,c1=0,c2=0,c3=0, c, dy;
  double fact, fact2, lefse;

  th=800*samps_per_frame;
  XFillRectangle(theDisplay, bitmap, theGC100, 0, 0, 1000, th);
  SetWatchCursor(topLevel);

  if (horiz_num>0) drawscale();

  fact=900./sec_pr_scr;
  br=fact*sec_pr_bin;

 for (c=0; c<samps_per_frame; c++) {
  point=c*horiz_num*numchannels; dy=c*800;
  for (j=0; j<horiz_num; j++) {
    grafx = (int)(fact*(j*sec_pr_bin-leftsec))+50;
    if (grafx>950) break;
    if ((grafx!=grafold) && (grafx>50)) {
      for (i=0; i<numchannels; i++) {
        if (megamp[point]*gain>120) {
          int y = theheight-(int)(megfreq[point]/binfreq)+10+dy;
          if (megamp[point]*gain>1000) {
             ADD_LINE(grafx,y,grafx+br,y,list0,c0,theGC0);
           } else if (megamp[point]*gain>500){ 
             ADD_LINE(grafx,y,grafx+br,y,list1,c1,theGC25);
           } else if (megamp[point]*gain>250) {
             ADD_LINE(grafx,y,grafx+br,y,list2,c2,theGC50);
           } else {
             ADD_LINE(grafx,y,grafx+br,y,list3,c3,theGC75);
           }
         }
         point++;
       }

      grafold=grafx;
    } else point+=numchannels;
  }
  FLUSH_LINES(list0,c0,theGC0);
  FLUSH_LINES(list1,c1,theGC25);
  FLUSH_LINES(list2,c2,theGC50);
  FLUSH_LINES(list3,c3,theGC75);

  grafold=0; lefse=fact*leftsec; fact2=fact/R; point=framecnt*c;
  if (sec_pr_scr>2.)
   for (j=(int)(leftsec*R); j<framecnt; j++) {
    grafx = (int)(fact2*j-lefse);
    if (grafx>900) break;
    if (megosc[j+point]>max) max=megosc[j+point];
    if (megosc[j+point]<min) min=megosc[j+point];
    if (grafx!=grafold) {
      ADD_LINE(grafx+50,500-max/3+dy,grafx+50,500-min/3+dy,list0,c0,theGC0);
      grafold=grafx; max=-999; min=999;
    }
  } else for (j=(int)(leftsec*R); j<framecnt; j++) {
    grafx = (int)(fact*((double)j/R-leftsec));
    if (grafx>900) break;
    ADD_LINE(grafold+50,500-oldy/3+dy,grafx+50,500-megosc[j+point]/3+dy,
      list0,c0,theGC0);
    grafold=grafx; oldy=megosc[j+point];
  }
  FLUSH_LINES(list0,c0,theGC0);

  oldy=0; grafold=50; point=horiz_num*c;
  for (j=0; j<horiz_num; j++) {
    grafx = (int)(fact*(j*sec_pr_bin-leftsec))+50;
    if (grafx>950) break;
    if ((grafx!=grafold) && (grafx>50)) {
      if ((megpitch[j+point]>0) && (megpitch[j+point]-oldy<50) && (oldy>0)) {
        ADD_LINE(grafold,700-oldy+dy,grafx,700-megpitch[j+point]+dy,
          list0,c0,theGC0);
      }
      grafold=grafx; oldy=megpitch[j+point];
    }
  }
  FLUSH_LINES(list0,c0,theGC0);

  oldy=0; grafold=50;
  for (j=0; j<horiz_num; j++) {
    grafx = (int)(fact*(j*sec_pr_bin-leftsec))+50;
    if (grafx>950) break;
    if ((grafx!=grafold) && (grafx>50)) {
      if (((noisest?megnoise2[j+point]:megnoise[j+point])>0) && (oldy>0)) {
        ADD_LINE(grafold,790-oldy/2+dy,grafx,
          790-(noisest?megnoise2[j+point]:megnoise[j+point])/2+dy,list0,c0,theGC0);
      }
      grafold=grafx; oldy=(noisest?megnoise2[j+point]:megnoise[j+point]);
    }
  }
  FLUSH_LINES(list0,c0,theGC0);

  if (settings.showcentro) {
    oldy=0; grafold=50;
    for (j=0; j<horiz_num; j++) {
      grafx = (int)(fact*(j*sec_pr_bin-leftsec))+50;
      if (grafx>950) break;
      if ((grafx!=grafold) && (grafx>50)) {
        int y = theheight-(int)(megcentro[j+point]/binfreq)+10;
        if ((y<theheight+10) && (megcentro[j+point]>10)
          &&(oldy>10)&&(oldy<theheight+10)) {
            ADD_LINE(grafold,oldy+dy,grafx,y+dy,list0,c0,funcGC);
        }
        grafold=grafx; oldy=y;
      }
    }
    FLUSH_LINES(list0,c0,funcGC);
  }
 }
  RestoreWin(NULL,NULL,NULL); ResetCursor(topLevel);
}

static void Keydown(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  KeySym keysym;
  char key_string[1];
  int key_string_len, i;

  key_string_len=XLookupString((XKeyEvent *) event, key_string,
    sizeof key_string, &keysym, (XComposeStatus *) NULL);
  if ((selected>1) && (selected<numsquare) &&
    ((keysym==XK_BackSpace) || (keysym==XK_Delete))) {
      for (i=selected; i<numsquare; i++) {
        xpos[i]=xpos[i+1]; ypos[i]=ypos[i+1];
      }
      numsquare--; selected=0;
      RestoreWin(NULL,NULL,NULL);
    }
}

static void AreaClick(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y;
  double tid;

  XButtonEvent *motion=(XButtonEvent *)event;
  x=motion->x; y=motion->y;

  if (realtiming) return;
  if ((x<50) || (x>950)) return;
  tid=((x-50)/900.)*sec_pr_scr+leftsec;
  if (marked) { marked=0; mstart=0.; mstop=960.; RestoreWin(NULL,NULL,NULL); }
  mstart=tid; xold=x; marking=1;
}

static void AreaDrop(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  XButtonEvent *motion=(XButtonEvent *)event;
  int x=motion->x;
  double tid;

  if (realtiming) return;
  if (marking==0) return;
  if (x<50) x=50; if (x>950) x=950;
  tid=((x-50)/900.)*sec_pr_scr+leftsec;

  mstop=tid;
  if (mstop<mstart) { mstop=mstart; mstart=tid; }
  if (mstart!=mstop) marked=1; else { marked=0; RestoreWin(NULL,NULL,NULL); }
  marking=0;
}

static void AreaDrag(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, c;
  XButtonEvent *motion=(XButtonEvent *)event;
  double tid;

  if (realtiming) return;
  if (marking==0) return;
  x=motion->x; y=motion->y;
  if (x<50) x=50; if (x>950) x=950;
  tid=((x-50)/900.)*sec_pr_scr+leftsec;
  XSetForeground(theDisplay, markerGC, red.pixel);
  XSetFunction(theDisplay,markerGC,GXxor);
  if (x>xold) for (c=0; c<samps_per_frame; c++)
    XFillRectangle(theDisplay, XtWindow(sketchpad), markerGC, xold, 250+c*800,
      x-xold, 550);
  else for (c=0; c<samps_per_frame; c++)
    XFillRectangle(theDisplay, XtWindow(sketchpad), markerGC, x, 250+c*800,
      xold-x, 550);
  xold=x; mstop=tid;
}

void notnav(double pitch, char *navn)
{
  char *n, *q;

  switch((int)(pitch+0.5) % 12) {
    case 0: n = strdup("C"); break;
    case 1: n = strdup("C#"); break;
    case 2: n = strdup("D"); break;
    case 3: n = strdup("D#"); break;
    case 4: n = strdup("E"); break;
    case 5: n = strdup("F"); break;
    case 6: n = strdup("F#"); break;
    case 7: n = strdup("G"); break;
    case 8: n = strdup("G#"); break;
    case 9: n = strdup("A"); break;
    case 10: n = strdup("A#"); break;
    case 11: n = strdup("B"); break;
  }
  if (pitch-(int)(pitch) < -0.33) q = strdup("-");
  else if (pitch-(int)(pitch) < 0.33) q = strdup("");
  else q = strdup("+");
  sprintf(navn, "%s%d%s", n, (int)(pitch+0.5)/12+3, q);
}

void Labelen(float tid, int y)
{
  char streng[100], notnavn[10];
  XmString st;
  int point=(int)(tid/sec_pr_bin)*numchannels;
  int c=0;

  if (y>800) { y-=800; c=1; point+=horiz_num*numchannels; }
  if ((horiz_num==0) || (tid>=duration))
    sprintf(streng,"Time: %6.3fs", tid);
  else if ((y>0) && (y<theheight+10)) {
    sprintf(streng,"T=%6.3fs  Frq=%dHz", tid,
      megfreq[point+theheight+11-y]);
  } else if ((y<543) && (y>457)) {
    sprintf(streng,"T=%6.3fs  Samp=%5.3f", tid,
      megosc[(int)(tid*R)]/128.+c*framecnt);
  } else if ((y<700) && (y>556)) {
    notnav(megpitch[(int)(tid/sec_pr_bin)+c*horiz_num]/4., notnavn);
    sprintf(streng,"T=%6.3fs  Ptch=%s", tid, notnavn);
  } else sprintf(streng,"Time: %6.3fs", tid);
  st = XmStringCreateLocalized(streng);  
  XtVaSetValues(posL, XmNlabelString, st, NULL); XtFree(st);
}

static void CursorClick(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, i, j, xs, th;
  double tid;

  XButtonEvent *motion=(XButtonEvent *)event;
  x=motion->x; y=motion->y;

  if (realtiming) return;
  if ((x<50) || (x>950)) return;
  tid=((x-50)/900.)*sec_pr_scr+leftsec;
  markering=-1;

  for (i=0; i<nummark; i++) {
    xs=(int)(900.*(markers[i]-leftsec)/sec_pr_scr)+50;
    if ((x>xs-5) && (x<xs+5) && (y>24) && (y<31)) {
      markertid=tid; markerxold=xs; markering=i;
      XSetForeground(theDisplay, markerGC, red.pixel);
      XSetFunction(theDisplay,markerGC,GXxor);
      Labelen(tid, y);
      if (removemark) {
        removemark=0; markering=-1;
        for (j=i; j<nummark-1; j++) markers[j]=markers[j+1];
        nummark--; RestoreWin(NULL,NULL,NULL);
      }
      break;
    }
  }
  th=800*samps_per_frame;
  if (markering==-1) {
    XSetForeground(theDisplay, markerGC, green.pixel);
    XSetFunction(theDisplay,markerGC,GXxor);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, cxold, 30, cxold, th);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, x, 30, x, th);
    curstid=tid; cxold=x; cursoring=1;
    Labelen(curstid, y);
  }
}

static void CursorDrop(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  XButtonEvent *motion=(XButtonEvent *)event;
  int x=motion->x;
  int th;
  double tid;

  if (realtiming) return;
  if (x<50) x=50; if (x>950) x=950;
  tid=((x-50)/900.)*sec_pr_scr+leftsec;

  th=800*samps_per_frame;
  if (cursoring) {
    XSetForeground(theDisplay, markerGC, green.pixel);
    XSetFunction(theDisplay,markerGC,GXxor);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, cxold,30,cxold,th);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, x, 30, x, th);
    curstid=tid;
    cursoring=0;
    cxold=x;
    if (XtIsManaged(specForm)) RedrawSpec();
  } else if (markering>-1) {
    XSetForeground(theDisplay, markerGC, red.pixel);
    XSetFunction(theDisplay,markerGC,GXxor);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC,
      markerxold, 30, markerxold, th);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC,
      markerxold-5, 25, markerxold+5, 25);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC,
      markerxold-5, 25, markerxold, 30);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC,
      markerxold+5, 25, markerxold, 30);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, x, 30, x, th);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, x-5, 25, x+5, 25);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, x-5, 25, x, 30);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, x+5, 25, x, 30);
    markertid=tid; markers[markering]=tid;
    markering=-1; markerxold=x;
  }
}

static void CursorDrag(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, th;
  XButtonEvent *motion=(XButtonEvent *)event;
  double tid;

  if (realtiming) return;

  x=motion->x; y=motion->y;
  if (x<50) x=50; if (x>950) x=950;
  tid=((x-50)/900.)*sec_pr_scr+leftsec;
  th=800*samps_per_frame;
  if (cursoring) {
    XSetForeground(theDisplay, markerGC, green.pixel);
    XSetFunction(theDisplay,markerGC,GXxor);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, cxold,30,cxold,th);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, x, 30, x, th);
    cxold=x; curstid=tid;
    Labelen(tid, y);
    if (XtIsManaged(specForm)) RedrawSpec();
  } else {
    XSetForeground(theDisplay, markerGC, red.pixel);
    XSetFunction(theDisplay,markerGC,GXxor);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC,
      markerxold, 30, markerxold, th);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC,
      markerxold-5, 25, markerxold+5, 25);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC,
      markerxold-5, 25, markerxold, 30);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC,
      markerxold+5, 25, markerxold, 30);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, x, 30, x, th);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, x-5, 25, x+5, 25);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, x-5, 25, x, 30);
    XDrawLine(theDisplay, XtWindow(sketchpad), markerGC, x+5, 25, x, 30);
    markertid=tid; markerxold=x;
    Labelen(tid, y);
    if (XtIsManaged(specForm)) RedrawSpec();   
  }
}

void Cancel(w, client, call)
Widget w;
XtPointer client, call;
{
  Widget dialog=(Widget) client;
  XtUnmanageChild(dialog);
}

void ShowHelp(w, client, call)
Widget w;
XtPointer client, call;
{
  Widget dialog=(Widget) client;
  XtManageChild(dialog);
}

void Quit(w, client, call)
Widget w;
XtPointer client, call;
{
  exit(0);
}

void Load(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(loadFileBox);
}

void LoadOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  char *filename;
  XPoint list0[LSIZE],list1[LSIZE],list2[LSIZE],list3[LSIZE];
  int c0=0,c1=0,c2=0,c3=0, fd, c;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  int in=-Nw,  i, j, eof=0, grafx, grafold=-1, maxbin, maxbin2, note, pos;
  long point;
  double max, maxamp, maxamp2, freq, freq2, fnote, centro, norm, noise;

  XtUnmanageChild(loadFileBox); XmUpdateDisplay(w);
  XmStringGetLtoR(calldata->value, charset, &filename);

  fd=open(filename,O_RDONLY);
  if (fd==-1) {
    XtManageChild(fileWarning);
    XtFree(filename);
    return;
  }
  //AFopenfd funker ikke p linux
  close(fd);
  in_AFsetup=NULL;
  //infile=AFopenfd(fd, "r", in_AFsetup);
  infile=AFopenfile(filename,"rb", in_AFsetup);
  if (infile==NULL) {
    close(fd);
    XtManageChild(fileWarning);
    XtFree(filename);
    return;
  }

  SetWatchCursor(topLevel);
  strcpy(infil,filename);

  samps_per_frame   = AFgetchannels(infile, AF_DEFAULT_TRACK);
  R                 = (int)AFgetrate(infile, AF_DEFAULT_TRACK);
  compression       = AFgetcompression(infile, AF_DEFAULT_TRACK);
  filefmt           = AFgetfilefmt(infile, &vers);   
  AFgetsampfmt(infile, AF_DEFAULT_TRACK, &samp_type, &bits_per_samp);
  framecnt          = AFgetframecnt(infile,  AF_DEFAULT_TRACK);

  horiz_num = (framecnt/D)+4;
  if (numchannels*horiz_num*samps_per_frame > 160000000) {
    ResetCursor(topLevel);
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    XtFree(filename);
    return;
  } 
  duration = (double)framecnt/R;
  binfreq = (double)R/N; sec_pr_bin=(double)D/R;

  if (megfreq!=NULL) free(megfreq);
  if (megamp!=NULL) free(megamp);
  if (megosc!=NULL) free(megosc);
  if (megpitch!=NULL) free(megpitch);
  if (megcentro!=NULL) free(megcentro);
  if (megnoise!=NULL) free(megnoise);
  if (megnoise2!=NULL) free(megnoise2);
  megfreq=megamp=megpitch=megcentro=megnoise=megnoise2=NULL; megosc=NULL;
  svec(megfreq, numchannels*horiz_num*samps_per_frame);
  if (megfreq==NULL) {
    ResetCursor(topLevel);
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    XtFree(filename);
    return;
  }
  svec(megamp, numchannels*horiz_num*samps_per_frame);
  if (megamp==NULL) {
    ResetCursor(topLevel);
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    XtFree(filename);
    return;
  }
  bvec(megosc, framecnt*samps_per_frame);
  if (megosc==NULL) {
    ResetCursor(topLevel);
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    XtFree(filename);
    return;
  }
  svec(megpitch, horiz_num*samps_per_frame);
  if (megpitch==NULL) {
    ResetCursor(topLevel);
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    XtFree(filename);
    return;
  }
  svec(megcentro, horiz_num*samps_per_frame);
  if (megcentro==NULL) {
    ResetCursor(topLevel);
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    XtFree(filename);
    return;
  }
  svec(megnoise, horiz_num*samps_per_frame);
  if (megnoise==NULL) {
    ResetCursor(topLevel);
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    XtFree(filename);
    return;
  }
  svec(megnoise2, horiz_num*samps_per_frame);
  if (megnoise2==NULL) {
    ResetCursor(topLevel);
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    XtFree(filename);
    return;
  }

  XtVaSetValues(frame, XmNheight, samps_per_frame*800+64, NULL);
  XtVaSetValues(sketchpad, XmNheight, samps_per_frame*800, NULL);

 for (c=0; c<samps_per_frame; c++) {
  AFseekframe(infile, AF_DEFAULT_TRACK, 0);
  point=c*horiz_num*numchannels; j=c*horiz_num;
  eof=0; si_valid=-1;
  while (!eof) {
    in+=D;
    eof=shiftin(input,  Nw,  D, &max, c);
    fold(input,  Wanal,  Nw,  buffer,  N,  in);
    noise=10000.*lpa(buffer, N2, 10);
    if (noise>100.) noise=100.;
    rfft(buffer,  N2,  FORWARD);
    convert(buffer,  channel,  N2,  D,  R);
    for (i=0; i<numchannels; i++) {
	megamp[point]=(short int)(channel[i+i]*60000000.);
	megfreq[point]=(short int)channel[i+i+1];
        point++;
    }

    maxamp=0.; maxbin=1;
    for (i=2; i<30; i++) if (channel[i+i]>maxamp) {
      maxamp=channel[i+i]; maxbin=i;
    }
    freq=channel[maxbin*2+1];
    freq2=0.;
    if ((freq<60.) || (max<threshold)) freq=0.; else {
      maxamp2=0.; maxbin2=1;
      for (i=2; i<maxbin-1; i++) if (channel[i+i]>maxamp2) {
        maxamp2=channel[i+i]; maxbin2=i;
      }
      freq2=channel[maxbin2*2+1];
      if ((fabs(freq/freq2-2.)<0.1) && (maxamp/maxamp2<16.)) freq=freq2;
      else if ((fabs(freq/freq2-3.)<0.1) && (maxamp/maxamp2<16.)) freq=freq2;
      else if ((fabs(freq/freq2-1.5)<0.1) && (maxamp/maxamp2<16.)) freq=freq2/2.;
    }
    if (freq>0.) {
      fnote=12.*log(freq/afreq)/log(2.)+57.; fnote-=octave*12.;
      if ((fnote>36.) || (fnote<0.)) fnote=0.;
    } else fnote=0.;
    megpitch[j]=(short int)(fnote*4.+0.5);

    megnoise[j]=(short int)noise;

    centro=0.; norm=0.;
    for (i=1; i<numchannels-1; i++) {
      centro+=channel[i+i]*channel[i+i+1];
      norm+=channel[i+i];
    }
    centro=(norm<=0.?0.:centro/norm);
    if (max<0.01) centro=0.;
    megcentro[j++]=(short int)(centro);
  }

  readosc(c);
  entropy(c);
 }
  RedrawWin();
  AFclosefile(infile);
  XtFree(filename);
  fileDirty = FALSE;
  ResetCursor(topLevel);
}

void LoadMarkers(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(loadMarkersFileBox);
}

void LoadMarkersOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  char *filename;
  FILE *fil;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  float val;

  XtUnmanageChild(loadMarkersFileBox); XmUpdateDisplay(w);
  XmStringGetLtoR(calldata->value, charset, &filename);

  fil=fopen(filename,"r");
  if (fil==NULL) {
    XtManageChild(fileWarning);
    XtFree(filename);
    return;
  }

  nummark=0;
  while (!feof(fil)) {
    fscanf(fil, "%f", &val);
    if (feof(fil)) break;
    if (nummark>=9999) break;
    markers[nummark++] = val;
  }

  fclose(fil);
  XtFree(filename);
  RestoreWin(NULL, NULL, NULL);
}

void Play(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(playFileBox);
}

void PlayOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename, streng[100];
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

  XmStringGetLtoR(calldata->value, charset, &filename);
  strcpy(streng,"sfplay ");
  strcat(streng, filename);
  strcat(streng," &");
  system(streng);

  XtFree(filename);
}

void Realtime(w, client, call)
Widget w;
XtPointer client, call;
{
#ifndef NOAUDIO
  int i, j, sampsread, sample, x, fill, buffwarning=0, t, outsamp, *pek, val;
  int rcode;
  double time, blocklen=1024./R, timeceil, gtime;
  ALport audioport;
  ALconfig config;
  long pvbuf[8];
  char buf[50], *filename;
#ifdef LINUX  
  long tempBufOut [4] = {0,0,0,0};
#endif  

  if (realtiming) return;

  realtiming=1; xold=-999999;
  XtVaSetValues(playB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,playpixmap,NULL);
  XSetFunction(theDisplay, markerGC, GXxor);
  XSetForeground(theDisplay, markerGC, red.pixel);

  pvbuf[0] = AL_OUTPUT_RATE;  pvbuf[1] = R;
  pvbuf[2] = AL_INPUT_RATE;   pvbuf[3] = R;
  pvbuf[4] = AL_CHANNEL_MODE;
  pvbuf[5]=AL_STEREO;
  ALsetparams(AL_DEFAULT_DEVICE, pvbuf, 6);
  config=ALnewconfig();
  ALsetqueuesize(config, (int)(R*settings.buffsize*2));
  ALsetchannels(config, 2);
  audioport=ALopenport("Realtime Mix", "w", config);

  infile=AFopenfile(infil,"rb",in_AFsetup);
  if (infile==NULL) {
    printf("Couldn't open %s\n",infil);
    return;
  }

  do {
    time=0.; if (marked) time=mstart;
    t=(int)(time*R/1024.); timeceil=time+blocklen;

    if (marked) AFseekframe(infile, AF_DEFAULT_TRACK,(int)(mstart*R));
      else AFseekframe(infile, AF_DEFAULT_TRACK,0);

    do {
      i=2047;
      do *(outbuff+i)=0; while (i--);
      i=1023; do *(auxbuff+i)=0; while (i--);
      sampsread=AFreadframes(infile, AF_DEFAULT_TRACK, framebuff, 1024);;
      if (samps_per_frame==1) {
        for (j=0; j<sampsread; j++) {
          *(auxbuff+j)+=(int)(*(framebuff+j));
        }
        for (j=0; j<sampsread; j++) {
          pek=outbuff+j+j;
          *pek    +=*(framebuff+j);
          *(pek+1)+=*(framebuff+j);
        }
      } else {
        for (j=0; j<1024; j++) {
          *(auxbuff+j)+=(int)(*(framebuff+j+j)+*(framebuff+j+j+1))>>1;
        }
        for (j=0; j<sampsread; j++) {
          pek=outbuff+j+j;
          *pek    +=*(framebuff+j+j);
          *(pek+1)+=*(framebuff+j+j+1);
        }
      }
/*    do_effect(); */
      for (i=0; i<2048; i++) {
        outsamp=(*(outbuff+i)+*(auxbuff+(i>>1)))>>1;
        *(mixbuff+i)=(short int)outsamp;
      }
      fill=ALgetfilled(audioport);
//#ifdef LINUX
//    ALwritesamps(audioport,tempBufOut,2);
//#else      
      ALwritesamps(audioport, mixbuff, 2048);
//#endif      
      gtime=time-(double)fill/R/2;
      if (marked) while (gtime<mstart) gtime+=mstop-mstart;
      x=(int)((gtime-leftsec)*900./sec_pr_scr)+50;
      if (x!=xold) {
        if (x<-10000) x=-10000; if (x>10000) x=10000;
        XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,xold,30,xold,800);
        XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,x,30,x,800);
        xold=x;
      }
      if (buffwarning==0) {
        if (fill<3000) {
          buffwarning=1;
          XtVaSetValues(stopB,XmNlabelPixmap,stoppixmap,NULL);
        } 
      } else if (fill>3000) {
        buffwarning=0;
        XtVaSetValues(stopB,XmNlabelPixmap,nostoppixmap,NULL);
      }
      XFlush(theDisplay);
      if (settings.scroll) {
        if ((x>=1000) || (x<0)) {
          leftsec=floor(time);
          if (leftsec+sec_pr_scr>960.) leftsec=960.-sec_pr_scr;
          RedrawWin();
          XtVaSetValues(horiscroll,XmNvalue,(int)(leftsec*10.),NULL);
        }
      }
      time+=blocklen; timeceil+=blocklen;
      if (XtAppPending(app_context) && ((fill>(int)(settings.buffsize*R/2.))
        || (t%10==0))) {
        XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
      }
      t++;
    } while ((sampsread==1024) && realtiming &&
      ((marked==0) || (time<mstop)));
  } while ((XmToggleButtonGetState(loopToggle)) && realtiming);

  while (fill=ALgetfilled(audioport)) {
#ifdef LINUX
    ALwritesamps(audioport,tempBufOut,2);
#endif  
    gtime=time-(double)fill/R/2;
    if (marked) while (gtime<mstart) gtime+=mstop-mstart;
    x=(int)((gtime-leftsec)*900./sec_pr_scr)+50;
    if (x!=xold) {
      XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,xold,30,xold,800);
      XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,x,30,x,800);
      xold=x; XFlush(theDisplay);
    }
  }

  AFclosefile(infile);
  ALcloseport(audioport);
  realtiming=0;

  XtVaSetValues(playB,XmNlabelPixmap,noplaypixmap,NULL);
  XtVaSetValues(stopB,XmNlabelPixmap,nostoppixmap,NULL);

  RestoreWin(NULL,NULL,NULL);
#endif /*NOAUDIO*/
}

void StopRealT(w, client, call)
Widget w;
XtPointer client, call;
{
  realtiming=0;
}

void Tocentroid(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(tocentroidFileBox);
}

void TocentroidOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *centroidfile;
  int i;
  double tid;

  XmStringGetLtoR(calldata->value, charset, &filename);
  centroidfile=fopen(filename,"w");
  if (centroidfile==NULL) {
    XtManageChild(centroidWarning);
    XtFree(filename);
    return;
  }

  for (i=0; i<horiz_num; i++) {
    tid=i*sec_pr_bin;
    if ((tid>=mstart) && (tid<mstop))
      fprintf(centroidfile,"%f\t%d\n",tid,megcentro[i]);
  }

  fclose(centroidfile);
  XtFree(filename);
  XtUnmanageChild(tocentroidFileBox);
}

void Toamplitude(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(toamplitudeFileBox);
}

void ToamplitudeOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *amplitudefile;
  int i, j;
  long point=0;
  double sum, tid;

  XmStringGetLtoR(calldata->value, charset, &filename);
  amplitudefile=fopen(filename,"w");
  if (amplitudefile==NULL) {
    XtManageChild(amplitudeWarning);
    XtFree(filename);
    return;
  }

  for (i=0; i<horiz_num; i++) {
    tid=i*sec_pr_bin;
    sum=0.;
    point++;
    for (j=1; j<numchannels; j++) sum+=megamp[point++];
    if ((tid>=mstart) && (tid<mstop))
      fprintf(amplitudefile,"%f\t%.2f\n", tid, (float)sum/1000.);
  }

  fclose(amplitudefile);
  XtFree(filename);
  XtUnmanageChild(toamplitudeFileBox);
}

void Topitch(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(topitchFileBox);
}

void TopitchOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *pitchfile;
  int i;
  double tid;

  XmStringGetLtoR(calldata->value, charset, &filename);
  pitchfile=fopen(filename,"w");
  if (pitchfile==NULL) {
    XtManageChild(pitchWarning);
    XtFree(filename);
    return;
  }

  for (i=0; i<horiz_num; i++) {
    tid=i*sec_pr_bin;
    if ((tid>=mstart) && (tid<mstop))
      fprintf(pitchfile,"%f\t%.2f\n", tid, megpitch[i]/4.);
  }

  fclose(pitchfile);
  XtFree(filename);
  XtUnmanageChild(topitchFileBox);
}

void Tonoise(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(tonoiseFileBox);
}

void TonoiseOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *noisefile;
  int i;
  double tid;

  XmStringGetLtoR(calldata->value, charset, &filename);
  noisefile=fopen(filename,"w");
  if (noisefile==NULL) {
    XtManageChild(noiseWarning);
    XtFree(filename);
    return;
  }

  for (i=0; i<horiz_num; i++) {
    tid=i*sec_pr_bin;
    if ((tid>=mstart) && (tid<mstop))
      fprintf(noisefile,"%f\t%d\n",i*sec_pr_bin,
        (noisest?megnoise2[i]:megnoise[i]));
  }

  fclose(noisefile);
  XtFree(filename);
  XtUnmanageChild(tonoiseFileBox);
}

void Tomarkers(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(tomarkersFileBox);
}

void TomarkersOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *markersfile;
  int i;

  XmStringGetLtoR(calldata->value, charset, &filename);
  markersfile=fopen(filename,"w");
  if (markersfile==NULL) {
    XtManageChild(markersWarning);
    XtFree(filename);
    return;
  }

  for (i=0; i<nummark; i++) {
    fprintf(markersfile,"%.3f\n",markers[i]);
  }

  fclose(markersfile);
  XtFree(filename);
  XtUnmanageChild(tomarkersFileBox);
}

void Printpage(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(printpageFileBox);
}

void printsono(fil, c)
FILE *fil; int c;
{
  int i, j;
  long point;
  double fact, br, grafx, val, y;

  point=c*horiz_num*numchannels;
  fprintf(fil,"gsave newpath\n");
  fact=900./sec_pr_scr;
  br=fact*sec_pr_bin;
  for (j=0; j<horiz_num; j++) {
    grafx = fact*(j*sec_pr_bin-leftsec)+50.;
    if (grafx>950.) break;
    if (grafx>50.) {
      for (i=0; i<numchannels; i++) {
        if (megamp[point]*gain>100) {
          y = megfreq[point]/binfreq+800.;
          val=1.-(log10(megamp[point]*gain)-2.); if (val<0.) val=0.;
          fprintf(fil,"%4.2f 0 %5.2f %5.2f %3.2f l\n",br,grafx,y,val);
        }
        point++;
      }
    } else point+=numchannels;
  }
  fprintf(fil,"grestore\n");
}

void printamp(fil, c)
FILE *fil; long c;
{
  int i, j, grafold, grafx, max=-999, min=999, oldy=0;
  double fact;

  c*=framecnt;
  fact=900./sec_pr_scr; grafold=50;
  if (sec_pr_scr>2.)
   for (j=(int)(leftsec*R); j<framecnt; j++) {
    grafx = (int)(fact*((double)j/R-leftsec))+50;
    if (grafx>950) break;
    if (megosc[c+j]>max) max=megosc[c+j]; if (megosc[c+j]<min) min=megosc[c+j];
    if (grafx!=grafold) {
      fprintf(fil,"0 %f %d %f 0 l\n",(max-min)/2.,grafx,650.+min/2.);
      grafold=grafx; max=-999; min=999;
    }
  } else for (j=(int)(leftsec*R); j<framecnt; j++) {
    grafx = (int)(fact*((double)j/R-leftsec))+50;
    if (grafx>950) break;
    fprintf(fil,"%d %f %d %f 0 l\n",grafx-grafold,(megosc[c+j]-oldy)/4.,
      grafold,650.+oldy/4.);
    grafold=grafx; oldy=megosc[c+j];
  }
}

void printpitch(fil, c)
FILE *fil; int c;
{
  int oldy=0, grafold=50, grafx, j;
  double fact=900./sec_pr_scr;

  c*=horiz_num;
  for (j=0; j<horiz_num; j++) {
    grafx = (int)(fact*(j*sec_pr_bin-leftsec))+50;
    if (grafx>950) break;
    if ((grafx!=grafold) && (grafx>50)) {
      if ((megpitch[c+j]>0) && (megpitch[c+j]-oldy<50) && (oldy>0)) {
        fprintf(fil,"%d %d %d %d s\n",grafx,400+megpitch[c+j],grafold,400+oldy);
      }
      grafold=grafx; oldy=megpitch[c+j];
    }
  }
}

void printnoise(fil, c)
FILE *fil; int c;
{
  int oldy=0, grafold=50, grafx, j;
  double fact=900./sec_pr_scr;

  c*=horiz_num;
  for (j=0; j<horiz_num; j++) {
    grafx = (int)(fact*(j*sec_pr_bin-leftsec))+50;
    if (grafx>950) break;
    if ((grafx!=grafold) && (grafx>50)) {
      if (((noisest?megnoise2[c+j]:megnoise[c+j])>0) && (oldy>0)) {
        fprintf(fil,"%d %d %d %d s\n",
          grafx,200+(noisest?megnoise2[c+j]:megnoise[c+j])/2,
          grafold,200+oldy/2);
      }
      grafold=grafx; oldy=(noisest?megnoise2[c+j]:megnoise[c+j]);
    }
  }
}

void printscale(fil, c)
FILE *fil; int c;
{
  int i, j, x, y, sec;

  fprintf(fil, "newpath\n");
  fprintf(fil, "40 %d moveto 950 0 rlineto\n", 799);
  fprintf(fil, "40 %d moveto 0 400 rlineto\n", 799);

  for (i=0; i<=(int)(sec_pr_scr); i++) {
    x=(i-leftsec+floor(leftsec))*900./sec_pr_scr+50; sec=i+floor(leftsec);
    if (sec%10==0) fprintf(fil, "%d %d moveto 0 10 rlineto\n", x, 799-10);
    if (sec%60==0) fprintf(fil, "%d %d moveto 0 15 rlineto\n", x, 799-15);
    if (sec_pr_scr<960.) fprintf(fil, "%d %d moveto 0 5 rlineto\n", x, 799-5);

/*
    if ((sec_pr_scr<240.) && ((sec%10==0) || (sec_pr_scr<15.))) {
      sprintf(tall, "%2d:%02d", sec/60,sec%60);
      XDrawString(theDisplay,bitmap,theGC,x-29,theheight+33,tall,strlen(tall));
    }
    else if (sec%60==0) {
      sprintf(tall, "%2d", sec/60);
      XDrawString(theDisplay,bitmap,theGC,x-15,theheight+33,tall,strlen(tall));
    }
    if ((settings.showgrid) && ((sec_pr_scr<120.) || (sec%10==0))) {
      XSetLineAttributes(theDisplay, theGC, 0, LineOnOffDash, CapRound,
        JoinMiter); 
      XSetDashes(theDisplay, theGC, 0, dashes, 2);
      XDrawLine(theDisplay,bitmap,theGC,x,10,x,800);
      XSetLineAttributes(theDisplay, theGC, 0, LineSolid, CapRound,
        JoinMiter); 
    } */
  }

  for (i=1; i<=(int)(numchannels*binfreq/1000.); i++) {
    y=800+(int)(i*1000./binfreq);
    fprintf(fil, "35 %d moveto 10 0 rlineto\n", y);
/*    sprintf(tall, "%2d", i);
    XDrawString(theDisplay,bitmap,theGC,
      2,y+5,tall,strlen(tall)); */
  }

/*
  XSetLineAttributes(theDisplay, theGC, 0, LineOnOffDash, CapRound,
    JoinMiter); 
  XSetDashes(theDisplay, theGC, 0, dashes, 2);
  for (i=0; i<4; i++)
    XDrawLine(theDisplay,bitmap,theGC,50,700-i*48,950,700-i*48);
  XSetLineAttributes(theDisplay, theGC, 0, LineSolid, CapRound,
    JoinMiter);
*/
  fprintf(fil, "stroke\n");
}

void PrintpageOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *fil;
  int i, j, grafx, grafold=-1, max=-999, min=999, br, oldy=0, c;
  long point=0;
  double fact;

  XmStringGetLtoR(calldata->value, charset, &filename);
  fil=fopen(filename,"w");
  if (fil==NULL) {
    XtManageChild(printWarning);
    XtFree(filename);
    return;
  }

  SetWatchCursor(topLevel);

  fprintf(fil,"%%!PS-Adobe-3.0\n");
  fprintf(fil,"%%%%Creator: NoTAM BIO\n");
  fprintf(fil,"%%%%Title: BIO page print\n");
  fprintf(fil,"%%%%BoundingBox: 0 0 1000 1000\n");
  fprintf(fil,"%%%%EndComments\n");
  fprintf(fil,"/l { newpath setgray moveto rlineto stroke } def\n");
  fprintf(fil,"/s { newpath moveto lineto stroke } def\n");
  for (c=0; c<samps_per_frame; c++) {
    fprintf(fil,"%%%%Page: %d %d\n", c+1, samps_per_frame);
    fprintf(fil,"0.6 0.6 scale\n");

    printscale(fil, c);
    printsono(fil, c);
    printamp(fil, c);
    printpitch(fil, c);
    printnoise(fil, c);

    fprintf(fil,"showpage\n");
  }
  fprintf(fil,"%%%%EOF\n");

  fclose(fil);
  XtFree(filename);
  XtUnmanageChild(printpageFileBox);
  ResetCursor(topLevel);
}

void Analy(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(analyForm);
}

void AnalyOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  Boolean ne;

  valstring=XmTextGetString(analyTextfield);
  gain=atof(valstring);
  XtFree(valstring);

  valstring=XmTextGetString(analyTextfield);
  octave=atoi(valstring);
  XtFree(valstring);

  ne=XmToggleButtonGetState(analyToggle1);
  if (ne) noisest=0; else noisest=1;
  RedrawWin();
}

void setgreycolors(void)
{
   if (farge==1) return;
   XSetForeground(theDisplay, theGC0, gray0.pixel);
   XSetForeground(theDisplay, showGC0, gray0.pixel);
   XSetForeground(theDisplay, theGC25, gray25.pixel);
   XSetForeground(theDisplay, showGC25, gray25.pixel);
   XSetForeground(theDisplay, theGC50, gray50.pixel);
   XSetForeground(theDisplay, showGC50, gray50.pixel);
   XSetForeground(theDisplay, theGC75, gray75.pixel);
   XSetForeground(theDisplay, showGC75, gray75.pixel);
   XSetForeground(theDisplay, theGC100, gray100.pixel);
   XSetForeground(theDisplay, showGC100, gray100.pixel);
   XtVaSetValues(sketchpad,XmNbackground, gray100.pixel,NULL);
   XSetForeground(theDisplay, funcGC, red.pixel);
   RedrawWin(); farge=1;
}

void sethotcolors(void)
{
   if (farge==2) return;
   XSetForeground(theDisplay, theGC0, gray100.pixel);
   XSetForeground(theDisplay, showGC0, gray100.pixel);
   XSetForeground(theDisplay, theGC25, yellow.pixel);
   XSetForeground(theDisplay, showGC25, yellow.pixel);
   XSetForeground(theDisplay, theGC50, orange.pixel);
   XSetForeground(theDisplay, showGC50, orange.pixel);
   XSetForeground(theDisplay, theGC75, red.pixel);
   XSetForeground(theDisplay, showGC75, red.pixel);
   XSetForeground(theDisplay, theGC100, gray0.pixel);
   XSetForeground(theDisplay, showGC100, gray0.pixel);
   XtVaSetValues(sketchpad,XmNbackground, gray0.pixel,NULL);
   XSetForeground(theDisplay, funcGC, green.pixel);
   RedrawWin(); farge=2;
}

void setcoldcolors(void)
{
   if (farge==3) return;
   XSetForeground(theDisplay, theGC0, gray100.pixel);
   XSetForeground(theDisplay, showGC0, gray100.pixel);
   XSetForeground(theDisplay, theGC25, lightblue.pixel);
   XSetForeground(theDisplay, showGC25, lightblue.pixel);
   XSetForeground(theDisplay, theGC50, blue.pixel);
   XSetForeground(theDisplay, showGC50, blue.pixel);
   XSetForeground(theDisplay, theGC75, darkblue.pixel);
   XSetForeground(theDisplay, showGC75, darkblue.pixel);
   XSetForeground(theDisplay, theGC100, gray0.pixel);
   XSetForeground(theDisplay, showGC100, gray0.pixel);
   XtVaSetValues(sketchpad,XmNbackground, gray0.pixel,NULL);
   XSetForeground(theDisplay, funcGC, green.pixel);
   RedrawWin(); farge=3; 
}

void Displaymenu(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(displayForm);
}

void DisplayOk(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(displayToggleGrey)) setgreycolors();
  if (XmToggleButtonGetState(displayToggleHot)) sethotcolors();
  if (XmToggleButtonGetState(displayToggleCold)) setcoldcolors();
}

void Zoomin(w, client, call)
Widget w;
XtPointer client, call;
{
  if (sec_pr_scr>10./128.) {
    sec_pr_scr/=2.;
    XtVaSetValues(horiscroll,XmNsliderSize,(int)ceil(sec_pr_scr*10.),NULL);
    RedrawWin();
  }
  if (sec_pr_scr<60.) XtVaSetValues(horiscroll,XmNincrement,1,NULL);
}

void Zoomout(w, client, call)
Widget w;
XtPointer client, call;
{
  if (sec_pr_scr<640.) {
    sec_pr_scr*=2;
    if (leftsec+sec_pr_scr>640.) {
      leftsec=640.-sec_pr_scr;
      XtVaSetValues(horiscroll,XmNvalue,(int)(leftsec*10.),NULL);
    }
    XtVaSetValues(horiscroll,XmNsliderSize,(int)ceil(sec_pr_scr*10.),NULL);
    RedrawWin();
  }
  if (sec_pr_scr>=60.) XtVaSetValues(horiscroll,XmNincrement,10,NULL);
}

void Hslider(w, client, call)
Widget w;
XtPointer client, call;
{
  XmScrollBarCallbackStruct *calldata=(XmScrollBarCallbackStruct *) call;
  leftsec=(calldata->value)/10.;

  RedrawWin();
}

void Grid(w, client, call)
Widget w;
XtPointer client, call;
{
  settings.showgrid=(int)XmToggleButtonGetState(w);
  RedrawWin();
}

void Centro(w, client, call)
Widget w;
XtPointer client, call;
{
  settings.showcentro=(int)XmToggleButtonGetState(w);
  RedrawWin();
}

void Spec(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(specForm);
  RedrawSpec();
}

void SpecOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XtUnmanageChild(specForm);
}

void printspecscale(fil, dy)
FILE *fil; int dy;
{
  int i, x, y, ch;

  fprintf(fil, "%d %d %d %d s\n", 39, dy+40, 39, dy+290);
  fprintf(fil, "%d %d %d %d s\n", 39, dy+40, numchannels+39, dy+40);

  for (i=1; i<=(int)(numchannels*binfreq/1000.); i++) {
    x=39+(int)(i*1000./binfreq);
    fprintf(fil, "%d %d %d %d s\n", x, dy+35, x, dy+40);
    if (i%2==0)
      fprintf(fil, "%d %d moveto (%d) show\n", x-14, dy+20, i*1000);
  }
/*  XSetLineAttributes(theDisplay, theGC, 0, LineOnOffDash, CapRound,
    JoinMiter); 
  XSetDashes(theDisplay, theGC, 0, dashes, 2);
  for (i=1; i<11; i++)
    XDrawLine(theDisplay,specbitmap,theGC,39,dy-40-i*25,numchannels+39,
      dy-40-i*25);
  XSetLineAttributes(theDisplay, theGC, 0, LineSolid, CapRound, JoinMiter);*/
}

void PrintspecOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *fil;
  int i, ch, val, lasty, dy;
  long point;

  XmStringGetLtoR(calldata->value, charset, &filename);
  fil=fopen(filename,"w");
  if (fil==NULL) {
    XtManageChild(specprintWarning);
    XtFree(filename);
    return;
  }

  SetWatchCursor(topLevel);

  fprintf(fil,"%%!PS-Adobe-3.0\n");
  fprintf(fil,"%%%%Creator: NoTAM BIO\n");
  fprintf(fil,"%%%%Title: BIO spectrum print\n");
  fprintf(fil,"%%%%BoundingBox: 0 0 600 %d\n", samps_per_frame*300);
  fprintf(fil,"%%%%EndComments\n");
  fprintf(fil,"/s { newpath moveto lineto stroke } def\n");
  fprintf(fil,"%%%%Page: 1 1\n");

  fprintf(fil, "/Helvetica findfont 10 scalefont setfont\n");

  if (horiz_num) for (ch=0; ch<samps_per_frame; ch++) {
    point=numchannels*(ch*horiz_num+(long)(curstid/sec_pr_bin));
    dy=samps_per_frame*300-ch*300;
    printspecscale(fil, dy);
    for (i=0; i<numchannels; i++) {
/*      val=(int)(log(megamp[point+i]+1)*20.); */
      val=(int)(megamp[point+i]/50.)*gain;
      if (i>0) fprintf(fil, "%d %d %d %d s\n",
        i+39, lasty+dy+40, i+40, val+dy+40);
      lasty=val;
    }
  }

  fprintf(fil,"showpage\n");
  fprintf(fil,"%%%%EOF\n");

  fclose(fil);
  XtFree(filename);
  XtUnmanageChild(printspecFileBox);
  ResetCursor(topLevel);
}

void ExportspecOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *fil;
  int i;
  long point;

  XmStringGetLtoR(calldata->value, charset, &filename);
  fil=fopen(filename,"w");
  if (fil==NULL) {
    XtManageChild(specexportWarning);
    XtFree(filename);
    return;
  }

  if (horiz_num) {
    point=numchannels*(long)(curstid/sec_pr_bin);
    for (i=0; i<numchannels; i++) {
      fprintf(fil, "%d\t%d\n", megfreq[point+i], megamp[point+i]);
    }
  }

  fclose(fil);
  XtFree(filename);
  XtUnmanageChild(exportspecFileBox);
}

void SpecPrint(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(printspecFileBox);
}

void SpecExport(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(exportspecFileBox);
}

void Corr(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(corrForm);
  RedrawCorr();
}

void CorrOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XtUnmanageChild(corrForm);
}

void PrintcorrOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *fil;
  int i, j, val, lasty;
  long point;

  XmStringGetLtoR(calldata->value, charset, &filename);
  fil=fopen(filename,"w");
  if (fil==NULL) {
    XtManageChild(corrprintWarning);
    XtFree(filename);
    return;
  }

  SetWatchCursor(topLevel);

  fprintf(fil,"%%!PS-Adobe-3.0\n");
  fprintf(fil,"%%%%Creator: NoTAM BIO\n");
  fprintf(fil,"%%%%Title: BIO cross-correlation print\n");
  fprintf(fil,"%%%%BoundingBox: 0 0 600 300\n");
  fprintf(fil,"%%%%EndComments\n");
  fprintf(fil,"/s { newpath moveto lineto stroke } def\n");
  fprintf(fil,"%%%%Page: 1 1\n");

  fprintf(fil, "/Helvetica findfont 10 scalefont setfont\n");
  if (horiz_num) {
    fprintf(fil, "570 120 50 120 s\n");
    fprintf(fil, "50 320 50 120 s\n");
    for (i=1000; i<(int)(R/2.); i+=1000) {
      val=(int)((float)i/binfreq)+50;
      fprintf(fil, "%d 120 %d 115 s\n", val, val);
      fprintf(fil, "%d 105 moveto (%d) show\n", val-10, i);
    }
    point=numchannels*(long)(curstid/sec_pr_bin);
    for (i=0; i<numchannels; i++) {
      val=(int)(log(megamp[point+i]+1)*20.);
      if (i>0) fprintf(fil, "%d %d %d %d s\n", i+50, val+120, i+49, lasty+120);
      lasty=val;
    }
  }

  fprintf(fil,"showpage\n");
  fprintf(fil,"%%%%EOF\n");

  fclose(fil);
  XtFree(filename);
  XtUnmanageChild(printcorrFileBox);
  ResetCursor(topLevel);
}

void ExportOneCorr(c1, c2, fil)
int c1, c2; FILE *fil;
{
  int lag, maxlag, i, grafx, s1, s2;
  long start, stop, p1, p2, offset1, offset2;
  double sum, maxlagtime, maxval, y;
  double ytab[2001];

  for (i=0; i<=2000; i++) ytab[i]=0.; 
  maxlagtime=0.1; maxlag=R*maxlagtime;
  start=mstart*R; stop=mstop*R;
  offset1=c1*framecnt; offset2=c2*framecnt;
  for (lag=-maxlag; lag<maxlag; lag++) {
    sum=0.;
    for (p1=start; p1<stop; p1++) {
      s1=megosc[offset1+p1];
      p2=p1-lag;
      if ((p2<0) || (p2>=framecnt)) s2=0;
        else s2=megosc[offset2+p2];
      sum+=s1*s2;
    }
    grafx = (int)(1000+lag*(double)1000./maxlag);
    y=sum/maxlag;
    if (fabs(y)>fabs(ytab[grafx])) ytab[grafx]=y;
  }
  maxval=0.;
  for (i=0; i<=2000; i++) if (fabs(ytab[i])>maxval) maxval=fabs(ytab[i]);
  for (i=0; i<=2000; i++) ytab[i]/=maxval;
  for (i=0; i<=2000; i++)
    fprintf(fil, "%f %f\n", maxlagtime*(i-1000)/1000.,ytab[i]);
}

void ExportcorrOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *fil;
  int i;
  long point;

  if (curstid>=duration) return;

  XmStringGetLtoR(calldata->value, charset, &filename);
  fil=fopen(filename,"w");
  if (fil==NULL) {
    XtManageChild(correxportWarning);
    XtFree(filename);
    return;
  }

  if ((horiz_num) && (samps_per_frame>1) && (mstop>mstart)) {
    if (samps_per_frame==2) ExportOneCorr(0,1,fil);
    else if (samps_per_frame==4) {
      ExportOneCorr(0,1,fil); fprintf(fil, "-----------------------------\n");
      ExportOneCorr(0,2,fil); fprintf(fil, "-----------------------------\n");
      ExportOneCorr(0,3,fil); fprintf(fil, "-----------------------------\n");
      ExportOneCorr(1,2,fil); fprintf(fil, "-----------------------------\n");
      ExportOneCorr(1,3,fil); fprintf(fil, "-----------------------------\n");
      ExportOneCorr(2,3,fil);
    }
  }

  fclose(fil);
  XtFree(filename);
  XtUnmanageChild(exportcorrFileBox);
}

void CorrPrint(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(printcorrFileBox);
}

void CorrExport(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(exportcorrFileBox);
}


void AddMarker(w, client, call)
Widget w;
XtPointer client, call;
{
  if (nummark>=9999) return;
  markers[nummark++] = curstid;
}

void RemoveMarker(w, client, call)
Widget w;
XtPointer client, call;
{
  removemark=1;
}

main(argc, argv)
int argc;
char **argv;
{
  XmString text;

  static XtActionsRec actions[]={
    {"CursorClick", CursorClick},
    {"CursorDrag",  CursorDrag},
    {"CursorDrop",  CursorDrop},
    {"AreaClick",   AreaClick},
    {"AreaDrag",    AreaDrag},
    {"AreaDrop",    AreaDrop},
    {"Keydown",     Keydown},
  };


  /* INITIALIZE PHASE VOCODER */

  N=1024;  /* FFT length */
  Nw=N*2;    /* Window size */
  D=Nw/4;  /* Decimation factor */
  I=D;     /* Interpolation factor */
  P=1.;    /* Oscbank pitch factor */
  synt=0.; /* Synthesis threshold */
  
  PI=4.*atan(1.); TWOPI=8.*atan(1.);
  obank=0.;
  N2=N>>1; theheight=400;
  Nw2=Nw>>1;
  numchannels=420;
  fvec(Wanal,  Nw);
  fvec(Wsyn,  Nw);
  fvec(input,  Nw);
  fvec(buffer,  N);
  fvec(channel,  N+2);
  fvec(output,  Nw);
  fvec(tempamp,numchannels);
  fvec(tempfreq,numchannels);
  fvec(last, numchannels); fvec(this, numchannels);

  svec(framebuff,4096);
  svec(mixbuff,4096);
  if ((outbuff =(int *)calloc(4096,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate out buffer!\n");
    exit(-1); }
  if ((auxbuff =(int *)calloc(2048,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate aux buffer!\n");
    exit(-1); }

  
  makewindows(Wanal,  Wsyn,  Nw,  N,  I);
  
  printf("Starting Bio ...\n");

/*  in_AFsetup=AFnewfilesetup(); */
  settings.buffsize=0.5; settings.scroll=0;

  /* INITIALIZE X */
/*  XtSetLanguageProc(NULL, (XtLanguageProc)NULL, NULL); */
  topLevel=XtVaAppInitialize(
    &app_context,"Bio",NULL,0,&argc,argv,NULL,NULL);
  dw = XmGetXmDisplay(XtDisplay(topLevel));
  XtVaSetValues(dw, XmNdragInitiatorProtocolStyle, XmDRAG_NONE, NULL);


  /* CREATE MAIN WINDOW & IMMEDIATE CHILDREN */
  mainWindow=XtVaCreateManagedWidget(
    "mainWindow",xmMainWindowWidgetClass,topLevel,
    XmNscrollingPolicy, XmAUTOMATIC, NULL);

  menuBar=XmCreateMenuBar(mainWindow,"menuBar",NULL,0);
  XtManageChild(menuBar);

  frame=XtVaCreateManagedWidget(
    "frame",xmFrameWidgetClass,mainWindow,XmNwidth,864,NULL);

  XmMainWindowSetAreas(mainWindow, menuBar, NULL, NULL, NULL, frame);
  XtVaSetValues(mainWindow,XmNwidth,1160,XmNheight,870,NULL);
  largeBB = XtVaCreateManagedWidget(
    "largeBB", xmBulletinBoardWidgetClass, frame,
    XmNmarginHeight, 6, XmNmarginWidth, 0, NULL);


  /* CREATE GRAPHICS */
   sketchpad = XtVaCreateManagedWidget(
     "sketchpad",xmDrawingAreaWidgetClass, largeBB,
     XmNwidth, 1000, XmNheight, 800, NULL);

   theDisplay = XtDisplay(sketchpad);
   theWindow = XtWindow(sketchpad);
   screen = DefaultScreen(theDisplay);
   theRoot = RootWindowOfScreen(XtScreen(sketchpad));
   theCmap = DefaultColormap( theDisplay, DefaultScreen(theDisplay) );
   bitmap = XCreatePixmap(XtDisplay(sketchpad),theRoot,1000,3200,
     DefaultDepthOfScreen(XtScreen(sketchpad)));
   specbitmap = XCreatePixmap(XtDisplay(sketchpad),theRoot,1000,800,
     DefaultDepthOfScreen(XtScreen(sketchpad)));
   corrbitmap = XCreatePixmap(XtDisplay(sketchpad),theRoot,1000,800,
     DefaultDepthOfScreen(XtScreen(sketchpad)));
   theGC = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC = XCreateGC( theDisplay, theRoot, 0, 0 );
   theGC0 = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC0 = XCreateGC( theDisplay, theRoot, 0, 0 );
   theGC25 = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC25 = XCreateGC( theDisplay, theRoot, 0, 0 );
   theGC50 = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC50 = XCreateGC( theDisplay, theRoot, 0, 0 );
   theGC75 = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC75 = XCreateGC( theDisplay, theRoot, 0, 0 );
   theGC100 = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC100 = XCreateGC( theDisplay, theRoot, 0, 0 );
   funcGC = XCreateGC( theDisplay, theRoot, 0, 0);
   gridGC = XCreateGC( theDisplay, theRoot, 0, 0);
   markerGC = XCreateGC( theDisplay, theRoot, 0, 0 );


   XParseColor(theDisplay,theCmap,"gray0",&gray0);
   XAllocColor( theDisplay, theCmap,&gray0);
   XParseColor(theDisplay,theCmap,"gray25",&gray25);
   XAllocColor( theDisplay, theCmap,&gray25);
   XParseColor(theDisplay,theCmap,"gray50",&gray50);
   XAllocColor( theDisplay, theCmap,&gray50);
   XParseColor(theDisplay,theCmap,"gray75",&gray75);
   XAllocColor( theDisplay, theCmap,&gray75);
   XParseColor(theDisplay,theCmap,"gray100",&gray100);
   XAllocColor( theDisplay, theCmap,&gray100);
   XParseColor(theDisplay,theCmap,"light blue",&lightblue);
   XAllocColor( theDisplay, theCmap,&lightblue);
   XParseColor(theDisplay,theCmap,"blue",&blue);
   XAllocColor( theDisplay, theCmap,&blue);
   XParseColor(theDisplay,theCmap,"navy",&darkblue);
   XAllocColor( theDisplay, theCmap,&darkblue);
   XParseColor(theDisplay,theCmap,"yellow",&yellow);
   XAllocColor( theDisplay, theCmap,&yellow);
   XParseColor(theDisplay,theCmap,"red",&red);
   XAllocColor( theDisplay, theCmap,&red);
   XParseColor(theDisplay,theCmap,"orange",&orange);
   XAllocColor( theDisplay, theCmap,&orange);
   XParseColor(theDisplay,theCmap,"green",&green);
   XAllocColor( theDisplay, theCmap,&green);
   XParseColor(theDisplay,theCmap,"coral",&brown);
   XAllocColor( theDisplay, theCmap,&brown);

   XSetForeground(theDisplay, gridGC, brown.pixel);

   XtAppAddActions(app_context, actions, XtNumber(actions));
   XtAddCallback(sketchpad,XmNexposeCallback,RestoreWin,0);

   xpos[1]=50; ypos[1]=100; xpos[2]=850; ypos[2]=100;

  horiscroll=XtVaCreateManagedWidget(
    "horiscroll",xmScrollBarWidgetClass,largeBB,XmNincrement,10,
     XmNorientation, XmHORIZONTAL, XmNx, 350, XmNy, 10,
     XmNmaximum, 9600, XmNwidth, 500, XmNheight, 20, XmNsliderSize, 2400, NULL);
  XtAddCallback(horiscroll, XmNdragCallback, Hslider, 0);
  XtAddCallback(horiscroll, XmNvalueChangedCallback, Hslider, 0);

  zoomBB=XtVaCreateManagedWidget("zoomBB",xmBulletinBoardWidgetClass,largeBB,NULL);
  zoominB=XtVaCreateManagedWidget("zoominB",xmPushButtonWidgetClass,
    zoomBB,NULL);
  XtAddCallback(zoominB,XmNactivateCallback,Zoomin,NULL);
  zoomoutB=XtVaCreateManagedWidget("zoomoutB",xmPushButtonWidgetClass,
    zoomBB,NULL);
  XtAddCallback(zoomoutB,XmNactivateCallback,Zoomout,NULL);
  text=XmStringCreateLocalized(" ");
  posL=XtVaCreateManagedWidget("posL",xmLabelWidgetClass,
    zoomBB,XmNlabelString,text,NULL);

  playBB=XtVaCreateManagedWidget("playBB",xmBulletinBoardWidgetClass,largeBB,NULL);
  playB=XtVaCreateManagedWidget("playB",xmPushButtonWidgetClass,playBB,NULL);
  XtAddCallback(playB,XmNactivateCallback,Realtime,NULL);
  stopB=XtVaCreateManagedWidget("stopB",xmPushButtonWidgetClass,playBB,NULL);
  XtAddCallback(stopB,XmNactivateCallback,StopRealT,NULL);
  loopToggle=XtVaCreateManagedWidget("loopToggle",xmToggleButtonWidgetClass,
    playBB,NULL);

  leftBB=XtVaCreateManagedWidget("leftBB",xmBulletinBoardWidgetClass,
    largeBB,NULL);
  leftLabSon=XtVaCreateManagedWidget("leftLabSon",xmLabelWidgetClass,
    leftBB,NULL);
  leftLabAmp=XtVaCreateManagedWidget("leftLabAmp",xmLabelWidgetClass,
    leftBB,NULL);
  leftLabPitch=XtVaCreateManagedWidget("leftLabPitch",xmLabelWidgetClass,
    leftBB,NULL);
  leftLabNoise=XtVaCreateManagedWidget("leftLabNoise",xmLabelWidgetClass,
    leftBB,NULL);


  /* CREATE FILE MENU */
  fileButton=XtVaCreateManagedWidget(
    "fileButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("File");
  XtVaSetValues(fileButton,XmNlabelString,text,NULL);

  fileMenu=XmCreatePulldownMenu(menuBar,"fileMenu",NULL,0);
  XtVaSetValues(fileButton,XmNsubMenuId,fileMenu,NULL);


  /* CREATE LOAD & ANALYZE BUTTON */
  load=XtVaCreateManagedWidget(
    "load",xmPushButtonWidgetClass,fileMenu,NULL);
  text=XmStringCreateLocalized("Load & Analyze");
  XtVaSetValues(load,XmNlabelString,text,NULL);

  loadFileBox=XmCreateFileSelectionDialog(mainWindow,"loadFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(loadFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Load & Analyze");
  XtVaSetValues(loadFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
/*  text=XmStringCreateSimple("/sounds/");
  XtVaSetValues(loadFileBox,XmNdirectory,text,NULL); */

  XtAddCallback(loadFileBox,XmNcancelCallback,Cancel,loadFileBox);
  XtAddCallback(loadFileBox,XmNokCallback,LoadOk,NULL);
  XtAddCallback(load,XmNactivateCallback,Load,NULL);


  /* CREATE LOAD MARKERS BUTTON */
  loadMarkers=XtVaCreateManagedWidget(
    "loadMarkers",xmPushButtonWidgetClass,fileMenu,NULL);
  text=XmStringCreateLocalized("Load markers");
  XtVaSetValues(loadMarkers,XmNlabelString,text,NULL);

  loadMarkersFileBox=XmCreateFileSelectionDialog(mainWindow,
    "loadMarkersFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(loadMarkersFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Load markers");
  XtVaSetValues(loadMarkersFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);

  XtAddCallback(loadMarkersFileBox,XmNcancelCallback,Cancel,loadMarkersFileBox);
  XtAddCallback(loadMarkersFileBox,XmNokCallback,LoadMarkersOk,NULL);
  XtAddCallback(loadMarkers,XmNactivateCallback,LoadMarkers,NULL);


  /* CREATE PLAY BUTTON */
  play=XtVaCreateManagedWidget(
    "play",xmPushButtonWidgetClass,fileMenu,NULL);
  text=XmStringCreateLocalized("Play");
  XtVaSetValues(play,XmNlabelString,text,NULL);

  playFileBox=XmCreateFileSelectionDialog(mainWindow,"playFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(playFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Play");
  XtVaSetValues(playFileBox,XmNdialogTitle,text,XmNautoUnmanage,False,
    XmNwidth,500,NULL);
  text=XmStringCreateSimple("/sounds/");
  XtVaSetValues(playFileBox,XmNdirectory,text,NULL);
  XtAddCallback(playFileBox,XmNcancelCallback,Cancel,playFileBox);
  XtAddCallback(playFileBox,XmNokCallback,PlayOk,NULL);
  XtAddCallback(play,XmNactivateCallback,Play,NULL);

  /* CREATE QUIT BUTTON */
  quit=XtVaCreateManagedWidget(
    "quit",xmPushButtonWidgetClass,fileMenu,
    XmNmnemonic, XStringToKeysym("Q"), XmNaccelerator, "Alt<Key>Q", NULL);
  text=XmStringCreateLocalized("Quit");
  XtVaSetValues(quit,XmNlabelString,text,NULL);
  XtAddCallback(quit,XmNactivateCallback,Quit,0);

  /* CREATE FILE OPENING WARNING */
  fileWarning=XmCreateWarningDialog(mainWindow,"fileWarning",NULL,0);
  temp=XmMessageBoxGetChild(fileWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(fileWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(fileWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open file");
  XtVaSetValues(fileWarning,XmNmessageString,text,NULL);

  /* CREATE FILE SIZE WARNING */
  sizeWarning=XmCreateWarningDialog(mainWindow,"sizeWarning",NULL,0);
  temp=XmMessageBoxGetChild(sizeWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(sizeWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(sizeWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateLtoR(
    "Sorry, can't handle\nsoundfiles that large",
    XmFONTLIST_DEFAULT_TAG);
  XtVaSetValues(sizeWarning,XmNmessageString,text,NULL);


  /* CREATE EXPORT MENU */
  exportButton=XtVaCreateManagedWidget(
    "exportButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Export");
  XtVaSetValues(exportButton,XmNlabelString,text,NULL);
  exportMenu=XmCreatePulldownMenu(menuBar,"exportMenu",NULL,0);
  XtVaSetValues(exportButton,XmNsubMenuId,exportMenu,NULL);

  /* CREATE TOCENTROID BUTTON */
  tocentroid=XtVaCreateManagedWidget(
    "tocentroid",xmPushButtonWidgetClass,exportMenu,NULL);
  text=XmStringCreateLocalized("Centroid vector");
  XtVaSetValues(tocentroid,XmNlabelString,text,NULL);

  tocentroidFileBox=XmCreateFileSelectionDialog(mainWindow,
    "tocentroidFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(tocentroidFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write centroid vector file");
  XtVaSetValues(tocentroidFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(tocentroidFileBox,XmNcancelCallback,Cancel,tocentroidFileBox);
  XtAddCallback(tocentroidFileBox,XmNokCallback,TocentroidOk,NULL);
  XtAddCallback(tocentroid,XmNactivateCallback,Tocentroid,NULL);

  /* CREATE CENTROID OPENING WARNING */
  centroidWarning=XmCreateWarningDialog(mainWindow,"centroidWarning",NULL,0);
  temp=XmMessageBoxGetChild(centroidWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(centroidWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(centroidWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open centroid vector file");
  XtVaSetValues(centroidWarning,XmNmessageString,text,NULL);

  /* CREATE TOAMPLITUDE BUTTON */
  toamplitude=XtVaCreateManagedWidget(
    "toamplitude",xmPushButtonWidgetClass,exportMenu,NULL);
  text=XmStringCreateLocalized("Amplitude vector");
  XtVaSetValues(toamplitude,XmNlabelString,text,NULL);

  toamplitudeFileBox=XmCreateFileSelectionDialog(mainWindow,
    "toamplitudeFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(toamplitudeFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write amplitude vector file");
  XtVaSetValues(toamplitudeFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(toamplitudeFileBox,XmNcancelCallback,Cancel,toamplitudeFileBox);
  XtAddCallback(toamplitudeFileBox,XmNokCallback,ToamplitudeOk,NULL);
  XtAddCallback(toamplitude,XmNactivateCallback,Toamplitude,NULL);

  /* CREATE AMPLITUDE OPENING WARNING */
  amplitudeWarning=XmCreateWarningDialog(mainWindow,"amplitudeWarning",NULL,0);
  temp=XmMessageBoxGetChild(amplitudeWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(amplitudeWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(amplitudeWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open amplitude vector file");
  XtVaSetValues(amplitudeWarning,XmNmessageString,text,NULL);

  /* CREATE TOPITCH BUTTON */
  topitch=XtVaCreateManagedWidget(
    "topitch",xmPushButtonWidgetClass,exportMenu,NULL);
  text=XmStringCreateLocalized("Pitch vector");
  XtVaSetValues(topitch,XmNlabelString,text,NULL);

  topitchFileBox=XmCreateFileSelectionDialog(mainWindow,
    "topitchFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(topitchFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write pitch vector file");
  XtVaSetValues(topitchFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(topitchFileBox,XmNcancelCallback,Cancel,topitchFileBox);
  XtAddCallback(topitchFileBox,XmNokCallback,TopitchOk,NULL);
  XtAddCallback(topitch,XmNactivateCallback,Topitch,NULL);

  /* CREATE PITCH OPENING WARNING */
  pitchWarning=XmCreateWarningDialog(mainWindow,"pitchWarning",NULL,0);
  temp=XmMessageBoxGetChild(pitchWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(pitchWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(pitchWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open pitch vector file");
  XtVaSetValues(pitchWarning,XmNmessageString,text,NULL);

  /* CREATE TONOISE BUTTON */
  tonoise=XtVaCreateManagedWidget(
    "tonoise",xmPushButtonWidgetClass,exportMenu,NULL);
  text=XmStringCreateLocalized("Noise vector");
  XtVaSetValues(tonoise,XmNlabelString,text,NULL);

  tonoiseFileBox=XmCreateFileSelectionDialog(mainWindow,
    "tonoiseFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(tonoiseFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write noise vector file");
  XtVaSetValues(tonoiseFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(tonoiseFileBox,XmNcancelCallback,Cancel,tonoiseFileBox);
  XtAddCallback(tonoiseFileBox,XmNokCallback,TonoiseOk,NULL);
  XtAddCallback(tonoise,XmNactivateCallback,Tonoise,NULL);

  /* CREATE NOISE OPENING WARNING */
  noiseWarning=XmCreateWarningDialog(mainWindow,"noiseWarning",NULL,0);
  temp=XmMessageBoxGetChild(noiseWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(noiseWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(noiseWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open noise vector file");
  XtVaSetValues(noiseWarning,XmNmessageString,text,NULL);

  /* CREATE TOMARKERS BUTTON */
  tomarkers=XtVaCreateManagedWidget(
    "tomarkers",xmPushButtonWidgetClass,exportMenu,NULL);
  text=XmStringCreateLocalized("Markers");
  XtVaSetValues(tomarkers,XmNlabelString,text,NULL);

  tomarkersFileBox=XmCreateFileSelectionDialog(mainWindow,
    "tomarkersFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(tomarkersFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write markers file");
  XtVaSetValues(tomarkersFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(tomarkersFileBox,XmNcancelCallback,Cancel,tomarkersFileBox);
  XtAddCallback(tomarkersFileBox,XmNokCallback,TomarkersOk,NULL);
  XtAddCallback(tomarkers,XmNactivateCallback,Tomarkers,NULL);

  /* CREATE MARKERS OPENING WARNING */
  markersWarning=XmCreateWarningDialog(mainWindow,"markersWarning",NULL,0);
  temp=XmMessageBoxGetChild(markersWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(markersWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(markersWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open markers file");
  XtVaSetValues(markersWarning,XmNmessageString,text,NULL);


  /* CREATE PRINT MENU */
  printButton=XtVaCreateManagedWidget(
    "printButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Print");
  XtVaSetValues(printButton,XmNlabelString,text,NULL);
  printMenu=XmCreatePulldownMenu(menuBar,"printMenu",NULL,0);
  XtVaSetValues(printButton,XmNsubMenuId,printMenu,NULL);

  /* CREATE PRINTPAGE BUTTON */
  printpage=XtVaCreateManagedWidget(
    "printpage",xmPushButtonWidgetClass,printMenu,NULL);
  text=XmStringCreateLocalized("Current page");
  XtVaSetValues(printpage,XmNlabelString,text,NULL);

  printpageFileBox=XmCreateFileSelectionDialog(mainWindow,
    "printpageFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(printpageFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write Postscript file");
  XtVaSetValues(printpageFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(printpageFileBox,XmNcancelCallback,Cancel,printpageFileBox);
  XtAddCallback(printpageFileBox,XmNokCallback,PrintpageOk,NULL);
  XtAddCallback(printpage,XmNactivateCallback,Printpage,NULL);

  /* CREATE PRINT OPENING WARNING */
  printWarning=XmCreateWarningDialog(mainWindow,"printWarning",NULL,0);
  temp=XmMessageBoxGetChild(printWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(printWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(printWarning,XmNdialogTitle,text,XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open file");
  XtVaSetValues(printWarning,XmNmessageString,text,NULL);


  /* CREATE SETTINGS MENU */

  settingsButton=XtVaCreateManagedWidget(
    "settingsButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Settings");
  XtVaSetValues(settingsButton,XmNlabelString,text,NULL);
  settingsMenu=XmCreatePulldownMenu(menuBar,"settingsMenu",NULL,0);
  XtVaSetValues(settingsButton,XmNsubMenuId,settingsMenu,NULL);

  /* ANALYSIS SETTINGS */

  analyButton=XtVaCreateManagedWidget(
    "analyButton",xmPushButtonWidgetClass,settingsMenu,NULL);
  text=XmStringCreateLocalized("Analysis");
  XtVaSetValues(analyButton,XmNlabelString,text,NULL);

  analyForm=XmCreateBulletinBoardDialog(mainWindow,"analyForm",NULL,0);
  analyOkButton=XtVaCreateManagedWidget("analyOk",xmPushButtonWidgetClass,
    analyForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(analyOkButton,XmNlabelString,text,XmNx,50,XmNy,150,
    XmNwidth,90,NULL);
  analyCancelButton=XtVaCreateManagedWidget("analyCancel",
    xmPushButtonWidgetClass,analyForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(analyCancelButton,XmNlabelString,text,XmNx,200,XmNy,150,NULL);

  analyTextfield=XtVaCreateManagedWidget("analyTextfield",
    xmTextFieldWidgetClass,analyForm,NULL);
  XtVaSetValues(analyTextfield,XmNwidth,100,XmNx,200,XmNy,10,
    XmNvalue,"1",NULL);
  analyLabel1=XtVaCreateManagedWidget("analyLabel1",xmLabelWidgetClass,
    analyForm,NULL);
  text=XmStringCreateLocalized("Gain:");
  XtVaSetValues(analyLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  analyTextfield2=XtVaCreateManagedWidget("analyTextfield2",
    xmTextFieldWidgetClass,analyForm,NULL);
  XtVaSetValues(analyTextfield2,XmNwidth,100,XmNx,200,XmNy,50,
    XmNvalue,"3",NULL);
  analyLabel2=XtVaCreateManagedWidget("analyLabel2",xmLabelWidgetClass,
    analyForm,NULL);
  text=XmStringCreateLocalized("Pitch octave:");
  XtVaSetValues(analyLabel2,XmNlabelString,text,XmNx,0,XmNy,54,NULL);

  analyRadio=XtVaCreateManagedWidget("analyRadio",
    xmRowColumnWidgetClass,analyForm,NULL);
  XtVaSetValues(analyRadio,XmNy,80,XmNradioBehavior,True,
    XmNradioAlwaysOne,True,NULL);
  analyToggle1=XtVaCreateManagedWidget("analyToggle1",
    xmToggleButtonWidgetClass,analyRadio,NULL);
  text=XmStringCreateLocalized("LPC noise estimate");
  XtVaSetValues(analyToggle1,XmNlabelString,text,XmNset,True,NULL);
  analyToggle2=XtVaCreateManagedWidget("analyToggle2",
    xmToggleButtonWidgetClass,analyRadio,NULL);
  text=XmStringCreateLocalized("Entropy noise estimate");
  XtVaSetValues(analyToggle2,XmNlabelString,text,NULL);

  text=XmStringCreateLocalized("Analysis settings");
  XtVaSetValues(analyForm,XmNdialogTitle,text,
    XmNcancelButton,analyCancelButton,
    XmNdefaultButton,analyOkButton,NULL);

  XtAddCallback(analyOkButton,XmNactivateCallback,AnalyOk,0);
  XtAddCallback(analyButton,XmNactivateCallback,Analy,0);


  /* CREATE DISPLAY BUTTON */
  displayButton=XtVaCreateManagedWidget(
    "displayButton",xmPushButtonWidgetClass,settingsMenu,NULL);
  text=XmStringCreateLocalized("Display");
  XtVaSetValues(displayButton,XmNlabelString,text,NULL);

  displayForm=XmCreateBulletinBoardDialog(mainWindow,"displayForm",NULL,0);
  displayOkButton=XtVaCreateManagedWidget("displayOk",xmPushButtonWidgetClass,
    displayForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(displayOkButton,XmNlabelString,text,XmNx,50,XmNy,200,
    XmNwidth,90,NULL);
  displayCancelButton=XtVaCreateManagedWidget("displayCancel",
    xmPushButtonWidgetClass,displayForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(displayCancelButton,XmNlabelString,text,XmNx,200,XmNy,200,NULL);

  colorRadio=XtVaCreateManagedWidget("colorRadio",
    xmRowColumnWidgetClass,displayForm,NULL);
  XtVaSetValues(colorRadio,XmNy,100,XmNradioBehavior,True,
    XmNradioAlwaysOne,True,NULL);
  displayToggleGrey=XtVaCreateManagedWidget("displayToggleGrey",
    xmToggleButtonWidgetClass,colorRadio,NULL);
  text=XmStringCreateLocalized("Grey colors");
  XtVaSetValues(displayToggleGrey,XmNlabelString,text,XmNset,True,NULL);
  displayToggleHot=XtVaCreateManagedWidget("displayToggleHot",
    xmToggleButtonWidgetClass,colorRadio,NULL);
  text=XmStringCreateLocalized("Hot colors");
  XtVaSetValues(displayToggleHot,XmNlabelString,text,NULL);
  displayToggleCold=XtVaCreateManagedWidget("displayToggleCold",
    xmToggleButtonWidgetClass,colorRadio,NULL);
  text=XmStringCreateLocalized("Cool colors");
  XtVaSetValues(displayToggleCold,XmNlabelString,text,NULL);

  displayToggleGrid=XtVaCreateManagedWidget("displayToggleGrid",
    xmToggleButtonWidgetClass,displayForm,NULL);
  text=XmStringCreateLocalized("Show grid");
  XtVaSetValues(displayToggleGrid,XmNlabelString,text,NULL);
  XtAddCallback(displayToggleGrid,XmNvalueChangedCallback,Grid,0);

  displayToggleCentro=XtVaCreateManagedWidget("displayToggleCentro",
    xmToggleButtonWidgetClass,displayForm,NULL);
  text=XmStringCreateLocalized("Show centroid");
  XtVaSetValues(displayToggleCentro,XmNlabelString,text,XmNy,40,NULL);
  XtAddCallback(displayToggleCentro,XmNvalueChangedCallback,Centro,0);

  text=XmStringCreateLocalized("Display settings");
  XtVaSetValues(displayForm,XmNdialogTitle,text,
    XmNcancelButton,displayCancelButton,
    XmNdefaultButton,displayOkButton,NULL);

  XtAddCallback(displayOkButton,XmNactivateCallback,DisplayOk,0);
  XtAddCallback(displayButton,XmNactivateCallback,Displaymenu,0);




  /* CREATE SPEC BUTTON */
  specButton=XtVaCreateManagedWidget(
    "specButton",xmPushButtonWidgetClass,settingsMenu,NULL);
  text=XmStringCreateLocalized("Display spectrum");
  XtVaSetValues(specButton,XmNlabelString,text,NULL);

  specForm=XmCreateBulletinBoardDialog(mainWindow,"specForm",NULL,0);
  specOkButton=XtVaCreateManagedWidget("specOk",xmPushButtonWidgetClass,
    specForm,NULL);
  text=XmStringCreateLocalized("Dismiss");
  XtVaSetValues(specOkButton,XmNlabelString,text,XmNx,100,XmNy,350,
    XmNwidth,90,NULL);
  specPrintButton=XtVaCreateManagedWidget("specPrint",
    xmPushButtonWidgetClass,specForm,NULL);
  text=XmStringCreateLocalized("Print");
  XtVaSetValues(specPrintButton,XmNlabelString,text,XmNx,300,XmNy,350,NULL);
  specExportButton=XtVaCreateManagedWidget("specExport",
    xmPushButtonWidgetClass,specForm,NULL);
  text=XmStringCreateLocalized("Export");
  XtVaSetValues(specExportButton,XmNlabelString,text,XmNx,220,XmNy,350,NULL);


  text=XmStringCreateLocalized("Spectrum at cursor position");
  XtVaSetValues(specForm,XmNdialogTitle,text,XmNdefaultButton,specOkButton,NULL);

  XtAddCallback(specOkButton,XmNactivateCallback,SpecOk,0);
  XtAddCallback(specPrintButton,XmNactivateCallback,SpecPrint,0);
  XtAddCallback(specExportButton,XmNactivateCallback,SpecExport,0);
  XtAddCallback(specButton,XmNactivateCallback, Spec, 0);

  specSketchpad = XtVaCreateManagedWidget(
     "specSketchpad",xmDrawingAreaWidgetClass, specForm, XmNheight, 300,
     XmNwidth, 500, XmNx, 20, XmNy, 20, NULL);
  XtAddCallback(specSketchpad,XmNexposeCallback,RestoreSpec,0);

  printspecFileBox=XmCreateFileSelectionDialog(mainWindow,
    "printspecFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(printspecFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write Postscript file");
  XtVaSetValues(printspecFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(printspecFileBox,XmNcancelCallback,Cancel,printspecFileBox);
  XtAddCallback(printspecFileBox,XmNokCallback,PrintspecOk,NULL);

  /* CREATE SPECPRINT OPENING WARNING */
  specprintWarning=XmCreateWarningDialog(mainWindow,"specprintWarning",NULL,0);
  temp=XmMessageBoxGetChild(specprintWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(specprintWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(specprintWarning,XmNdialogTitle,text,XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open file");
  XtVaSetValues(specprintWarning,XmNmessageString,text,NULL);

  exportspecFileBox=XmCreateFileSelectionDialog(mainWindow,
    "exportspecFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(exportspecFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Export linear spectral data");
  XtVaSetValues(exportspecFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(exportspecFileBox,XmNcancelCallback,Cancel,exportspecFileBox);
  XtAddCallback(exportspecFileBox,XmNokCallback,ExportspecOk,NULL);

  /* CREATE SPECEXPORT OPENING WARNING */
  specexportWarning=XmCreateWarningDialog(mainWindow,"specexportWarning",NULL,0);
  temp=XmMessageBoxGetChild(specexportWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(specexportWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(specexportWarning,XmNdialogTitle,text,XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open file");
  XtVaSetValues(specexportWarning,XmNmessageString,text,NULL);



  /* CREATE CORR BUTTON */
  corrButton=XtVaCreateManagedWidget(
    "corrButton",xmPushButtonWidgetClass,settingsMenu,NULL);
  text=XmStringCreateLocalized("Display cross-correllograms");
  XtVaSetValues(corrButton,XmNlabelString,text,NULL);

  corrForm=XmCreateBulletinBoardDialog(mainWindow,"corrForm",NULL,0);
  corrOkButton=XtVaCreateManagedWidget("corrOk",xmPushButtonWidgetClass,
    corrForm,NULL);
  text=XmStringCreateLocalized("Dismiss");
  XtVaSetValues(corrOkButton,XmNlabelString,text,XmNx,100,XmNy,650,
    XmNwidth,90,NULL);
  corrPrintButton=XtVaCreateManagedWidget("corrPrint",
    xmPushButtonWidgetClass,corrForm,NULL);
  text=XmStringCreateLocalized("Print");
  XtVaSetValues(corrPrintButton,XmNlabelString,text,XmNx,300,XmNy,650,NULL);
  corrExportButton=XtVaCreateManagedWidget("corrExport",
    xmPushButtonWidgetClass,corrForm,NULL);
  text=XmStringCreateLocalized("Export");
  XtVaSetValues(corrExportButton,XmNlabelString,text,XmNx,220,XmNy,650,NULL);


  text=XmStringCreateLocalized("Cross-channel correllograms");
  XtVaSetValues(corrForm,XmNdialogTitle,text,XmNdefaultButton,corrOkButton,NULL);

  XtAddCallback(corrOkButton,XmNactivateCallback,CorrOk,0);
  XtAddCallback(corrPrintButton,XmNactivateCallback,CorrPrint,0);
  XtAddCallback(corrExportButton,XmNactivateCallback,CorrExport,0);
  XtAddCallback(corrButton,XmNactivateCallback, Corr, 0);

  corrSketchpad = XtVaCreateManagedWidget(
     "corrSketchpad",xmDrawingAreaWidgetClass, corrForm, XmNheight, 600,
     XmNwidth, 1000, XmNx, 20, XmNy, 20, NULL);
  XtAddCallback(corrSketchpad,XmNexposeCallback,RestoreCorr,0);

  printcorrFileBox=XmCreateFileSelectionDialog(mainWindow,
    "printcorrFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(printcorrFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write Postscript file");
  XtVaSetValues(printcorrFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(printcorrFileBox,XmNcancelCallback,Cancel,printcorrFileBox);
  XtAddCallback(printcorrFileBox,XmNokCallback,PrintcorrOk,NULL);

  /* CREATE CORRPRINT OPENING WARNING */
  corrprintWarning=XmCreateWarningDialog(mainWindow,"corrprintWarning",NULL,0);
  temp=XmMessageBoxGetChild(corrprintWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(corrprintWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(corrprintWarning,XmNdialogTitle,text,XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open file");
  XtVaSetValues(corrprintWarning,XmNmessageString,text,NULL);

  exportcorrFileBox=XmCreateFileSelectionDialog(mainWindow,
    "exportcorrFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(exportcorrFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Export cross-correlation data");
  XtVaSetValues(exportcorrFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(exportcorrFileBox,XmNcancelCallback,Cancel,exportcorrFileBox);
  XtAddCallback(exportcorrFileBox,XmNokCallback,ExportcorrOk,NULL);

  /* CREATE CORREXPORT OPENING WARNING */
  correxportWarning=XmCreateWarningDialog(mainWindow,"correxportWarning",NULL,0);
  temp=XmMessageBoxGetChild(correxportWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(correxportWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(correxportWarning,XmNdialogTitle,text,XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open file");
  XtVaSetValues(correxportWarning,XmNmessageString,text,NULL);



  /* CREATE MARKERS MENU */

  markersButton=XtVaCreateManagedWidget(
    "markersButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Markers");
  XtVaSetValues(markersButton,XmNlabelString,text,NULL);
  markersMenu=XmCreatePulldownMenu(menuBar,"markersMenu",NULL,0);
  XtVaSetValues(markersButton,XmNsubMenuId,markersMenu,NULL);


  /* CREATE ADD BUTTON */
  addMarker=XtVaCreateManagedWidget(
    "addMarker",xmPushButtonWidgetClass,markersMenu,NULL);
  text=XmStringCreateLocalized("Add");
  XtVaSetValues(addMarker,XmNlabelString,text,NULL);

  XtAddCallback(addMarker,XmNactivateCallback,AddMarker,0);


  /* CREATE REMOVE BUTTON */
  removeMarker=XtVaCreateManagedWidget(
    "removeMarker",xmPushButtonWidgetClass,markersMenu,NULL);
  text=XmStringCreateLocalized("Remove");
  XtVaSetValues(removeMarker,XmNlabelString,text,NULL);

  XtAddCallback(removeMarker,XmNactivateCallback,RemoveMarker,0);



  /* CREATE HELP THINGS */
  helpButton=XtVaCreateManagedWidget(
    "helpButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Help");
  XtVaSetValues(helpButton,XmNlabelString,text,NULL);
  XtVaSetValues(menuBar,XmNmenuHelpWidget,helpButton,NULL);

  helpMenu=XmCreatePulldownMenu(menuBar,"helpMenu",NULL,0);
  help=XtVaCreateManagedWidget(
    "helpMenu",xmPushButtonWidgetClass,helpMenu,NULL);
  text=XmStringCreateLocalized("About");
  XtVaSetValues(help,XmNlabelString,text,NULL);  
  XtVaSetValues(helpButton,XmNsubMenuId,helpMenu,NULL);

  helpBox=XmCreateMessageDialog(help,"helpBox",NULL,0);
  text=XmStringCreateSimple("Help");
  XtVaSetValues(helpBox,XmNdialogTitle,text,NULL);
  text=XmStringCreateLtoR(
    "BIO is an audio analysis tool aimed\n\
at bioacoustical applications.\n\n\
The phase vocoder core is based on code from\n\
F.Richard Moore: Elements of Computer Music\n\n\
Version 0.02,   Oyvind Hammer 1999\n\
Please send your comments to oyvindha@notam.uio.no.",
     XmFONTLIST_DEFAULT_TAG);
  XtVaSetValues(helpBox,XmNmessageString,text,XmNwidth,600,
    XmNx,100,XmNy,200,NULL);

  temp=XmMessageBoxGetChild(helpBox,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(helpBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  XtAddCallback(help,XmNactivateCallback,ShowHelp,helpBox);


  XtRealizeWidget(topLevel);
  setgreycolors();

  defdepth=DefaultDepthOfScreen(XtScreen(topLevel));
  pixelmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel), zoomin_bits, 16, 16, 0, 3, defdepth);
  XtVaSetValues(zoominB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,pixelmap,NULL);
  pixelmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),zoomout_bits, 16, 16, 0, 3, defdepth);
  XtVaSetValues(zoomoutB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,pixelmap,NULL);
  playpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),play_bits, 16, 16, 0, 1, defdepth);
  noplaypixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),play_bits, 16, 16, 0, 3, defdepth);
  stoppixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),stop_bits, 16, 16, 0, 1, defdepth);
  nostoppixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),stop_bits, 16, 16, 0, 3, defdepth);
  XtVaSetValues(playB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
    noplaypixmap,NULL);
  XtVaSetValues(stopB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
    nostoppixmap,NULL);
    
  #ifdef LINUX
  initialiseAudio();
  #endif    

  XtAppMainLoop(app_context);
  
  #ifdef LINUX
  finishAudio();
  #endif  
}
