/* CERES: A combination of FFT and granular synthesis */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <malloc.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include "../config.h"
#else
#define SGI
#include <audiofile.h>
#include <audio.h>
#endif

#ifdef SGI
#include <sys/fpu.h>
#endif

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Display.h>
#include <Xm/RowColumn.h>
#include <Xm/Frame.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/CascadeB.h>
#include <Xm/MessageB.h>
#include <Xm/FileSB.h>
#include <Xm/DrawingA.h>
#include <Xm/Scale.h>
#include <Xm/SelectioB.h>
#include <Xm/BulletinB.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/Label.h>
#include <Xm/ToggleB.h>
#include <Xm/Separator.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>


#define FORWARD 1
#define INVERSE 0
#define MAX_FFT_SIZE 4096
#define MAX_PAINT 5000
#define MAXNOD 800

#define fvec(name, size)\
if ((name=(double *)calloc(size, sizeof(double)))==NULL) {\
  printf("Insufficient memory\n");\
}
#define ffvec(name, size)\
if ((name=(float *)calloc(size, sizeof(float)))==NULL) {\
  printf("Insufficient memory\n");\
}
#define LSIZE 512
#define FLUSH_LIST(list, count, showGC, theGC)        \
          XDrawPoints(theDisplay,XtWindow(sketchpad),showGC,list,count, \
              CoordModeOrigin);       \
          XDrawPoints(theDisplay,bitmap,theGC,list,count,     \
              CoordModeOrigin);       \
        count=0

#define ADD_POINT(xx, yy, list, count, showGC, theGC) \
      list[count].x = xx;     \
      list[count].y = yy;     \
      if (++(count)==LSIZE) { \
              FLUSH_LIST(list,count,showGC,theGC);    \
      }       

extern double lpamp(double, double, double *, int);
extern double lpa(double *, int, double *, int, int);

Widget loadFileBox, saveFileBox, playFileBox, tocsoundFileBox, toparmerudFileBox;
Widget loadLpc, loadLpcBox;
Widget tostfFileBox, toltasFileBox, topeakFileBox, imageFileBox, brkFileBox,
  brkSaveFileBox;
Widget sketchpad, topLevel, dw;
Widget sshiftForm, sshiftTextfield, sshiftTextfield2, sshiftToggle1;
Widget mirrorForm, mirrorTextfield, mirrorTextfield2, mirrorToggle;
Widget filterForm, filterTextfield, filterTextfield2;
Widget gain, gainForm, gainTextfield, gainOkButton, gainCancelButton, gainLabel,
  gainToggle;
Widget threshold, thresholdForm, thresholdTextfield, thresholdToggle,
  thresholdOkButton, thresholdCancelButton, thresholdLabel;
Widget noisered, noiseredForm, noiseredTextfield, noiseredOkButton,
  noiseredCancelButton, noiseredLabel, noiseredToggle, noiseredToggle2,
  noiseredToggle3;
Widget fileWarning, sizeWarning, csoundWarning, stfWarning, writeWarning,
  ltasWarning;
Widget settingsButton;
Widget pitchForm, pitchTextfield, pitchTextfield2, pitchTextfield3,
         pitchToggle1;
Widget expoForm, expoTextfield, expoTextfield2, expoTextfield3, expoToggle;
Widget spreadForm, spreadTextfield, spreadTextfield2, spreadTextfield3, spreadToggle;
Widget magnetForm, magnetTextfield, magnetTextfield2, magnetTextfield3,
         magnetToggle1;
Widget sieveForm, sieveTextfield, sieveTextfield2, sieveTextfield3,
         sieveToggle1, sieveToggle2, sieveToggle3, sieveToggle4;
Widget shearForm, shearTextfield1, shearTextfield2, shearOkButton,
  shearCancelButton, shear, shearToggle1, shearLabel1, shearLabel2;
Widget blurForm, blurTextfield1, blurTextfield2, blurOkButton,
  blurCancelButton, blur, blurToggle1, blurLabel1, blurLabel2,
  blurTextfield3, blurLabel3;
Widget combForm, combTextfield1, combTextfield2, combOkButton, combCancelButton,
  comb, combToggle1, combLabel1, combLabel2;
Widget settingsForm, settingsTextfield, settingsToggle1, settingsToggle2,
  settingsToggle3,
  analyForm, analyToggleHanning, analyToggleRect, analyToggleHamming,
  analyToggleBartlett, analyLabel1, analyL1, analyTextfield,
  analyL2, analyTextfield2, stabilToggle, analySep;
Widget displayForm, displayToggleGrey, displayToggleHot, displayToggleCold,
  displayToggleFunc, displayToggleScale, displayToggleSpots,
  displayGainLabel, displayGainText, displayScaleLabel, displayScaleText;
Widget gridForm, gridTextfield, gridToggleMajor, gridToggleMinor, gridTogglePenta,
  gridToggleChroma, gridToggleWhole, gridToggleHarm, gridToggleEvenharm, gridToggleOddharm, 
  gridToggleFile, freqfile, freqFileBox;
Widget amplitude, pitch, centroid, grScale, grMove, grNonlin;
Widget grScaleForm, grScaleTextfield, grScaleOkButton, grScaleCancelButton,
  grScaleLabel;
Widget grMoveForm, grMoveTextfield, grMoveOkButton, grMoveCancelButton,
  grMoveLabel;

XtAppContext app_context;
Widget mainWindow, menuBar, frame;
Widget fileButton, fileMenu, quit, load, loadimage, loadbrk, savebrk, save, play,
  help, helpButton, helpMenu, helpBox, temp;
Widget transButton, transMenu, sieve, sshift, filter, pshift, nullphase,
  trpeaks, mirror, expo, spread;
Widget ltas, magnet, convolve;
Widget exportButton, exportMenu, tocsound, toparmerud, tostf, toltas, topeak;
Widget settingsButton, settingsMenu, resyntButton,
  analyButton, displayButton, magnetButton;
Widget analyRadio, analyOkButton, analyCancelButton;
Widget pitchOkButton, pitchCancelButton, pitchLabel1, pitchLabel2, pitchLabel3;
Widget sshiftOkButton, sshiftCancelButton, sshiftLabel1, sshiftLabel2;
Widget mirrorOkButton, mirrorCancelButton, mirrorLabel1, mirrorLabel2;
Widget expoOkButton, expoCancelButton, expoLabel1, expoLabel2, expoLabel3;
Widget spreadOkButton, spreadCancelButton, spreadLabel1, spreadLabel2, spreadLabel3;
Widget magnetOkButton, magnetCancelButton, magnetLabel1, magnetLabel2, magnetLabel3;
Widget convolveButton, convolveOkButton, convolveCancelButton, convolveForm,
  convolveRadio, convolveToggleLaplace, convolveToggleMean, 
  convolveToggleTimeMean, convolveToggleFile;
Widget sieveOkButton, sieveCancelButton, sieveLabel1, sieveLabel2, 
  sieveLabel3, sieveRadio, sieveLabel4, sieveLabel5;
Widget filterOkButton, filterCancelButton, filterLabel1, filterLabel2,
  filterFilToggle;
Widget settingsOkButton, settingsCancelButton, settingsLabel1,
  settingsRadio, settingsLabel2, settingsLabel3, settingsLabel4,
  settingsTextfield2, settingsLabel5, settingsTextfield3;
Widget displayOkButton, displayCancelButton, colorRadio;
Widget gridButton, gridOkButton, gridCancelButton, gridRadio, gridLabel1;

Widget spotsButton, spotsMenu, spotsRestart, spotsKeep, spotsRemove;
Widget parmerudRowCol, parmL1, parmL2, parmL3, parmL4,
  parmT1, parmT2, parmT3, parmT4;
Widget extractButton, extractMenu, extractAmp, extractPitch, extractCentro;
Widget graphButton, graphMenu,  grClear, grInvert, grRetro;
Widget progressDialog, progressScale;

Widget lpcButton, lpcMenu, lpcMult, lpcInvert;

Boolean fileDirty = FALSE;
#define PI 3.141592654
#define TWOPI 6.2831853

AFfilehandle infile,  outfile;
AFfilesetup in_AFsetup,  out_AFsetup;

long compression,  filefmt;
int bits_per_samp, samp_type, samps_per_frame=1, vers;
long framecnt;

double *largamp=NULL;
float *lpc_coef=NULL,
      *megfreq=NULL, 
      *megamp=NULL;         /* Enormous arrays */
long horiz_num=0;	    /* Number of analysis frames */
double duration;		    /* Duration in secs */
int numchannels;	    /* Number of phase vocoder channels */
double binfreq;		    /* Frequency pr. bin */
int theheight;
int areat1=0, areat2=0, areaf1=0, areaf2=0, areach=0;
int spotst[MAX_PAINT], spotsf[MAX_PAINT];
int si_valid=-1, playing=0, xold, numspots=0;
double stretchfac=1., dgain=1., dscale=1.;

int lpc_R=44100, numcoef=60, numcoef2=0, lpc_samps_per_frame=1;
long lpc_horiz_num=0, lpc_framecnt;
double lpc_duration;

int R=44100,  Dn,  N,  Nw,  N2,  I, overlap, NP;
int Nw2, obank=0;
double P, Pinc, synt,  *Wanal,  *Wsyn,  *input,  *buffer,  *channel;
double output[4][8192], lastphase[4][4097],
  lastamp[4][4097], lastfreq[4][4097], indx[4][4097];
double *tempamp, *tempfreq, *last, *this;

XGCValues values;
Window theRoot, theWindow;
Colormap theCmap;
Display *theDisplay;
int screen;
Pixmap bitmap, spotsmap;
XColor gray0, gray25, gray50, gray75, gray100;
XColor red, orange, yellow, green, brown;
XColor darkblue, blue, lightblue;

GC theGC, showGC, funtheGC, funshowGC;
GC theGC0, theGC25, theGC50, theGC75, theGC100;
GC showGC0, showGC25, showGC50, showGC75, showGC100;
GC funcGC, gridGC, spotsGC, markerGC;

int xpos[MAXNOD+1], ypos[MAXNOD+1], numsquare=2, selected=0, funcpt;
int farge=0;
double gridfreqs[600], gridfreqs2[600]; int numfreqs;
double filterfreqs[1000];
char playfile[200];


void flush_all_underflows_to_zero()
  {
#ifdef SGI
    union fpc_csr f;
    f.fc_word = get_fpc_csr();
    f.fc_struct.flush = 1;
    set_fpc_csr(f.fc_word);
#endif
  }

void init_again(void)
{
  int i, j;

  for (i=0; i<4; i++) for (j=0; j<8192; j++) output[i][j]=0.;
  for (i=0; i<4; i++) for (j=0; j<4097; j++) {
    lastphase[i][j]=0.; lastfreq[i][j]=0.; lastamp[i][j]=0.; indx[i][j]=0.;
  }
}

/* FFT ROUTINES */

/* If forward is true, rfft replaces 2*N real data points in x with
   N complex values representing the positive frequency half of their
   Fourier spectrum, with x[1] replaced with the real part of the Nyquist
   frequency value.  If forward is false, rfft expects x to contain a
   positive frequency spectrum arranged as before, and replaces it with
   2*N real values.  N MUST be a power of 2. */

rfft( x, N, forward )
double x[]; int N, forward;
{
  double 	c1,c2,
  		h1r,h1i,
		h2r,h2i,
		wr,wi,
		wpr,wpi,
  		temp,
		theta;
  double 	xr,xi;
  int 		i,
		i1,i2,i3,i4,
		N2p1;

    theta = PI/N;
    wr = 1.;
    wi = 0.;
    c1 = 0.5;
    if ( forward ) {
	c2 = -0.5;
	cfft( x, N, forward );
	xr = x[0];
	xi = x[1];
    } else {
	c2 = 0.5;
	theta = -theta;
	xr = x[1];
	xi = 0.;
	x[1] = 0.;
    }
    wpr = -2.*powf( sin( 0.5*theta ), 2. );
    wpi = sin( theta );
    N2p1 = (N<<1) + 1;
    for ( i = 0; i <= N>>1; i++ ) {
	i1 = i<<1;
	i2 = i1 + 1;
	i3 = N2p1 - i2;
	i4 = i3 + 1;
	if ( i == 0 ) {
	    h1r =  c1*(x[i1] + xr );
	    h1i =  c1*(x[i2] - xi );
	    h2r = -c2*(x[i2] + xi );
	    h2i =  c2*(x[i1] - xr );
	    x[i1] =  h1r + wr*h2r - wi*h2i;
	    x[i2] =  h1i + wr*h2i + wi*h2r;
	    xr =  h1r - wr*h2r + wi*h2i;
	    xi = -h1i + wr*h2i + wi*h2r;
	} else {
	    h1r =  c1*(x[i1] + x[i3] );
	    h1i =  c1*(x[i2] - x[i4] );
	    h2r = -c2*(x[i2] + x[i4] );
	    h2i =  c2*(x[i1] - x[i3] );
	    x[i1] =  h1r + wr*h2r - wi*h2i;
	    x[i2] =  h1i + wr*h2i + wi*h2r;
	    x[i3] =  h1r - wr*h2r + wi*h2i;
	    x[i4] = -h1i + wr*h2i + wi*h2r;
	}
	wr = (temp = wr)*wpr - wi*wpi + wr;
	wi = wi*wpr + temp*wpi + wi;
    }
    if ( forward )
	x[1] = xr;
    else
	cfft( x, N, forward );
}

/* cfft replaces double array x containing NC complex values
   (2*NC double values alternating real, imagininary, etc.)
   by its Fourier transform if forward is true, or by its
   inverse Fourier transform if forward is false, using a
   recursive Fast Fourier transform method due to Danielson
   and Lanczos.  NC MUST be a power of 2. */

cfft( x, NC, forward )
double x[]; int NC, forward;
{
  double 	wr,wi,
		wpr,wpi,
		theta,
		scale;
  int 		mmax,
		ND,
		m,
		i,j,
		delta;

    ND = NC<<1;
    bitreverse( x, ND );
    for ( mmax = 2; mmax < ND; mmax = delta ) {
	delta = mmax<<1;
	theta = TWOPI/( forward? mmax : -mmax );
	wpr = -2.*powf( sin( 0.5*theta ), 2. );
	wpi = sin( theta );
	wr = 1.;
	wi = 0.;
	for ( m = 0; m < mmax; m += 2 ) {
	  double rtemp, itemp;
	    for ( i = m; i < ND; i += delta ) {
		j = i + mmax;
		rtemp = wr*x[j] - wi*x[j+1];
		itemp = wr*x[j+1] + wi*x[j];
		x[j] = x[i] - rtemp;
		x[j+1] = x[i+1] - itemp;
		x[i] += rtemp;
		x[i+1] += itemp;
	    }
	    wr = (rtemp = wr)*wpr - wi*wpi + wr;
	    wi = wi*wpr + rtemp*wpi + wi;
	}
    }

/* scale output */

    scale = forward ? 1./ND : 2.;
    for (i=0; i<ND; i++) x[i]*=scale;
}

/* bitreverse places double array x containing N/2 complex values
   into bit-reversed order */

bitreverse( x, N )
double x[]; int N;
{
  double 	rtemp,itemp;
  int 		i,j,
		m;

    for ( i = j = 0; i < N; i += 2, j += m ) {
	if ( j > i ) {
	    rtemp = x[j]; itemp = x[j+1]; /* complex exchange */
	    x[j] = x[i]; x[j+1] = x[i+1];
	    x[i] = rtemp; x[i+1] = itemp;
	}
	for ( m = N>>1; m >= 2 && j >= m; m >>= 1 )
	    j -= m;
    }
}


/* PHASE VOCODER INTERNALS */

/*
 * make balanced pair of analysis (A) and synthesis (S) windows;
 * window lengths are Nw, FFT length is N, synthesis interpolation
 * factor is I
 */
makewindows(A, S, Nw, N, I)
    double A[], S[] ; int Nw, N, I;
{
 int i ;
 double sum ;


  if (XmToggleButtonGetState(analyToggleHamming))
    for ( i = 0 ; i < Nw ; i++ )
      A[i] = S[i] = 0.54 - 0.46*cos( TWOPI*i/(Nw - 1) ) ;
  else if (XmToggleButtonGetState(analyToggleHanning))
    for ( i = 0 ; i < Nw ; i++ )
      A[i] = S[i] = 0.5 - 0.5*cos( TWOPI*i/(Nw - 1) ) ;
  if (XmToggleButtonGetState(analyToggleRect))
    for ( i = 0 ; i < Nw ; i++ )
      A[i] = S[i] = 1. ;
  else if (XmToggleButtonGetState(analyToggleBartlett))
    for ( i = 0 ; i < Nw ; i++ )
      A[i] = S[i] = 1.-fabs(i-(Nw-1.)/2.)/((Nw-1.)/2.);


/*
 * when Nw > N, also apply interpolating (sinc) windows to
 * ensure that window are 0 at increments of N (the FFT length)
 * away from the center of the analysis window and of I away
 * from the center of the synthesis window
 */
    if ( Nw > N ) {
     double x ;

/*
 * take care to create symmetrical windows
 */
	x = -(Nw - 1)/2. ;
	for ( i = 0 ; i < Nw ; i++, x += 1. )
	    if ( x != 0. ) {
		A[i] *= N*sin( PI*x/N )/(PI*x) ;
		S[i] *= I*sin( PI*x/I )/(PI*x) ;
	    }
    }
/*
 * normalize windows for unity gain across unmodified
 * analysis-synthesis procedure
 */
    for ( sum = i = 0 ; i < Nw ; i++ )
	sum += A[i] ;

    for ( i = 0 ; i < Nw ; i++ ) {
     double afac = 2./sum ;
     double sfac = Nw > N ? 1./afac : afac ;
	A[i] *= afac ;
	S[i] *= sfac ;
    }

    if ( Nw <= N) {
	for ( sum = i = 0 ; i < Nw ; i += I )
	    sum += S[i]*S[i] ;
	for ( sum = 1./sum, i = 0 ; i < Nw ; i++ )
	    S[i] *= sum ;
    }
}

short int *framebuff;

int shiftin(A,  N,  D, max, ch)
double A[];
int N, D;
double *max;
int ch;
{
  int i;
  double *Aptr = A, mmax=0.;
    
  if (si_valid<0) si_valid=N;
  i = N-D;
  while (i--) {
    *Aptr = *(Aptr+D);
    Aptr++;
  }
  if (si_valid==N) {
    short int *frames = framebuff;
    int sampsread=AFreadframes(infile, AF_DEFAULT_TRACK, frames, D);
    if (samps_per_frame==1) {
      si_valid = sampsread+N-D;
      while(sampsread--) {
        *(Aptr) = (double) *(frames++) / 32768.0;
        if (fabs(*Aptr)>mmax) mmax=fabs(*Aptr); Aptr++;
      }
    } else {
      si_valid = sampsread+N-D;
      while(sampsread--) {
        *(Aptr) = (double) (*(frames+ch)) / 32768.0;
        frames+=samps_per_frame;
        if (fabs(*Aptr)>mmax) mmax=fabs(*Aptr); Aptr++;
      }
    }
  }
  if (si_valid<N) {
    for (i=si_valid; i<N; i++) A[i]=0.;
    si_valid-=D;
  }
  *max=mmax;
  return (si_valid<=0);
}


/*
 * multiply current input I by window W (both of length Nw);
 * using modulus arithmetic, fold and rotate windowed input
 * into output array O of (FFT) length N according to current
 * input time n
 */
fold( I, W, Nw, O, N, n )
    double I[], W[], O[] ; int Nw, N, n ;
{
 
    int i;

    for ( i = 0; i < N; i++ )
	O[i] = 0.;

    while ( n < 0 )
	n += N;
    n %= N;
    for ( i = 0; i < Nw; i++ ) {
	O[n] += I[i]*W[i];
	if ( ++n == N )
	    n = 0;
    }
}


/* S is a spectrum in rfft format, i.e., it contains N real values
   arranged as real followed by imaginary values, except for first
   two values, which are real parts of 0 and Nyquist frequencies;
   convert first changes these into N/2+1 PAIRS of magnitude and
   phase values to be stored in output array C; the phases are then
   unwrapped and successive phase differences are used to compute
   estimates of the instantaneous frequencies for each phase vocoder
   analysis channel; decimation rate D and sampling rate R are used
   to render these frequency values directly in Hz. */

convert( S, C, N2, D, R, ch )
double S[], C[]; int N2, D, R, ch;
{
  static int 	first = 1;
  static double fundamental,
		factor;
  double 	phase,
		phasediff;
  int 		real,
		imag,
		amp,
		freq;
  double 	a,
		b;
  int 		i;

/* first pass: allocate zeroed space for previous phase
   values for each channel and compute constants */

    if ( first ) {
      first = 0;
      fundamental = (double) R/(N2<<1);
      factor = R/(D*TWOPI);
    } 

/* unravel rfft-format spectrum: note that N2+1 pairs of
   values are produced */

    for ( i = 0; i <= N2; i++ ) {
      imag = freq = ( real = amp = i<<1 ) + 1;
      a = ( i == N2 ? S[1] : S[real] );
      b = ( i == 0 || i == N2 ? 0. : S[imag] );

/* compute magnitude value from real and imaginary parts */

      C[amp] = hypot( a, b );

/* compute phase value from real and imaginary parts and take
   difference between this and previous value for each channel */

      if ( C[amp] == 0. )
	phasediff = 0.;
      else {
	phasediff = ( phase = -atan2( b, a ) ) - lastphase[ch][i];
	lastphase[ch][i] = phase;
	
/* unwrap phase differences */

	while ( phasediff > PI )
	  phasediff -= TWOPI;
	while ( phasediff < -PI )
	  phasediff += TWOPI;
      }

/* convert each phase difference to Hz */

      C[freq] = phasediff*factor + i*fundamental;
    }
}

/* oscillator bank resynthesizer for phase vocoder analyzer
  uses sum of N+1 cosinusoidal table lookup oscillators to 
  compute I (interpolation factor) samples of output O
  from N+1 amplitude and frequency value-pairs in C;
  frequencies are scaled by P */

oscbank( C, N, R, I, O, ch, Nw, coef, Np, a0 )
double C[], O[], coef[], a0; int N, R, I, ch, Nw, Np;
{
  static int 	L = 8192,
		first = 1;
  static double	*table;
  int 		amp, freq, n, chan;
  double	Iinv;

/* first pass: allocate memory for and compute cosine table */

    if ( first ) {
	first = 0;
	fvec(table,L);

	for ( n = 0; n < L; n++ )
	    table[n] = N*cos( TWOPI*(double)n/L);
    }

  Iinv=1./I;

/* for each channel, compute I samples using linear
   interpolation on the amplitude and frequency
   control values */

    for ( chan = 0; chan < NP; chan++ ) {

      double	 	a,
			ainc,
			f,
			finc,
			address;

	freq = ( amp = ( chan << 1 ) ) + 1;
	C[freq] *= Pinc;
	finc = ( C[freq] - ( f = lastfreq[ch][chan] ) )*Iinv;
	if (C[amp]<synt) C[amp] = 0.;
        else if (Np) C[amp]*=lpamp(chan*P*PI/N, a0, coef, Np)/
          lpamp(chan*PI/N, a0, coef, Np);
	ainc = ( C[amp] - ( a = lastamp[ch][chan] ) )*Iinv;
	address = indx[ch][chan];

/* accumulate the I samples from each oscillator into
   output array O (initially assumed to be zero);
   f is frequency in Hz scaled by oscillator increment
   factor and pitch (Pinc); a is amplitude; */

	if (ainc!=0. || a !=0.) for ( n = 0; n < I; n++ ) {
	    O[n] += a*table[ (int) address ];

	    address += f;

	    while ( address >= L )
		address -= L;

	    while ( address < 0 )
		address += L;

	    a += ainc;
	    f += finc;
	}

/* save current values for next iteration */
	lastfreq[ch][chan] = C[freq];
	lastamp[ch][chan] = C[amp];
	indx[ch][chan] = address;
    }
}


/* unconvert essentially undoes what convert does, i.e., it
  turns N2+1 PAIRS of amplitude and frequency values in
  C into N2 PAIR of complex spectrum data (in rfft format)
  in output array S; sampling rate R and interpolation factor
  I are used to recompute phase values from frequencies */

unconvert( C, S, N2, I, R, ch )
double C[], S[]; int N2, I, R, ch;
{
  static int 	first = 1;
  static double fundamental,
		factor;
  int 		i,
		real,
		imag,
		amp,
		freq;
  double 	mag,
		phase;

/* first pass: allocate memory and compute constants */

    if ( first ) {
	first = 0;
	fundamental = (double) R/(N2<<1);
        factor = TWOPI*I/R;
    } 

/* subtract out frequencies associated with each channel,
   compute phases in terms of radians per I samples, and
   convert to complex form */

    for ( i = 0; i <= N2; i++ ) {
	imag = freq = ( real = amp = i<<1 ) + 1;
	if ( i == N2 )
	    real = 1;
	mag = C[amp];
	lastphase[ch][i] += C[freq] - i*fundamental;
	phase = lastphase[ch][i]*factor;
	S[real] = mag*cos( phase );
	if ( i != N2 )
	    S[imag] = -mag*sin( phase );
    }
}


/*
 * input I is a folded spectrum of length N; output O and
 * synthesis window W are of length Nw--overlap-add windowed,
 * unrotated, unfolded input data into output O
 */
overlapadd( I, N, W, O, Nw, n )
    double I[], W[], O[] ; int N, Nw, n ; 
{
 int i ;
    while ( n < 0 )
	n += N ;
    n %= N ;
    for ( i = 0 ; i < Nw ; i++ ) {
	O[i] += I[n]*W[i] ;
	if ( ++n == N )
	    n = 0 ;
    }
}


void shiftout(N,  I,  n)
int N,  I,  n;
{
    int i, ch;

    if (n>=0) {
      for (i=0; i<I; i++) {
        for (ch=0; ch<samps_per_frame; ch++)
          *(framebuff+i*samps_per_frame+ch)=(short)(output[ch][i]*32767.);
      }
      AFwriteframes(outfile, AF_DEFAULT_TRACK, framebuff, I);
    }
    for (ch=0; ch<samps_per_frame; ch++) {
      for (i=0; i<N-I; i++) output[ch][i]=output[ch][i+I];
      for (i=N-I; i<N; i++) output[ch][i]=0.;
    }
}


void rightbins()
{
   double nyquist;
   long point=0;
   int i,  j, thebin;

   nyquist=R/2.;
   for (i=0; i<horiz_num*samps_per_frame; i++) {
     for (j=0; j<numchannels; j++) tempamp[j]=0.;
     for (j=0; j<numchannels; j++) {
       if ((megfreq[point+j]>0.) && (megfreq[point+j]<nyquist)) {
       thebin=(int)(0.5+megfreq[point+j]/binfreq);
if (thebin>=numchannels) printf("OOPS: %d\n", thebin);
         if (megamp[point+j]>tempamp[thebin]) {
	   tempamp[thebin]=megamp[point+j];
	   tempfreq[thebin]=megfreq[point+j];
         }
       }
     }
     for (j=0; j<numchannels; j++) {
	 megfreq[point+j]=tempfreq[j];
	 megamp[point+j]=tempamp[j];
     }
     point+=numchannels;
   }
}

void clipfreq()
{
  double nyquist;
  long point=0;
  int i, j, thebin;

  nyquist=R/2.;
  for (i=0; i<horiz_num*samps_per_frame; i++) {
    for (j=0; j<numchannels; j++) {
      if ((megfreq[point]<0.) || (megfreq[point]>=nyquist)) megamp[point]=0.;
      point++;
    }
  }
} 

void SetWatchCursor(w)
Widget w;
{
        static Cursor watch = NULL;

        if(!watch)
                watch = XCreateFontCursor(XtDisplay(w),XC_watch);

        XDefineCursor(XtDisplay(w),XtWindow(w),watch);
        XmUpdateDisplay(w);
}

void ResetCursor(w)
Widget w;
{
        XUndefineCursor(XtDisplay(w),XtWindow(w));
        XmUpdateDisplay(w);
}

void Playlast(void)
{
#ifndef NOAUDIO
  int sampsread, chnls, t=1, numfram, i, fill, x;
  ALport audioport;
  ALconfig config;
  AFfilehandle filehandle;
  AFfilesetup filesetup;
  short int frames[4096];
  XmString text;
  double time=0.;
#ifdef LINUX  
  long tempBufOut [4] = {0,0,0,0};
#endif

  if (playing) { playing=0; return; }
  if (playfile[0]=='\0') return;

  filehandle=AFopenfile(playfile, "rb", NULL);
  if (filehandle==NULL) {
    XtManageChild(fileWarning);
    return;
  }

  XSetFunction(theDisplay, markerGC, GXxor);
  XSetForeground(theDisplay, markerGC, gray100.pixel);

  playing=1; xold=-999999;
  chnls=AFgetchannels(filehandle, AF_DEFAULT_TRACK);
  config=ALnewconfig();
  ALsetchannels(config, chnls);
  audioport=ALopenport("Ceres Play", "w", config);

  do {
    sampsread=AFreadframes(filehandle, AF_DEFAULT_TRACK, frames, 1024);
    numfram=sampsread*chnls;
    ALwritesamps(audioport, frames, numfram);
    fill=ALgetfilled(audioport);
#ifdef LINUX
    ALwritesamps(audioport,tempBufOut,chnls);
#endif 
    x=(int)((time-(double)fill/R/chnls)*800./duration)+50;
    if (x!=xold) {
      if (x<-10000) x=-10000; if (x>10000) x=10000;
      XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,xold,10,xold,10+theheight);
      XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,x,10,x,10+theheight);
      xold=x;
    }
    time+=1024/(double)R;

    if (XtAppPending(app_context) && ((t++)%3==0))
      XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
  } while ((playing) && (sampsread==1024));

  while (fill=ALgetfilled(audioport)) {
#ifdef LINUX
    ALwritesamps(audioport,tempBufOut,chnls);
#endif  
    x=(int)((time-(double)fill/R/chnls)*800./duration)+50;
    if (x!=xold) {
      XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,xold,10,xold,10+theheight);
      XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,x,10,x,10+theheight);
      xold=x; XFlush(theDisplay);
    }
  }
  XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,xold,10,xold,10+theheight);

  ALcloseport(audioport);
  playing=0;
#endif /*NOAUDIO*/
}

void drawscale()
{
  int i, grafx, step=1, ch;
  char tall[10];

  theGC=theGC0; showGC=showGC0;
  XDrawLine(theDisplay,XtWindow(sketchpad),showGC,
    40,theheight+20,850,theheight+20);
  XDrawLine(theDisplay,bitmap,theGC,40,theheight+20,850,theheight+20);
  XDrawLine(theDisplay,XtWindow(sketchpad),showGC,40,theheight+20,40,10);
  XDrawLine(theDisplay,bitmap,theGC,40,theheight+20,40,10);
  if (duration>25.) step=2;
  if (duration>50.) step=4;
  for (i=1; i<=(int)(duration); i+=step) {
    grafx=(int)(i*800./duration)+50;
    XDrawLine(theDisplay,XtWindow(sketchpad),showGC,
      grafx,theheight+15,grafx,theheight+25);
    XDrawLine(theDisplay,bitmap,theGC,grafx,theheight+15,grafx,theheight+25);
    sprintf(tall, "%d", i);
    XDrawString(theDisplay,XtWindow(sketchpad),showGC,
      grafx-5,theheight+40,tall,strlen(tall));
    XDrawString(theDisplay,bitmap,theGC,
      grafx-5,theheight+40,tall,strlen(tall));
  }
  for (ch=0; ch<samps_per_frame; ch++)
    for (i=1; i<=(int)(R/2000)/dscale; i+=samps_per_frame) {
      grafx=theheight-(int)((dscale*i*1000./binfreq)/samps_per_frame)+10
        -ch*theheight/samps_per_frame;
      XDrawLine(theDisplay,XtWindow(sketchpad),showGC,35,grafx,45,grafx);
      XDrawLine(theDisplay,bitmap,theGC,35,grafx,45,grafx);
      sprintf(tall, "%2d", i);
      XDrawString(theDisplay,XtWindow(sketchpad),showGC,
        2,grafx+5,tall,strlen(tall));
      XDrawString(theDisplay,bitmap,theGC,
        2,grafx+5,tall,strlen(tall));
    }
}

double getfuncval(hnum)
int hnum;
{
  double hnumx;

  hnumx=hnum*800./horiz_num+50.;
  while ((int)floor(hnumx)>xpos[funcpt+1]) funcpt++;
  return theheight-(ypos[funcpt+1]-ypos[funcpt])*(hnumx-xpos[funcpt])
    /(xpos[funcpt+1]-xpos[funcpt])-ypos[funcpt]+10;

}


/* CALLBACK ROUTINES */

void RedrawWin(void)
{
    int i, j, grafx, grafold, ch, y;
    long point=0;
    XPoint list0[LSIZE],list1[LSIZE],list2[LSIZE],list3[LSIZE];
    int c0=0,c1=0,c2=0,c3=0;
    double a;

    XFillRectangle(theDisplay, bitmap, theGC100, 0, 0, 800+70, theheight+70);
    XClearArea(theDisplay, XtWindow(sketchpad), 0, 0, 0, 0, True);
    SetWatchCursor(topLevel);

    if (horiz_num>0) drawscale();

  for (ch=0; ch<samps_per_frame; ch++) {
    grafold=-1; point=ch*numchannels*horiz_num;
    for (j=0; j<horiz_num; j++) {
      grafx = (int)(j*800./horiz_num)+50;
      if (grafx!=grafold) {
        for (i=0; i<numchannels; i++) {
          a=megamp[point]*dgain;
          if (a>0.0000012) {
            y=(int)(dscale*megfreq[point]/binfreq);
            if (y<theheight) {
            y=((ch+1)*theheight-y)/samps_per_frame+10;
            if (a>0.0000100) {
              ADD_POINT(grafx,y,list0,c0,showGC0,theGC0);
            } else if (a>0.0000050){ 
              ADD_POINT(grafx,y,list1,c1,showGC25,theGC25);
            } else if (a>0.0000025) {
              ADD_POINT(grafx,y,list2,c2,showGC50,theGC50);
            } else {
              ADD_POINT(grafx,y,list3,c3,showGC75,theGC75);
            }
          }}
          point++;
        }

        grafold=grafx;
      } else point+=numchannels;
    }
  }
   FLUSH_LIST(list0,c0,showGC0,theGC0);
   FLUSH_LIST(list1,c1,showGC25,theGC25);
   FLUSH_LIST(list2,c2,showGC50,theGC50);
   FLUSH_LIST(list3,c3,showGC75,theGC75);

  for (ch=0; ch<samps_per_frame; ch++) for (i=0; i<numspots; i++) {
    y=dscale*spotsf[i];
    if (y<theheight)
      XFillRectangle(theDisplay,bitmap,spotsGC,
        spotst[i]*800./horiz_num+50-3,
        theheight-(ch*theheight+y)/samps_per_frame+10-3, 6,6);
  }

  ResetCursor(topLevel);
}

void RedrawFunc(void)
{
  int i;

  if (XmToggleButtonGetState(displayToggleFunc)==False) return;
  for (i=1; i<=numsquare; i++) {
    XDrawRectangle(theDisplay,XtWindow(sketchpad),funcGC,
      xpos[i]-4,ypos[i]-4,8,8);
  }
  for (i=1; i<numsquare; i++) {
    XDrawLine(theDisplay,XtWindow(sketchpad),funcGC,
      xpos[i],ypos[i],xpos[i+1],ypos[i+1]);
  }
  if (selected>0) {
    XFillRectangle(theDisplay,XtWindow(sketchpad),funcGC,
      xpos[selected]-4,ypos[selected]-4,8,8);
  }
}

void RedrawArea(void)
{
  int i, t1, t2, f1, f2;

  if ((areat1==0) && (areat2==horiz_num)
    && (areaf1==0) && (areaf2==numchannels)) return;
  if (areat1<areat2) { t1=areat1; t2=areat2; } else { t1=areat2; t2=areat1; }
  if (areaf1<areaf2) { f1=areaf1; f2=areaf2; } else { f1=areaf2; f2=areaf1; }
  f1*=dscale; f2*=dscale;
  if (f1>theheight) f1=theheight;
  if (f2>theheight) f2=theheight;
  for (i=0; i<samps_per_frame; i++)
    XDrawRectangle(theDisplay,XtWindow(sketchpad),funcGC,
      t1*800./horiz_num+50,
      theheight-i*theheight/samps_per_frame-f2/samps_per_frame+10,
      (t2-t1)*800./horiz_num, (f2-f1)/samps_per_frame);
}

void RedrawScale(void)
{
  int i, f, ch;

  if (XmToggleButtonGetState(displayToggleScale)==False) return;
  for (ch=0; ch<samps_per_frame; ch++)
    for (i=0; i<numfreqs; i++) if ((gridfreqs[i]>0.) && (gridfreqs[i]<R/2.))  {
      f=(int)(dscale*gridfreqs[i]/binfreq);
      if (f>theheight) break;
      f=theheight-(ch*theheight+f)/samps_per_frame+10;
      XDrawLine(theDisplay,XtWindow(sketchpad),gridGC,50,f,800,f);
    }
}

void RestoreWin(w, client, call)
Widget w;
XtPointer client, call;
{
  XCopyArea(theDisplay, bitmap, XtWindow(sketchpad),
    DefaultGCOfScreen(XtScreen(sketchpad)), 0, 0, 800+70, theheight+70, 0, 0);
  RedrawFunc();
  RedrawArea();
  RedrawScale();
}

static void Keydown(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  KeySym keysym;
  char key_string[1];
  int key_string_len, i;

  key_string_len=XLookupString((XKeyEvent *) event, key_string,
    sizeof key_string, &keysym, (XComposeStatus *) NULL);
  if ((selected>1) && (selected<numsquare) &&
    ((keysym==XK_BackSpace) || (keysym==XK_Delete))) {
      for (i=selected; i<numsquare; i++) {
        xpos[i]=xpos[i+1]; ypos[i]=ypos[i+1];
      }
      numsquare--; selected=0;
      RestoreWin(NULL,NULL,NULL);
    }
  if ((keysym==XK_p) || (keysym==XK_space)) Playlast();
}

static void GraphClick(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, i, which=0, addpos;

  XButtonEvent *motion=(XButtonEvent *)event;
  x=motion->x; y=motion->y;

  if (y>theheight+10) y=theheight+10;
  if (y<10) y=10;
  for (i=1; i<=numsquare; i++)
    if ((x>xpos[i]-4) && (x<xpos[i]+4)) {
      which=i;
      break;
    }
  if (which>0) selected=which;
  else if ((x>50) && (x<850) && (numsquare<MAXNOD)) { 
    for (i=1; i<=numsquare; i++)
      if (x<xpos[i]) {
        addpos=i; break;
      }
    numsquare++;
    for (i=numsquare; i>addpos; i--) {
      xpos[i]=xpos[i-1]; ypos[i]=ypos[i-1];
    }
    xpos[addpos]=x; ypos[addpos]=y;
    selected=addpos;
  }
  RestoreWin(NULL,NULL,NULL);
}

static void GraphDrag(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y;

  XButtonEvent *motion=(XButtonEvent *)event;
  x=motion->x; y=motion->y;

  if (y>theheight+10) y=theheight+10;
  if (y<10) y=10;
  if (selected>0) {
    if (selected==1) ypos[1]=y;
    else if (selected==numsquare) ypos[numsquare]=y;
    else if ((x>xpos[selected-1]) && (x<xpos[selected+1])) {
      xpos[selected]=x; ypos[selected]=y;
    }
    RestoreWin(NULL,NULL,NULL);
  }
}

static void AreaClick(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y;

  XButtonEvent *motion=(XButtonEvent *)event;
  x=motion->x; y=motion->y;

  if (y>theheight+20) y=theheight+20;
  if (y<0) y=0;
  areat1=((x-50)*horiz_num)/800;
  if (areat1<0) areat1=0; if (areat1>horiz_num) areat1=horiz_num;
  areat2=areat1;
  areaf1=theheight-y+10;
  areach=areaf1*samps_per_frame/numchannels;
  areaf1=(areaf1%(numchannels/samps_per_frame))*samps_per_frame;
  areaf1/=dscale;
  if (areaf1<0) areaf1=0; if (areaf1>numchannels) areaf1=numchannels;
  areaf2=areaf1;

  RestoreWin(NULL,NULL,NULL);
}

static void AreaDrop(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int swap;

  if ((areat1==areat2) && (areaf1==areaf2)) {
    areat1=0; areat2=horiz_num;
    areaf1=0; areaf2=numchannels;
    return;
  }
  if (areat1>areat2) { swap=areat1; areat1=areat2; areat2=swap; }
  if (areaf1>areaf2) { swap=areaf1; areaf1=areaf2; areaf2=swap; }
}

static void AreaDrag(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, swap, ch;
  XButtonEvent *motion=(XButtonEvent *)event;

  x=motion->x; y=motion->y;
  if (y>theheight+20) y=theheight+20;
  if (y<0) y=0;
  if (x<40) x=40;
  if (XmToggleButtonGetState(displayToggleSpots)==False) {
    areat2=((x-50)*horiz_num)/800.;
    if (areat2<0) areat2=0; if (areat2>horiz_num) areat2=horiz_num;
    areaf2=theheight-y+10;
    if (areaf2<areach*numchannels/samps_per_frame)
      areaf2=areach*numchannels/samps_per_frame;
    if (areaf2>=(areach+1)*numchannels/samps_per_frame) areaf2=numchannels;
      else areaf2=(areaf2%(numchannels/samps_per_frame))*samps_per_frame;
    areaf2/=dscale;
    RestoreWin(NULL,NULL,NULL);
  } else if (numspots<MAX_PAINT) {
    spotst[numspots]=((x-50)*horiz_num)/800.;
    if (spotst[numspots]<0) spotst[numspots]=0;
    if (spotst[numspots]>horiz_num-1) spotst[numspots]=horiz_num-1;
    spotsf[numspots]=theheight-y+10;
    spotsf[numspots]=(spotsf[numspots]%(numchannels/samps_per_frame))*samps_per_frame;
    spotsf[numspots]/=dscale;
    for (ch=0; ch<samps_per_frame; ch++) {
      XFillRectangle(theDisplay,bitmap,spotsGC,
        spotst[numspots]*800./horiz_num+50-3,
        theheight-ch*theheight/samps_per_frame
          -dscale*spotsf[numspots]/samps_per_frame+10-3,
        6,6);
      XFillRectangle(XtDisplay(sketchpad),XtWindow(sketchpad),spotsGC,
        spotst[numspots]*800./horiz_num+50-3,
        theheight-ch*theheight/samps_per_frame
          -dscale*spotsf[numspots]/samps_per_frame+10-3,
        6,6);
    }
    numspots++;
  }
}

void Cancel(w, client, call)
Widget w;
XtPointer client, call;
{
  Widget dialog=(Widget) client;
  XtUnmanageChild(dialog);
}

void ShowHelp(w, client, call)
Widget w;
XtPointer client, call;
{
  Widget dialog=(Widget) client;
  XtManageChild(dialog);
}

void Quit(w, client, call)
Widget w;
XtPointer client, call;
{
  exit(0);
}

void loadlpc(char *filename)
{
  int in, i, ch, eof, pr, lastpr=-1, S=0;
  long point, j=0;
  double amp, a0;
  double *coef;

  if (XmToggleButtonGetState(stabilToggle)) S=1;
  infile=AFopenfile(filename, "rb", in_AFsetup);
  if (infile==NULL) {
    XtManageChild(fileWarning);
    return;
  }
  lpc_samps_per_frame   = AFgetchannels(infile, AF_DEFAULT_TRACK);
  lpc_R                 = (int)AFgetrate(infile, AF_DEFAULT_TRACK);
  compression       = AFgetcompression(infile, AF_DEFAULT_TRACK);
  filefmt           = AFgetfilefmt(infile, &vers);   
  AFgetsampfmt(infile, AF_DEFAULT_TRACK, &samp_type, &bits_per_samp);
  lpc_framecnt          = AFgetframecnt(infile,  AF_DEFAULT_TRACK);
  
  lpc_horiz_num = (lpc_framecnt/Dn)+overlap;
  if (numcoef*lpc_horiz_num*lpc_samps_per_frame > 32000000) {
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    return;
  } 
  lpc_duration = (double)lpc_framecnt/lpc_R;
  if (lpc_coef!=NULL) free(lpc_coef);
  lpc_coef=NULL;
  ffvec(lpc_coef, (numcoef+2)*lpc_horiz_num*lpc_samps_per_frame);
  if (lpc_coef==NULL) {
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    return;
  }
  fvec(coef, numcoef+2);

  SetWatchCursor(topLevel);
  XtVaSetValues(progressScale, XmNvalue, 0, NULL);
  XtManageChild(progressDialog);
  while (XtAppPending(app_context)) {
     XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
  }

  for (ch=0; ch<lpc_samps_per_frame; ch++) {
    eof=0; si_valid=-1; point=ch*(numcoef+2)*lpc_horiz_num;
    in=-Nw;
    AFseekframe(infile, AF_DEFAULT_TRACK, 0);
    while (!eof) {
      in+=Dn;
      eof=shiftin(input,  Nw,  Dn, &amp, ch);
      coef[numcoef+1]=a0=lpa(input, Nw, coef, numcoef, S);
      for (i=0; i<numcoef+2; i++) {
 	lpc_coef[point]=coef[i];
        point++;
      }
      j++;
      pr=(int)(100.*(double)j/(lpc_samps_per_frame*lpc_horiz_num));
      if (pr!=lastpr) {
        XtVaSetValues(progressScale, XmNvalue, pr, NULL);
        XmUpdateDisplay(topLevel);
        lastpr=pr;
      }
    }
  }

  numcoef2=numcoef;
  AFclosefile(infile);
  free(coef);
  XtUnmanageChild(progressDialog);
  ResetCursor(topLevel);
}

void loadana(char *filename)
{
  int in, i, ch, eof, grafx, grafold, j, y;
  long point, p;
  double amp;
  XPoint list0[LSIZE],list1[LSIZE],list2[LSIZE],list3[LSIZE];
  int c0=0,c1=0,c2=0,c3=0;
  double a;

  infile=AFopenfile(filename, "rb", in_AFsetup);
  if (infile==NULL) {
    XtManageChild(fileWarning);
    return;
  }
  samps_per_frame   = AFgetchannels(infile, AF_DEFAULT_TRACK);
  R                 = (int)AFgetrate(infile, AF_DEFAULT_TRACK);
  compression       = AFgetcompression(infile, AF_DEFAULT_TRACK);
  filefmt           = AFgetfilefmt(infile, &vers);   
  AFgetsampfmt(infile, AF_DEFAULT_TRACK, &samp_type, &bits_per_samp);
  framecnt          = AFgetframecnt(infile,  AF_DEFAULT_TRACK);

  init_again();
  horiz_num = (framecnt/Dn)+overlap;
  if (numchannels*horiz_num*samps_per_frame > 32000000) {
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    return;
  } 
  duration = (double)framecnt/R;
  binfreq = (double)R/N;
  if (megfreq!=NULL) free(megfreq);
  if (megamp!=NULL) free(megamp);
  if (largamp!=NULL) free(largamp);
  megfreq=megamp=NULL; largamp=NULL;
  ffvec(megfreq, numchannels*horiz_num*samps_per_frame);
  if (megfreq==NULL) {
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    return;
  }
  ffvec(megamp, numchannels*horiz_num*samps_per_frame);
  if (megamp==NULL) {
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    return;
  }
  fvec(largamp, horiz_num);
  if (largamp==NULL) {
    XtManageChild(sizeWarning);
    AFclosefile(infile); 
    return;
  }

  SetWatchCursor(topLevel);
  areat1=0; areat2=horiz_num;
  areaf1=0; areaf2=numchannels;

  XFillRectangle(XtDisplay(sketchpad), bitmap, theGC100,
    0, 0, 800+70, theheight+70);
  XClearArea(XtDisplay(sketchpad), XtWindow(sketchpad), 0, 0, 0, 0, True);

  drawscale();

  for (ch=0; ch<samps_per_frame; ch++) {
    eof=0; si_valid=-1; point=ch*numchannels*horiz_num;
    p=0; in=-Nw, j=0; grafold=-1;
    AFseekframe(infile, AF_DEFAULT_TRACK, 0);
    while (!eof) {
      in+=Dn;
      eof=shiftin(input,  Nw,  Dn, &amp, ch);
      if (ch==0) largamp[p++]=amp;
      fold(input,  Wanal,  Nw,  buffer,  N,  in);
      rfft(buffer,  N2,  FORWARD);
      convert(buffer,  channel,  N2,  Dn,  R, ch);
      grafx = (int)(j++*800./horiz_num)+50;
      for (i=0; i<numchannels; i++) { 
 	megamp[point]=channel[i+i];
	megfreq[point]=channel[i+i+1];
        point++;
      }

      if (grafx!=grafold) {
        point-=numchannels;
          for (i=0; i<numchannels; i++) {
            a=dgain*megamp[point];
            if (a>0.0000012) {
              y=(int)(dscale*megfreq[point]/binfreq);
              if (y<theheight) {        
                y = ((ch+1)*theheight-y)/samps_per_frame+10;
                if (a>0.0000100) {
                  ADD_POINT(grafx,y,list0,c0,showGC0,theGC0);
                } else if (a>0.0000050){ 
                  ADD_POINT(grafx,y,list1,c1,showGC25,theGC25);
                } else if (a>0.0000025) {
                  ADD_POINT(grafx,y,list2,c2,showGC50,theGC50);
                } else {
                  ADD_POINT(grafx,y,list3,c3,showGC75,theGC75);
                }
              }
            }
            point++;
          }
        grafold=grafx;
      }
    }
  }
  FLUSH_LIST(list0,c0,showGC0,theGC0);
  FLUSH_LIST(list1,c1,showGC25,theGC25);
  FLUSH_LIST(list2,c2,showGC50,theGC50);
  FLUSH_LIST(list3,c3,showGC75,theGC75);

  AFclosefile(infile);
  fileDirty = FALSE;
  strcpy(playfile, filename);
  RedrawFunc();
  ResetCursor(topLevel);
}

void Load(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(loadFileBox);
}

void LoadOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  long int pvbuf[8];

  XtUnmanageChild(loadFileBox); XmUpdateDisplay(w);
  XmStringGetLtoR(calldata->value, charset, &filename);

  loadana(filename);
  XtFree(filename);
#ifndef NOAUDIO
  pvbuf[0] = AL_OUTPUT_RATE;  pvbuf[1] = R;
  pvbuf[2] = AL_INPUT_RATE;   pvbuf[3] = R;
  pvbuf[4] = AL_CHANNEL_MODE;
  if (samps_per_frame==1) pvbuf[5]=AL_MONO;
    else if (samps_per_frame==4) pvbuf[5]=AL_4CHANNEL; else pvbuf[5]=AL_STEREO;
  ALsetparams(AL_DEFAULT_DEVICE, pvbuf, 6);
#endif
}

void Lpcmult(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, j, ch;
  long point, point2;
  double *coef, f;

  if (numcoef2==0) return;
  SetWatchCursor(topLevel);
  fvec(coef, numcoef2+1);
  for (ch=0; ch<lpc_samps_per_frame; ch++) {
    for (j=0; j<lpc_horiz_num; j++) {
      if (j>=horiz_num) break;
      point=ch*(numcoef2+2)*lpc_horiz_num+j*(numcoef2+2);
      point2=ch*horiz_num*numchannels+j*numchannels;
      for (i=0; i<numcoef2+1; i++) coef[i]=lpc_coef[point+i];
      for (i=0; i<numchannels; i++) {
        f=megfreq[point2+i]*TWOPI/R;
        megamp[point2+i]*=lpamp(f, lpc_coef[point+numcoef2+1], coef, numcoef2)
          *0.1/numcoef2;
        if (megamp[point2+i]<1e-20) megamp[point2+i]=0.;
      }
    }
  }
  free(coef);
  RedrawWin();
  ResetCursor(topLevel);
}

void Lpcinvert(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, j, ch;
  long point, point2;
  double *coef, f;

  if (numcoef2==0) return;
  SetWatchCursor(topLevel);
  fvec(coef, numcoef2+1);
  for (ch=0; ch<lpc_samps_per_frame; ch++) {
    for (j=0; j<lpc_horiz_num; j++) {
      if (j>=horiz_num) break;
      point=ch*(numcoef2+2)*lpc_horiz_num+j*(numcoef2+2);
      point2=ch*horiz_num*numchannels+j*numchannels;
      for (i=0; i<numcoef2+1; i++) coef[i]=lpc_coef[point+i];
      for (i=0; i<numchannels; i++) {
        f=megfreq[point2+i]*TWOPI/R;
        megamp[point2+i]/=(lpamp(f, lpc_coef[point+numcoef2+1], coef, numcoef2)
          *0.1/numcoef2);
        megamp[point2+i]*=sqrt(lpc_coef[point+numcoef2+1]);
      }
    }
  }
  free(coef);
  RedrawWin();
  ResetCursor(topLevel);
}

void LoadLpc(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(loadLpcBox);
}

void LoadLpcOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

  XtUnmanageChild(loadLpcBox); XmUpdateDisplay(w);
  XmStringGetLtoR(calldata->value, charset, &filename);

  loadlpc(filename);
  XtFree(filename);
}

void Image(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(imageFileBox);
}

void ImageOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  char *filename, line[200];
  XPoint list0[LSIZE],list1[LSIZE],list2[LSIZE],list3[LSIZE];
  int c0=0,c1=0,c2=0,c3=0;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  int i, j=0, grafx, grafold=-1, x, y, xc, yc, val1, val2, val3;
  long point=0;
  FILE *infile;
  double fr, a;
  long int pvbuf[8];

  XtUnmanageChild(imageFileBox);
  SetWatchCursor(topLevel);
  XmStringGetLtoR(calldata->value, charset, &filename);

  infile=fopen(filename, "r");
  if (infile==NULL) {
    ResetCursor(topLevel);
    XtManageChild(fileWarning);
    XtFree(filename);
    return;
  }

  fgets(line, 200, infile); /* P3 */
  fgets(line, 200, infile); /* # CREATOR */
  fgets(line, 200, infile); if (line[0]=='#') fgets(line, 200, infile);
  sscanf(line,"%d %d",&x,&y); printf("%d %d\n",x,y);
  fgets(line, 200, infile);

  samps_per_frame   = 1;
  R                 = 44100;
  compression       = AF_COMPRESSION_NONE;
  filefmt           = AF_FILE_AIFF;  
  samp_type         = AF_SAMPFMT_TWOSCOMP;
  bits_per_samp     = 16;

  horiz_num=x;
  framecnt=(horiz_num-overlap)*Dn;
  if (numchannels*horiz_num > 16000000) {
    ResetCursor(topLevel);
    XtManageChild(sizeWarning);
    fclose(infile); 
    XtFree(filename);
    return;
  } 
  duration = (double)framecnt/R;
  binfreq = (double)R/N;
  if (megfreq!=NULL) free(megfreq);
  if (megamp!=NULL) free(megamp);
  megfreq=megamp=NULL;
  ffvec(megfreq, numchannels*horiz_num);
  if (megfreq==NULL) {
    ResetCursor(topLevel);
    XtManageChild(sizeWarning);
    fclose(infile); 
    XtFree(filename);
    return;
  }
  ffvec(megamp, numchannels*horiz_num);
  if (megamp==NULL) {
    ResetCursor(topLevel);
    XtManageChild(sizeWarning);
    fclose(infile); 
    XtFree(filename);
    return;
  }

  areat1=0; areat2=horiz_num;
  areaf1=0; areaf2=numchannels;

  XFillRectangle(XtDisplay(sketchpad), bitmap, theGC100,
    0, 0, 800+70, theheight+70);
  XClearArea(XtDisplay(sketchpad), XtWindow(sketchpad), 0, 0, 0, 0, True);

  drawscale();

  fr=(y+1)*binfreq; point=0;
  for (xc=0; xc<x; xc++) for (yc=0; yc<numchannels; yc++) megfreq[point++]=50.;
  for (yc=0; yc<y; yc++) {
    fr-=binfreq;
    for (i=0; i<x; i++) {
        fscanf(infile,"%d %d %d", &val1, &val2, &val3);
        val1=255-val1; val2=255-val2; val3=255-val3;
        if (y-yc<511) {
	  megamp[i*numchannels+y-yc]=(double)val1*(double)val2*(double)val3
            *0.000000000001;
	  megfreq[i*numchannels+y-yc]=fr+rand()/32768.*binfreq;
        }
    }
  }
  for (xc=0; xc<x; xc++) {
    grafx = (int)(j++*800./horiz_num)+50;
    if (grafx!=grafold) {
      point=xc*numchannels;
        for (i=0; i<numchannels; i++) {
          a=dgain*megamp[point];
          if (a>0.0000012) {        
            int y = theheight-(int)(megfreq[point]/binfreq)+10;
            if (a>0.0000100) {
           ADD_POINT(grafx,y,list0,c0,showGC0,theGC0);
            } else if (a>0.0000050){ 
           ADD_POINT(grafx,y,list1,c1,showGC25,theGC25);
            } else if (a>0.0000025) {
           ADD_POINT(grafx,y,list2,c2,showGC50,theGC50);
            } else {
           ADD_POINT(grafx,y,list3,c3,showGC75,theGC75);
            }
          }
          point++;
        }
      grafold=grafx;
    }
  }
  FLUSH_LIST(list0,c0,showGC0,theGC0);
  FLUSH_LIST(list1,c1,showGC25,theGC25);
  FLUSH_LIST(list2,c2,showGC50,theGC50);
  FLUSH_LIST(list3,c3,showGC75,theGC75);

  rightbins();

  fclose(infile); 
  XtFree(filename);
  fileDirty = FALSE;
  RedrawFunc();
  ResetCursor(topLevel);

#ifndef NOAUDIO
  pvbuf[0] = AL_OUTPUT_RATE;  pvbuf[1] = R;
  pvbuf[2] = AL_INPUT_RATE;   pvbuf[3] = R;
  pvbuf[4] = AL_CHANNEL_MODE;
  if (samps_per_frame==1) pvbuf[5]=AL_MONO;
    else if (samps_per_frame==4) pvbuf[5]=AL_4CHANNEL; else pvbuf[5]=AL_STEREO;
  ALsetparams(AL_DEFAULT_DEVICE, pvbuf, 6);
#endif
}

void LoadBrk(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(brkFileBox);
}

void LoadBrkOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  float x, y;
  int ant=0;
  FILE *infile;

  XtUnmanageChild(brkFileBox);
  XmStringGetLtoR(calldata->value, charset, &filename);

  infile=fopen(filename, "r");
  if (infile==NULL) {
    ResetCursor(topLevel);
    XtManageChild(fileWarning);
    XtFree(filename);
    return;
  }

  while (1) {
    fscanf(infile, "%f", &x);
    if (feof(infile)) break;
    fscanf(infile, "%f", &y);
    if (feof(infile)) break;
    if (ant>=MAXNOD) break;
    xpos[ant+1]=x*800+50;
    ypos[ant+1]=(1.-y)*theheight+10;
    ant++;
  }
  fclose(infile); 
  XtFree(filename);

  if (ant>0) {
    xpos[1]=50; xpos[ant]=850;
    numsquare=ant;
  }

  RedrawFunc();
}

void SaveBrk(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(brkSaveFileBox);
}

void SaveBrkOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  double x, y;
  int ant=0;
  FILE *outfile;

  XtUnmanageChild(brkSaveFileBox);
  XmStringGetLtoR(calldata->value, charset, &filename);

  outfile=fopen(filename, "w");
  if (outfile==NULL) {
    ResetCursor(topLevel);
    XtManageChild(fileWarning);
    XtFree(filename);
    return;
  }

  for (ant=1; ant<=numsquare; ant++) {
    fprintf(outfile, "%f\t%f\n", (xpos[ant]-50)/800.,
      1.-(ypos[ant]-10.)/theheight);
  }
  fclose(outfile); 
  XtFree(filename);
}

void Save(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(saveFileBox);
}

void SaveOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  int on=(-Nw*I)/Dn, in=-Nw, i, j, pr, lastpr=-1, ch, keepform=0;
  long point=0, point2=0;
  double *coef, a0;
  long pvbuf[8];

  SetWatchCursor(topLevel);
  XtUnmanageChild(saveFileBox); XmUpdateDisplay(w);

  XmStringGetLtoR(calldata->value, charset, &filename);

  out_AFsetup=AFnewfilesetup();
  AFinitchannels(out_AFsetup, AF_DEFAULT_TRACK, samps_per_frame);
  AFinitrate(out_AFsetup, AF_DEFAULT_TRACK, R);
  AFinitcompression(out_AFsetup, AF_DEFAULT_TRACK, compression);
  AFinitfilefmt(out_AFsetup, filefmt);
  AFinitsampfmt(out_AFsetup, AF_DEFAULT_TRACK, samp_type, bits_per_samp);
#ifdef CYGWIN
  outfile=AFopenfile(filename, "wb", out_AFsetup);
#else
  outfile=AFopenfile(filename, "w", out_AFsetup);
#endif

  if (outfile==NULL) {
    ResetCursor(topLevel);
    XtManageChild(writeWarning);
    XtFree(filename);
    return;
  }

  XtVaSetValues(progressScale, XmNvalue, 0, NULL);
  XtManageChild(progressDialog);
  while (XtAppPending(app_context)) {
     XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
  }
  if (XmToggleButtonGetState(settingsToggle3) && numcoef2!=0) keepform=1;

  init_again();
  fvec(coef, numcoef2+2);
  for (j=0; j<horiz_num; j++) {
    on+=I; in+=Dn;
    for (ch=0; ch<samps_per_frame; ch++) {
      point=ch*horiz_num*numchannels+j*numchannels;
      point2=ch*lpc_horiz_num*(numcoef2+2)+j*(numcoef2+2);
      for (i=0; i<numchannels; i++) {
        channel[i+i]=megamp[point+i];
        channel[i+i+1]=megfreq[point+i];
      }
      if (obank) {
        if (j==0) for (i=0; i<N2+1; i++) {
          lastfreq[ch][i]=channel[i+i+1];
          lastamp[ch][i]=channel[i+i];
          indx[ch][i]=0.;
        }
        if (j>=lpc_horiz_num) keepform=0;
        if (keepform) {
          for (i=0; i<numcoef2+2; i++) {
            coef[i]=lpc_coef[point2]; point2++;
          }
          a0=coef[numcoef2+1];
        }
        oscbank(channel, N2, R, I, output[ch], ch, Nw,coef, numcoef2*keepform,a0);
      } else {
        unconvert(channel,  buffer,  N2,  I,  R, ch);
        rfft(buffer,  N2,  INVERSE);
        overlapadd(buffer,  N,  Wsyn,  output[ch],  Nw,  on);
      }
    }
    if (obank) shiftout(Nw,  I,  in+Nw2+Dn);
      else shiftout(Nw,  I,  on);

    pr=(int)(100.*(double)j/horiz_num);
    if (pr!=lastpr) {
      XtVaSetValues(progressScale, XmNvalue, pr, NULL);
      XmUpdateDisplay(topLevel);
      lastpr=pr;
    }
  }

  AFclosefile(outfile);

  fileDirty = FALSE;
  strcpy(playfile, filename);
  XtFree(filename);
  free(coef);
  XtUnmanageChild(progressDialog);
  ResetCursor(topLevel);
#ifndef NOAUDIO  
  pvbuf[0] = AL_OUTPUT_RATE;  pvbuf[1] = R;
  pvbuf[2] = AL_INPUT_RATE;   pvbuf[3] = R;
  pvbuf[4] = AL_CHANNEL_MODE;
  if (samps_per_frame==1) pvbuf[5]=AL_MONO;
    else if (samps_per_frame==4) pvbuf[5]=AL_4CHANNEL; else pvbuf[5]=AL_STEREO;
  ALsetparams(AL_DEFAULT_DEVICE, pvbuf, 6);
#endif
}

void Play(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(playFileBox);
}

void PlayOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename, streng[100];
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

  XmStringGetLtoR(calldata->value, charset, &filename);
  strcpy(streng,"sfplay ");
  strcat(streng, filename);
  strcat(streng," &");
  system(streng);

  XtFree(filename);
}

void Sieve(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(sieveForm);
}

void SieveOk(w, client, call)
Widget w;
XtPointer client, call;
{
  int numparint, numparint2, i, j, k, maxnum, ch, pr, lastpr=-1;
  long point;
  double *tempamp, *megamp2, max, mult_pr_horiz, numpar, sievew, rscale;
  char *valstring;
  Boolean ampsel, funcsel, nullsel;

  XmUpdateDisplay(w);
  SetWatchCursor(topLevel);

  fvec(tempamp,numchannels);
  fvec(megamp2,numchannels);

  valstring=XmTextGetString(sieveTextfield);
  numparint2=atoi(valstring);
  XtFree(valstring);
  valstring=XmTextGetString(sieveTextfield2);
  mult_pr_horiz=powf(atof(valstring),duration/horiz_num);
  XtFree(valstring);
  valstring=XmTextGetString(sieveTextfield3);
  sievew=atof(valstring);
  XtFree(valstring);
  ampsel=XmToggleButtonGetState(sieveToggle1);
  funcsel=XmToggleButtonGetState(sieveToggle3);
  nullsel=XmToggleButtonGetState(sieveToggle4);

  XtVaSetValues(progressScale, XmNvalue, 0, NULL);
  XtManageChild(progressDialog);
  while (XtAppPending(app_context)) {
     XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
  }

  for (ch=0; ch<samps_per_frame; ch++) {
    funcpt=1; maxnum=1; numparint=numparint2; numpar=(double)numparint;
    rscale=(double)(areaf2-areaf1)/32768.;
    point=ch*numchannels*horiz_num+areat1*numchannels;
    for (i=areat1; i<areat2; i++) {
      if (funcsel) numparint=(int)getfuncval(i);
      if ((!ampsel) && (numparint>numchannels-10)) numparint=numchannels-10;
      for (j=0; j<numchannels; j++) {
        if (nullsel) tempamp[j]=megamp[point+j]; else tempamp[j]=0.;
        megamp2[j]=megamp[point+j]*(1.+sievew*j);
      }
      if (ampsel) for (k=0; k<numparint; k++) {
        max=0.;
	if (ampsel) for (j=areaf1; j<areaf2; j++) {
	  if (megamp2[j]>=max) {
	    max=megamp2[j]; maxnum=j;
	  }
        }
        if (nullsel) tempamp[maxnum]=0.;
          else tempamp[maxnum]=megamp[point+maxnum];
        megamp2[maxnum]=-1.;
      } else for (k=0; k<numparint; k++) {
        j=0;
        do { maxnum=(int)((double)rand()*rscale)+areaf1; j++; }
          while ((megamp2[maxnum]==-1) && (j<numchannels));
        if (nullsel) tempamp[maxnum]=0.;
          else tempamp[maxnum]=megamp[point+maxnum];
        megamp2[maxnum]=-1.;
      }
      for (j=areaf1; j<areaf2; j++) megamp[point+j]=tempamp[j];
      point+=numchannels;
      if (!funcsel) { numpar*=mult_pr_horiz; numparint=(int)(floor(numpar)); }
      pr=(int)(100.*(double)(i-areat1)/(areat2-areat1));
      if (pr!=lastpr) {
        XtVaSetValues(progressScale, XmNvalue, pr, NULL);
        XmUpdateDisplay(topLevel);
        lastpr=pr;
      }
    }
  }

  RedrawWin();
  free(tempamp); free(megamp2);
  XtUnmanageChild(progressDialog);
  ResetCursor(topLevel);
}

void Sshift(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(sshiftForm);
}

void SshiftOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  int i, j, ch;
  long point;
  double shval, shval2, mult_pr_horiz;
  Boolean funcsel;

  valstring=XmTextGetString(sshiftTextfield);
  shval2=atof(valstring);
  XtFree(valstring);
  valstring=XmTextGetString(sshiftTextfield2);
  mult_pr_horiz=powf(atof(valstring),duration/horiz_num);
  XtFree(valstring);
  funcsel=XmToggleButtonGetState(sshiftToggle1);

  XmUpdateDisplay(w);
  SetWatchCursor(topLevel);

  for (ch=0; ch<samps_per_frame; ch++) {
    point=areat1*numchannels+ch*horiz_num*numchannels; funcpt=1;
    shval=shval2;
    for (i=areat1; i<areat2; i++) {
      if (funcsel) shval=1000.*(getfuncval(i)/(double)theheight)-500.;
      for (j=areaf1; j<areaf2; j++) {
        megfreq[point+j]+=shval;
        if (megfreq[point+j]<20.) megamp[point+j]*=megfreq[point+j]/20.;
      }
      point+=numchannels;
      shval*=mult_pr_horiz;
    }
  }
  rightbins();
  RedrawWin();
  ResetCursor(topLevel);
}

void Shear(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(shearForm);
}

void ShearOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  int i, j, ch;
  long delay, moveto, start;
  double shval, shval2, mult_pr_horiz, fact;
  Boolean funcsel;

  valstring=XmTextGetString(shearTextfield1);
  shval2=atof(valstring);
  XtFree(valstring);
  valstring=XmTextGetString(shearTextfield2);
  mult_pr_horiz=powf(atof(valstring),duration/horiz_num);
  XtFree(valstring);
  funcsel=XmToggleButtonGetState(shearToggle1);

  XmUpdateDisplay(w);
  SetWatchCursor(topLevel);

  for (ch=0; ch<samps_per_frame; ch++) {
    funcpt=1; start=ch*numchannels*horiz_num;
    if (shval2<0) for (i=areat1; i<areat2; i++) {
      shval=-shval2*powf(mult_pr_horiz,i);
      if (funcsel) shval=getfuncval(i)/(double)theheight;
      fact=R*shval/(1000.*Dn);
      for (j=areaf1; j<areaf2; j++) {
        delay=(long)(fact*megfreq[i*numchannels+j+start]);
        moveto=i-delay; if (moveto>=horiz_num) moveto=horiz_num-1;
        if (moveto<0) moveto=0;
        megfreq[moveto*numchannels+j+start]=megfreq[i*numchannels+j+start];
        megamp[moveto*numchannels+j+start]=megamp[i*numchannels+j+start];
        megfreq[i*numchannels+j+start]=0.; megamp[i*numchannels+j+start]=0.;
      }
    } else for (i=areat2-1; i>=areat1; i--) {
      shval=shval2*powf(mult_pr_horiz,i-areat1);
      if (funcsel) shval=getfuncval(i)/(double)theheight;
      fact=R*shval/(1000.*Dn);
      for (j=areaf1; j<areaf2; j++) {
        delay=(long)(fact*megfreq[i*numchannels+j+start]);
        moveto=i+delay; if (moveto>=horiz_num) moveto=horiz_num-1;
        if (moveto<0) moveto=0;
        megfreq[moveto*numchannels+j+start]=megfreq[i*numchannels+j+start];
        megamp[moveto*numchannels+j+start]=megamp[i*numchannels+j+start];
        megfreq[i*numchannels+j+start]=0.; megamp[i*numchannels+j+start]=0.;
      }
    }
  }
  RedrawWin();
  ResetCursor(topLevel);
}

void Blur(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(blurForm);
}

void BlurOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  int i, j, p, ch, stepn;
  long start, t;
  double blurval, blurval2, mult_pr_horiz, ya, yf, step;
  Boolean funcsel;

  valstring=XmTextGetString(blurTextfield1);
  blurval2=atof(valstring);
  XtFree(valstring);
  valstring=XmTextGetString(blurTextfield2);
  mult_pr_horiz=powf(atof(valstring),duration/horiz_num);
  XtFree(valstring);
  valstring=XmTextGetString(blurTextfield3);
  step=atof(valstring); if (step==0.) step=1e-9;
  stepn=(int)((R/(double)Dn)/step);
  XtFree(valstring);  
  funcsel=XmToggleButtonGetState(blurToggle1);

  XmUpdateDisplay(w);
  SetWatchCursor(topLevel);

  for (ch=0; ch<samps_per_frame; ch++) {
    start=ch*numchannels*horiz_num;
    for (j=areaf1; j<areaf2; j++) {
      t=0;
      blurval=blurval2;
      funcpt=1;
      if (blurval>0.) for (i=areat1; i<areat2; i++) {
        p=i*numchannels+j+start;
        if (funcsel) blurval=getfuncval(i)/(double)theheight;
        if (blurval>1.0) blurval=1.0;
        if (t%stepn==0) {
          ya=megamp[p]; yf=megfreq[p];
        } else {
          ya=blurval*ya+(1.-blurval)*megamp[p]; megamp[p]=ya;
          yf=blurval*yf+(1.-blurval)*megfreq[p]; megfreq[p]=yf;
        }
        blurval*=mult_pr_horiz; t++;
      } else for (i=areat1+1; i<areat2; i++) {
        p=i*numchannels+j+start;
        if (funcsel) blurval=getfuncval(i)/(double)theheight;
        if (blurval<-1.0) blurval=-1.0;
        if (t%stepn==0) {
          ya=megamp[p]; yf=megfreq[p];
        } else {
          ya=blurval*ya+(1.+blurval)*megamp[p]; megamp[p]=ya;
        }
        blurval*=mult_pr_horiz; t++;
      }
    }
  }
  rightbins();
  RedrawWin();
  ResetCursor(topLevel);
}

void Comb(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(combForm);
}

void CombOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  int i, j, ch, bin, basebin;
  long point;
  double combval, combval2, mult_pr_horiz;
  Boolean funcsel;

  valstring=XmTextGetString(combTextfield1);
  combval2=atof(valstring);
  XtFree(valstring);
  valstring=XmTextGetString(combTextfield2);
  mult_pr_horiz=powf(atof(valstring),duration/horiz_num);
  XtFree(valstring);
  funcsel=XmToggleButtonGetState(combToggle1);

  XmUpdateDisplay(w);
  SetWatchCursor(topLevel);

  for (ch=0; ch<samps_per_frame; ch++) {
    point=ch*numchannels*horiz_num+areat1*numchannels;
    funcpt=1; combval=combval2;
    for (i=areat1; i<areat2; i++) {
      for (j=areaf1; j<areaf2; j++) {
        if (funcsel) combval=1000.*getfuncval(i)/(double)theheight;
        basebin=(int)(combval/binfreq+0.5);
        if (basebin>=numchannels) basebin=numchannels-1;
        if (basebin) {
          if (j%basebin) megamp[point]=0.;
        }
        combval*=mult_pr_horiz; point++;
      }
    }
  }
  RedrawWin();
  ResetCursor(topLevel);
}

void Mirror(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(mirrorForm);
}

void MirrorOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  int i, j, ch;
  long point;
  double mval, mval2, mult_pr_horiz, f;
  Boolean funcsel;

  valstring=XmTextGetString(mirrorTextfield);
  mval2=atof(valstring);
  XtFree(valstring);
  valstring=XmTextGetString(mirrorTextfield2);
  mult_pr_horiz=powf(atof(valstring),duration/horiz_num);
  XtFree(valstring);
  funcsel=XmToggleButtonGetState(mirrorToggle);

  XmUpdateDisplay(w);
  SetWatchCursor(topLevel);

  for (ch=0; ch<samps_per_frame; ch++) {
    point=areat1*numchannels+ch*horiz_num*numchannels; funcpt=1; mval=mval2;
    for (i=areat1; i<areat2; i++) {
      if (funcsel) mval=binfreq*getfuncval(i);
      for (j=areaf1; j<areaf2; j++) {
        f=megfreq[point+j];
        f=mval-(f-mval);
        if (f<0.) f=0.;
        if (f<20.) f*=f/20.;
        megfreq[point+j]=f;
      }
      point+=numchannels;
      mval*=mult_pr_horiz;
    }
  }
  rightbins();
  RedrawWin();
  ResetCursor(topLevel);
}

void Expo(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(expoForm);
}

void ExpoOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  int i, j, ch;
  long point;
  double eval, eval2, mult_pr_horiz, f, mul;
  Boolean funcsel;

  valstring=XmTextGetString(expoTextfield);
  eval2=atof(valstring);
  XtFree(valstring);
  valstring=XmTextGetString(expoTextfield2);
  mult_pr_horiz=powf(atof(valstring),duration/horiz_num);
  XtFree(valstring);
  valstring=XmTextGetString(expoTextfield3);
  mul=atof(valstring);
  XtFree(valstring);
  funcsel=XmToggleButtonGetState(expoToggle);

  XmUpdateDisplay(w);
  SetWatchCursor(topLevel);

  for (ch=0; ch<samps_per_frame; ch++) {
    point=areat1*numchannels+ch*numchannels*horiz_num; funcpt=1; eval=eval2;
    for (i=areat1; i<areat2; i++) {
      if (funcsel) eval=2.*getfuncval(i)/(double)theheight;
      for (j=areaf1; j<areaf2; j++) {
        f=megfreq[point+j];
        f=powf(f*mul, eval);
        if (f<0.) f=0.;
        if (f<20.) f*=f/20.;
        megfreq[point+j]=f;
      }
      point+=numchannels;
      eval*=mult_pr_horiz;
    }
  }
  rightbins();
  RedrawWin();
  ResetCursor(topLevel);
}

void Spread(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(spreadForm);
}

void SpreadOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  int i, j, ch;
  long point;
  double eval, eval2, mult_pr_horiz, f, mul;
  Boolean funcsel;

  valstring=XmTextGetString(spreadTextfield);
  eval2=atof(valstring);
  XtFree(valstring);
  valstring=XmTextGetString(spreadTextfield2);
  mult_pr_horiz=powf(atof(valstring),duration/horiz_num);
  XtFree(valstring);
  valstring=XmTextGetString(spreadTextfield3);
  mul=atof(valstring);
  XtFree(valstring);
  funcsel=XmToggleButtonGetState(spreadToggle);

  XmUpdateDisplay(w);
  SetWatchCursor(topLevel);

  for (ch=0; ch<samps_per_frame; ch++) {
    point=areat1*numchannels+ch*numchannels*horiz_num; funcpt=1; eval=eval2;
    for (i=areat1; i<areat2; i++) {
      if (funcsel) eval=2.*getfuncval(i)/(double)theheight;
      for (j=areaf1; j<areaf2; j++) {
        f=megfreq[point+j];
        f*=powf(2., eval*(rand()/16384.-1.)*(1.-mul+mul*megamp[point+j]*10000.));
        if (f<0.) f=0.;
        if (f<20.) f*=f/20.;
        megfreq[point+j]=f;
      }
      point+=numchannels;
      eval*=mult_pr_horiz;
    }
  }
  rightbins();
  RedrawWin();
  ResetCursor(topLevel);
}

void Pshift(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(pitchForm);
}

void PshiftOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  int i, j, ch;
  long point;
  double fact, fact2, mult_pr_horiz, reference, diff;
  Boolean funcsel;

  valstring=XmTextGetString(pitchTextfield);
  fact2=atof(valstring);
  XtFree(valstring);
  valstring=XmTextGetString(pitchTextfield2);
  mult_pr_horiz=powf(atof(valstring),duration/horiz_num);
  XtFree(valstring);
  valstring=XmTextGetString(pitchTextfield3);
  reference=atof(valstring);
  XtFree(valstring);
  funcsel=XmToggleButtonGetState(pitchToggle1);

  XmUpdateDisplay(w);
  SetWatchCursor(topLevel);

  for (ch=0; ch<samps_per_frame; ch++) {
    point=areat1*numchannels+ch*numchannels*horiz_num; funcpt=1; fact=fact2;
    for (i=areat1; i<areat2; i++) {
      if (funcsel) fact=2.*getfuncval(i)/(double)theheight;
      for (j=areaf1; j<areaf2; j++) {
        diff=megfreq[point+j]-reference;
        megfreq[point+j]=fact*diff+reference;
        if (megfreq[point+j]<20.) megamp[point+j]*=megfreq[point+j]/20.;
      }
      point+=numchannels;
      fact*=mult_pr_horiz;
    }
  }
  rightbins();
  RedrawWin();
  ResetCursor(topLevel);
}

void Filter(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(filterForm);
}

int makefilterfile(void)
{
  int i;
  float f;
  FILE *fil;

  i=0; fil=fopen("frequencies","r");
  if (fil==NULL) {
    printf("Couldn't open frequencies file\n");
    return 0;
  }
  while ((!feof(fil)) && (i<100)) {
    fscanf(fil,"%f",&f);
    if (feof(fil)) break;
    filterfreqs[i++]=f;
  }
  fclose(fil);
  return i;
}

void filterOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  long point, ch;
  int i, j, k, filter_lower, filter_upper, ant, keep;

  if (XmToggleButtonGetState(filterFilToggle)) {
    for (ch=0; ch<samps_per_frame; ch++) {
      point=areat1*numchannels+ch*numchannels*horiz_num;
      ant=makefilterfile();
      if (ant>0) for (i=areat1; i<areat2; i++) {
        for (j=areaf1; j<areaf2; j++) {
          keep=0;
          for (k=0; k<ant; k++) {
            filter_lower=filterfreqs[k]-binfreq/2.;
            filter_upper=filterfreqs[k]+binfreq/2.;
            if ((megfreq[point+j]>=filter_lower)
              && (megfreq[point+j]<=filter_upper)) keep=1;
          }
          if (keep==0) megamp[point+j]=0.;
        }
        point+=numchannels;
      }
    } 
  } else {
    valstring=XmTextGetString(filterTextfield);
    filter_lower=atoi(valstring);
    XtFree(valstring);
    if (filter_lower==0.) filter_lower=-99999.;
    valstring=XmTextGetString(filterTextfield2);
    filter_upper=atoi(valstring);
    XtFree(valstring);

    for (ch=0; ch<samps_per_frame; ch++) {
      point=areat1*numchannels+ch*numchannels*horiz_num;
      for (i=areat1; i<areat2; i++) {
        for (j=areaf1; j<areaf2; j++)
          if ((megfreq[point+j]>=filter_lower) && (megfreq[point+j]<=filter_upper))
            megamp[point+j]=0.;
        point+=numchannels;
      }
    }
    XtFree(valstring);
  }

  RedrawWin(); 
}

void Gain(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(gainForm);
}

void GainOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  double val;
  long point;
  int i, j, ch;
  Boolean funcsel;

  funcsel=XmToggleButtonGetState(gainToggle);

  valstring=XmTextGetString(gainTextfield);
  val=atof(valstring);
  XtFree(valstring);

  for (ch=0; ch<samps_per_frame; ch++) {
    point=areat1*numchannels+ch*numchannels*horiz_num;
    for (i=areat1; i<areat2; i++) {
      if (funcsel) val=getfuncval(i)/(double)theheight;
      for (j=areaf1; j<areaf2; j++)
        megamp[point+j]*=val;
      point+=numchannels;
    }
  }

  RedrawWin(); 
  XtFree(valstring);
}

void Threshold(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(thresholdForm);
}

void ThresholdOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  double val, maxamp;
  long point;
  int i, j, ch;
  Boolean funcsel;

  funcsel=XmToggleButtonGetState(thresholdToggle);

  valstring=XmTextGetString(thresholdTextfield);
  val=atof(valstring);
  XtFree(valstring);

  maxamp=-1.;
  for (ch=0; ch<samps_per_frame; ch++) {
    point=ch*numchannels*horiz_num;
    for (i=0; i<horiz_num; i++) {
      for (j=0; j<numchannels; j++)
        if (megamp[point+j]>maxamp) maxamp=megamp[point+j];
      point+=numchannels;
    }
  }

  val*=maxamp;
  if (maxamp>0.) for (ch=0; ch<samps_per_frame; ch++) {
    point=areat1*numchannels+ch*numchannels*horiz_num;
    for (i=areat1; i<areat2; i++) {
      if (funcsel) val=maxamp*getfuncval(i)/(double)theheight;
      for (j=areaf1; j<areaf2; j++)
        if (megamp[point+j]<val) megamp[point+j]=0.;
      point+=numchannels;
    }
  }

  RedrawWin(); 
  XtFree(valstring);
}

void Noisered(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(noiseredForm);
}

void envelopenoise(ampvec)
double *ampvec;
{
  int i, j;
  double max;
  
  for (i=1; i<numchannels/8-1; i++) {
    max=0.;
    for (j=0; j<8; j++) if (ampvec[i*8+j]>max) max=ampvec[i*8+j];
    for (j=0; j<8; j++) ampvec[i*8+j]=max;
  }
}

void NoiseredOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  double val, maxamp, *tempamp;
  long point;
  int i, j, ch;
  Boolean funcsel, envelopesel, despecklesel;

  funcsel=XmToggleButtonGetState(noiseredToggle);
  envelopesel=XmToggleButtonGetState(noiseredToggle2);
  despecklesel=XmToggleButtonGetState(noiseredToggle3);

  valstring=XmTextGetString(noiseredTextfield);
  val=atof(valstring);
  XtFree(valstring);

  fvec(tempamp, numchannels);

  for (ch=0; ch<samps_per_frame; ch++) {
    point=areat1*numchannels+ch*numchannels*horiz_num;
    for (j=0; j<numchannels; j++) tempamp[j]=0.;
    for (i=areat1; i<areat2; i++) {
      for (j=0; j<numchannels; j++) tempamp[j]+=megamp[point+j];
      point+=numchannels;
    }
    for (j=0; j<numchannels; j++) tempamp[j]/=(areat2-areat1);
    if (envelopesel) envelopenoise(tempamp);
    point=ch*numchannels*horiz_num;
    for (i=0; i<horiz_num; i++) {
      if (funcsel) val=2.*getfuncval(i)/(double)theheight;
      for (j=areaf1; j<areaf2; j++) {
        megamp[point+j]-=val*tempamp[j];
        if (megamp[point+j]<0.) megamp[point+j]=0.;
      }
      point+=numchannels;
    }
    if (despecklesel) {
      point=ch*numchannels*horiz_num+numchannels;
      for (i=1; i<horiz_num-1; i++) {
        for (j=areaf1; j<areaf2; j++) {
          if ((megamp[point+j-numchannels]==0.) &&
            (megamp[point+j+numchannels]==0.)) megamp[point+j]=0.;
        }
        point+=numchannels;
      }
      point=ch*numchannels*horiz_num;
      for (i=0; i<horiz_num; i++) {
        for (j=areaf1+1; j<areaf2-1; j++) {
          if ((megamp[point+j-1]==0.) &&
            (megamp[point+j+1]==0.)) megamp[point+j]=0.;
        }
        point+=numchannels;
      }
      point=ch*numchannels*horiz_num+numchannels;
      for (i=1; i<horiz_num-1; i++) {
        for (j=areaf1; j<areaf2; j++) {
          if ((megamp[point+j-numchannels]==0.) &&
            (megamp[point+j+numchannels]==0.)) megamp[point+j]=0.;
        }
        point+=numchannels;
      }
    }
  }

  RedrawWin(); 
  XtFree(valstring);
  free(tempamp);
}

void Nullphase(w, client, call)
Widget w;
XtPointer client, call;
{
  long point;
  int i, j, ch;

  for (ch=0; ch<samps_per_frame; ch++) {
    point=areat1*numchannels+ch*numchannels*horiz_num;  
    for (i=areat1; i<areat2; i++) {
      for (j=areaf1; j<areaf2; j++)
        megfreq[point+j]=j*binfreq;
      point+=numchannels;
    }
  }
  RedrawWin();
}

void Trpeaks(w, client, call)
Widget w;
XtPointer client, call;
{
  long point;
  int i, j, keep, ch;
  double *tempamp;

  fvec(tempamp,numchannels);

  for (ch=0; ch<samps_per_frame; ch++) {
    point=areat1*numchannels+ch*horiz_num*numchannels;
    for (i=areat1; i<areat2; i++) {
      for (j=areaf1; j<areaf2; j++) tempamp[j]=megamp[point+j];
      for (j=areaf1; j<areaf2-1; j++) {
        keep=0; if (j==areaf1) keep=1; else {
          if ((tempamp[j-1]<tempamp[j]) && (tempamp[j+1]<tempamp[j]))
            keep=1;
        }
        if ((tempamp[j]<=0.000001) || (megfreq[point+j]<20.) || (keep==0))
          megamp[point+j] = 0.;
      }
      point += numchannels;
    }
  }
  free(tempamp);
  RedrawWin();
}

void LTAS(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, j, ch, start;
  double middleamp, middlefreq;

  for (ch=0; ch<samps_per_frame; ch++) {
    start=ch*horiz_num*numchannels;
    for (j=0; j<numchannels; j++) {
      middleamp=middlefreq=0.;
      for (i=0; i<horiz_num; i++) {
        middleamp+=megamp[i*numchannels+j+start];
        middlefreq+=megfreq[i*numchannels+j+start];
      }
      middleamp/=horiz_num;
      middlefreq/=horiz_num;
      for (i=0; i<horiz_num; i++) {
        megamp[i*numchannels+j+start]=middleamp;
        megfreq[i*numchannels+j+start]=middlefreq;
      }
    }
  }
  RedrawWin();
}

void makepenta()
{
  char *valstring;
  float val;
  int i;

  valstring=XmTextGetString(gridTextfield);
  val=atof(valstring);
  XtFree(valstring);

  gridfreqs[0]=val; gridfreqs[1]=gridfreqs[0]*1.122;
  gridfreqs[2]=gridfreqs[0]*1.335;
  gridfreqs[3]=gridfreqs[0]*1.498;
  gridfreqs[4]=gridfreqs[0]*1.682;
  for (i=5; i<500; i++) {
    gridfreqs[i]=gridfreqs[i-5]*2.;
    numfreqs=i+1;
    if (gridfreqs[i]>23000) break;
    }
}

void makemajor(void)
{
  char *valstring;
  float val;
  int i;

  valstring=XmTextGetString(gridTextfield);
  val=atof(valstring);
  XtFree(valstring);

  gridfreqs[0]=val; gridfreqs[1]=gridfreqs[0]*1.122;
  gridfreqs[2]=gridfreqs[0]*1.260;
  gridfreqs[3]=gridfreqs[0]*1.335;
  gridfreqs[4]=gridfreqs[0]*1.498;
  gridfreqs[5]=gridfreqs[0]*1.682;
  gridfreqs[6]=gridfreqs[0]*1.888;
  for (i=7; i<500; i++) {
    gridfreqs[i]=gridfreqs[i-7]*2.;
    numfreqs=i+1;
    if (gridfreqs[i]>23000) break;    
    }
}

void makeminor(void)
{
  char *valstring;
  float val;
  int i;

  valstring=XmTextGetString(gridTextfield);
  val=atof(valstring);
  XtFree(valstring);

  gridfreqs[0]=val; gridfreqs[1]=gridfreqs[0]*1.122;
  gridfreqs[2]=gridfreqs[0]*1.189;
  gridfreqs[3]=gridfreqs[0]*1.335;
  gridfreqs[4]=gridfreqs[0]*1.498;
  gridfreqs[5]=gridfreqs[0]*1.587;
  gridfreqs[6]=gridfreqs[0]*1.888;
  for (i=7; i<500; i++) {
    gridfreqs[i]=gridfreqs[i-7]*2.;
    numfreqs=i+1;
    if (gridfreqs[i]>23000) break;
    }
}

void makechroma(void)
{
  char *valstring;
  float val;
  int i;

  valstring=XmTextGetString(gridTextfield);
  val=atof(valstring);
  XtFree(valstring);

  gridfreqs[0]=val; gridfreqs[1]=gridfreqs[0]*1.0594631;
  gridfreqs[2]=gridfreqs[0]*1.1224621;
  gridfreqs[3]=gridfreqs[0]*1.189;
  gridfreqs[4]=gridfreqs[0]*1.260;
  gridfreqs[5]=gridfreqs[0]*1.335;
  gridfreqs[6]=gridfreqs[0]*1.414;
  gridfreqs[7]=gridfreqs[0]*1.498;
  gridfreqs[8]=gridfreqs[0]*1.587;
  gridfreqs[9]=gridfreqs[0]*1.682;
  gridfreqs[10]=gridfreqs[0]*1.782;
  gridfreqs[11]=gridfreqs[0]*1.888;
  for (i=12; i<500; i++) {
    gridfreqs[i]=gridfreqs[i-12]*2;
    numfreqs=i+1;
    if (gridfreqs[i]>23000) break;
    }
}

void makewhole(void)
{
  char *valstring;
  float val;
  int i;

  valstring=XmTextGetString(gridTextfield);
  val=atof(valstring);
  XtFree(valstring);

  gridfreqs[0]=val; gridfreqs[1]=gridfreqs[0]*1.1224621;
  gridfreqs[2]=gridfreqs[0]*1.260;
  gridfreqs[3]=gridfreqs[0]*1.414;
  gridfreqs[4]=gridfreqs[0]*1.587;
  gridfreqs[5]=gridfreqs[0]*1.782;
  for (i=6; i<500; i++) {
    gridfreqs[i]=gridfreqs[i-6]*2;
    numfreqs=i+1;
    if (gridfreqs[i]>23000) break;
    }
}

void makeharmseries(void)
{
  char *valstring;
  float val;
  int i;

  valstring=XmTextGetString(gridTextfield);
  val=atof(valstring);
  XtFree(valstring);

  gridfreqs[0]=val;
  numfreqs=1;
  for (i=1; i<500; i++) {
    gridfreqs[i]=gridfreqs[i-1]+val; 
    numfreqs=i+1;
    if (gridfreqs[i]>23000) break;
    }  
}

void makeevenharmseries(void)
{
  char *valstring;
  float val;
  int i;

  valstring=XmTextGetString(gridTextfield);
  val=atof(valstring);
  XtFree(valstring);

  gridfreqs[0]=val;
  gridfreqs[1]=2*val;
  for (i=2; i<500; i++) {
    gridfreqs[i]=gridfreqs[i-1]+(2*val);
    numfreqs=i+1;
    if (gridfreqs[i]>23000) break;
    }
}

void makeoddharmseries(void)
{
  char *valstring;
  float val;
  int i;

  valstring=XmTextGetString(gridTextfield);
  val=atof(valstring);
  XtFree(valstring);

  gridfreqs[0]=val;
  for (i=1; i<500; i++) {
    gridfreqs[i]=gridfreqs[i-1]+(2*val);
    numfreqs=i+1;
    if (gridfreqs[i]>23000) break;
    }
}

void PGFreqfile(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(freqFileBox);
}

void FreqfileOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  static XmStringCharSet charset = 
                            (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  char *filename;
  float f;
  FILE *infile;

  XtUnmanageChild(freqFileBox);
  SetWatchCursor(topLevel);
  XmStringGetLtoR(calldata->value, charset, &filename);

  infile=fopen(filename, "r");
  if (infile==NULL) {
    ResetCursor(topLevel);
    XtManageChild(fileWarning);
    XtFree(filename);
    return;
  }

  numfreqs=0;
  while ((!feof(infile)) && (numfreqs<500)) {
    fscanf(infile,"%f",&f); 
    gridfreqs[numfreqs]=f; numfreqs++;
  }
  numfreqs--;
  fclose(infile);
  XtFree(filename);
  fileDirty = FALSE;
  ResetCursor(topLevel); 
}

void gridsort(double *gridfreqs2, int left, int right)
{
  int i, j;
  float x, y;

  i=left; j=right;
  x=gridfreqs2[(left+right)/2];

  do {
    while (gridfreqs2[i]<x && i <right) i++;
    while (x<gridfreqs2[j] && j>left) j--;

    if(i<=j) {
      y=gridfreqs2[i];
      gridfreqs2[i]=gridfreqs2[j];
      gridfreqs2[j]=y;
      i++; j--;
    }
  } while (i<=j);

  if (left<j) gridsort(gridfreqs2, left, j);
  if (i<right) gridsort(gridfreqs2, i, right);
}


void Magnet(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(magnetForm);
}

void MagnetOk(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, j, k, nearest, ch, totf;
  double mindiff, f, proba, proba2, mult_pr_horiz;
  long point=0;
  char *valstring;
  Boolean funcsel;

  valstring=XmTextGetString(magnetTextfield);
  proba2=atof(valstring);
  XtFree(valstring);
  valstring=XmTextGetString(magnetTextfield2);
  mult_pr_horiz=powf(atof(valstring),duration/horiz_num);
  XtFree(valstring);
  funcsel=XmToggleButtonGetState(magnetToggle1);

  SetWatchCursor(topLevel);

  totf=numfreqs;
  for (i=0; i<totf; i++) gridfreqs2[i]=gridfreqs[i];

  gridsort(gridfreqs2, 0, totf-1);

  for (i=0; i<totf; i++) {
    while (gridfreqs2[i]==gridfreqs2[i+1]) { 
      for (j=i; j<numfreqs-1; j++) gridfreqs2[j]=gridfreqs2[j+1];
      totf--;  
    }
  }

  for (ch=0; ch<samps_per_frame; ch++) {
    point=areat1*numchannels+ch*numchannels*horiz_num; funcpt=1; proba=proba2;
    for (i=areat1; i<areat2; i++) {
      if (funcsel) proba=getfuncval(i)/(double)theheight;
      for (j=areaf1; j<areaf2; j++) {
        if ((double)rand()/32768.<proba) {
          nearest=totf-1; mindiff=99999.;
          for (k=0; k<100; k++) {
            f=fabs(megfreq[point+j]-gridfreqs2[k]);
            if (f<mindiff) {
              nearest=k; mindiff=f;
            } else break;
          }
          megfreq[point+j]=gridfreqs2[nearest];
          if (megfreq[point+j]<20.) megamp[point+j]*=megfreq[point+j]/20.;
        }
      }
      point+=numchannels;
      proba*=mult_pr_horiz;
    }
  }
  rightbins();
  RedrawWin();
  ResetCursor(topLevel);
}

void Convolve(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(convolveForm);
}

void ConvolveOk(w, client, call)
Widget w;
XtPointer client, call;
{
  double laplace[3][3]={
    { 0, -1,  0},
    {-1,  4, -1},
    { 0, -1,  0}};
  double mean[3][3]={
    { 0.11, 0.11, 0.11},
    { 0.11, 0.11, 0.11},
    { 0.11, 0.11, 0.11}};
  double timemean[3][3]={
    { 0., 0., 0.},
    { 0.33, 0.33, 0.33},
    { 0., 0., 0.}};
  double op[3][3];

  int i, j, i1, j1, ch;
  long point;
  double sum;

  XmUpdateDisplay(w);
  SetWatchCursor(topLevel);

  for (i=0; i<=2; i++) for (j=0; j<=2; j++) {
   if (XmToggleButtonGetState(convolveToggleLaplace)) op[i][j]=laplace[j][i];
   if (XmToggleButtonGetState(convolveToggleMean)) op[i][j]=mean[j][i];
   if (XmToggleButtonGetState(convolveToggleTimeMean)) op[i][j]=timemean[j][i];
  }

  for (ch=0; ch<samps_per_frame; ch++) {
    for (j=0; j<numchannels; j++) this[j]=0.;
    point=areat1*numchannels+ch*numchannels*horiz_num;
    for (i=areat1; i<areat2; i++) {
      for (j=0; j<numchannels; j++) last[j]=this[j];
      for (j=0; j<numchannels; j++) this[j]=megamp[point+j];
      for (j=areaf1; j<areaf2; j++) {
        sum=0.;
        for (j1=-1; j1<=1; j1++) {
          if ((j+j1>=0) && (j+j1<numchannels))
            sum+=op[0][j1+1]*last[j+j1];
        }
        for (j1=-1; j1<=1; j1++) {
          if ((j+j1>=0) && (j+j1<numchannels))
            sum+=op[1][j1+1]*this[j+j1];
        }
        for (j1=-1; j1<=1; j1++) {
          if ((i+1<horiz_num) && (j+j1>=0) && (j+j1<numchannels))
            sum+=op[2][j1+1]*megamp[point+numchannels+j+j1];
        }
        if (sum>=0.) megamp[point+j]=sum;
      }
      point+=numchannels;
    }
  }
  RedrawWin();
  ResetCursor(topLevel);
}

void Tocsound(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(tocsoundFileBox);
}

void TocsoundOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *scorefile;
  int i, j;
  long point;
  double distance=stretchfac*duration/(double)horiz_num;
  double dur=stretchfac/binfreq;
  double onset;

  XmStringGetLtoR(calldata->value, charset, &filename);
  scorefile=fopen(filename,"w");
  if (scorefile==NULL) {
    XtManageChild(csoundWarning);
    XtFree(filename);
    return;
  }

  point=areat1*numchannels;
  for (i=areat1; i<areat2; i++) {
     onset=i*distance;
     for (j=areaf1; j<areaf2; j++) {
       if ((megamp[point+j]>0.) && (megfreq[point+j]>20.))
         fprintf(scorefile,"i1    %8.4f    %8.4f    %7.2f    %7.2f\n",
           onset, dur, megamp[point+j]*10000000., megfreq[point+j]);
     }
     point+=numchannels;
  }

  fprintf(scorefile,"e\n");

  fclose(scorefile);
  XtFree(filename);
  XtUnmanageChild(tocsoundFileBox);
}

void Toparmerud(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(toparmerudFileBox);
}

void ToparmerudOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename, *valstring;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *scorefile;
  int i, j, k, strings, state, maxpoint;
  long point=0;
  double distance=stretchfac*duration/(double)horiz_num;
  double dur=stretchfac/binfreq;
  double onset, saveamp;
  double triglevel, gatetime, resontime, strfrq, range, maxamp, timeren;

  XmStringGetLtoR(calldata->value, charset, &filename);
  scorefile=fopen(filename,"w");
  if (scorefile==NULL) {
    XtManageChild(csoundWarning);
    XtFree(filename);
    return;
  }

  valstring=XmTextGetString(parmT1);
  triglevel=atof(valstring);
  valstring=XmTextGetString(parmT2);
  gatetime=atof(valstring);
  valstring=XmTextGetString(parmT3);
  strings=atoi(valstring);
  valstring=XmTextGetString(parmT4);
  resontime=atof(valstring);

  strfrq=30.;
  for (i=0; i<strings*8; i++) {
    state=0;
    range=(strfrq*powf(2., (double)strings/12.)-strfrq)/2.;
    for (j=0; j<horiz_num; j++) {
      point=j*numchannels;
      maxamp=-100.; maxpoint=-1;
      for (k=0; k<numchannels; k++) {
        if ((megfreq[point]>strfrq-range) && (megfreq[point]<strfrq+range)
        && (megamp[point]*10000000.>triglevel)) {
          if (megamp[point]>maxamp) {
            maxamp=megamp[point]; maxpoint=point;
          }
        }
        point++;
      }
      if (state==0) {
        if (maxpoint>-1) {
          saveamp=maxamp*10000000.;
          onset=j*distance;
          state=1;
        }
      } else if (state==1) {
        if (maxpoint==-1) {
          fprintf(scorefile,"i1    %8.4f    %8.4f    %7.2f    %7.2f\n",
            onset, (resontime>j*distance-onset?resontime:j*distance-onset),
            saveamp, strfrq);
          state=2; timeren=0.;
        }
      } else {
        timeren+=distance;
        if (timeren>gatetime) state=0;
      }
    }
    strfrq*=powf(2., 1./(double)strings);
  }

  fprintf(scorefile,"e\n");

  fclose(scorefile);
  XtFree(filename);
  XtUnmanageChild(toparmerudFileBox);
}

void Topeak(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(topeakFileBox);
}

void TopeakOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *scorefile;
  int i, j, keep;
  long point;
  double distance=stretchfac*duration/(double)horiz_num;
  double dur=stretchfac/binfreq;
  double onset;

  XmStringGetLtoR(calldata->value, charset, &filename);
  scorefile=fopen(filename,"w");
  if (scorefile==NULL) {
    XtManageChild(csoundWarning);
    XtFree(filename);
    return;
  }

  point=areat1*numchannels;
  for (i=areat1; i<areat2; i++) {
     onset=i*distance;
     for (j=0; j<numchannels; j++) {
       keep=0; if (j==0) keep=1; else {
       if ((megamp[point-1]<megamp[point]) && (megamp[point+1]<megamp[point]))
         keep=1; }
       if ((megamp[point]>0.000001) && (megfreq[point]>20.) && keep)
         fprintf(scorefile,"%8.4f    %7.2f    %7.2f\n",
           onset, megamp[point]*10000000., megfreq[point]);
       point++;
     }
  }

  fclose(scorefile);
  XtFree(filename);
  XtUnmanageChild(topeakFileBox);
}

void Tostf(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(tostfFileBox);
}

void TostfOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *utfil;
  int i, j, hori;
  long point=0;

  XmStringGetLtoR(calldata->value, charset, &filename);
  utfil=fopen(filename,"w");
  if (utfil==NULL) {
    XtManageChild(stfWarning);
    XtFree(filename);
    return;
  }
  hori=horiz_num/4-1;

  fprintf(utfil, "#Inventor V2.0 ascii\n");
  fprintf(utfil, "Separator {\n");
  fprintf(utfil, "  DirectionalLight { direction  -1 -1 -0.9  }\n");


  fprintf(utfil, "  Separator {\n");
  fprintf(utfil, "    Material {\n");
  fprintf(utfil, "        diffuseColor [ .6 .8 0.5 ]\n");
  fprintf(utfil, "        specularColor [ .1 .1 .1 ]\n");
  fprintf(utfil, "        shininess 0.99\n");
  fprintf(utfil, "        transparency 0.0\n");
  fprintf(utfil, "    }\n");
  fprintf(utfil, "    Coordinate3 {\n");
  fprintf(utfil, "      point [\n");

  for (i=0; i<hori; i++) {
    point=i*4*numchannels;
    for (j=3; j<numchannels/2; j++) {
      fprintf(utfil, "%d %d %.2f, ", j*50, i*50,
        megamp[point++]*10000000.);
    }
    fprintf(utfil, "\n");
  }

  fprintf(utfil, "      ]\n");
  fprintf(utfil, "    }\n");
  fprintf(utfil, "    ShapeHints {\n");
  fprintf(utfil, "      vertexOrdering COUNTERCLOCKWISE\n");
  fprintf(utfil, "    }\n");
  fprintf(utfil, "    QuadMesh {\n");
  fprintf(utfil, "      verticesPerColumn %d\n", hori);
  fprintf(utfil, "      verticesPerRow    %d\n", numchannels/2-3);
  fprintf(utfil, "    }\n");
  fprintf(utfil, "  }\n");

  fprintf(utfil, "}\n");


  fclose(utfil);
  XtFree(filename);
  XtUnmanageChild(tostfFileBox);
}

void Toltas(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(toltasFileBox);
}

void ToltasOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  FILE *ltasfile;
  int i, j;
  long point;

  XmStringGetLtoR(calldata->value, charset, &filename);
  ltasfile=fopen(filename,"w");
  if (ltasfile==NULL) {
    XtManageChild(ltasWarning);
    XtFree(filename);
    return;
  }

 point=4*numchannels;
 for (j=0; j<numchannels; j++)
   fprintf(ltasfile,"%f\n",megamp[point++]*10000);

  fclose(ltasfile);
  XtFree(filename);
  XtUnmanageChild(toltasFileBox);
}


void Settings(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(settingsForm);
}

void SettingsOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;

  valstring=XmTextGetString(settingsTextfield);
  stretchfac=atof(valstring);
  XtFree(valstring);
  I=(int)(Dn*stretchfac);
  
  valstring=XmTextGetString(settingsTextfield2);
  P=atof(valstring);
  Pinc = P*8192./R;
  if (P>1.) NP=N2/P; else NP=N2;
  XtFree(valstring);
  
  valstring=XmTextGetString(settingsTextfield3);
  synt=atof(valstring)/N;
  XtFree(valstring);

  if (XmToggleButtonGetState(settingsToggle1)) obank=0;
  if (XmToggleButtonGetState(settingsToggle2)) obank=1;
}

void Analy(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(analyForm);
}

void AnalyOk(w, client, call)
Widget w;
XtPointer client, call;
{
  int i;
  char *valstring;
  double *A=Wanal, *S=Wsyn;

  valstring=XmTextGetString(analyTextfield);
  overlap=atoi(valstring); XtFree(valstring);
  if (overlap<1) overlap=1; if (overlap>32) overlap=32;

  Dn=Nw/overlap;
  I=Dn;
  makewindows(Wanal,  Wsyn,  Nw,  N,  I);

  valstring=XmTextGetString(analyTextfield2);
  numcoef=atoi(valstring); XtFree(valstring);
  if (numcoef<1) numcoef=1; if (numcoef>100) numcoef=100;
  if (numcoef!=numcoef2) numcoef2=0;
}

void setgreycolors(void)
{
   if (farge==1) return;
   XSetForeground(theDisplay, markerGC, gray0.pixel);
   XSetForeground(theDisplay, theGC0, gray0.pixel);
   XSetForeground(theDisplay, showGC0, gray0.pixel);
   XSetForeground(theDisplay, theGC25, gray25.pixel);
   XSetForeground(theDisplay, showGC25, gray25.pixel);
   XSetForeground(theDisplay, theGC50, gray50.pixel);
   XSetForeground(theDisplay, showGC50, gray50.pixel);
   XSetForeground(theDisplay, theGC75, gray75.pixel);
   XSetForeground(theDisplay, showGC75, gray75.pixel);
   XSetForeground(theDisplay, theGC100, gray100.pixel);
   XSetForeground(theDisplay, showGC100, gray100.pixel);
   XtVaSetValues(sketchpad,XmNbackground, gray100.pixel,NULL);
   XSetForeground(theDisplay, funcGC, red.pixel);
   XSetForeground(theDisplay, spotsGC, blue.pixel);
   farge=1;
}

void sethotcolors(void)
{
   if (farge==2) return;
   XSetForeground(theDisplay, markerGC, gray100.pixel);
   XSetForeground(theDisplay, theGC0, gray100.pixel);
   XSetForeground(theDisplay, showGC0, gray100.pixel);
   XSetForeground(theDisplay, theGC25, yellow.pixel);
   XSetForeground(theDisplay, showGC25, yellow.pixel);
   XSetForeground(theDisplay, theGC50, orange.pixel);
   XSetForeground(theDisplay, showGC50, orange.pixel);
   XSetForeground(theDisplay, theGC75, red.pixel);
   XSetForeground(theDisplay, showGC75, red.pixel);
   XSetForeground(theDisplay, theGC100, gray0.pixel);
   XSetForeground(theDisplay, showGC100, gray0.pixel);
   XtVaSetValues(sketchpad,XmNbackground, gray0.pixel,NULL);
   XSetForeground(theDisplay, funcGC, green.pixel);
   XSetForeground(theDisplay, spotsGC, green.pixel);
   farge=2;
}

void setcoldcolors(void)
{
   if (farge==3) return;
   XSetForeground(theDisplay, markerGC, gray100.pixel);
   XSetForeground(theDisplay, theGC0, gray100.pixel);
   XSetForeground(theDisplay, showGC0, gray100.pixel);
   XSetForeground(theDisplay, theGC25, lightblue.pixel);
   XSetForeground(theDisplay, showGC25, lightblue.pixel);
   XSetForeground(theDisplay, theGC50, blue.pixel);
   XSetForeground(theDisplay, showGC50, blue.pixel);
   XSetForeground(theDisplay, theGC75, darkblue.pixel);
   XSetForeground(theDisplay, showGC75, darkblue.pixel);
   XSetForeground(theDisplay, theGC100, gray0.pixel);
   XSetForeground(theDisplay, showGC100, gray0.pixel);
   XtVaSetValues(sketchpad,XmNbackground, gray0.pixel,NULL);
   XSetForeground(theDisplay, funcGC, green.pixel);
   XSetForeground(theDisplay, spotsGC, green.pixel);
   farge=3; 
}

void Displaymenu(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(displayForm);
}

void DisplayOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;

  if (XmToggleButtonGetState(displayToggleSpots)==False) {
    if (numspots) farge=-1;
    numspots=0;
  }
  valstring=XmTextGetString(displayGainText);
  dgain=atof(valstring);
  XtFree(valstring);
  valstring=XmTextGetString(displayScaleText);
  dscale=atof(valstring);
  XtFree(valstring);
  if (XmToggleButtonGetState(displayToggleGrey)) setgreycolors();
  if (XmToggleButtonGetState(displayToggleHot)) sethotcolors();
  if (XmToggleButtonGetState(displayToggleCold)) setcoldcolors();
  RedrawWin();
}

void Gridmenu(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(gridForm);
}

void GridmenuOk(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(gridTogglePenta)) makepenta();
    else if (XmToggleButtonGetState(gridToggleMajor)) makemajor();
    else if (XmToggleButtonGetState(gridToggleMinor)) makeminor();
    else if (XmToggleButtonGetState(gridToggleChroma)) makechroma();
    else if (XmToggleButtonGetState(gridToggleWhole)) makewhole();
    else if (XmToggleButtonGetState(gridToggleHarm)) makeharmseries();
    else if (XmToggleButtonGetState(gridToggleEvenharm)) makeevenharmseries();
    else if (XmToggleButtonGetState(gridToggleOddharm)) makeoddharmseries();
    else PGFreqfile(NULL,NULL,NULL);
  RestoreWin(NULL,NULL,NULL);
}

void SpotsRestart(w, client, call)
Widget w;
XtPointer client, call;
{
  numspots=0;
  RedrawWin();
}

void SpotsKeep(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, j, k, f, t, ch;
  long start;

  for (ch=0; ch<samps_per_frame; ch++) {
    start=ch*numchannels*horiz_num;
    for (i=0; i<numspots; i++) {
      for (j=-3; j<4; j++) {
        f=spotsf[i]+j;
        if ((f>0) && (f<numchannels)) {
          for (k=-3*horiz_num/800.; k<4*horiz_num/800.; k++) {
            t=spotst[i]+k;
            if ((t>=0) && (t<horiz_num))
              megamp[t*numchannels+f+start]=-megamp[t*numchannels+f+start];
          }
        }
      }
    }

    for (i=0; i<horiz_num; i++) {
      for (j=0; j<numchannels; j++) {
        megamp[start]=(megamp[start]<0?-megamp[start]:0.); start++;
      }
    }
  }
  RedrawWin();
}

void SpotsRemove(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, j, k, f, t, ch;

  for (ch=0; ch<samps_per_frame; ch++)
    for (i=0; i<numspots; i++) {
      for (j=-3; j<4; j++) {
        f=spotsf[i]+j;
        if ((f>0) && (f<numchannels)) {
          for (k=-3*horiz_num/800.; k<4*horiz_num/800.; k++) {
            t=spotst[i]+k;
            if ((t>=0) && (t<horiz_num))
              megamp[ch*horiz_num*numchannels+t*numchannels+f]=0.;
          }
        }
      }
    }

  RedrawWin();
}

void Amplitude(w, client, call)
Widget w;
XtPointer client, call;
{
  double amp;
  int j, lastx, lasty, antnod=1, x, lastxx=-1;

  for (j=0; j<horiz_num; j++) {
    x=(int)(j*800./horiz_num)+50;
    if (x!=lastxx) {
      amp=largamp[j];
      if (amp>1.) amp=1.;
      amp = (amp/2.)+0.25;
      amp = (1.-amp)*theheight+10.;

      if (j==0) { lastx=50; lasty=(int)amp; xpos[1]=50; ypos[1]=(int)amp; }
      else if ((antnod<MAXNOD) && ((fabs(amp-lasty)>50.) || (x-lastx>20))) {
        antnod++;
        lastx=x; lasty=(int)amp;
        xpos[antnod]=x; ypos[antnod]=(int)amp;
      }
      lastxx=x;
    }
  }
  xpos[antnod]=850;
  numsquare=antnod;
}

void Pitch(w, client, call)
Widget w;
XtPointer client, call;
{
  double maxamp, maxamp2, freq, freq2;
  int i, j, point=0, lastx, lasty, antnod=1, x, lastxx=-1, maxbin, maxbin2;

  for (j=0; j<horiz_num; j++) {
    x=(int)(j*800./horiz_num)+50;
    if (x!=lastxx) {

      maxamp=0.; maxbin=1;
      for (i=2; i<40; i++) if (megamp[point+i]>maxamp) {
        maxamp=megamp[point+i]; maxbin=i;
      }
      freq=megfreq[maxbin+point];
      freq2=0.;
      if (freq<60.) freq=0.; else {
        maxamp2=0.; maxbin2=1;
        for (i=2; i<maxbin-1; i++) if (megamp[point+i]>maxamp2) {
          maxamp2=megamp[point+i]; maxbin2=i;
        }
        freq2=megfreq[maxbin2+point];
        if ((fabs(freq/freq2-2.)<0.1) && (maxamp/maxamp2<16.)) freq=freq2;
        else if ((fabs(freq/freq2-3.)<0.1) && (maxamp/maxamp2<16.)) freq=freq2;
        else if ((fabs(freq/freq2-1.5)<0.1) && (maxamp/maxamp2<16.)) freq=freq2/2.;
      }

      freq=freq/1000.;
      if (freq>1.) freq=1.;
      freq = (1.-freq)*theheight+10.;

      if (j==0) { lastxx=50; lasty=(int)freq; xpos[1]=50; ypos[1]=(int)freq; }
      else if ((antnod<MAXNOD) && ((fabs(freq-lasty)>50.) || (x-lastx>20))) {
        antnod++;
        lastx=x; lasty=(int)freq;
        xpos[antnod]=x; ypos[antnod]=(int)freq;
      }
      lastxx=x;
    }
    point+=numchannels;
  }
  xpos[antnod]=850;
  numsquare=antnod;
}

void Centroid(w, client, call)
Widget w;
XtPointer client, call;
{
  double centro, norm;
  int i, j, point=0, lastx, lasty, antnod=1, x, lastxx=-1;

  for (j=0; j<horiz_num; j++) {
    x=(int)(j*800./horiz_num)+50;
    if (x!=lastxx) {
      centro=0.; norm=0.;
      for (i=1; i<numchannels-1; i++) {
        centro+=megamp[point+i]*megfreq[point+i];
        norm+=megamp[point+i];
      }
      centro=(norm<=0.?0.:centro/norm);
      /* if (max<0.01) centro=0.; */
      centro/=5000.;
      if (centro>1.) centro=1.;
      centro = (centro/2.)+0.25;
      centro = (1.-centro)*theheight+10.;

      if (j==0) { lastxx=50; lasty=(int)centro; xpos[1]=50; ypos[1]=(int)centro; }
      else if ((antnod<MAXNOD) && ((fabs(centro-lasty)>50.) || (x-lastx>20))) {
        antnod++;
        lastx=x; lasty=(int)centro;
        xpos[antnod]=x; ypos[antnod]=(int)centro;
      }
      lastxx=x;
    }
    point+=numchannels;
  }
  xpos[antnod]=850;
  numsquare=antnod;
}

void GrScale(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(grScaleForm);
}

void GrScaleOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  double val, y;
  int i;

  valstring=XmTextGetString(grScaleTextfield);
  val=atof(valstring);
  XtFree(valstring);

  for (i=1; i<=numsquare; i++) {
    y=val*(1.-(ypos[i]-10.)/theheight);
    if (y<0.) y=0.; if (y>1.) y=1.;
    ypos[i]=(1.-y)*theheight+10;
  }
}

void GrMove(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(grMoveForm); 
}

void GrMoveOk(w, client, call)
Widget w;
XtPointer client, call;
{
  char *valstring;
  double val, y;
  int i;

  valstring=XmTextGetString(grMoveTextfield);
  val=atof(valstring);
  XtFree(valstring);

  for (i=1; i<=numsquare; i++) {
    y=val+(1.-(ypos[i]-10.)/theheight);
    if (y<0.) y=0.; if (y>1.) y=1.;
    ypos[i]=(1.-y)*theheight+10;
  }
}

void GrNonlin(w, client, call)
Widget w;
XtPointer client, call;
{
}

void GrClear(w, client, call)
Widget w;
XtPointer client, call;
{
  ypos[1]=10+theheight/2;
  ypos[2]=ypos[1];
  xpos[2]=850;
  numsquare=2;
  selected=0; 
}

void GrInvert(w, client, call)
Widget w;
XtPointer client, call;
{
  double y;
  int i;

  for (i=1; i<=numsquare; i++) {
    y=1.-(1.-(ypos[i]-10.)/theheight);
    if (y<0.) y=0.; if (y>1.) y=1.;
    ypos[i]=(1.-y)*theheight+10;
  }
}

void GrRetro(w, client, call)
Widget w;
XtPointer client, call;
{
}

void create_transforms(void)
{
  XmString text;

  /* CREATE TRANSFORM MENU */
  transButton=XtVaCreateManagedWidget(
    "transButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Transform");
  XtVaSetValues(transButton,XmNlabelString,text,NULL);
  transMenu=XmCreatePulldownMenu(menuBar,"transMenu",NULL,0);
  XtVaSetValues(transButton,XmNsubMenuId,transMenu,NULL);

  /* CREATE SIEVE BUTTON */
  sieve=XtVaCreateManagedWidget(
    "sieve",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Sieve");
  XtVaSetValues(sieve,XmNlabelString,text,NULL);

  sieveForm=XmCreateBulletinBoardDialog(mainWindow,"sieveForm",NULL,0);
  sieveOkButton=XtVaCreateManagedWidget("sieveOk",xmPushButtonWidgetClass,
    sieveForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(sieveOkButton,XmNlabelString,text,XmNx,50,XmNy,340,
    XmNwidth,80,NULL);
  sieveCancelButton=XtVaCreateManagedWidget("sieveCancel",
    xmPushButtonWidgetClass,sieveForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(sieveCancelButton,XmNlabelString,text,XmNx,200,XmNy,340,NULL);

  sieveTextfield=XtVaCreateManagedWidget("sieveTextfield",
    xmTextFieldWidgetClass,sieveForm,NULL);
  XtVaSetValues(sieveTextfield,XmNwidth,100,XmNx,240,XmNy,10,
    XmNvalue,"10",NULL);
  sieveLabel1=XtVaCreateManagedWidget("sieveLabel1",xmLabelWidgetClass,
    sieveForm,NULL);
  text=XmStringCreateLocalized("Number of harmonics:");
  XtVaSetValues(sieveLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  sieveTextfield2=XtVaCreateManagedWidget("sieveTextfield2",
    xmTextFieldWidgetClass,sieveForm,NULL);
  XtVaSetValues(sieveTextfield2,XmNwidth,100,XmNx,240,XmNy,60,
    XmNvalue,"1",NULL);
  sieveLabel2=XtVaCreateManagedWidget("sieveLabel2",xmLabelWidgetClass,
    sieveForm,NULL);
  text=XmStringCreateLocalized("Multiplication pr. sec.:");
  XtVaSetValues(sieveLabel2,XmNlabelString,text,XmNx,0,XmNy,64,NULL);

  sieveToggle3=XtVaCreateManagedWidget("sieveToggle3",
    xmToggleButtonWidgetClass,sieveForm,NULL);
  text=XmStringCreateLocalized("Control function -> Num. harm. (0-all)");
  XtVaSetValues(sieveToggle3,XmNlabelString,text,XmNset,False,
    XmNy,110,NULL);

  sieveTextfield3=XtVaCreateManagedWidget("sieveTextfield3",
    xmTextFieldWidgetClass,sieveForm,NULL);
  XtVaSetValues(sieveTextfield3,XmNwidth,100,XmNx,240,XmNy,160,
    XmNvalue,"0",NULL);
  sieveLabel3=XtVaCreateManagedWidget("sieveLabel3",xmLabelWidgetClass,
    sieveForm,NULL);
  text=XmStringCreateLocalized("High frequency boost:");
  XtVaSetValues(sieveLabel3,XmNlabelString,text,XmNx,0,XmNy,164,NULL);

  sieveRadio=XtVaCreateManagedWidget("sieveRadio",
    xmRowColumnWidgetClass,sieveForm,NULL);
  XtVaSetValues(sieveRadio,XmNy,200,XmNradioBehavior,True,
    XmNradioAlwaysOne,True,NULL);
  sieveToggle1=XtVaCreateManagedWidget("sieveToggle1",
    xmToggleButtonWidgetClass,sieveRadio,NULL);
  text=XmStringCreateLocalized("Amplitude selection");
  XtVaSetValues(sieveToggle1,XmNlabelString,text,XmNset,True,NULL);
  sieveToggle2=XtVaCreateManagedWidget("sieveToggle2",
    xmToggleButtonWidgetClass,sieveRadio,NULL);
  text=XmStringCreateLocalized("Random selection");
  XtVaSetValues(sieveToggle2,XmNlabelString,text,NULL);
  
  sieveToggle4=XtVaCreateManagedWidget("sieveToggle4",
    xmToggleButtonWidgetClass,sieveForm,NULL);
  text=XmStringCreateLocalized("Null out selected partials");
  XtVaSetValues(sieveToggle4,XmNlabelString,text,XmNset,False,
    XmNy,280,NULL);

  text=XmStringCreateLocalized("Sieve");
  XtVaSetValues(sieveForm,XmNdialogTitle,text,
    XmNcancelButton,sieveCancelButton,
    XmNdefaultButton,sieveOkButton,NULL);

  XtAddCallback(sieveOkButton,XmNactivateCallback,SieveOk,NULL);
  XtAddCallback(sieve,XmNactivateCallback,Sieve,0);

  
  /* CREATE SPECTRUM SHIFT BUTTON */
  sshift=XtVaCreateManagedWidget(
    "sshift",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Spectrum shift");
  XtVaSetValues(sshift,XmNlabelString,text,NULL);

  sshiftForm=XmCreateBulletinBoardDialog(mainWindow,"sshiftForm",NULL,0);
  sshiftOkButton=XtVaCreateManagedWidget("sshiftOk",xmPushButtonWidgetClass,
    sshiftForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(sshiftOkButton,XmNlabelString,text,XmNx,50,XmNy,170,
    XmNwidth,80,NULL);
  sshiftCancelButton=XtVaCreateManagedWidget("sshiftCancel",
    xmPushButtonWidgetClass,sshiftForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(sshiftCancelButton,XmNlabelString,text,XmNx,200,XmNy,170,NULL);

  sshiftTextfield=XtVaCreateManagedWidget("sshiftTextfield",
    xmTextFieldWidgetClass,sshiftForm,NULL);
  XtVaSetValues(sshiftTextfield,XmNwidth,100,XmNx,260,XmNy,10,
    XmNvalue,"50",NULL);
  sshiftLabel1=XtVaCreateManagedWidget("sshiftLabel1",xmLabelWidgetClass,
    sshiftForm,NULL);
  text=XmStringCreateLocalized("Shift value (Hz):");
  XtVaSetValues(sshiftLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  sshiftTextfield2=XtVaCreateManagedWidget("sshiftTextfield2",
    xmTextFieldWidgetClass,sshiftForm,NULL);
  XtVaSetValues(sshiftTextfield2,XmNwidth,100,XmNx,260,XmNy,60,
    XmNvalue,"1",NULL);
  sshiftLabel2=XtVaCreateManagedWidget("sshiftLabel2",xmLabelWidgetClass,
    sshiftForm,NULL);
  text=XmStringCreateLocalized("Multiplication pr. sec.:");
  XtVaSetValues(sshiftLabel2,XmNlabelString,text,XmNx,0,XmNy,64,NULL);

  sshiftToggle1=XtVaCreateManagedWidget("sshiftToggle1",
    xmToggleButtonWidgetClass,sshiftForm,NULL);
  text=XmStringCreateLocalized("Control function -> Shift (-500 - 500)");
  XtVaSetValues(sshiftToggle1,XmNlabelString,text,XmNset,False,
    XmNy,110,NULL);

  text=XmStringCreateLocalized("Spectrum shift");
  XtVaSetValues(sshiftForm,XmNdialogTitle,text,
    XmNcancelButton,sshiftCancelButton,
    XmNdefaultButton,sshiftOkButton,NULL);

  XtAddCallback(sshiftOkButton,XmNactivateCallback,SshiftOk,NULL);
  XtAddCallback(sshift,XmNactivateCallback,Sshift,0);


  /* CREATE SHEAR BUTTON */
  shear=XtVaCreateManagedWidget(
    "shear",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Shear");
  XtVaSetValues(shear,XmNlabelString,text,NULL);

  shearForm=XmCreateBulletinBoardDialog(mainWindow,"shearForm",NULL,0);
  shearOkButton=XtVaCreateManagedWidget("shearOk",xmPushButtonWidgetClass,
    shearForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(shearOkButton,XmNlabelString,text,XmNx,50,XmNy,170,
    XmNwidth,80,NULL);
  shearCancelButton=XtVaCreateManagedWidget("shearCancel",
    xmPushButtonWidgetClass,shearForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(shearCancelButton,XmNlabelString,text,XmNx,200,XmNy,170,NULL);

  shearTextfield1=XtVaCreateManagedWidget("shearTextfield1",
    xmTextFieldWidgetClass,shearForm,NULL);
  XtVaSetValues(shearTextfield1,XmNwidth,100,XmNx,260,XmNy,10,
    XmNvalue,"0.1",NULL);
  shearLabel1=XtVaCreateManagedWidget("shearLabel1",xmLabelWidgetClass,
    shearForm,NULL);
  text=XmStringCreateLocalized("Seconds pr. kHz:");
  XtVaSetValues(shearLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  shearTextfield2=XtVaCreateManagedWidget("shearTextfield2",
    xmTextFieldWidgetClass,shearForm,NULL);
  XtVaSetValues(shearTextfield2,XmNwidth,100,XmNx,260,XmNy,60,
    XmNvalue,"1",NULL);
  shearLabel2=XtVaCreateManagedWidget("shearLabel2",xmLabelWidgetClass,
    shearForm,NULL);
  text=XmStringCreateLocalized("Multiplication pr. sec.:");
  XtVaSetValues(shearLabel2,XmNlabelString,text,XmNx,0,XmNy,64,NULL);

  shearToggle1=XtVaCreateManagedWidget("shearToggle1",
    xmToggleButtonWidgetClass,shearForm,NULL);
  text=XmStringCreateLocalized("Control function -> Shear (0-1)");
  XtVaSetValues(shearToggle1,XmNlabelString,text,XmNset,False,
    XmNy,110,NULL);

  text=XmStringCreateLocalized("Shear");
  XtVaSetValues(shearForm,XmNdialogTitle,text,
    XmNcancelButton,shearCancelButton,
    XmNdefaultButton,shearOkButton,NULL);

  XtAddCallback(shearOkButton,XmNactivateCallback,ShearOk,NULL);
  XtAddCallback(shear,XmNactivateCallback,Shear,0);


  /* CREATE BLUR BUTTON */
  blur=XtVaCreateManagedWidget(
    "blur",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Blur");
  XtVaSetValues(blur,XmNlabelString,text,NULL);

  blurForm=XmCreateBulletinBoardDialog(mainWindow,"blurForm",NULL,0);
  blurOkButton=XtVaCreateManagedWidget("blurOk",xmPushButtonWidgetClass,
    blurForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(blurOkButton,XmNlabelString,text,XmNx,50,XmNy,220,
    XmNwidth,80,NULL);
  blurCancelButton=XtVaCreateManagedWidget("blurCancel",
    xmPushButtonWidgetClass,blurForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(blurCancelButton,XmNlabelString,text,XmNx,200,XmNy,220,NULL);

  blurTextfield1=XtVaCreateManagedWidget("blurTextfield1",
    xmTextFieldWidgetClass,blurForm,NULL);
  XtVaSetValues(blurTextfield1,XmNwidth,100,XmNx,260,XmNy,10,
    XmNvalue,"0.9",NULL);
  blurLabel1=XtVaCreateManagedWidget("blurLabel1",xmLabelWidgetClass,
    blurForm,NULL);
  text=XmStringCreateLocalized("Blur (0-1):");
  XtVaSetValues(blurLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  blurTextfield2=XtVaCreateManagedWidget("blurTextfield2",
    xmTextFieldWidgetClass,blurForm,NULL);
  XtVaSetValues(blurTextfield2,XmNwidth,100,XmNx,260,XmNy,60,
    XmNvalue,"1",NULL);
  blurLabel2=XtVaCreateManagedWidget("blurLabel2",xmLabelWidgetClass,
    blurForm,NULL);
  text=XmStringCreateLocalized("Multiplication pr. sec.:");
  XtVaSetValues(blurLabel2,XmNlabelString,text,XmNx,0,XmNy,64,NULL);

  blurTextfield3=XtVaCreateManagedWidget("blurTextfield3",
    xmTextFieldWidgetClass,blurForm,NULL);
  XtVaSetValues(blurTextfield3,XmNwidth,100,XmNx,260,XmNy,110,
    XmNvalue,"0",NULL);
  blurLabel3=XtVaCreateManagedWidget("blurLabel3",xmLabelWidgetClass,
    blurForm,NULL);
  text=XmStringCreateLocalized("Re-init frequency (0-20):");
  XtVaSetValues(blurLabel3,XmNlabelString,text,XmNx,0,XmNy,114,NULL);

  blurToggle1=XtVaCreateManagedWidget("blurToggle1",
    xmToggleButtonWidgetClass,blurForm,NULL);
  text=XmStringCreateLocalized("Control function -> Blur (0-1)");
  XtVaSetValues(blurToggle1,XmNlabelString,text,XmNset,False,
    XmNy,160,NULL);

  text=XmStringCreateLocalized("Blur");
  XtVaSetValues(blurForm,XmNdialogTitle,text,
    XmNcancelButton,blurCancelButton,
    XmNdefaultButton,blurOkButton,NULL);

  XtAddCallback(blurOkButton,XmNactivateCallback,BlurOk,NULL);
  XtAddCallback(blur,XmNactivateCallback,Blur,0);


  /* CREATE COMB BUTTON */
  comb=XtVaCreateManagedWidget(
    "comb",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Comb filter");
  XtVaSetValues(comb,XmNlabelString,text,NULL);

  combForm=XmCreateBulletinBoardDialog(mainWindow,"combForm",NULL,0);
  combOkButton=XtVaCreateManagedWidget("combOk",xmPushButtonWidgetClass,
    combForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(combOkButton,XmNlabelString,text,XmNx,50,XmNy,170,
    XmNwidth,80,NULL);
  combCancelButton=XtVaCreateManagedWidget("combCancel",
    xmPushButtonWidgetClass,combForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(combCancelButton,XmNlabelString,text,XmNx,200,XmNy,170,NULL);

  combTextfield1=XtVaCreateManagedWidget("combTextfield1",
    xmTextFieldWidgetClass,combForm,NULL);
  XtVaSetValues(combTextfield1,XmNwidth,100,XmNx,260,XmNy,10,
    XmNvalue,"200",NULL);
  combLabel1=XtVaCreateManagedWidget("combLabel1",xmLabelWidgetClass,
    combForm,NULL);
  text=XmStringCreateLocalized("Fundamental frequency (Hz):");
  XtVaSetValues(combLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  combTextfield2=XtVaCreateManagedWidget("combTextfield2",
    xmTextFieldWidgetClass,combForm,NULL);
  XtVaSetValues(combTextfield2,XmNwidth,100,XmNx,260,XmNy,60,
    XmNvalue,"1",NULL);
  combLabel2=XtVaCreateManagedWidget("combLabel2",xmLabelWidgetClass,
    combForm,NULL);
  text=XmStringCreateLocalized("Multiplication pr. sec.:");
  XtVaSetValues(combLabel2,XmNlabelString,text,XmNx,0,XmNy,64,NULL);

  combToggle1=XtVaCreateManagedWidget("combToggle1",
    xmToggleButtonWidgetClass,combForm,NULL);
  text=XmStringCreateLocalized("Control function -> Fundamental (0-1000)");
  XtVaSetValues(combToggle1,XmNlabelString,text,XmNset,False,
    XmNy,110,NULL);

  text=XmStringCreateLocalized("Comb filter");
  XtVaSetValues(combForm,XmNdialogTitle,text,
    XmNcancelButton,combCancelButton,
    XmNdefaultButton,combOkButton,NULL);

  XtAddCallback(combOkButton,XmNactivateCallback,CombOk,NULL);
  XtAddCallback(comb,XmNactivateCallback,Comb,0);


  /* CREATE MIRROR BUTTON */
  mirror=XtVaCreateManagedWidget(
    "mirror",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Mirror");
  XtVaSetValues(mirror,XmNlabelString,text,NULL);

  mirrorForm=XmCreateBulletinBoardDialog(mainWindow,"mirrorForm",NULL,0);
  mirrorOkButton=XtVaCreateManagedWidget("mirrorOk",xmPushButtonWidgetClass,
    mirrorForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(mirrorOkButton,XmNlabelString,text,XmNx,50,XmNy,170,
    XmNwidth,80,NULL);
  mirrorCancelButton=XtVaCreateManagedWidget("mirrorCancel",
    xmPushButtonWidgetClass,mirrorForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(mirrorCancelButton,XmNlabelString,text,XmNx,200,XmNy,170,NULL);

  mirrorTextfield=XtVaCreateManagedWidget("mirrorTextfield",
    xmTextFieldWidgetClass,mirrorForm,NULL);
  XtVaSetValues(mirrorTextfield,XmNwidth,100,XmNx,260,XmNy,10,
    XmNvalue,"500",NULL);
  mirrorLabel1=XtVaCreateManagedWidget("mirrorLabel1",xmLabelWidgetClass,
    mirrorForm,NULL);
  text=XmStringCreateLocalized("Frequency of mirror (Hz):");
  XtVaSetValues(mirrorLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  mirrorTextfield2=XtVaCreateManagedWidget("mirrorTextfield2",
    xmTextFieldWidgetClass,mirrorForm,NULL);
  XtVaSetValues(mirrorTextfield2,XmNwidth,100,XmNx,260,XmNy,60,
    XmNvalue,"1",NULL);
  mirrorLabel2=XtVaCreateManagedWidget("mirrorLabel2",xmLabelWidgetClass,
    mirrorForm,NULL);
  text=XmStringCreateLocalized("Multiplication pr. sec.:");
  XtVaSetValues(mirrorLabel2,XmNlabelString,text,XmNx,0,XmNy,64,NULL);

  mirrorToggle=XtVaCreateManagedWidget("mirrorToggle",
    xmToggleButtonWidgetClass,mirrorForm,NULL);
  text=XmStringCreateLocalized("Control function -> Mirror (0-Fs/2)");
  XtVaSetValues(mirrorToggle,XmNlabelString,text,XmNset,False,
    XmNy,110,NULL);

  text=XmStringCreateLocalized("Mirror");
  XtVaSetValues(mirrorForm,XmNdialogTitle,text,
    XmNcancelButton,mirrorCancelButton,
    XmNdefaultButton,mirrorOkButton,NULL);

  XtAddCallback(mirrorOkButton,XmNactivateCallback,MirrorOk,NULL);
  XtAddCallback(mirror,XmNactivateCallback,Mirror,0);


  /* CREATE EXPO BUTTON */
  expo=XtVaCreateManagedWidget(
    "expo",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Exponentiate");
  XtVaSetValues(expo,XmNlabelString,text,NULL);

  expoForm=XmCreateBulletinBoardDialog(mainWindow,"expoForm",NULL,0);
  expoOkButton=XtVaCreateManagedWidget("expoOk",xmPushButtonWidgetClass,
    expoForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(expoOkButton,XmNlabelString,text,XmNx,50,XmNy,220,
    XmNwidth,80,NULL);
  expoCancelButton=XtVaCreateManagedWidget("expoCancel",
    xmPushButtonWidgetClass,expoForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(expoCancelButton,XmNlabelString,text,XmNx,200,XmNy,220,NULL);

  expoTextfield=XtVaCreateManagedWidget("expoTextfield",
    xmTextFieldWidgetClass,expoForm,NULL);
  XtVaSetValues(expoTextfield,XmNwidth,100,XmNx,260,XmNy,10,
    XmNvalue,"0.7",NULL);
  expoLabel1=XtVaCreateManagedWidget("expoLabel1",xmLabelWidgetClass,
    expoForm,NULL);
  text=XmStringCreateLocalized("Frequency exponent (0-2):");
  XtVaSetValues(expoLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  expoTextfield2=XtVaCreateManagedWidget("expoTextfield2",
    xmTextFieldWidgetClass,expoForm,NULL);
  XtVaSetValues(expoTextfield2,XmNwidth,100,XmNx,260,XmNy,60,
    XmNvalue,"1",NULL);
  expoLabel2=XtVaCreateManagedWidget("expoLabel2",xmLabelWidgetClass,
    expoForm,NULL);
  text=XmStringCreateLocalized("Multiplication pr. sec.:");
  XtVaSetValues(expoLabel2,XmNlabelString,text,XmNx,0,XmNy,64,NULL);

  expoToggle=XtVaCreateManagedWidget("expoToggle",
    xmToggleButtonWidgetClass,expoForm,NULL);
  text=XmStringCreateLocalized("Control function -> Exponent (0-2)");
  XtVaSetValues(expoToggle,XmNlabelString,text,XmNset,False,
    XmNy,110,NULL);

  expoTextfield3=XtVaCreateManagedWidget("expoTextfield3",
    xmTextFieldWidgetClass,expoForm,NULL);
  XtVaSetValues(expoTextfield3,XmNwidth,100,XmNx,260,XmNy,160,
    XmNvalue,"1",NULL);
  expoLabel3=XtVaCreateManagedWidget("expoLabel3",xmLabelWidgetClass,
    expoForm,NULL);
  text=XmStringCreateLocalized("Multiplication factor:");
  XtVaSetValues(expoLabel3,XmNlabelString,text,XmNx,0,XmNy,164,NULL);

  text=XmStringCreateLocalized("Exponentiate");
  XtVaSetValues(expoForm,XmNdialogTitle,text,
    XmNcancelButton,expoCancelButton,
    XmNdefaultButton,expoOkButton,NULL);

  XtAddCallback(expoOkButton,XmNactivateCallback,ExpoOk,NULL);
  XtAddCallback(expo,XmNactivateCallback,Expo,0);

  /* CREATE SPREAD BUTTON */
  spread=XtVaCreateManagedWidget(
    "spread",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Spread");
  XtVaSetValues(spread,XmNlabelString,text,NULL);

  spreadForm=XmCreateBulletinBoardDialog(mainWindow,"spreadForm",NULL,0);
  spreadOkButton=XtVaCreateManagedWidget("spreadOk",xmPushButtonWidgetClass,
    spreadForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(spreadOkButton,XmNlabelString,text,XmNx,50,XmNy,220,
    XmNwidth,80,NULL);
  spreadCancelButton=XtVaCreateManagedWidget("spreadCancel",
    xmPushButtonWidgetClass,spreadForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(spreadCancelButton,XmNlabelString,text,XmNx,200,XmNy,220,NULL);

  spreadTextfield=XtVaCreateManagedWidget("spreadTextfield",
    xmTextFieldWidgetClass,spreadForm,NULL);
  XtVaSetValues(spreadTextfield,XmNwidth,100,XmNx,260,XmNy,10,
    XmNvalue,"0.1",NULL);
  spreadLabel1=XtVaCreateManagedWidget("spreadLabel1",xmLabelWidgetClass,
    spreadForm,NULL);
  text=XmStringCreateLocalized("Random spread (0-2 octaves):");
  XtVaSetValues(spreadLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  spreadTextfield2=XtVaCreateManagedWidget("spreadTextfield2",
    xmTextFieldWidgetClass,spreadForm,NULL);
  XtVaSetValues(spreadTextfield2,XmNwidth,100,XmNx,260,XmNy,60,
    XmNvalue,"1",NULL);
  spreadLabel2=XtVaCreateManagedWidget("spreadLabel2",xmLabelWidgetClass,
    spreadForm,NULL);
  text=XmStringCreateLocalized("Multiplication pr. sec.:");
  XtVaSetValues(spreadLabel2,XmNlabelString,text,XmNx,0,XmNy,64,NULL);

  spreadToggle=XtVaCreateManagedWidget("spreadToggle",
    xmToggleButtonWidgetClass,spreadForm,NULL);
  text=XmStringCreateLocalized("Control function -> Spread (0-2)");
  XtVaSetValues(spreadToggle,XmNlabelString,text,XmNset,False,
    XmNy,110,NULL);

  spreadTextfield3=XtVaCreateManagedWidget("spreadTextfield3",
    xmTextFieldWidgetClass,spreadForm,NULL);
  XtVaSetValues(spreadTextfield3,XmNwidth,100,XmNx,260,XmNy,160,
    XmNvalue,"0",NULL);
  spreadLabel3=XtVaCreateManagedWidget("spreadLabel3",xmLabelWidgetClass,
    spreadForm,NULL);
  text=XmStringCreateLocalized("Amplitude sensitivity (0-1):");
  XtVaSetValues(spreadLabel3,XmNlabelString,text,XmNx,0,XmNy,164,NULL);

  text=XmStringCreateLocalized("Spread");
  XtVaSetValues(spreadForm,XmNdialogTitle,text,
    XmNcancelButton,spreadCancelButton,
    XmNdefaultButton,spreadOkButton,NULL);

  XtAddCallback(spreadOkButton,XmNactivateCallback,SpreadOk,NULL);
  XtAddCallback(spread,XmNactivateCallback,Spread,0);


  /* CREATE PITCH SHIFT BUTTON */
  pshift=XtVaCreateManagedWidget(
    "pshift",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Pitch shift");
  XtVaSetValues(pshift,XmNlabelString,text,NULL);


  pitchForm=XmCreateBulletinBoardDialog(mainWindow,"pitchForm",NULL,0);
  pitchOkButton=XtVaCreateManagedWidget("pitchOk",xmPushButtonWidgetClass,
    pitchForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(pitchOkButton,XmNlabelString,text,XmNx,50,XmNy,220,
    XmNwidth,80,NULL);
  pitchCancelButton=XtVaCreateManagedWidget("pitchCancel",
    xmPushButtonWidgetClass,pitchForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(pitchCancelButton,XmNlabelString,text,XmNx,200,XmNy,220,NULL);

  pitchTextfield=XtVaCreateManagedWidget("pitchTextfield",
    xmTextFieldWidgetClass,pitchForm,NULL);
  XtVaSetValues(pitchTextfield,XmNwidth,100,XmNx,260,XmNy,10,
    XmNvalue,"1",NULL);
  pitchLabel1=XtVaCreateManagedWidget("pitchLabel1",xmLabelWidgetClass,
    pitchForm,NULL);
  text=XmStringCreateLocalized("Transposition factor:");
  XtVaSetValues(pitchLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  pitchTextfield2=XtVaCreateManagedWidget("pitchTextfield2",
    xmTextFieldWidgetClass,pitchForm,NULL);
  XtVaSetValues(pitchTextfield2,XmNwidth,100,XmNx,260,XmNy,60,
    XmNvalue,"1",NULL);
  pitchLabel2=XtVaCreateManagedWidget("pitchLabel2",xmLabelWidgetClass,
    pitchForm,NULL);
  text=XmStringCreateLocalized("Multiplication pr. sec.:");
  XtVaSetValues(pitchLabel2,XmNlabelString,text,XmNx,0,XmNy,64,NULL);

  pitchToggle1=XtVaCreateManagedWidget("pitchToggle1",
    xmToggleButtonWidgetClass,pitchForm,NULL);
  text=XmStringCreateLocalized("Control function -> Tr. factor (0-2)");
  XtVaSetValues(pitchToggle1,XmNlabelString,text,XmNset,False,
    XmNy,110,NULL);

  pitchTextfield3=XtVaCreateManagedWidget("pitchTextfield3",
    xmTextFieldWidgetClass,pitchForm,NULL);
  XtVaSetValues(pitchTextfield3,XmNwidth,100,XmNx,260,XmNy,160,
    XmNvalue,"0",NULL);
  pitchLabel3=XtVaCreateManagedWidget("pitchLabel3",xmLabelWidgetClass,
    pitchForm,NULL);
  text=XmStringCreateLocalized("Static frequency:");
  XtVaSetValues(pitchLabel3,XmNlabelString,text,XmNx,0,XmNy,164,NULL);

  text=XmStringCreateLocalized("Pitch shift");
  XtVaSetValues(pitchForm,XmNdialogTitle,text,
    XmNcancelButton,pitchCancelButton,
    XmNdefaultButton,pitchOkButton,NULL);

  XtAddCallback(pitchOkButton,XmNactivateCallback,PshiftOk,NULL);
  XtAddCallback(pshift,XmNactivateCallback,Pshift,0);

  /* CREATE FILTER BUTTON */
  filter=XtVaCreateManagedWidget(
    "filter",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Filter");
  XtVaSetValues(filter,XmNlabelString,text,NULL);

  filterForm=XmCreateBulletinBoardDialog(mainWindow,"filterForm",NULL,0);
  filterOkButton=XtVaCreateManagedWidget("filterOk",xmPushButtonWidgetClass,
    filterForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(filterOkButton,XmNlabelString,text,XmNx,50,XmNy,160,
    XmNwidth,80,NULL);
  filterCancelButton=XtVaCreateManagedWidget("filterCancel",
    xmPushButtonWidgetClass,filterForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(filterCancelButton,XmNlabelString,text,XmNx,200,XmNy,160,NULL);

  filterTextfield=XtVaCreateManagedWidget("filterTextfield",
    xmTextFieldWidgetClass,filterForm,NULL);
  XtVaSetValues(filterTextfield,XmNwidth,100,XmNx,260,XmNy,10,
    XmNvalue,"0",NULL);
  filterLabel1=XtVaCreateManagedWidget("filterLabel1",xmLabelWidgetClass,
    filterForm,NULL);
  text=XmStringCreateLocalized("Lower cutoff frequency:");
  XtVaSetValues(filterLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  filterTextfield2=XtVaCreateManagedWidget("filterTextfield2",
    xmTextFieldWidgetClass,filterForm,NULL);
  XtVaSetValues(filterTextfield2,XmNwidth,100,XmNx,260,XmNy,60,
    XmNvalue,"0",NULL);
  filterLabel2=XtVaCreateManagedWidget("filterLabel2",xmLabelWidgetClass,
    filterForm,NULL);
  text=XmStringCreateLocalized("Upper cutoff frequency:");
  XtVaSetValues(filterLabel2,XmNlabelString,text,XmNx,0,XmNy,64,NULL);

  filterFilToggle=XtVaCreateManagedWidget("filterFilToggle",
    xmToggleButtonWidgetClass, filterForm, NULL);
  text=XmStringCreateLocalized("Read from file 'frequencies'");
  XtVaSetValues(filterFilToggle,XmNlabelString,text,XmNset,False,
    XmNy,110,NULL);

  text=XmStringCreateLocalized("Bandstop Filter");
  XtVaSetValues(filterForm,XmNdialogTitle,text,
    XmNcancelButton,filterCancelButton,
    XmNdefaultButton,filterOkButton,NULL);

  XtAddCallback(filterOkButton,XmNactivateCallback,filterOk,NULL);
  XtAddCallback(filter,XmNactivateCallback,Filter,0);


  /* CREATE GAIN BUTTON */
  gain=XtVaCreateManagedWidget(
    "gain",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Gain");
  XtVaSetValues(gain,XmNlabelString,text,NULL);

  gainForm=XmCreateBulletinBoardDialog(mainWindow,"gainForm",NULL,0);
  gainOkButton=XtVaCreateManagedWidget("gainOk",xmPushButtonWidgetClass,
    gainForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(gainOkButton,XmNlabelString,text,XmNx,40,XmNy,100,
    XmNwidth,80,NULL);
  gainCancelButton=XtVaCreateManagedWidget("gainCancel",
    xmPushButtonWidgetClass,gainForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(gainCancelButton,XmNlabelString,text,XmNx,200,XmNy,100,NULL);

  gainTextfield=XtVaCreateManagedWidget("gainTextfield",
    xmTextFieldWidgetClass,gainForm,NULL);
  XtVaSetValues(gainTextfield,XmNwidth,100,XmNx,200,XmNy,10,
    XmNvalue,"0",NULL);
  gainLabel=XtVaCreateManagedWidget("gainLabel",xmLabelWidgetClass,
    gainForm,NULL);
  text=XmStringCreateLocalized("Gain factor:");
  XtVaSetValues(gainLabel,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  gainToggle=XtVaCreateManagedWidget("gainToggle",
    xmToggleButtonWidgetClass,gainForm,NULL);
  text=XmStringCreateLocalized("Control function -> Gain (0-1)");
  XtVaSetValues(gainToggle,XmNlabelString,text,XmNset,False,
    XmNy,60,NULL);

  text=XmStringCreateLocalized("Gain");
  XtVaSetValues(gainForm,XmNdialogTitle,text,
    XmNcancelButton,gainCancelButton,
    XmNdefaultButton,gainOkButton,NULL);

  XtAddCallback(gainOkButton,XmNactivateCallback,GainOk,NULL);
  XtAddCallback(gain,XmNactivateCallback,Gain,0);
  
  
  /* CREATE THRESHOLD BUTTON */
  threshold=XtVaCreateManagedWidget(
    "threshold",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Threshold");
  XtVaSetValues(threshold,XmNlabelString,text,NULL);

  thresholdForm=XmCreateBulletinBoardDialog(mainWindow,"thresholdForm",NULL,0);
  thresholdOkButton=XtVaCreateManagedWidget("thresholdOk",xmPushButtonWidgetClass,
    thresholdForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(thresholdOkButton,XmNlabelString,text,XmNx,40,XmNy,100,
    XmNwidth,80,NULL);
  thresholdCancelButton=XtVaCreateManagedWidget("thresholdCancel",
    xmPushButtonWidgetClass,thresholdForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(thresholdCancelButton,XmNlabelString,text,XmNx,200,XmNy,100,NULL);

  thresholdTextfield=XtVaCreateManagedWidget("thresholdTextfield",
    xmTextFieldWidgetClass,thresholdForm,NULL);
  XtVaSetValues(thresholdTextfield,XmNwidth,100,XmNx,200,XmNy,10,
    XmNvalue,"0.1",NULL);
  thresholdLabel=XtVaCreateManagedWidget("thresholdLabel",xmLabelWidgetClass,
    thresholdForm,NULL);
  text=XmStringCreateLocalized("Threshold (0-1):");
  XtVaSetValues(thresholdLabel,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  thresholdToggle=XtVaCreateManagedWidget("thresholdToggle",
    xmToggleButtonWidgetClass,thresholdForm,NULL);
  text=XmStringCreateLocalized("Control function -> Threshold (0-1)");
  XtVaSetValues(thresholdToggle,XmNlabelString,text,XmNset,False,
    XmNy,60,NULL);

  text=XmStringCreateLocalized("Threshold");
  XtVaSetValues(thresholdForm,XmNdialogTitle,text,
    XmNcancelButton,thresholdCancelButton,
    XmNdefaultButton,thresholdOkButton,NULL);

  XtAddCallback(thresholdOkButton,XmNactivateCallback,ThresholdOk,NULL);
  XtAddCallback(threshold,XmNactivateCallback,Threshold,0);
  
  
  /* CREATE SPECTRAL SUBTRACTION BUTTON */
  noisered=XtVaCreateManagedWidget(
    "noisered",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Spectral subtraction");
  XtVaSetValues(noisered,XmNlabelString,text,NULL);

  noiseredForm=XmCreateBulletinBoardDialog(mainWindow,"noiseredForm",NULL,0);
  noiseredOkButton=XtVaCreateManagedWidget("noiseredOk",xmPushButtonWidgetClass,
    noiseredForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(noiseredOkButton,XmNlabelString,text,XmNx,40,XmNy,180,
    XmNwidth,80,NULL);
  noiseredCancelButton=XtVaCreateManagedWidget("noiseredCancel",
    xmPushButtonWidgetClass,noiseredForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(noiseredCancelButton,XmNlabelString,text,XmNx,200,XmNy,180,NULL);

  noiseredTextfield=XtVaCreateManagedWidget("noiseredTextfield",
    xmTextFieldWidgetClass,noiseredForm,NULL);
  XtVaSetValues(noiseredTextfield,XmNwidth,100,XmNx,200,XmNy,10,
    XmNvalue,"0.8",NULL);
  noiseredLabel=XtVaCreateManagedWidget("noiseredLabel",xmLabelWidgetClass,
    noiseredForm,NULL);
  text=XmStringCreateLocalized("Amount (0-2):");
  XtVaSetValues(noiseredLabel,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  noiseredToggle=XtVaCreateManagedWidget("noiseredToggle",
    xmToggleButtonWidgetClass,noiseredForm,NULL);
  text=XmStringCreateLocalized("Control function -> Amount (0-2)");
  XtVaSetValues(noiseredToggle,XmNlabelString,text,XmNset,False,
    XmNy,60,NULL);
    
  noiseredToggle2=XtVaCreateManagedWidget("noiseredToggle2",
    xmToggleButtonWidgetClass,noiseredForm,NULL);
  text=XmStringCreateLocalized("Envelope noise spectrum");
  XtVaSetValues(noiseredToggle2,XmNlabelString,text,XmNset,False,
    XmNy,100,NULL);
    
  noiseredToggle3=XtVaCreateManagedWidget("noiseredToggle3",
    xmToggleButtonWidgetClass,noiseredForm,NULL);
  text=XmStringCreateLocalized("Despeckle");
  XtVaSetValues(noiseredToggle3,XmNlabelString,text,XmNset,False,
    XmNy,130,NULL);

  text=XmStringCreateLocalized("Subtract selected area");
  XtVaSetValues(noiseredForm,XmNdialogTitle,text,
    XmNcancelButton,noiseredCancelButton,
    XmNdefaultButton,noiseredOkButton,NULL);

  XtAddCallback(noiseredOkButton,XmNactivateCallback,NoiseredOk,NULL);
  XtAddCallback(noisered,XmNactivateCallback,Noisered,0);


  /* CREATE NULL PHASE BUTTON */
  nullphase=XtVaCreateManagedWidget(
    "nullphase",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Null phase");
  XtVaSetValues(nullphase,XmNlabelString,text,NULL);
  XtAddCallback(nullphase,XmNactivateCallback,Nullphase,0);

  /* CREATE TRPEAKS BUTTON */
  trpeaks=XtVaCreateManagedWidget(
    "trpeaks",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Keep peaks only");
  XtVaSetValues(trpeaks,XmNlabelString,text,NULL);
  XtAddCallback(trpeaks,XmNactivateCallback,Trpeaks,0);

  /* CREATE LTAS BUTTON */
  ltas=XtVaCreateManagedWidget(
    "ltas",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Average");
  XtVaSetValues(ltas,XmNlabelString,text,NULL);
  XtAddCallback(ltas,XmNactivateCallback,LTAS,0);

  /* CREATE MAGNET BUTTON */
  magnet=XtVaCreateManagedWidget(
    "magnet",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Move to pitch grid");
  XtVaSetValues(magnet,XmNlabelString,text,NULL);

  magnetForm=XmCreateBulletinBoardDialog(mainWindow,"magnetForm",NULL,0);
  magnetOkButton=XtVaCreateManagedWidget("magnetOk",xmPushButtonWidgetClass,
    magnetForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(magnetOkButton,XmNlabelString,text,XmNx,50,XmNy,170,
    XmNwidth,80,NULL);
  magnetCancelButton=XtVaCreateManagedWidget("magnetCancel",
    xmPushButtonWidgetClass,magnetForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(magnetCancelButton,XmNlabelString,text,XmNx,200,XmNy,170,NULL);

  magnetTextfield=XtVaCreateManagedWidget("magnetTextfield",
    xmTextFieldWidgetClass,magnetForm,NULL);
  XtVaSetValues(magnetTextfield,XmNwidth,100,XmNx,260,XmNy,10,
    XmNvalue,"1",NULL);
  magnetLabel1=XtVaCreateManagedWidget("magnetLabel1",xmLabelWidgetClass,
    magnetForm,NULL);
  text=XmStringCreateLocalized("Probability:");
  XtVaSetValues(magnetLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  magnetTextfield2=XtVaCreateManagedWidget("magnetTextfield2",
    xmTextFieldWidgetClass,magnetForm,NULL);
  XtVaSetValues(magnetTextfield2,XmNwidth,100,XmNx,260,XmNy,60,
    XmNvalue,"1",NULL);
  magnetLabel2=XtVaCreateManagedWidget("magnetLabel2",xmLabelWidgetClass,
    magnetForm,NULL);
  text=XmStringCreateLocalized("Multiplication pr. sec.:");
  XtVaSetValues(magnetLabel2,XmNlabelString,text,XmNx,0,XmNy,64,NULL);

  magnetToggle1=XtVaCreateManagedWidget("magnetToggle1",
    xmToggleButtonWidgetClass,magnetForm,NULL);
  text=XmStringCreateLocalized("Control function -> Probability (0-1)");
  XtVaSetValues(magnetToggle1,XmNlabelString,text,XmNset,False,
    XmNy,110,NULL);

  text=XmStringCreateLocalized("Move to pitch grid");
  XtVaSetValues(magnetForm,XmNdialogTitle,text,
    XmNcancelButton,magnetCancelButton,
    XmNdefaultButton,magnetOkButton,NULL);

  XtAddCallback(magnetOkButton,XmNactivateCallback,MagnetOk,NULL);
  XtAddCallback(magnet,XmNactivateCallback,Magnet,0);


  /* CREATE CONVOLVE BUTTON */
  convolveButton=XtVaCreateManagedWidget(
    "convolveButton",xmPushButtonWidgetClass,transMenu,NULL);
  text=XmStringCreateLocalized("Convolve");
  XtVaSetValues(convolveButton,XmNlabelString,text,NULL);

  convolveForm=XmCreateBulletinBoardDialog(mainWindow,"convolveForm",NULL,0);
  convolveOkButton=XtVaCreateManagedWidget("convolveOk",xmPushButtonWidgetClass,
    convolveForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(convolveOkButton,XmNlabelString,text,XmNx,50,XmNy,200,
    XmNwidth,80,NULL);
  convolveCancelButton=XtVaCreateManagedWidget("convolveCancel",
    xmPushButtonWidgetClass,convolveForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(convolveCancelButton,XmNlabelString,text,XmNx,200,XmNy,200,NULL);

  convolveRadio=XtVaCreateManagedWidget("convolveRadio",
    xmRowColumnWidgetClass,convolveForm,NULL);
  XtVaSetValues(convolveRadio,XmNy,20,XmNradioBehavior,True,
    XmNradioAlwaysOne,True,NULL);
  convolveToggleLaplace=XtVaCreateManagedWidget("convolveToggleLaplace",
    xmToggleButtonWidgetClass,convolveRadio,NULL);
  text=XmStringCreateLocalized("Laplace");
  XtVaSetValues(convolveToggleLaplace,XmNlabelString,text,XmNset,True,NULL);
  convolveToggleMean=XtVaCreateManagedWidget("convolveToggleMean",
    xmToggleButtonWidgetClass,convolveRadio,NULL);
  text=XmStringCreateLocalized("Mean");
  XtVaSetValues(convolveToggleMean,XmNlabelString,text,NULL);
  convolveToggleTimeMean=XtVaCreateManagedWidget("convolveToggleTimeMean",
    xmToggleButtonWidgetClass,convolveRadio,NULL);
  text=XmStringCreateLocalized("Time Mean");
  XtVaSetValues(convolveToggleTimeMean,XmNlabelString,text,NULL);
  convolveToggleFile=XtVaCreateManagedWidget("convolveToggleFile",
    xmToggleButtonWidgetClass,convolveRadio,NULL);
  text=XmStringCreateLocalized("Read from file 'convolve'");
  XtVaSetValues(convolveToggleFile,XmNlabelString,text,NULL);

  text=XmStringCreateLocalized("Convolve with local operator");
  XtVaSetValues(convolveForm,XmNdialogTitle,text,
    XmNcancelButton,convolveCancelButton,XmNdefaultButton,convolveOkButton,NULL);

  XtAddCallback(convolveButton,XmNactivateCallback,Convolve,0);
  XtAddCallback(convolveOkButton,XmNactivateCallback,ConvolveOk,0);

}


void main(int argc, char **argv)
{
  XmString text;
  int i, j;

  static XtActionsRec actions[]={
    {"GraphClick",  GraphClick},
    {"GraphDrag",   GraphDrag},
    {"AreaClick",   AreaClick},
    {"AreaDrag",    AreaDrag},
    {"AreaDrop",    AreaDrop},
    {"Keydown",     Keydown},
  };
  static char drawTranslations[]=
    "<Btn2Down>:   GraphClick()\n\
     <Btn2Motion>: GraphDrag()\n\
     <Btn3Down>:   AreaClick()\n\
     <Btn3Up>:     AreaDrop()\n\
     <Btn3Motion>: AreaDrag()\n\
     <Key>:        Keydown()";


  /* INITIALIZE PHASE VOCODER */

  /* FFT length */
  N=1024;
  if (argc>1) {
    N=atoi(argv[1]);
    if ((N!=512) && (N!=1024) && (N!=2048) && (N!=4096)) {
      printf("Illegal value for N. Selected 1024.\n"); N=1024;
    }
  }
  overlap=8;

  Nw=N;    /* Window size */
  Dn=Nw/overlap;  /* Decimation factor */
  I=Dn;     /* Interpolation factor */
  N2=N>>1; theheight=N2;
  P=1.;    /* Oscbank pitch factor */
  Pinc = P*8192./R;
  NP=N2;
  synt=0.; /* Synthesis threshold */
  binfreq = (double)R/N;
  obank=0.;
  Nw2=Nw>>1;
  numchannels=N/2+1; areaf2=numchannels;
  fvec(Wanal,  Nw);
  fvec(Wsyn,  Nw);
  fvec(input,  Nw);
  fvec(buffer,  N);
  fvec(channel,  N+2);
  init_again();
  fvec(tempamp,numchannels);
  fvec(tempfreq,numchannels);
  fvec(last, numchannels); fvec(this, numchannels);

  if ((framebuff =(short int *)calloc(Nw*2*4,sizeof(short int)))==NULL) {
    fprintf(stderr,"Unable to allocate AIFF framebuffer!\n");
    exit(-1);
  }
  
  printf("Starting Ceres ...\n");


  /* INITIALIZE X */
/*  XtSetLanguageProc(NULL, (XtLanguageProc)NULL, NULL); */
  topLevel=XtVaAppInitialize(
    &app_context,"Ceres",NULL,0,&argc,argv,NULL,NULL);
  dw = XmGetXmDisplay(XtDisplay(topLevel));
  XtVaSetValues(dw, XmNdragInitiatorProtocolStyle, XmDRAG_NONE, NULL);


  /* CREATE MAIN WINDOW & IMMEDIATE CHILDREN */
  mainWindow=XtVaCreateManagedWidget(
    "mainWindow",xmMainWindowWidgetClass,topLevel,
    XmNscrollingPolicy, XmAUTOMATIC, NULL);

  menuBar=XmCreateMenuBar(mainWindow,"menuBar",NULL,0);
  XtManageChild(menuBar);

  frame=XtVaCreateManagedWidget(
    "frame",xmFrameWidgetClass,mainWindow,NULL);

  XmMainWindowSetAreas(mainWindow, menuBar, NULL, NULL, NULL, frame);
  XtVaSetValues(mainWindow,XmNwidth,800+69,XmNheight,theheight+85, NULL);


  /* CREATE GRAPHICS */
   sketchpad = XtVaCreateManagedWidget(
     "sketchpad",xmDrawingAreaWidgetClass, frame,
      XmNwidth,800+70,XmNheight,theheight+70,NULL);

   theDisplay = XtDisplay(sketchpad);
   theWindow = XtWindow(sketchpad);
   screen = DefaultScreen(theDisplay);
   theRoot = RootWindowOfScreen(XtScreen(sketchpad));
   theCmap = DefaultColormap( theDisplay, DefaultScreen(theDisplay) );
   bitmap = XCreatePixmap(XtDisplay(sketchpad),theRoot,800+70,theheight+70,
     DefaultDepthOfScreen(XtScreen(sketchpad)));
   spotsmap = XCreatePixmap(XtDisplay(sketchpad),theRoot,800+70,theheight+70,
     DefaultDepthOfScreen(XtScreen(sketchpad)));
   theGC = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC = XCreateGC( theDisplay, theRoot, 0, 0 );
   theGC0 = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC0 = XCreateGC( theDisplay, theRoot, 0, 0 );
   theGC25 = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC25 = XCreateGC( theDisplay, theRoot, 0, 0 );
   theGC50 = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC50 = XCreateGC( theDisplay, theRoot, 0, 0 );
   theGC75 = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC75 = XCreateGC( theDisplay, theRoot, 0, 0 );
   theGC100 = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC100 = XCreateGC( theDisplay, theRoot, 0, 0 );
   funcGC = XCreateGC( theDisplay, theRoot, 0, 0);
   gridGC = XCreateGC( theDisplay, theRoot, 0, 0);
   spotsGC = XCreateGC( theDisplay, theRoot, 0, 0);
   markerGC = XCreateGC( theDisplay, theRoot, 0, 0 );

   XParseColor(theDisplay,theCmap,"gray0",&gray0);
   XAllocColor( theDisplay, theCmap,&gray0);
   XParseColor(theDisplay,theCmap,"gray25",&gray25);
   XAllocColor( theDisplay, theCmap,&gray25);
   XParseColor(theDisplay,theCmap,"gray50",&gray50);
   XAllocColor( theDisplay, theCmap,&gray50);
   XParseColor(theDisplay,theCmap,"gray75",&gray75);
   XAllocColor( theDisplay, theCmap,&gray75);
   XParseColor(theDisplay,theCmap,"gray100",&gray100);
   XAllocColor( theDisplay, theCmap,&gray100);
   XParseColor(theDisplay,theCmap,"light blue",&lightblue);
   XAllocColor( theDisplay, theCmap,&lightblue);
   XParseColor(theDisplay,theCmap,"blue",&blue);
   XAllocColor( theDisplay, theCmap,&blue);
   XParseColor(theDisplay,theCmap,"navy",&darkblue);
   XAllocColor( theDisplay, theCmap,&darkblue);
   XParseColor(theDisplay,theCmap,"yellow",&yellow);
   XAllocColor( theDisplay, theCmap,&yellow);
   XParseColor(theDisplay,theCmap,"red",&red);
   XAllocColor( theDisplay, theCmap,&red);
   XParseColor(theDisplay,theCmap,"orange",&orange);
   XAllocColor( theDisplay, theCmap,&orange);
   XParseColor(theDisplay,theCmap,"green",&green);
   XAllocColor( theDisplay, theCmap,&green);
   XParseColor(theDisplay,theCmap,"coral",&brown);
   XAllocColor( theDisplay, theCmap,&brown);

   XSetForeground(theDisplay, gridGC, brown.pixel);

   XtAppAddActions(app_context, actions, XtNumber(actions));
   XtOverrideTranslations(sketchpad,XtParseTranslationTable(drawTranslations));
   XtAddCallback(sketchpad,XmNexposeCallback,RestoreWin,0);

   xpos[1]=50; ypos[1]=100; xpos[2]=850; ypos[2]=100;
   playfile[0]='\0';


  /* CREATE FILE MENU */
  fileButton=XtVaCreateManagedWidget(
    "fileButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("File");
  XtVaSetValues(fileButton,XmNlabelString,text,NULL);

  fileMenu=XmCreatePulldownMenu(menuBar,"fileMenu",NULL,0);
  XtVaSetValues(fileButton,XmNsubMenuId,fileMenu,NULL);


  /* CREATE LOAD & ANALYZE BUTTON */
  load=XtVaCreateManagedWidget(
    "load",xmPushButtonWidgetClass,fileMenu,NULL);
  text=XmStringCreateLocalized("Load & Analyze");
  XtVaSetValues(load,XmNlabelString,text,NULL);

  loadFileBox=XmCreateFileSelectionDialog(mainWindow,"loadFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(loadFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Load & Analyze");
  XtVaSetValues(loadFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);

  XtAddCallback(loadFileBox,XmNcancelCallback,Cancel,loadFileBox);
  XtAddCallback(loadFileBox,XmNokCallback,LoadOk,NULL);
  XtAddCallback(load,XmNactivateCallback,Load,NULL);


  /* CREATE SYNTH & SAVE BUTTON */
  save=XtVaCreateManagedWidget(
    "save",xmPushButtonWidgetClass,fileMenu,NULL);
  text=XmStringCreateLocalized("Synth & Save");
  XtVaSetValues(save,XmNlabelString,text,NULL);

  saveFileBox=XmCreateFileSelectionDialog(mainWindow,"saveFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(saveFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Synth & Save");
  XtVaSetValues(saveFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(saveFileBox,XmNcancelCallback,Cancel,saveFileBox);
  XtAddCallback(saveFileBox,XmNokCallback,SaveOk,NULL);
  XtAddCallback(save,XmNactivateCallback,Save,NULL);


  /* CREATE LOADLPC BUTTON */
  loadLpc=XtVaCreateManagedWidget(
    "loadLpc",xmPushButtonWidgetClass,fileMenu,NULL);
  text=XmStringCreateLocalized("Load & Analyze LPC");
  XtVaSetValues(loadLpc,XmNlabelString,text,NULL);

  loadLpcBox=XmCreateFileSelectionDialog(mainWindow,"loadLpcBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(loadLpcBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Load & Analyze LPC");
  XtVaSetValues(loadLpcBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);

  XtAddCallback(loadLpcBox,XmNcancelCallback,Cancel,loadLpcBox);
  XtAddCallback(loadLpcBox,XmNokCallback,LoadLpcOk,NULL);
  XtAddCallback(loadLpc,XmNactivateCallback,LoadLpc,NULL);


  /* CREATE LOAD IMAGE BUTTON */
  loadimage=XtVaCreateManagedWidget(
    "loadimage",xmPushButtonWidgetClass,fileMenu,NULL);
  text=XmStringCreateLocalized("Load PBM ascii image");
  XtVaSetValues(loadimage,XmNlabelString,text,NULL);

  imageFileBox=XmCreateFileSelectionDialog(mainWindow,"imageFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(imageFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Load PBM ascii image");
  XtVaSetValues(imageFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);

  XtAddCallback(imageFileBox,XmNcancelCallback,Cancel,imageFileBox);
  XtAddCallback(imageFileBox,XmNokCallback,ImageOk,NULL);
  XtAddCallback(loadimage,XmNactivateCallback,Image,NULL);


  /* CREATE LOAD BREAKPOINT BUTTON */
  loadbrk=XtVaCreateManagedWidget(
    "loadbrk",xmPushButtonWidgetClass,fileMenu,NULL);
  text=XmStringCreateLocalized("Load control function");
  XtVaSetValues(loadbrk,XmNlabelString,text,NULL);

  brkFileBox=XmCreateFileSelectionDialog(mainWindow,"brkFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(brkFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Load control function");
  XtVaSetValues(brkFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);

  XtAddCallback(brkFileBox,XmNcancelCallback,Cancel,brkFileBox);
  XtAddCallback(brkFileBox,XmNokCallback,LoadBrkOk,NULL);
  XtAddCallback(loadbrk,XmNactivateCallback,LoadBrk,NULL);

  /* CREATE SAVE BREAKPOINT BUTTON */
  savebrk=XtVaCreateManagedWidget(
    "savebrk",xmPushButtonWidgetClass,fileMenu,NULL);
  text=XmStringCreateLocalized("Save control function");
  XtVaSetValues(savebrk,XmNlabelString,text,NULL);

  brkSaveFileBox=XmCreateFileSelectionDialog(mainWindow,"brkSaveFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(brkSaveFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Save control function");
  XtVaSetValues(brkSaveFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);

  XtAddCallback(brkSaveFileBox,XmNcancelCallback,Cancel,brkSaveFileBox);
  XtAddCallback(brkSaveFileBox,XmNokCallback,SaveBrkOk,NULL);
  XtAddCallback(savebrk,XmNactivateCallback,SaveBrk,NULL);


  /* CREATE PLAY BUTTON */
  play=XtVaCreateManagedWidget(
    "play",xmPushButtonWidgetClass,fileMenu,NULL);
  text=XmStringCreateLocalized("Play");
  XtVaSetValues(play,XmNlabelString,text,NULL);

  playFileBox=XmCreateFileSelectionDialog(mainWindow,"playFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(playFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Play");
  XtVaSetValues(playFileBox,XmNdialogTitle,text,XmNautoUnmanage,False,
    XmNwidth,500,NULL);
  XtAddCallback(playFileBox,XmNcancelCallback,Cancel,playFileBox);
  XtAddCallback(playFileBox,XmNokCallback,PlayOk,NULL);
  XtAddCallback(play,XmNactivateCallback,Play,NULL);

  /* CREATE QUIT BUTTON */
  quit=XtVaCreateManagedWidget(
    "quit",xmPushButtonWidgetClass,fileMenu,
    XmNmnemonic,XStringToKeysym("Q"),XmNaccelerator,"Alt<Key>Q",
    XmNacceleratorText,XmStringCreateLocalized("Alt+Q"),NULL);
  text=XmStringCreateLocalized("Quit");
  XtVaSetValues(quit,XmNlabelString,text,NULL);
  XtAddCallback(quit,XmNactivateCallback,Quit,0);

  /* CREATE FILE OPENING WARNING */
  fileWarning=XmCreateWarningDialog(mainWindow,"fileWarning",NULL,0);
  temp=XmMessageBoxGetChild(fileWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(fileWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(fileWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open file");
  XtVaSetValues(fileWarning,XmNmessageString,text,NULL);

  /* CREATE FILE WRITE WARNING */
  writeWarning=XmCreateWarningDialog(mainWindow,"writeWarning",NULL,0);
  temp=XmMessageBoxGetChild(writeWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(writeWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(writeWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't write to file");
  XtVaSetValues(writeWarning,XmNmessageString,text,NULL);

  /* CREATE FILE SIZE WARNING */
  sizeWarning=XmCreateWarningDialog(mainWindow,"sizeWarning",NULL,0);
  temp=XmMessageBoxGetChild(sizeWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(sizeWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(sizeWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateLtoR(
    "Sorry, can't handle\nsoundfiles that large",
    XmFONTLIST_DEFAULT_TAG);
  XtVaSetValues(sizeWarning,XmNmessageString,text,NULL);

  create_transforms();
 
  /* CREATE EXPORT MENU */
  exportButton=XtVaCreateManagedWidget(
    "exportButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Export");
  XtVaSetValues(exportButton,XmNlabelString,text,NULL);
  exportMenu=XmCreatePulldownMenu(menuBar,"exportMenu",NULL,0);
  XtVaSetValues(exportButton,XmNsubMenuId,exportMenu,NULL);

  /* CREATE PARMERUD BUTTON */
  toparmerud=XtVaCreateManagedWidget(
    "toparmerud",xmPushButtonWidgetClass,exportMenu,NULL);
  text=XmStringCreateLocalized("Parmerud Csound scorefile");
  XtVaSetValues(toparmerud,XmNlabelString,text,NULL);

  toparmerudFileBox=XmCreateFileSelectionDialog(mainWindow,
    "toparmerudFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(toparmerudFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write Parmerud Csound score file");
  XtVaSetValues(toparmerudFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  parmerudRowCol=XtVaCreateManagedWidget("parmerudRowCol",xmRowColumnWidgetClass,
    toparmerudFileBox,NULL);
  text=XmStringCreateLocalized("Trigging threshold");
  parmL1=XtVaCreateManagedWidget("parmL1",xmLabelWidgetClass,
    parmerudRowCol,XmNlabelString,text,NULL); XtFree(text);
  parmT1=XtVaCreateManagedWidget("parmT1",
    xmTextFieldWidgetClass,parmerudRowCol,NULL);
  XtVaSetValues(parmT1,XmNwidth,100,XmNvalue,"10",NULL);
  text=XmStringCreateLocalized("Hysteresis time");
  parmL2=XtVaCreateManagedWidget("parmL2",xmLabelWidgetClass,
    parmerudRowCol,XmNlabelString,text,NULL); XtFree(text);
  parmT2=XtVaCreateManagedWidget("parmT2",
    xmTextFieldWidgetClass,parmerudRowCol,NULL);
  XtVaSetValues(parmT2,XmNwidth,100,XmNvalue,"1",NULL);
  text=XmStringCreateLocalized("Strings pr. octave");
  parmL3=XtVaCreateManagedWidget("parmL3",xmLabelWidgetClass,
    parmerudRowCol,XmNlabelString,text,NULL); XtFree(text);
  parmT3=XtVaCreateManagedWidget("parmT3",
    xmTextFieldWidgetClass,parmerudRowCol,NULL);
  XtVaSetValues(parmT3,XmNwidth,100,XmNvalue,"6",NULL);
  text=XmStringCreateLocalized("Resonance time");
  parmL4=XtVaCreateManagedWidget("parmL4",xmLabelWidgetClass,
    parmerudRowCol,XmNlabelString,text,NULL); XtFree(text);
  parmT4=XtVaCreateManagedWidget("parmT4",
    xmTextFieldWidgetClass,parmerudRowCol,NULL);
  XtVaSetValues(parmT4,XmNwidth,100,XmNvalue,"1",NULL);
  XtAddCallback(toparmerudFileBox,XmNcancelCallback,Cancel,toparmerudFileBox);
  XtAddCallback(toparmerudFileBox,XmNokCallback,ToparmerudOk,NULL);
  XtAddCallback(toparmerud,XmNactivateCallback,Toparmerud,NULL);


  /* CREATE PEAK BUTTON */
  topeak=XtVaCreateManagedWidget(
    "topeak",xmPushButtonWidgetClass,exportMenu,NULL);
  text=XmStringCreateLocalized("Spectral peaks");
  XtVaSetValues(topeak,XmNlabelString,text,NULL);

  topeakFileBox=XmCreateFileSelectionDialog(mainWindow,
    "topeakFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(topeakFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write spectral peaks");
  XtVaSetValues(topeakFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);

  XtAddCallback(topeakFileBox,XmNcancelCallback,Cancel,topeakFileBox);
  XtAddCallback(topeakFileBox,XmNokCallback,TopeakOk,NULL);
  XtAddCallback(topeak,XmNactivateCallback,Topeak,NULL);

  /* CREATE CSOUND BUTTON */
  tocsound=XtVaCreateManagedWidget(
    "tocsound",xmPushButtonWidgetClass,exportMenu,NULL);
  text=XmStringCreateLocalized("Csound scorefile");
  XtVaSetValues(tocsound,XmNlabelString,text,NULL);

  tocsoundFileBox=XmCreateFileSelectionDialog(mainWindow,
    "tocsoundFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(tocsoundFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write Csound score file");
  XtVaSetValues(tocsoundFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(tocsoundFileBox,XmNcancelCallback,Cancel,tocsoundFileBox);
  XtAddCallback(tocsoundFileBox,XmNokCallback,TocsoundOk,NULL);
  XtAddCallback(tocsound,XmNactivateCallback,Tocsound,NULL);

  /* CREATE CSOUND OPENING WARNING */
  csoundWarning=XmCreateWarningDialog(mainWindow,"csoundWarning",NULL,0);
  temp=XmMessageBoxGetChild(csoundWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(csoundWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(csoundWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open file");
  XtVaSetValues(csoundWarning,XmNmessageString,text,NULL);

  /* CREATE STF BUTTON */
  tostf=XtVaCreateManagedWidget(
    "tostf",xmPushButtonWidgetClass,exportMenu,NULL);
  text=XmStringCreateLocalized("Inventor visualization file");
  XtVaSetValues(tostf,XmNlabelString,text,NULL);

  tostfFileBox=XmCreateFileSelectionDialog(mainWindow,
    "tostfFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(tostfFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write Inventor visualization file");
  XtVaSetValues(tostfFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(tostfFileBox,XmNcancelCallback,Cancel,tostfFileBox);
  XtAddCallback(tostfFileBox,XmNokCallback,TostfOk,NULL);
  XtAddCallback(tostf,XmNactivateCallback,Tostf,NULL);

  /* CREATE STF OPENING WARNING */
  stfWarning=XmCreateWarningDialog(mainWindow,"stfWarning",NULL,0);
  temp=XmMessageBoxGetChild(stfWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(stfWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(stfWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open file");
  XtVaSetValues(stfWarning,XmNmessageString,text,NULL);

  /* CREATE TOLTAS BUTTON */
  toltas=XtVaCreateManagedWidget(
    "toltas",xmPushButtonWidgetClass,exportMenu,NULL);
  text=XmStringCreateLocalized("Single LTAS frame");
  XtVaSetValues(toltas,XmNlabelString,text,NULL);

  toltasFileBox=XmCreateFileSelectionDialog(mainWindow,
    "toltasFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(toltasFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Write single LTAS frame file");
  XtVaSetValues(toltasFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(toltasFileBox,XmNcancelCallback,Cancel,toltasFileBox);
  XtAddCallback(toltasFileBox,XmNokCallback,ToltasOk,NULL);
  XtAddCallback(toltas,XmNactivateCallback,Toltas,NULL);

  /* CREATE LTAS OPENING WARNING */
  ltasWarning=XmCreateWarningDialog(mainWindow,"ltasWarning",NULL,0);
  temp=XmMessageBoxGetChild(ltasWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(ltasWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Error!");
  XtVaSetValues(ltasWarning,XmNdialogTitle,text,
    XmNwidth,400,NULL);
  text=XmStringCreateSimple("Can't open file");
  XtVaSetValues(ltasWarning,XmNmessageString,text,NULL);


  /* CREATE SETTINGS MENU */

  settingsButton=XtVaCreateManagedWidget(
    "settingsButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Settings");
  XtVaSetValues(settingsButton,XmNlabelString,text,NULL);
  settingsMenu=XmCreatePulldownMenu(menuBar,"settingsMenu",NULL,0);
  XtVaSetValues(settingsButton,XmNsubMenuId,settingsMenu,NULL);


  /* CREATE ANALYSIS BUTTON */
  analyButton=XtVaCreateManagedWidget(
    "analyButton",xmPushButtonWidgetClass,settingsMenu,NULL);
  text=XmStringCreateLocalized("Analysis");
  XtVaSetValues(analyButton,XmNlabelString,text,NULL);

  analyForm=XmCreateBulletinBoardDialog(mainWindow,"analyForm",NULL,0);
  XtVaSetValues(analyForm, XmNwidth, 300, NULL);
  analyOkButton=XtVaCreateManagedWidget("analyOk",xmPushButtonWidgetClass,
    analyForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(analyOkButton,XmNlabelString,text,XmNx,30,XmNy,400,
    XmNwidth,80,NULL);
  analyCancelButton=XtVaCreateManagedWidget("analyCancel",
    xmPushButtonWidgetClass,analyForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(analyCancelButton,XmNlabelString,text,XmNx,190,XmNy,400,NULL);

  analyLabel1=XtVaCreateManagedWidget("analyLabel1",xmLabelWidgetClass,
    analyForm,NULL);
  text=XmStringCreateLocalized("Window function:");
  XtVaSetValues(analyLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  analyRadio=XtVaCreateManagedWidget("analyRadio",
    xmRowColumnWidgetClass,analyForm,NULL);
  XtVaSetValues(analyRadio,XmNy,50,XmNradioBehavior,True,
    XmNradioAlwaysOne,True,NULL);
  analyToggleHanning=XtVaCreateManagedWidget("analyToggleHanning",
    xmToggleButtonWidgetClass,analyRadio,NULL);
  text=XmStringCreateLocalized("Hanning");
  XtVaSetValues(analyToggleHanning,XmNlabelString,text,XmNset,True,NULL);
  analyToggleRect=XtVaCreateManagedWidget("analyToggleRect",
    xmToggleButtonWidgetClass,analyRadio,NULL);
  text=XmStringCreateLocalized("Rectangular (bad!)");
  XtVaSetValues(analyToggleRect,XmNlabelString,text,XmNset, False, NULL);
  analyToggleHamming=XtVaCreateManagedWidget("analyToggleHamming",
    xmToggleButtonWidgetClass,analyRadio,NULL);
  text=XmStringCreateLocalized("Hamming (bad!)");
  XtVaSetValues(analyToggleHamming,XmNlabelString,text,XmNset,False,NULL);
  analyToggleBartlett=XtVaCreateManagedWidget("analyToggleBartlett",
    xmToggleButtonWidgetClass,analyRadio,NULL);
  text=XmStringCreateLocalized("Bartlett");
  XtVaSetValues(analyToggleBartlett,XmNlabelString,text,XmNset,False,NULL);

  text=XmStringCreateLocalized("Overlap factor:");
  analyL1=XtVaCreateManagedWidget("analyL1",xmLabelWidgetClass,
    analyForm,XmNlabelString,text,XmNy,204,NULL); XtFree(text);
  analyTextfield=XtVaCreateManagedWidget("analyTextfield",
    xmTextFieldWidgetClass,analyForm,NULL);
  XtVaSetValues(analyTextfield,XmNwidth,60,XmNx,180,XmNy,200,
    XmNvalue,"8",NULL);

  analySep=XtVaCreateManagedWidget("analySep", xmSeparatorWidgetClass,
    analyForm, XmNorientation, XmHORIZONTAL, XmNy, 250, XmNwidth, 250, NULL);
  text=XmStringCreateLocalized("LPC poles:");
  analyL2=XtVaCreateManagedWidget("analyL2",xmLabelWidgetClass,
    analyForm,XmNlabelString,text,XmNy,284,NULL); XtFree(text);
  analyTextfield2=XtVaCreateManagedWidget("analyTextfield2",
    xmTextFieldWidgetClass,analyForm,NULL);
  XtVaSetValues(analyTextfield2,XmNwidth,60,XmNx,180,XmNy,280,
    XmNvalue,"50",NULL);

  stabilToggle=XtVaCreateManagedWidget("stabilToggle",
    xmToggleButtonWidgetClass, analyForm,NULL);
  text=XmStringCreateLocalized("Stabilize poles (slow)");
  XtVaSetValues(stabilToggle,XmNlabelString,text,XmNy, 330, NULL);

  text=XmStringCreateLocalized("Analysis settings");
  XtVaSetValues(analyForm,XmNdialogTitle,text,
    XmNcancelButton,analyCancelButton,
    XmNdefaultButton,analyOkButton,NULL);

  XtAddCallback(analyOkButton,XmNactivateCallback,AnalyOk,0);
  XtAddCallback(analyButton,XmNactivateCallback,Analy,0);


  /* CREATE RESYNT BUTTON */
  resyntButton=XtVaCreateManagedWidget(
    "resyntButton",xmPushButtonWidgetClass,settingsMenu,NULL);
  text=XmStringCreateLocalized("Resynthesis");
  XtVaSetValues(resyntButton,XmNlabelString,text,NULL);

  settingsForm=XmCreateBulletinBoardDialog(mainWindow,"settingsForm",NULL,0);
  settingsOkButton=XtVaCreateManagedWidget("settingsOk",xmPushButtonWidgetClass,
    settingsForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(settingsOkButton,XmNlabelString,text,XmNx,50,XmNy,260,
    XmNwidth,80,NULL);
  settingsCancelButton=XtVaCreateManagedWidget("settingsCancel",
    xmPushButtonWidgetClass,settingsForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(settingsCancelButton,XmNlabelString,text,XmNx,200,XmNy,260,NULL);

  settingsTextfield=XtVaCreateManagedWidget("settingsTextfield",
    xmTextFieldWidgetClass,settingsForm,NULL);
  XtVaSetValues(settingsTextfield,XmNwidth,100,XmNx,300,XmNy,10,
    XmNvalue,"1",NULL);
  settingsLabel1=XtVaCreateManagedWidget("settingsLabel1",xmLabelWidgetClass,
    settingsForm,NULL);
  text=XmStringCreateLocalized("Time stretch factor:");
  XtVaSetValues(settingsLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  settingsRadio=XtVaCreateManagedWidget("settingsRadio",
    xmRowColumnWidgetClass,settingsForm,NULL);
  XtVaSetValues(settingsRadio,XmNy,50,XmNradioBehavior,True,
    XmNradioAlwaysOne,True,NULL);
  settingsToggle1=XtVaCreateManagedWidget("settingsToggle1",
    xmToggleButtonWidgetClass,settingsRadio,NULL);
  text=XmStringCreateLocalized("IFFT resynthesis (low quality)");
  XtVaSetValues(settingsToggle1,XmNlabelString,text,XmNset,True,NULL);
  settingsToggle2=XtVaCreateManagedWidget("settingsToggle2",
    xmToggleButtonWidgetClass,settingsRadio,NULL);
  text=XmStringCreateLocalized("Additive resynthesis (good)");
  XtVaSetValues(settingsToggle2,XmNlabelString,text,NULL);
  
  settingsTextfield2=XtVaCreateManagedWidget("settingsTextfield2",
    xmTextFieldWidgetClass,settingsForm,NULL);
  XtVaSetValues(settingsTextfield2,XmNwidth,100,XmNx,300,XmNy,120,
    XmNvalue,"1", NULL);
  settingsLabel2=XtVaCreateManagedWidget("settingsLabel2",xmLabelWidgetClass,
    settingsForm,NULL);
  text=XmStringCreateLocalized("Pitch shift for additive synth:");
  XtVaSetValues(settingsLabel2,XmNlabelString,text,XmNx,0,XmNy,124,NULL);
  
  settingsTextfield3=XtVaCreateManagedWidget("settingsTextfield3",
    xmTextFieldWidgetClass,settingsForm,NULL);
  XtVaSetValues(settingsTextfield3,XmNwidth,100,XmNx,300,XmNy,160,
    XmNvalue,"0.000001", NULL);
  settingsLabel3=XtVaCreateManagedWidget("settingsLabel3",xmLabelWidgetClass,
    settingsForm,NULL);
  text=XmStringCreateLocalized("Additive synthesis threshold:");
  XtVaSetValues(settingsLabel3,XmNlabelString,text,XmNx,0,XmNy,164,NULL);

  settingsToggle3=XtVaCreateManagedWidget("settingsToggle3",
    xmToggleButtonWidgetClass,settingsForm,NULL);
  text=XmStringCreateLocalized("Keep formants for additive synthesis (needs LPC)");
  XtVaSetValues(settingsToggle3,XmNlabelString,text,XmNy,210,NULL);

  text=XmStringCreateLocalized("Resynthesis settings");
  XtVaSetValues(settingsForm,XmNdialogTitle,text,
    XmNcancelButton,settingsCancelButton,
    XmNdefaultButton,settingsOkButton,NULL);

  XtAddCallback(settingsOkButton,XmNactivateCallback,SettingsOk,0);
  XtAddCallback(resyntButton,XmNactivateCallback,Settings,0);

  /* CREATE DISPLAY BUTTON */
  displayButton=XtVaCreateManagedWidget(
    "displayButton",xmPushButtonWidgetClass,settingsMenu,NULL);
  text=XmStringCreateLocalized("Display");
  XtVaSetValues(displayButton,XmNlabelString,text,NULL);

  displayForm=XmCreateBulletinBoardDialog(mainWindow,"displayForm",NULL,0);
  displayOkButton=XtVaCreateManagedWidget("displayOk",xmPushButtonWidgetClass,
    displayForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(displayOkButton,XmNlabelString,text,XmNx,50,XmNy,290,
    XmNwidth,80,NULL);
  displayCancelButton=XtVaCreateManagedWidget("displayCancel",
    xmPushButtonWidgetClass,displayForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(displayCancelButton,XmNlabelString,text,XmNx,200,XmNy,290,NULL);

  colorRadio=XtVaCreateManagedWidget("colorRadio",
    xmRowColumnWidgetClass,displayForm,NULL);
  XtVaSetValues(colorRadio,XmNy,100,XmNradioBehavior,True,
    XmNradioAlwaysOne,True,NULL);
  displayToggleGrey=XtVaCreateManagedWidget("displayToggleGrey",
    xmToggleButtonWidgetClass,colorRadio,NULL);
  text=XmStringCreateLocalized("Grey colors");
  XtVaSetValues(displayToggleGrey,XmNlabelString,text,XmNset,True,NULL);
  displayToggleHot=XtVaCreateManagedWidget("displayToggleHot",
    xmToggleButtonWidgetClass,colorRadio,NULL);
  text=XmStringCreateLocalized("Hot colors");
  XtVaSetValues(displayToggleHot,XmNlabelString,text,NULL);
  displayToggleCold=XtVaCreateManagedWidget("displayToggleCold",
    xmToggleButtonWidgetClass,colorRadio,NULL);
  text=XmStringCreateLocalized("Cool colors");
  XtVaSetValues(displayToggleCold,XmNlabelString,text,NULL);

  displayToggleFunc=XtVaCreateManagedWidget("displayToggleFunc",
    xmToggleButtonWidgetClass,displayForm,NULL);
  text=XmStringCreateLocalized("Show control function");
  XtVaSetValues(displayToggleFunc,XmNlabelString,text,NULL);

  displayToggleScale=XtVaCreateManagedWidget("displayToggleScale",
    xmToggleButtonWidgetClass,displayForm,NULL);
  text=XmStringCreateLocalized("Show grid");
  XtVaSetValues(displayToggleScale,XmNlabelString,text,XmNy,45,NULL);

  displayToggleSpots=XtVaCreateManagedWidget("displayToggleSpots",
    xmToggleButtonWidgetClass,displayForm,NULL);
  text=XmStringCreateLocalized("Show paint");
  XtVaSetValues(displayToggleSpots,XmNlabelString,text,XmNy,70,NULL);
  
  displayGainText=XtVaCreateManagedWidget("displayGainText",
    xmTextFieldWidgetClass,displayForm,NULL);
  XtVaSetValues(displayGainText,XmNwidth,100,XmNx,300,XmNy,200,
    XmNvalue,"1",NULL);
  displayGainLabel=XtVaCreateManagedWidget("displayGainLabel",
    xmLabelWidgetClass, displayForm,NULL);
  text=XmStringCreateLocalized("Display gain factor:");
  XtVaSetValues(displayGainLabel,XmNlabelString,text,XmNx,0,XmNy,204,NULL);

  displayScaleText=XtVaCreateManagedWidget("displayScaleText",
    xmTextFieldWidgetClass,displayForm,NULL);
  XtVaSetValues(displayScaleText,XmNwidth,100,XmNx,300,XmNy,240,
    XmNvalue,"1",NULL);
  displayScaleLabel=XtVaCreateManagedWidget("displayScaleLabel",
    xmLabelWidgetClass, displayForm,NULL);
  text=XmStringCreateLocalized("Frequency zoom factor:");
  XtVaSetValues(displayScaleLabel,XmNlabelString,text,XmNx,0,XmNy,244,NULL);

  text=XmStringCreateLocalized("Display settings");
  XtVaSetValues(displayForm,XmNdialogTitle,text,
    XmNcancelButton,displayCancelButton,
    XmNdefaultButton,displayOkButton,NULL);

  XtAddCallback(displayOkButton,XmNactivateCallback,DisplayOk,0);
  XtAddCallback(displayButton,XmNactivateCallback,Displaymenu,0);


  /* CREATE GRID BUTTON */
  gridButton=XtVaCreateManagedWidget(
    "gridButton",xmPushButtonWidgetClass,settingsMenu,NULL);
  text=XmStringCreateLocalized("Pitch grid");
  XtVaSetValues(gridButton,XmNlabelString,text,NULL);

  gridForm=XmCreateBulletinBoardDialog(mainWindow,"gridForm",NULL,0);
  gridOkButton=XtVaCreateManagedWidget("gridOk",xmPushButtonWidgetClass,
    gridForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(gridOkButton,XmNlabelString,text,XmNx,50,XmNy,400,
    XmNwidth,80,NULL);
  gridCancelButton=XtVaCreateManagedWidget("gridCancel",
    xmPushButtonWidgetClass,gridForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(gridCancelButton,XmNlabelString,text,XmNx,200,XmNy,400,NULL);

  gridTextfield=XtVaCreateManagedWidget("gridTextfield",
    xmTextFieldWidgetClass,gridForm,NULL);
  XtVaSetValues(gridTextfield,XmNwidth,100,XmNx,200,XmNy,10,
    XmNvalue,"50",NULL);
  gridLabel1=XtVaCreateManagedWidget("gridLabel1",xmLabelWidgetClass,
    gridForm,NULL);
  text=XmStringCreateLocalized("Grid Starting Frequency");
  XtVaSetValues(gridLabel1,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  gridRadio=XtVaCreateManagedWidget("gridRadio",
    xmRowColumnWidgetClass,gridForm,NULL);
  XtVaSetValues(gridRadio,XmNy,50,XmNradioBehavior,True,
    XmNradioAlwaysOne,True,NULL);
  gridToggleMajor=XtVaCreateManagedWidget("gridToggleMajor",
    xmToggleButtonWidgetClass,gridRadio,NULL);
  text=XmStringCreateLocalized("Major");
  XtVaSetValues(gridToggleMajor,XmNlabelString,text,NULL);
  gridToggleMinor=XtVaCreateManagedWidget("gridToggleMinor",
    xmToggleButtonWidgetClass,gridRadio,NULL);
  text=XmStringCreateLocalized("Harmonic Minor");
  XtVaSetValues(gridToggleMinor,XmNlabelString,text,NULL);
  gridTogglePenta=XtVaCreateManagedWidget("gridTogglePenta",
    xmToggleButtonWidgetClass,gridRadio,NULL);
  text=XmStringCreateLocalized("Pentatonic");
  XtVaSetValues(gridTogglePenta,XmNlabelString,text,NULL);
  gridToggleChroma=XtVaCreateManagedWidget("gridToggleChroma",
    xmToggleButtonWidgetClass,gridRadio,NULL);
  text=XmStringCreateLocalized("Chromatic");
  XtVaSetValues(gridToggleChroma,XmNlabelString,text,NULL);
  gridToggleWhole=XtVaCreateManagedWidget("gridToggleWhole",
    xmToggleButtonWidgetClass,gridRadio,NULL);
  text=XmStringCreateLocalized("Whole-tone");
  XtVaSetValues(gridToggleWhole,XmNlabelString,text,NULL);
  gridToggleHarm=XtVaCreateManagedWidget("gridToggleHarm",
    xmToggleButtonWidgetClass,gridRadio,NULL);
  text=XmStringCreateLocalized("Harmonic Series");
  XtVaSetValues(gridToggleHarm,XmNlabelString,text,NULL);
  gridToggleEvenharm=XtVaCreateManagedWidget("gridToggleEvenharm",
    xmToggleButtonWidgetClass,gridRadio,NULL);
  text=XmStringCreateLocalized("Even-harmonic Series");
  XtVaSetValues(gridToggleEvenharm,XmNlabelString,text,NULL);
  gridToggleOddharm=XtVaCreateManagedWidget("gridToggleOddharm",
    xmToggleButtonWidgetClass,gridRadio,NULL);
  text=XmStringCreateLocalized("Odd-harmonic Series");
  XtVaSetValues(gridToggleOddharm,XmNlabelString,text,NULL);

/*  gridToggleFile=XtVaCreateManagedWidget("gridToggleFile",
    xmToggleButtonWidgetClass,gridRadio,NULL);
  text=XmStringCreateLocalized("Read selected frequency file'");
  XtVaSetValues(gridToggleFile,XmNlabelString,text,XmNset,True,NULL); */

 /* CREATE LOAD FREQUENCIES FILE BUTTON */
  freqfile=XtVaCreateManagedWidget(
    "freqfile",xmToggleButtonWidgetClass,gridRadio,NULL);
  text=XmStringCreateLocalized("Load pitch grid frequencies file");
  XtVaSetValues(freqfile,XmNlabelString,text,XmNset,True,NULL);

  freqFileBox=XmCreateFileSelectionDialog(mainWindow,"freqFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(freqFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  text=XmStringCreateSimple("Load pitch grid frequencies file");
  XtVaSetValues(freqFileBox,XmNdialogTitle,text,
    XmNwidth,500,NULL);
  XtAddCallback(freqFileBox,XmNcancelCallback,Cancel,freqFileBox);
  XtAddCallback(freqFileBox,XmNokCallback,FreqfileOk,NULL);

  text=XmStringCreateLocalized("Pitch grid");
  XtVaSetValues(gridForm,XmNdialogTitle,text,
    XmNcancelButton,gridCancelButton,XmNdefaultButton,gridOkButton,NULL);

  XtAddCallback(gridButton,XmNactivateCallback,Gridmenu,0);
  XtAddCallback(gridOkButton,XmNactivateCallback,GridmenuOk,0);



  /* CREATE SPOTS MENU */

  spotsButton=XtVaCreateManagedWidget(
    "spotsButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Paint");
  XtVaSetValues(spotsButton,XmNlabelString,text,NULL);
  spotsMenu=XmCreatePulldownMenu(menuBar,"spotsMenu",NULL,0);
  XtVaSetValues(spotsButton,XmNsubMenuId,spotsMenu,NULL);

  /* CREATE RESTART BUTTON */
  spotsRestart=XtVaCreateManagedWidget(
    "restartButton",xmPushButtonWidgetClass,spotsMenu,NULL);
  text=XmStringCreateLocalized("Restart");
  XtVaSetValues(spotsRestart,XmNlabelString,text,NULL);
  XtAddCallback(spotsRestart,XmNactivateCallback,SpotsRestart,0);

  /* CREATE KEEP BUTTON */
  spotsKeep=XtVaCreateManagedWidget(
    "keepButton",xmPushButtonWidgetClass,spotsMenu,NULL);
  text=XmStringCreateLocalized("Keep");
  XtVaSetValues(spotsKeep,XmNlabelString,text,NULL);
  XtAddCallback(spotsKeep,XmNactivateCallback,SpotsKeep,0);

  /* CREATE REMOVE BUTTON */
  spotsRemove=XtVaCreateManagedWidget(
    "removeButton",xmPushButtonWidgetClass,spotsMenu,NULL);
  text=XmStringCreateLocalized("Remove");
  XtVaSetValues(spotsRemove,XmNlabelString,text,NULL);
  XtAddCallback(spotsRemove,XmNactivateCallback,SpotsRemove,0);


  /* CREATE EXTRACT MENU */
  extractButton=XtVaCreateManagedWidget(
    "extractButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Extract");
  XtVaSetValues(extractButton,XmNlabelString,text,NULL);
  extractMenu=XmCreatePulldownMenu(menuBar,"extractMenu",NULL,0);
  XtVaSetValues(extractButton,XmNsubMenuId,extractMenu,NULL);

  /* CREATE AMPLITUDE BUTTON */
  amplitude=XtVaCreateManagedWidget(
    "amplitude",xmPushButtonWidgetClass,extractMenu,NULL);
  text=XmStringCreateLocalized("Amplitude");
  XtVaSetValues(amplitude,XmNlabelString,text,NULL);
  XtAddCallback(amplitude,XmNactivateCallback,Amplitude,0);

  /* CREATE PITCH BUTTON */
  pitch=XtVaCreateManagedWidget(
    "pitch",xmPushButtonWidgetClass,extractMenu,NULL);
  text=XmStringCreateLocalized("Pitch");
  XtVaSetValues(pitch,XmNlabelString,text,NULL);
  XtAddCallback(pitch,XmNactivateCallback,Pitch,0);

  /* CREATE CENTROID BUTTON */
  centroid=XtVaCreateManagedWidget(
    "centroid",xmPushButtonWidgetClass,extractMenu,NULL);
  text=XmStringCreateLocalized("Centroid");
  XtVaSetValues(centroid,XmNlabelString,text,NULL);
  XtAddCallback(centroid,XmNactivateCallback,Centroid,0);



  /* CREATE GRAPH MENU */
  graphButton=XtVaCreateManagedWidget(
    "graphButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Graph");
  XtVaSetValues(graphButton,XmNlabelString,text,NULL);
  graphMenu=XmCreatePulldownMenu(menuBar,"graphMenu",NULL,0);
  XtVaSetValues(graphButton,XmNsubMenuId,graphMenu,NULL);

  /* CREATE SCALE BUTTON */
  grScale=XtVaCreateManagedWidget(
    "grScale",xmPushButtonWidgetClass,graphMenu,NULL);
  text=XmStringCreateLocalized("Scale");
  XtVaSetValues(grScale,XmNlabelString,text,NULL);

  grScaleForm=XmCreateBulletinBoardDialog(mainWindow,"grScaleForm",NULL,0);
  grScaleOkButton=XtVaCreateManagedWidget("grScaleOk",xmPushButtonWidgetClass,
    grScaleForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(grScaleOkButton,XmNlabelString,text,XmNx,50,XmNy,100,
    XmNwidth,80,NULL);
  grScaleCancelButton=XtVaCreateManagedWidget("grScaleCancel",
    xmPushButtonWidgetClass,grScaleForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(grScaleCancelButton,XmNlabelString,text,XmNx,200,XmNy,100,NULL);

  grScaleTextfield=XtVaCreateManagedWidget("grScaleTextfield",
    xmTextFieldWidgetClass,grScaleForm,NULL);
  XtVaSetValues(grScaleTextfield,XmNwidth,100,XmNx,200,XmNy,10,
    XmNvalue,"0",NULL);
  grScaleLabel=XtVaCreateManagedWidget("grScaleLabel",xmLabelWidgetClass,
    grScaleForm,NULL);
  text=XmStringCreateLocalized("Multiplication factor:");
  XtVaSetValues(grScaleLabel,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  text=XmStringCreateLocalized("Graph scale");
  XtVaSetValues(grScaleForm,XmNdialogTitle,text,
    XmNcancelButton,grScaleCancelButton,
    XmNdefaultButton,grScaleOkButton,NULL);

  XtAddCallback(grScaleOkButton,XmNactivateCallback,GrScaleOk,NULL);
  XtAddCallback(grScale,XmNactivateCallback,GrScale,0);


  /* CREATE MOVE BUTTON */
  grMove=XtVaCreateManagedWidget(
    "grMove",xmPushButtonWidgetClass,graphMenu,NULL);
  text=XmStringCreateLocalized("Add");
  XtVaSetValues(grMove,XmNlabelString,text,NULL);

  grMoveForm=XmCreateBulletinBoardDialog(mainWindow,"grMoveForm",NULL,0);
  grMoveOkButton=XtVaCreateManagedWidget("grMoveOk",xmPushButtonWidgetClass,
    grMoveForm,NULL);
  text=XmStringCreateLocalized("Ok");
  XtVaSetValues(grMoveOkButton,XmNlabelString,text,XmNx,50,XmNy,100,
    XmNwidth,80,NULL);
  grMoveCancelButton=XtVaCreateManagedWidget("grMoveCancel",
    xmPushButtonWidgetClass,grMoveForm,NULL);
  text=XmStringCreateLocalized("Cancel");
  XtVaSetValues(grMoveCancelButton,XmNlabelString,text,XmNx,200,XmNy,100,NULL);

  grMoveTextfield=XtVaCreateManagedWidget("grMoveTextfield",
    xmTextFieldWidgetClass,grMoveForm,NULL);
  XtVaSetValues(grMoveTextfield,XmNwidth,100,XmNx,200,XmNy,10,
    XmNvalue,"0",NULL);
  grMoveLabel=XtVaCreateManagedWidget("grMoveLabel",xmLabelWidgetClass,
    grMoveForm,NULL);
  text=XmStringCreateLocalized("Addition value:");
  XtVaSetValues(grMoveLabel,XmNlabelString,text,XmNx,0,XmNy,14,NULL);

  text=XmStringCreateLocalized("Graph add");
  XtVaSetValues(grMoveForm,XmNdialogTitle,text,
    XmNcancelButton,grMoveCancelButton,
    XmNdefaultButton,grMoveOkButton,NULL);

  XtAddCallback(grMoveOkButton,XmNactivateCallback,GrMoveOk,NULL);
  XtAddCallback(grMove,XmNactivateCallback,GrMove,0);


  /* CREATE NONLIN BUTTON */
  grNonlin=XtVaCreateManagedWidget(
    "grNonlin",xmPushButtonWidgetClass,graphMenu,NULL);
  text=XmStringCreateLocalized("Nonlinear");
  XtVaSetValues(grNonlin,XmNlabelString,text,NULL);
  XtVaSetValues(grNonlin, XmNsensitive, False, NULL);
  XtAddCallback(grNonlin,XmNactivateCallback,GrNonlin,0);

  /* CREATE CLEAR BUTTON */
  grClear=XtVaCreateManagedWidget(
    "grClear",xmPushButtonWidgetClass,graphMenu,NULL);
  text=XmStringCreateLocalized("Clear");
  XtVaSetValues(grClear,XmNlabelString,text,NULL);
  XtAddCallback(grClear,XmNactivateCallback,GrClear,0);

  /* CREATE INVERT BUTTON */
  grInvert=XtVaCreateManagedWidget(
    "grInvert",xmPushButtonWidgetClass,graphMenu,NULL);
  text=XmStringCreateLocalized("Invert");
  XtVaSetValues(grInvert,XmNlabelString,text,NULL);
  XtAddCallback(grInvert,XmNactivateCallback,GrInvert,0);

  /* CREATE RETRO BUTTON */
  grRetro=XtVaCreateManagedWidget(
    "grRetro",xmPushButtonWidgetClass,graphMenu,NULL);
  text=XmStringCreateLocalized("Retrograde");
  XtVaSetValues(grRetro, XmNsensitive, False, NULL);
  XtVaSetValues(grRetro, XmNlabelString,text,NULL);
  XtAddCallback(grRetro, XmNactivateCallback,GrRetro,0);


  /* CREATE LPC MENU */

  lpcButton=XtVaCreateManagedWidget(
    "lpcButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("LPC");
  XtVaSetValues(lpcButton,XmNlabelString,text,NULL);
  lpcMenu=XmCreatePulldownMenu(menuBar,"lpcMenu",NULL,0);
  XtVaSetValues(lpcButton,XmNsubMenuId,lpcMenu,NULL);

  /* CREATE LPCMULT BUTTON */
  lpcMult=XtVaCreateManagedWidget(
    "lpcMult",xmPushButtonWidgetClass,lpcMenu,NULL);
  text=XmStringCreateLocalized("Filter current spectrum");
  XtVaSetValues(lpcMult,XmNlabelString,text,NULL);
  XtAddCallback(lpcMult,XmNactivateCallback,Lpcmult,0);

  /* CREATE INVERT BUTTON */
  lpcInvert=XtVaCreateManagedWidget(
    "lpcInvert",xmPushButtonWidgetClass,lpcMenu,NULL);
  text=XmStringCreateLocalized("Inverse-filter spectrum (whiten)");
  XtVaSetValues(lpcInvert,XmNlabelString,text,NULL);
  XtAddCallback(lpcInvert,XmNactivateCallback,Lpcinvert,0);


  /* CREATE HELP THINGS */
  helpButton=XtVaCreateManagedWidget(
    "helpButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  text=XmStringCreateLocalized("Help");
  XtVaSetValues(helpButton,XmNlabelString,text,NULL);
  XtVaSetValues(menuBar,XmNmenuHelpWidget,helpButton,NULL);

  helpMenu=XmCreatePulldownMenu(menuBar,"helpMenu",NULL,0);
  help=XtVaCreateManagedWidget(
    "helpMenu",xmPushButtonWidgetClass,helpMenu,NULL);
  text=XmStringCreateLocalized("About");
  XtVaSetValues(help,XmNlabelString,text,NULL);  
  XtVaSetValues(helpButton,XmNsubMenuId,helpMenu,NULL);

  helpBox=XmCreateMessageDialog(help,"helpBox",NULL,0);
  text=XmStringCreateSimple("Help");
  XtVaSetValues(helpBox,XmNdialogTitle,text,NULL);
  text=XmStringCreateLtoR(
    "CERES is a graphical sound editor working\n\
in the frequency domain.\n\n\
The phase vocoder core is based on code from\n\
F.Richard Moore: Elements of Computer Music\n\n\
Version 0.14,   Oyvind Hammer 1999\n\
Please send your comments to oyvindha@notam.uio.no.",
     XmFONTLIST_DEFAULT_TAG);
  XtVaSetValues(helpBox,XmNmessageString,text,XmNwidth,600,
    XmNx,100,XmNy,200,NULL);

  temp=XmMessageBoxGetChild(helpBox,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(helpBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  XtAddCallback(help,XmNactivateCallback,ShowHelp,helpBox);
  progressDialog=XmCreateWorkingDialog(mainWindow,"progressDialog",NULL,0);
  temp=XmMessageBoxGetChild(progressDialog,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(progressDialog,XmDIALOG_OK_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(progressDialog,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp); 
  progressScale=XtVaCreateManagedWidget("progressScale",
    xmScaleWidgetClass, progressDialog, NULL);
  text=XmStringCreateLocalized("Working");
  XtVaSetValues(progressDialog,XmNdialogTitle,text,NULL,NULL);
  text=XmStringCreateLocalized("Please wait ...");
  XtVaSetValues(progressDialog,XmNmessageString,text,
    XmNorientation, XmHORIZONTAL, NULL,NULL);

  flush_all_underflows_to_zero();
  XtRealizeWidget(topLevel);
  setgreycolors(); RedrawWin();
  makewindows(Wanal,  Wsyn,  Nw,  N,  I);
  makemajor();
  if (argc==3) loadana(argv[2]);
  
  #ifdef LINUX
  initialiseAudio();
  #endif  

  XtAppMainLoop(app_context);
  
  #ifdef LINUX
  finishAudio();
  #endif  
  
}
