/* MIX: A soundfile mixer for SGI */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <fcntl.h>
#include <math.h>
#include <malloc.h>
#include <string.h>


#ifdef HAVE_CONFIG_H
#include "../config.h"
#else
#define SGI
#include <audiofile.h>
#include <audio.h>
#endif

#ifdef SGI
#include <sys/fpu.h>
#include <dmedia/midi.h>
#include <bstring.h>
#include <sys/schedctl.h>
#endif

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/prctl.h>
#include <stropts.h>
#include <poll.h>
#include <limits.h>

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Display.h>
#include <Xm/RowColumn.h>
#include <Xm/Frame.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/CascadeB.h>
#include <Xm/MessageB.h>
#include <Xm/FileSB.h>
#include <Xm/DrawingA.h>
#include <Xm/Scale.h>
#include <Xm/SelectioB.h>
#include <Xm/BulletinB.h>
#include <Xm/Form.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/Label.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/ScrollBar.h>
#include <Xm/Separator.h>
#include <Xm/List.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>

#include <Matrix.h>
#include "pixmaps.h"


#define MAXLYD 3000	/* Max number of sounds */
#define MAXLEN 1920.	/* Max length in secs (larger -> slower and eats memory) */
                        /* Must be 960., 1920. or 3840. */
#define GRAPHSAMPLES 91000  /* Set to 46000, 91000 or 181000 depending on MAXLEN  */
#define MAXNODE 2001	/* Nodes in a graph */
#define MAXCLIENTS 64	/* Networking things */
#define MAXSENDTO 8
#define PORTNUM 7777

#define WAITING 0
#define MIXING 1
#define FINISHED 2
#define HIGHPASS 1
#define LOWPASS 2
#define BANDPASS 3
#define BANDSTOP 4

#define PI 3.141592654f
#define TWOPI 6.2831853f

#define Eecho 1
#define Echorus 2
#define Eharmonizer 3
#define Ering 4
#define Efilter 5
#define Ereverb 6
#define Efiltertype 7
#define Eecho_time 8
#define Eecho_feedback 9
#define Echorus_speed 10
#define Echorus_depth 11
#define Eharmonizer_interval 12
#define Ering_frequency 13
#define Efilter_freq 14
#define Efilter_Q 15
#define Ereverb_time 16
#define Ecompress 17
#define Ecompress_A 18
#define Equadpos 19
#define Esync 20
#define Eoctave 21
#define Ecsound 22

#define LSIZE 100
#define FLUSH_LIST(list, count, GC)        \
          XDrawSegments(theDisplay,bitmap,GC,list,count);   \
        count=0

#define ADD_LINE(xx1, yy1, xx2, yy2, list, count, GC) \
      list[count].x1 = xx1;     \
      list[count].y1 = yy1;     \
      list[count].x2 = xx2;     \
      list[count++].y2 = yy2;     \
      if (count==LSIZE) { \
              FLUSH_LIST(list,count,GC);    \
      }       

#define XtNwwwbrowser "wwwBrowser"
#define XtCwwwbrowser "WwwBrowser"
#define XtNwwwfile    "wwwFile"
#define XtCwwwfile    "WwwFile"
#define XtNprocessmenufile "processMenuFile"
#define XtCprocessmenufile "ProcessMenuFile"
#define XtNsoundeditor "soundEditor"
#define XtCsoundeditor "SoundEditor"

typedef struct {
  char *www_browser;
  char *www_file;
  char *processmenu_file;
  char *soundeditor;
} AppData;
AppData app_data;

static XtResource resources[] = {
  { XtNwwwbrowser, XtCwwwbrowser, XmRString, sizeof(char*),
    XtOffsetOf(AppData, www_browser), XmRImmediate, NULL },
  { XtNwwwfile, XtCwwwfile, XmRString, sizeof(char*),
    XtOffsetOf(AppData, www_file), XmRImmediate, NULL },
  { XtNprocessmenufile, XtCprocessmenufile, XmRString, sizeof(char*),
    XtOffsetOf(AppData, processmenu_file), XmRImmediate, NULL },
  { XtNsoundeditor, XtCsoundeditor, XmRString, sizeof(char*),
    XtOffsetOf(AppData, soundeditor), XmRImmediate, NULL },
};

void MakeSoundWidgets(int);
void MakeEventWidgets(int);
void RedrawWin(void);
void RedrawTrack(int);
void newsamp(int);
void readcontrol(int);
void backup(void);

Widget loadFileBox, saveFileBox, playFileBox, loadmix, loadmixBox, loadmRowCol,
  loadmixToggle, loadmixtext, loadmixlabel;
Widget loadtextchild, infoRowCol, infoL1, infoL2, infoL3,
  trackR, trackToggles[10], trackL, trackL2, trackT, trackS, lPlay;
Widget sketchpad, topLevel, mainWindow, menuBar, frame, largeBB, dw;
Widget fileWarning, connectWarning, writeWarning, srateWarning, clipWarning,
  slavefailWarning, bitsWarning;
Widget horiscroll, savemix, savemixBox, savemixToggle;
Widget fileButton, fileMenu, quit, load, save, play, refresh, new,
  help, helpButton, helpMenu, helpBox, showcaseB, hyperB, temp;
Widget meditButton, meditMenu, undo, cut, copy, paste, insert;
Widget toolsButton, toolsMenu, to_machine,
  to_srate, to_edit, to_print, printFileBox, to_inventor;
Widget effectsButton, effectsMenu, effects1, effects1BB;
Widget effects1OkButton, eff1RadLabel, eff1RadRing, eff1RadChor,
  eff1RadEcho, eff1RadHarm, eff1RadFilter;
Widget effects1RMfreq, eff1RMfreqLabel, eff1CHspeed, eff1CHspeedLabel,
  eff1CHdepth, eff1CHdepthLabel,
  eff1ECtimeLabel, eff1ECtime, eff1ECfeed, eff1ECfeedLabel,
  eff1FLTcenterLabel, eff1FLTcenter, eff1FLTcenterT, eff1FLTQLabel, eff1FLTQ,
  eff1FLTtypeRadio, eff1FLTlowpass, eff1FLThipass, eff1FLTbandpass,
  eff1FLTbandstop;
Widget eff1HAinter, eff1HAinterLabel, eff1sep1, eff1sep2, eff1sep3;
Widget effects2, effects2BB, effects2OkButton, eff2RadLabel;
Widget eff2RadReverb, eff2RadCompress, eff2RadOctave;
Widget eff2RVtime, eff2RVtimeLabel, eff2CMA, eff2CMALabel,
  eff2sep1, eff2sep2, eff2sep3;
Widget settingsButton, settingsMenu, displayButton, displayBB,
  dispgridToggle, displinToggle, dispnumToggle, dispscrollToggle,
  dispfastdrawToggle,
  displayOkButton, colorRadio, colorToggleWhite, colorToggleBlue, dispallToggle;
Widget audioButton, audioBB, audioOkButton, srateRadio, srate8, srate11025,
  srate16, srate2205, srate32, srate441, srate48, srateLabel,
  buffText, buffLabel, linvolToggle, nchnlsRadio, nchnls1, nchnls2, nchnls4,
  nchnlsLabel;
Widget recordButton, recordBox, recRowCol, monitorToggle, recstereoToggle;
Widget graphMenu, graphButton, showList, nodDelete;
Widget regionMenu, regionButton, newRegion;
Widget grpMenu, grpButton, grpDelete, grpUngroupall, grpGroupall,
  grpDuplicate, grpMute;
Widget eventMenu, eventButton, e_echo, e_chorus, e_harmonizer, e_ring,
  e_filter, e_reverb, e_echo_time, e_echo_feedback, e_chorus_speed,
  e_chorus_depth, e_harmonizer_interval, e_ring_frequency, e_filter_freq,
  e_filter_Q, e_reverb_time, e_compress, e_compress_A, e_octave,
  e_quadpos, e_sync, e_csound, e_filtertype;
Widget trackBB, zoomBB, zoominB, zoomoutB, posL;
Widget playBB, playB, fastB, stopB, recordB;
Widget soundMatrix, soverv, sovervBB, sovervOkButton, sovervSize;
Widget eventMatrix, eoverv, eovervBB, eovervOkButton;
Widget sendtoBB, sendtoButton, sendtoToggle[MAXSENDTO], sendtoText[MAXSENDTO],
  sendtoOkButton, sendtoWarning, sendtoTogLabel, sendtoTextLabel, sendtoAccept;
Widget progressDialog, progressScale;
Widget timeButton, timeBB, timeRadio, timeToggleSec, timeToggleBeats,
  timeBPMlabel, timeBPMtext, timeBPBlabel, timeBPBtext, timeOkButton;
Widget machineBB, machineQuantLabel, machineQuantRadio,
  machine1, machine1d2, machine1d4, machine1d8, machine1d16, machine1d3,
  machineQuantNow, machineAutoQuant, machineTimeLabel, machineTimeText,
  machineTimeApply, machineOkButton, machineSep;
Widget newRegionBB, newreginfoL1, newreginfoL2, newreginfoL3,
  newregR, newregToggles[10], newregL, newregL2, newregT, newregS,
  newregOkButton, newregRowCol;

XmFontList smallfontlist, largefontlist;
XmFontListEntry thin10p, thin14p;

String rowlabels[1]={"0"}, rowlabels2[1]={"0"};
String Enames[23]={"nil", "echo", "chorus", "harmonizer", "ring",
  "filter", "reverb", "filtertype", "echo_time", "echo_feedback",
  "chorus_speed", "chorus_depth", "harmonizer_interval", "ring_frequency",
  "filter_freq", "filter_Q", "reverb_time", "compress", "compress_A",
  "quadpos", "sync", "octave", "csound" };
String Edefaults[23]={"nil", "0", "0", "0", "0",
  "0", "0", "lowpass", "0.5", "0.2",
  "0.2", "0.05", "4.0", "500.",
  "500.", "10.", "5.", "0", "0.", "1", "0.", "0", "" };
char processStrings[30][100];

Boolean fileDirty = FALSE;

AFfilehandle infile, outfile, lPlayFilehandle;
AFfilesetup in_AFsetup, out_AFsetup, lPlayFilesetup;
short int drbuf[800000];

int samps_per_frame;
long compression=AF_COMPRESSION_NONE,  filefmt=AF_FILE_AIFF;
long framecnt;
int theheight, num_mixed, num_total, events_total, realtiming=0, fasting=0,
  accepting=0, recording=0, saving=0;
int bits_per_samp=16, samp_type=AF_SAMPFMT_TWOSCOMP;
int marking=0, regmarking=0, marked=0, regmarked=0, zoomtrack=0,
  playing=-1, ldplaying=0;
double mstart=0., regmstart=0., mstop=MAXLEN, regmstop=MAXLEN, pastelen;

short int *framebuff, *mixbuff, *recordbuff;
int *delay, *delay2, *delay3, *rvdel1, *rvdel2, *rvdel3, *rvdel4, *rvdel5,
  *auxbuff, *outbuff;

XGCValues values;
Window theRoot, theWindow;
XtAppContext app_context;
Colormap theCmap;
Display *theDisplay;
int screen;
Pixmap bitmap, mutepixmap, nomutepixmap, playpixmap, noplaypixmap,
  fastpixmap, nofastpixmap, backpixmap, frontpixmap, auxpixmap, noauxpixmap,
  volpixmap, novolpixmap, panpixmap, nopanpixmap, stoppixmap, nostoppixmap,
  mixiconpixmap, varpixmap, varselpixmap, recordpixmap, norecordpixmap,
  parpixmap, noparpixmap;
XColor red, blue, yellow, green, black, white, skyblue, brown, gray, purple,
  markcolor;
XColor backcolor;
GC showGC, bitGC, markerGC;

int farge=0, pasteOk=0;
int Srate=44100;
double sec_pr_scr=240., leftsec=0., graboffset, regoffset;
int grabnum=-1, evgrabnum=-1, regnum=-1, xm=-999, ym, xold, numgrouped;
int selnum, seltrack, seltype, dirtytrack, dirtytype;
char streng[150];
char newsnd_name[150], recordname[150];
int newsnd_trk, newsnd_lop, newsnd_loc, newsnd_num, newenum;
int phase3, delpt, delpt2, delpt3;
int rvpt1, rvpt2, rvpt3, rvpt4, rvpt5;
float harm[4096], phase, phase2;
int sintab[4096], d1, d2;
double cdel, newsnd_start;

#ifdef SGI
MDport MIDIport;
#endif
int quartercount, havemidi;
double MIDItime;

int server_socket, client_sockets[MAXCLIENTS];
struct timeval timeout;
struct sockaddr_in me_sockaddr, sendto_addresses[MAXSENDTO];
fd_set client_sockets_set;
int sendto_sockets[MAXSENDTO], sendto_dummy[MAXSENDTO],
  sendto_connected[MAXSENDTO];

struct {
  char filename[150];
  AFfilehandle filehandle, filehandle2;
  AFfilesetup filesetup, filesetup2;
  int track, sumnum, channels, used, mixstatus, muted, wmade, nodraw, finestart;
  int dummy, sfreq, grouped, posinmatrix, groupdifftrack, loop, loopcount,
    lock, sampsread;
  double startsec, dur, groupdiff;
  int *summary;
  XColor color;
  Widget bulletin, muteB, playB, deleteB, duplicateB, editB, refreshB, processB,
    startText, startLabel, stopText, stopLabel, trackText, trackLabel, cancelB,
    loopText, loopLabel, volScale, volLabel, lockToggle, processM, processMB[30];
  int regstart, regend;
} sounds[MAXLYD];

struct {
  char value[50];
  int name, track, used, mixstatus, muted, wmade, nodraw;
  int dummy, grouped, posinmatrix, groupdifftrack;
  double startsec, groupdiff;
  XColor color;
  Widget bulletin, muteB, deleteB, duplicateB,
    startText, startLabel, valueText, valueLabel, trackText, trackLabel, cancelB;
} events[MAXLYD];

struct {
  char filename[150];
  double startsec;
  int track, used, loop;
} pasteSounds[MAXLYD];

struct {
  char value[50];
  int name, track, used;
  double startsec;
} pasteEvents[MAXLYD];

struct {
  Widget button, muteB, panB, volB, volScale, volLabel, fbackB, auxB;
  Widget volSel, panSel, auxSel, zoomScale;
  int muted, pan, vol, aux, dummy, pair, volval;
  int vxpos[MAXNODE], vypos[MAXNODE], vnumsquare;
  int pxpos[MAXNODE], pypos[MAXNODE], pnumsquare;
  int axpos[MAXNODE], aypos[MAXNODE], anumsquare;
  XmString vlist[MAXNODE], plist[MAXNODE], alist[MAXNODE];
  short int fullvol[GRAPHSAMPLES], fullpan[GRAPHSAMPLES], fullaux[GRAPHSAMPLES];
  double zoomval;
} tracks[11];

struct {
  int vxpos[MAXNODE], vypos[MAXNODE], vnumsquare;
  int pxpos[MAXNODE], pypos[MAXNODE], pnumsquare;
  int axpos[MAXNODE], aypos[MAXNODE], anumsquare; 
} pasteTracks[11];

struct {
  int sumnum, wmade;
  short int summary[GRAPHSAMPLES][2];
  Widget bulletin, drawarea;
} master;

struct settings {
  int shownum, showlin, showgrid, showall, scroll, fastdraw, linvol, nchnls;
  double buffsize;
  int ringset, chorusset, harmonizerset, filterset, echoset, filtertype;
  int reverbset, compresset, octaveset, monitor, recstereo;
  int bpmscale, bpm, bpb, quant, autoquant;
  int centerT, smallfont;
  double RM1, ECtime, ECfeed, CHspeed, CHdepth, HAinter, FLTcenter, FLTQ;
  double RVtime, CMA;
};
struct settings sett, sett2;


void audioexit(int n)
{
  #ifdef LINUX
  finishAudio();
  #endif
  exit(n);
}

void flush_all_underflows_to_zero()
  {
#ifdef SGI
    union fpc_csr f;
    f.fc_word = get_fpc_csr();
    f.fc_struct.flush = 1;
    set_fpc_csr(f.fc_word);
#endif
  }

void openWarn(fil)
char *fil;
{
  XmString text;

  sprintf(streng,"Can't open file %s",fil);
  text=XmStringCreateSimple(streng);
  XtVaSetValues(fileWarning,XmNmessageString,text,NULL);
  XtManageChild(fileWarning);
  XtFree(text);
}

void connectWarn(host)
char *host;
{
  XmString text;

  sprintf(streng,"Accepted connection from %s",host);
  text=XmStringCreateSimple(streng);
  XtVaSetValues(connectWarning,XmNmessageString,text,NULL);
  XtManageChild(connectWarning);
  XtFree(text);
}

void slavefailWarn(host)
char *host;
{
  XmString text;

  sprintf(streng,"Couldn't contact Mix on %s",host);
  text=XmStringCreateSimple(streng);
  XtVaSetValues(slavefailWarning,XmNmessageString,text,NULL);
  XtManageChild(slavefailWarning);
  XtFree(text);
}

int shiftin(lmax, rmax, lmax2, rmax2)
int *lmax, *rmax, *lmax2, *rmax2;
{
  int i;
  int lmmax=0, rmmax=0, lmmax2=0, rmmax2=0;
  short int *frames = framebuff, lsamp, rsamp, lsamp2, rsamp2;
  int sampsread=AFreadframes(infile, AF_DEFAULT_TRACK, frames, 512);
  int eof=(sampsread<512);
      
  if (samps_per_frame==1)
    for (i=0; i<sampsread; i++) { 
      lsamp = frames[i];
      if (lsamp>lmmax) lmmax=lsamp;
      if (-lsamp>lmmax) lmmax=-lsamp;
    }
  else if (samps_per_frame==2)
    while(sampsread--) {
      lsamp = *(frames++); rsamp = *(frames++);
      if (lsamp>lmmax) lmmax=lsamp;
      if (-lsamp>lmmax) lmmax=-lsamp;
      if (rsamp>rmmax) rmmax=rsamp;
      if (-rsamp>rmmax) rmmax=-rsamp; 
    }
  else {
    while(sampsread--) {
      lsamp = *(frames++); rsamp = *(frames++);
      lsamp2 = *(frames++); rsamp2 = *(frames++);
      if (lsamp>lmmax) lmmax=lsamp;
      if (-lsamp>lmmax) lmmax=-lsamp;
      if (rsamp>rmmax) rmmax=rsamp;
      if (-rsamp>rmmax) rmmax=-rsamp;
      if (lsamp2>lmmax2) lmmax2=lsamp2;
      if (-lsamp2>lmmax2) lmmax2=-lsamp2;
      if (rsamp2>rmmax2) rmmax2=rsamp2;
      if (-rsamp2>rmmax2) rmmax2=-rsamp2;
    }
  }
  *lmax=lmmax; *rmax=rmmax; *lmax2=lmmax2; *rmax2=rmmax2; 
  return(eof);
}

int mixin(pt)
int pt;
{
  int sampsread, num, i;

  if (sounds[pt].finestart) {
    num=sounds[pt].finestart*sounds[pt].channels;
    for (i=0; i<num; i++) *(framebuff+i)=0;
    sampsread=AFreadframes(sounds[pt].filehandle, AF_DEFAULT_TRACK,
      framebuff+num,1024-sounds[pt].finestart);
    sounds[pt].finestart=0;
    return(1024);
  }

  sampsread=AFreadframes(sounds[pt].filehandle, AF_DEFAULT_TRACK,
    framebuff, 1024);

  if (sampsread<1024) {
    sounds[pt].loopcount--;
    if (sounds[pt].loopcount) {
      AFseekframe(sounds[pt].filehandle,AF_DEFAULT_TRACK,0);
      AFreadframes(sounds[pt].filehandle, AF_DEFAULT_TRACK,
        framebuff+sampsread*sounds[pt].channels, 1024-sampsread);
      sampsread=1024;
    }
  }
  return(sampsread);  
}

void SetWatchCursor(w)
Widget w;
{
    static Cursor watch = NULL;

    if(!watch) watch = XCreateFontCursor(XtDisplay(w),XC_watch);

    XDefineCursor(XtDisplay(w),XtWindow(w),watch);
    XDefineCursor(XtDisplay(topLevel),XtWindow(topLevel),watch);
    XmUpdateDisplay(w);
    XmUpdateDisplay(topLevel);
}

void SetGrabCursor(w)
Widget w;
{
    static Cursor grab = NULL;

    if(!grab) grab = XCreateFontCursor(XtDisplay(w),XC_hand2);

    XDefineCursor(XtDisplay(w),XtWindow(w),grab);
    XmUpdateDisplay(w);
}

void ResetCursor(w)
Widget w;
{
    XUndefineCursor(XtDisplay(w),XtWindow(w));
    XUndefineCursor(XtDisplay(topLevel),XtWindow(topLevel));
    XmUpdateDisplay(w);
    XmUpdateDisplay(topLevel);
}

void DrawGrid(void)
{
  static char dashes1[2]={4, 4}, dashes2[2]={1, 4};
  int x, i, sec, beat;
  double beats_pr_scr, bars_pr_scr, leftbeat;

  XSetForeground(theDisplay, bitGC, black.pixel);
  XSetLineAttributes(theDisplay, bitGC, 0, LineOnOffDash, CapRound,
    JoinMiter); 
  XSetDashes(theDisplay, bitGC, 0, dashes2, 2);
  if (sett.bpmscale) {
    beats_pr_scr=sett.bpm*sec_pr_scr/60.;
    bars_pr_scr=beats_pr_scr/sett.bpb;
    leftbeat=sett.bpm*leftsec/60.;
    for (beat=ceil(leftbeat); beat<(int)floor(leftbeat)+beats_pr_scr; beat++) {
      x=(beat-leftbeat)*1000./beats_pr_scr;
      if (
        (beats_pr_scr<120.) ||
        ((beat%4==0) && (beats_pr_scr<480.)) || (
          (beat%sett.bpb==0) && ((bars_pr_scr<120.) ||
          (((beat/sett.bpb)%2==0) && (bars_pr_scr<240.)) ||
          (((beat/sett.bpb)%4==0) && (bars_pr_scr<480.)) ||
          (((beat/sett.bpb)%8==0) && (bars_pr_scr<960.)) ||
          ((beat/sett.bpb)%64==0))
        )
      ) XDrawLine(theDisplay,bitmap,bitGC,x,30,x,735);
    }
  } else {
    for (sec=ceil(leftsec); sec<(int)floor(leftsec)+sec_pr_scr; sec++) {
      x=(sec-leftsec)*1000./sec_pr_scr;
      if ((sec_pr_scr<120.) || ((sec%10==0) && (sec_pr_scr<MAXLEN))
        || (sec%60==0)) 
          XDrawLine(theDisplay,bitmap,bitGC,x,30,x,735);
    }
  }
  XSetDashes(theDisplay, bitGC, 0, dashes1, 2);
  XSetLineAttributes(theDisplay, bitGC, 0, LineSolid, CapRound,
    JoinMiter); 
}

void DrawScale(void)
{
  int i, j, x, sec, beat;
  char tall[10];
  static char dashes1[2]={4, 4}, dashes2[2]={1, 4};
  double beats_pr_scr, bars_pr_scr, leftbeat;

  XSetForeground(theDisplay, bitGC, black.pixel);
  XDrawLine(theDisplay,bitmap,bitGC,0,15,1000,15);
  if (sett.bpmscale) {
    beats_pr_scr=sett.bpm*sec_pr_scr/60.;
    bars_pr_scr=beats_pr_scr/sett.bpb;
    leftbeat=sett.bpm*leftsec/60.;
    for (i=1; i<=(int)beats_pr_scr+1; i++) {
      x=(i-leftbeat+floor(leftbeat))*1000./beats_pr_scr;
      beat=i+floor(leftbeat);
      if (beat%sett.bpb==0) XDrawLine(theDisplay,bitmap,bitGC,x,15,x,30);
      if (beats_pr_scr<800) XDrawLine(theDisplay,bitmap,bitGC,x,15,x,20);
      if ((bars_pr_scr<17.) && ((beat%sett.bpb==0) || (beats_pr_scr<17.))) {
        sprintf(tall, "%2d:%d", beat/sett.bpb+1,beat%sett.bpb+1);
        XDrawString(theDisplay,bitmap,bitGC,x-29,12,tall,strlen(tall));
      }
      else if (beat%sett.bpb==0) {
        if (((bars_pr_scr<33) && ((beat/sett.bpb)%4==0)) ||
          ((bars_pr_scr<65) && ((beat/sett.bpb)%8==0)) ||
          ((bars_pr_scr<129) && ((beat/sett.bpb)%16==0)) ||
          ((bars_pr_scr<257) && ((beat/sett.bpb)%32==0)) ||
          ((beat/sett.bpb)%64==0)   ) {
            sprintf(tall, "%2d", beat/sett.bpb+1);
            XDrawString(theDisplay,bitmap,bitGC,x-15,12,tall,strlen(tall));
          }
      }
    }
    if (beats_pr_scr<30.) for (i=0; i<=(int)(beats_pr_scr*8)+8; i++) {
      x=(1000./beats_pr_scr)*(floor(leftbeat)-leftbeat+i/8.);
      XDrawLine(theDisplay,bitmap,bitGC,x,15,x,17);
    }  
  } else {
    for (i=1; i<=(int)sec_pr_scr+1; i++) {
      x=(i-leftsec+floor(leftsec))*1000./sec_pr_scr; sec=i+floor(leftsec);
      if (sec%10==0) XDrawLine(theDisplay,bitmap,bitGC,x,15,x,25);
      if (sec%60==0) XDrawLine(theDisplay,bitmap,bitGC,x,15,x,30);
      if (sec_pr_scr<960) XDrawLine(theDisplay,bitmap,bitGC,x,15,x,20);
      if ((sec_pr_scr<240.) && ((sec%10==0) || (sec_pr_scr<15.))) {
        sprintf(tall, "%2d:%02d", sec/60,sec%60);
        XDrawString(theDisplay,bitmap,bitGC,x-29,12,tall,strlen(tall));
      }
      else if (sec%60==0) {
        sprintf(tall, "%2d", sec/60);
        XDrawString(theDisplay,bitmap,bitGC,x-15,12,tall,strlen(tall));
      }
    }
    if (sec_pr_scr<30.) for (i=1; i<=(int)(sec_pr_scr*10); i++) {
      x=i*100./sec_pr_scr;
      XDrawLine(theDisplay,bitmap,bitGC,x,15,x,17);
    }
  }
  XSetLineAttributes(theDisplay, bitGC, 0, LineOnOffDash, CapRound,
    JoinMiter); 
  if (zoomtrack) XDrawLine(theDisplay,bitmap,bitGC,0,665,1000,665);
    else for (i=1; i<10; i++)
      XDrawLine(theDisplay,bitmap,bitGC,0,i*70+35,1000,i*70+35);
  XSetLineAttributes(theDisplay, bitGC, 0, LineSolid, CapRound,
    JoinMiter);
  if (sett.showgrid) DrawGrid();
}

void DrawSound(pt)
int pt;
{
  int i, x, x2, xold=-9999999, vert, height, hmaxl=-99999, hmaxr=-99999,
    fd, sampsread, lp, hmaxl2=-99999, hmaxr2=-99999, hminl=99999,
    hminr=99999, hminl2=99999, hminr2=99999;
  double sec, scale, scale2, accum, savestart, zoom, zoom2;
  char tall[10];
  XSegment list0[LSIZE];
  int c0=0;

  if (sounds[pt].nodraw) return;
  if (zoomtrack && (zoomtrack!=sounds[pt].track)) return;

  scale2=1000./sec_pr_scr;
  if (sounds[pt].grouped) XSetForeground(theDisplay, bitGC, brown.pixel);
    else XSetForeground(theDisplay, bitGC, sounds[pt].color.pixel);
  vert=sounds[pt].track*70;
  savestart=sounds[pt].startsec;
  zoom=tracks[sounds[pt].track].zoomval/256.; zoom2=zoom*0.25;
 for (lp=1; lp<=sounds[pt].loop; lp++) {
  accum=0.;
  if (lp==2) XSetForeground(theDisplay, bitGC, blue.pixel);
  if (sett.fastdraw) {
    x=(int)((sounds[pt].startsec-leftsec)*scale2);
    x2=(int)(sounds[pt].dur*scale2);
    if (sounds[pt].channels==1) {
      if (zoomtrack) XFillRectangle(theDisplay,bitmap,bitGC,x,350-160,x2,320);
      else XFillRectangle(theDisplay,bitmap,bitGC,x,vert-20,x2,40);
    } else {
      if (zoomtrack) {
        XFillRectangle(theDisplay,bitmap,bitGC,x,175-80,x2,160);
        XFillRectangle(theDisplay,bitmap,bitGC,x,525-80,x2,160);
      } else {
        XFillRectangle(theDisplay,bitmap,bitGC,x,vert-25,x2,20);
        XFillRectangle(theDisplay,bitmap,bitGC,x,vert+5,x2,20);
      }
    }
  } else if (sec_pr_scr>=7.) {
    scale=512./Srate;
    if (sounds[pt].channels==1)
      for (i=0; i<sounds[pt].sumnum; i++) {
        sec=sounds[pt].startsec+(accum+=scale);
        x=(int)((sec-leftsec)*scale2);
        if (x>1000) break;
        if (x>=0) {
          height=(int)(sounds[pt].summary[i]&65535);
          if (height>hmaxl) hmaxl=height;
          if (x!=xold) {
            if (zoomtrack)
              { ADD_LINE(x,350-hmaxl*0.01,x,350+hmaxl*0.01,list0,c0,bitGC); }
                else {
                  hmaxl*=zoom2; if (hmaxl>34) hmaxl=34;
                  { ADD_LINE(x,vert-hmaxl,x,vert+hmaxl,list0,c0,bitGC); }
                }
            xold=x; hmaxl=0;
          }
        }
      }
    else if (sounds[pt].channels==2) for (i=0; i<sounds[pt].sumnum; i++) {
      sec=sounds[pt].startsec+(accum+=scale);
      x=(int)((sec-leftsec)*scale2);
      if (x>1000) break;
      if (x>=0) {
        height=(int)(sounds[pt].summary[i]&0xffff);
        if (height>hmaxl) hmaxl=height;
        height=(int)(sounds[pt].summary[i]>>16);
        if (height>hmaxr) hmaxr=height;
        if (x!=xold) {
          if (zoomtrack) {
            { ADD_LINE(x,175-hmaxl*0.005,x,175+hmaxl*0.005,list0,c0,bitGC); }
            { ADD_LINE(x,525-hmaxr*0.005,x,525+hmaxr*0.005,list0,c0,bitGC); }
          } else {
            hmaxl*=zoom2*0.5; if (hmaxl>17) hmaxl=17;
            hmaxr*=zoom2*0.5; if (hmaxr>17) hmaxr=17;
            { ADD_LINE(x,vert-17-hmaxl,x,vert-17+hmaxl,list0,c0,bitGC); }
            { ADD_LINE(x,vert+17-hmaxr,x,vert+17+hmaxr,list0,c0,bitGC); }
          }
          xold=x; hmaxl=hmaxr=0;
        }
      }
    }
    else if (sounds[pt].channels==4) for (i=0; i<sounds[pt].sumnum; i++) {
      sec=sounds[pt].startsec+(accum+=scale);
      x=(int)((sec-leftsec)*scale2);
      if (x>1000) break;
      if (x>=0) {
        height=(int)(sounds[pt].summary[i]&255);
        if (height>hmaxl) hmaxl=height;
        height=(int)(sounds[pt].summary[i]>>8)&255;
        if (height>hmaxr) hmaxr=height;
        height=(int)(sounds[pt].summary[i]>>16)&255;
        if (height>hmaxl2) hmaxl2=height;
        height=(int)(sounds[pt].summary[i]>>24)&255;
        if (height>hmaxr2) hmaxr2=height;
        if (x!=xold) {
          if (zoomtrack) {
            { ADD_LINE(x,150-hmaxl,x,150+hmaxl,list0,c0,bitGC); }
            { ADD_LINE(x,300-hmaxr,x,300+hmaxr,list0,c0,bitGC); }
            { ADD_LINE(x,450-hmaxl2,x,450+hmaxl2,list0,c0,bitGC); }
            { ADD_LINE(x,600-hmaxr2,x,600+hmaxr2,list0,c0,bitGC); }
          } else {
            hmaxl*=zoom*0.5; if (hmaxl>9) hmaxl=9;
            hmaxr*=zoom*0.5; if (hmaxr>9) hmaxr=9;
            hmaxl2*=zoom*0.5; if (hmaxl2>9) hmaxl2=9;
            hmaxr2*=zoom*0.5; if (hmaxr2>9) hmaxr2=9;
            { ADD_LINE(x,vert-24-hmaxl,x,vert-24+hmaxl,list0,c0,bitGC); }
            { ADD_LINE(x,vert-8-hmaxr,x,vert-8+hmaxr,list0,c0,bitGC); }
            { ADD_LINE(x,vert+8-hmaxl2,x,vert+8+hmaxl2,list0,c0,bitGC); }
            { ADD_LINE(x,vert+24-hmaxr2,x,vert+24+hmaxr2,list0,c0,bitGC); }
          }
          xold=x; hmaxl=hmaxr=hmaxl2=hmaxr2=0;
        }
      }
    }
  } else if (sounds[pt].startsec+sounds[pt].dur>=leftsec) {
    int start;
    scale=scale2/(double)Srate;
    fd=open(sounds[pt].filename,O_RDONLY);
    if (fd==-1) return;
    close(fd); 
    //infile=AFopenfd(fd, "r", in_AFsetup);
    infile=AFopenfile(sounds[pt].filename, "rb", in_AFsetup);
    if (infile==NULL) { openWarn(sounds[pt].filename); close(fd); return; }
    sec=0.; if (leftsec>sounds[pt].startsec) sec=leftsec-sounds[pt].startsec;
    start=0;
    if (sounds[pt].startsec>leftsec)
      start=(int)((sounds[pt].startsec-leftsec)*1000./sec_pr_scr);
    AFseekframe(infile, AF_DEFAULT_TRACK, (int)(Srate*sec));
    sampsread=AFreadframes(infile,AF_DEFAULT_TRACK,drbuf,(int)(Srate*sec_pr_scr));
    if (sounds[pt].channels==1) for (i=0; i<sampsread; i++) {
      x=(int)(scale*i)+start;
      height=(int)(drbuf[i]);
      if (height>hmaxl) hmaxl=height; if (height<hminl) hminl=height;
      if (x!=xold) {
        if (zoomtrack)
          { ADD_LINE(x,350-hmaxl*0.01,x,350-hminl*0.01,list0,c0,bitGC); }
        else {
          hminl*=zoom2; if (hminl<-34) hminl=-34; if (hminl>34) hminl=34;
          hmaxl*=zoom2; if (hmaxl>34) hmaxl=34; if (hmaxl<-34) hmaxl=-34;
          { ADD_LINE(x,vert-hmaxl,x,vert-hminl,list0,c0,bitGC); }
        }
        xold=x; hmaxl=-99999; hminl=99999;
      }
    } else if (sounds[pt].channels==2) for (i=0; i<sampsread; i++) {
      x=(int)(scale*i)+start;
      height=(int)(drbuf[i+i]);
      if (height>hmaxl) hmaxl=height; if (height<hminl) hminl=height;
      height=(int)(drbuf[i+i+1]);
      if (height>hmaxr) hmaxr=height; if (height<hminr) hminr=height;
      if (x!=xold) {
        if (zoomtrack) {
          { ADD_LINE(x,175-hmaxl*0.005,x,175-hminl*0.005,list0,c0,bitGC); }
          { ADD_LINE(x,525-hmaxr*0.005,x,525-hminr*0.005,list0,c0,bitGC); }
        } else {
          hminl*=zoom2*0.5; if (hminl<-17) hminl=-17; if (hminl>17) hminl=17;
          hmaxl*=zoom2*0.5; if (hmaxl>17) hmaxl=17; if (hmaxl<-17) hmaxl=-17;
          hminr*=zoom2*0.5; if (hminr<-17) hminr=-17; if (hminr>17) hminr=17;
          hmaxr*=zoom2*0.5; if (hmaxr>17) hmaxr=17; if (hmaxr<-17) hmaxr=-17;
          { ADD_LINE(x,vert-17-hmaxl,x,vert-17-hminl,list0,c0,bitGC);}
          { ADD_LINE(x,vert+17-hmaxr,x,vert+17-hminr,list0,c0,bitGC);}
        }
        xold=x; hmaxl=-99999, hmaxr=-99999; hminl=99999; hminr=99999;
      }
    } else if (sounds[pt].channels==4) for (i=0; i<sampsread; i++) {
      x=(int)(scale*i)+start;
      height=(int)(drbuf[i+i]);
      if (height>hmaxl) hmaxl=height; if (height<hminl) hminl=height;
      height=(int)(drbuf[i+i+1]);
      if (height>hmaxr) hmaxr=height; if (height<hminr) hminr=height;
      height=(int)(drbuf[i+i+2]);
      if (height>hmaxl2) hmaxl2=height; if (height<hminl2) hminl2=height;
      height=(int)(drbuf[i+i+3]);
      if (height>hmaxr2) hmaxr2=height; if (height<hminr2) hminr2=height;
      if (x!=xold) {
        if (zoomtrack) {
          { ADD_LINE(x,150-hmaxl*0.0025,x,150-hminl*0.0025,list0,c0,bitGC); }
          { ADD_LINE(x,300-hmaxr*0.0025,x,300-hminr*0.0025,list0,c0,bitGC); }
          { ADD_LINE(x,450-hmaxl2*0.0025,x,450-hminl2*0.0025,list0,c0,bitGC); }
          { ADD_LINE(x,600-hmaxr2*0.0025,x,600-hminr2*0.0025,list0,c0,bitGC); }
        } else {
          hminl*=zoom2*0.25; if (hminl<-9) hminl=-9; if (hminl>9) hminl=9;
          hmaxl*=zoom2*0.25; if (hmaxl>9) hmaxl=9; if (hmaxl<-9) hmaxl=-9;
          hminr*=zoom2*0.25; if (hminr<-9) hminr=-9; if (hminr>9) hminr=9;
          hmaxr*=zoom2*0.25; if (hmaxr>9) hmaxr=9; if (hmaxr<-9) hmaxr=-9;
          hminl2*=zoom2*0.25; if (hminl2<-9) hminl2=-9; if (hminl2>9) hminl2=9;
          hmaxl2*=zoom2*0.25; if (hmaxl2>9) hmaxl2=9; if (hmaxl2<-9) hmaxl2=-9;
          hminr2*=zoom2*0.25; if (hminr2<-9) hminr2=-9; if (hminr2>9) hminr2=9;
          hmaxr2*=zoom2*0.25; if (hmaxr2>9) hmaxr2=9; if (hmaxr2<-9) hmaxr2=-9;
          { ADD_LINE(x,vert-24-hmaxl,x,vert-24-hminl,list0,c0,bitGC);}
          { ADD_LINE(x,vert-8-hmaxr,x,vert-8-hminr,list0,c0,bitGC);}
          { ADD_LINE(x,vert+8-hmaxl2,x,vert+8-hminl2,list0,c0,bitGC);}
          { ADD_LINE(x,vert+24-hmaxr2,x,vert+24-hminr2,list0,c0,bitGC);}
        }
        xold=x; hmaxl=-99999, hmaxr=-99999; hminl=99999; hminr=99999;
        hmaxl2=-99999, hmaxr2=-99999; hminl2=99999; hminr2=99999;
      }
    }
    AFclosefile(infile);
  }
  FLUSH_LIST(list0,c0,bitGC);
  sounds[pt].startsec+=sounds[pt].dur;
 }
  sounds[pt].startsec=savestart;
  if (sett.shownum) {
    sprintf(tall, "#%d", pt);
    x=(int)((sounds[pt].startsec-leftsec)*scale2);
    XSetForeground(theDisplay, bitGC, black.pixel);
    XDrawString(theDisplay,bitmap,bitGC,x,vert-20,tall,strlen(tall));
  }
}

void DrawEvent(pt)
int pt;
{
  int x, vert;
  double scale2;
  char tall[10];

  if (events[pt].nodraw) return;
  if (zoomtrack && (zoomtrack!=events[pt].track)) return;

  scale2=1000./sec_pr_scr;
  vert=events[pt].track*70;
  x=(int)((events[pt].startsec-leftsec)*scale2);
  if (events[pt].grouped)
    XCopyArea(theDisplay, varselpixmap, bitmap,bitGC, 0, 0, 20, 24, x, vert+1);
  else XCopyArea(theDisplay, varpixmap, bitmap,bitGC, 0, 0, 20, 24, x, vert+1);
  if (sett.shownum) {
    sprintf(tall, "#%d", pt);
    x=(int)((events[pt].startsec-leftsec)*scale2);
    XSetForeground(theDisplay, bitGC, black.pixel);
    XDrawString(theDisplay,bitmap,bitGC,x,vert-20,tall,strlen(tall));
  }
}

void DrawMix(void)
{
  int i, x, xold=-1, height, hmaxl=0, hmaxr=0, hmaxl2=0, hmaxr2=0;
  double scale, scale2, zoom, zoom2;

  scale=1024./Srate; scale2=1000./sec_pr_scr;
  XSetForeground(theDisplay, bitGC, blue.pixel);
  zoom=tracks[10].zoomval; zoom2=zoom*0.25;
  if (sett.nchnls==1) for (i=0; i<master.sumnum; i++) {
    x=(int)((i*scale-leftsec)*scale2);
    if (x>1000) break;
    if (x>=0.) {
      height=(int)(master.summary[i][0]&255);
      if (height>hmaxl) hmaxl=height;
      if (x!=xold) {
        if (zoomtrack==10) {
          XDrawLine(theDisplay,bitmap,bitGC,x,350-hmaxl*2,x,350+hmaxl*2);
        } else {
          hmaxl*=zoom2; if (hmaxl>34) hmaxl=34;
          XDrawLine(theDisplay,bitmap,bitGC,x,700-hmaxl,x,700+hmaxl);
        }
        xold=x; hmaxl=0;
      }
    }
  } else if (sett.nchnls==2) for (i=0; i<master.sumnum; i++) {
    x=(int)((i*scale-leftsec)*scale2);
    if (x>1000) break;
    if (x>=0.) {
      height=(int)(master.summary[i][0]&255);
      if (height>hmaxl) hmaxl=height;
      height=(int)(master.summary[i][0]>>8);
      if (height>hmaxr) hmaxr=height;
      if (x!=xold) {
        if (zoomtrack==10) {
          XDrawLine(theDisplay,bitmap,bitGC,x,175-hmaxl,x,175+hmaxl);
          XDrawLine(theDisplay,bitmap,bitGC,x,525-hmaxr,x,525+hmaxr);
          XFillRectangle(theDisplay,bitmap,bitGC,
            xold, 175-hmaxl, x-xold, hmaxl+hmaxl);
          XFillRectangle(theDisplay,bitmap,bitGC,
            xold, 525-hmaxr, x-xold, hmaxr+hmaxr);
        } else {
          hmaxl*=zoom2*0.5; if (hmaxl>17) hmaxl=17;
          hmaxr*=zoom2*0.5; if (hmaxr>17) hmaxr=17;
          XFillRectangle(theDisplay,bitmap,bitGC,
            xold, 700-17-hmaxl, x-xold, hmaxl+hmaxl);
          XFillRectangle(theDisplay,bitmap,bitGC,
            xold, 700+17-hmaxr, x-xold, hmaxr+hmaxr);
        }
        xold=x; hmaxl=hmaxr=0;
      }
    }
  } else if (sett.nchnls=4) for (i=0; i<master.sumnum; i++) {
    x=(int)((i*scale-leftsec)*scale2);
    if (x>1000) break;
    if (x>=0.) {
      height=(int)(master.summary[i][0]&255);
      if (height>hmaxl) hmaxl=height;
      height=(int)(master.summary[i][0]>>8);
      if (height>hmaxr) hmaxr=height;
      height=(int)(master.summary[i][1]&255);
      if (height>hmaxl2) hmaxl2=height;
      height=(int)(master.summary[i][1]>>8);
      if (height>hmaxr2) hmaxr2=height;
      if (x!=xold) {
        if (zoomtrack==10) {
          XDrawLine(theDisplay,bitmap,bitGC,x,87-hmaxl/2,x,87+hmaxl/2);
          XDrawLine(theDisplay,bitmap,bitGC,x,262-hmaxr/2,x,262+hmaxr/2);
          XDrawLine(theDisplay,bitmap,bitGC,x,437-hmaxl2/2,x,437+hmaxl2/2);
          XDrawLine(theDisplay,bitmap,bitGC,x,612-hmaxr2/2,x,612+hmaxr2/2);
        } else {
          XDrawLine(theDisplay,bitmap,bitGC,x,674-hmaxl/16,x,674+hmaxl/16);
          XDrawLine(theDisplay,bitmap,bitGC,x,691-hmaxr/16,x,691+hmaxr/16);
          XDrawLine(theDisplay,bitmap,bitGC,x,709-hmaxl2/16,x,709+hmaxl2/16);
          XDrawLine(theDisplay,bitmap,bitGC,x,726-hmaxr2/16,x,726+hmaxr2/16);
        }
        xold=x; hmaxl=hmaxr=hmaxl2=hmaxr2=0;
      }
    }
  }
}

void makelist(track, type)
int track, type;
{
  int i;
  double time;

  if (type==1) {
    for (i=1; i<=tracks[track].vnumsquare; i++) {
      if (tracks[track].vlist[i-1]) XtFree(tracks[track].vlist[i-1]);
      time=(tracks[track].vxpos[i]*1024./Srate);
      sprintf(streng,"%6.2f   %d",time,tracks[track].vypos[i]);
      tracks[track].vlist[i-1]=XmStringCreateLocalized(streng);
    }
    XtVaSetValues(tracks[track].volSel,XmNlistItems,tracks[track].vlist,
      XmNlistItemCount,tracks[track].vnumsquare,NULL);
  } else if (type==2) {
    for (i=1; i<=tracks[track].pnumsquare; i++) {
      if (tracks[track].plist[i-1]) XtFree(tracks[track].plist[i-1]);
      time=(tracks[track].pxpos[i]*1024./Srate);
      sprintf(streng,"%6.2f   %d",time,tracks[track].pypos[i]);
      tracks[track].plist[i-1]=XmStringCreateLocalized(streng);
    }
    XtVaSetValues(tracks[track].panSel,XmNlistItems,tracks[track].plist,
      XmNlistItemCount,tracks[track].pnumsquare,NULL);
  } else {
    for (i=1; i<=tracks[track].anumsquare; i++) {
      if (tracks[track].alist[i-1]) XtFree(tracks[track].alist[i-1]);
      time=(tracks[track].axpos[i]*1024./Srate);
      sprintf(streng,"%6.2f   %d",time,tracks[track].aypos[i]);
      tracks[track].alist[i-1]=XmStringCreateLocalized(streng);
    }
    XtVaSetValues(tracks[track].auxSel,XmNlistItems,tracks[track].alist,
      XmNlistItemCount,tracks[track].anumsquare,NULL);
  }
}

int pix_to_blk(pix)
int pix;
{
  return (int)((sec_pr_scr*pix/1000.+leftsec)*Srate/1024.);
}

int blk_to_pix(block)
int block;
{
  int a=(block*1024./Srate-leftsec)*1000./sec_pr_scr;
  if (a>10000) a=10000; if (a<-10000) a=-10000;
  return a;
}

void DrawVol(pt)
int pt;
{
  int i;

  XSetForeground(theDisplay, bitGC, green.pixel);

  for (i=1; i<=tracks[pt].vnumsquare; i++) {
    XDrawRectangle(theDisplay,bitmap,bitGC,
      blk_to_pix(tracks[pt].vxpos[i])-4,pt*70+26-tracks[pt].vypos[i],8,8);
  }
  for (i=1; i<tracks[pt].vnumsquare; i++) {
    XDrawLine(theDisplay,bitmap,bitGC,
      blk_to_pix(tracks[pt].vxpos[i]),pt*70+30-tracks[pt].vypos[i],
      blk_to_pix(tracks[pt].vxpos[i+1]),pt*70+30-tracks[pt].vypos[i+1]);
  }
  if ((pt==seltrack) && selnum && (seltype==1)) {
    XFillRectangle(theDisplay,bitmap,bitGC,
      blk_to_pix(tracks[pt].vxpos[selnum])-4,
      pt*70+26-tracks[pt].vypos[selnum],8,8);
  }
}

void DrawPan(pt)
int pt;
{
  int i;

  XSetForeground(theDisplay, bitGC, blue.pixel);

  for (i=1; i<=tracks[pt].pnumsquare; i++) {
    XDrawRectangle(theDisplay,bitmap,bitGC,
      blk_to_pix(tracks[pt].pxpos[i])-4,pt*70+26-tracks[pt].pypos[i],8,8);
  }
  for (i=1; i<tracks[pt].pnumsquare; i++) {
    XDrawLine(theDisplay,bitmap,bitGC,
      blk_to_pix(tracks[pt].pxpos[i]),pt*70+30-tracks[pt].pypos[i],
      blk_to_pix(tracks[pt].pxpos[i+1]),pt*70+30-tracks[pt].pypos[i+1]);
  }
  if ((pt==seltrack) && selnum && (seltype==2)) {
    XFillRectangle(theDisplay,bitmap,bitGC,
      blk_to_pix(tracks[pt].pxpos[selnum])-4,
      pt*70+26-tracks[pt].pypos[selnum],8,8);
  }
}

void DrawAux(pt)
int pt;
{
  int i;

  XSetForeground(theDisplay, bitGC, purple.pixel);

  for (i=1; i<=tracks[pt].anumsquare; i++) {
    XDrawRectangle(theDisplay,bitmap,bitGC,
      blk_to_pix(tracks[pt].axpos[i])-4,pt*70+26-tracks[pt].aypos[i],8,8);
  }
  for (i=1; i<tracks[pt].anumsquare; i++) {
    XDrawLine(theDisplay,bitmap,bitGC,
      blk_to_pix(tracks[pt].axpos[i]),pt*70+30-tracks[pt].aypos[i],
      blk_to_pix(tracks[pt].axpos[i+1]),pt*70+30-tracks[pt].aypos[i+1]);
  }
  if ((pt==seltrack) && selnum && (seltype==3)) {
    XFillRectangle(theDisplay,bitmap,bitGC,
      blk_to_pix(tracks[pt].axpos[selnum])-4,
      pt*70+26-tracks[pt].aypos[selnum],8,8);
  }
}

int newsound(filename, start, trk, loop, lock, num, partial)
char *filename;
double start;
int trk, loop, lock, num, partial;
{
  int i, pt=-1, lmax, rmax, lmax2, rmax2, eof=0, R, fd, vers;
  char startstr[20], stopstr[20], trackstr[20], labelstr[9];
  String newrow[1][4];
  String newlabel;

  strcpy(newsnd_name,filename);
  newsnd_trk=trk; newsnd_lop=loop; newsnd_start=start; newsnd_num=num;
  newsnd_loc=lock;
  fd=open(filename,O_RDONLY);
  if (fd==-1) { close(fd); return 1; }
  close(fd);
  //infile=AFopenfd(fd, "r", in_AFsetup);
  infile=AFopenfile(filename, "rb", in_AFsetup);
  if (infile==NULL) { close(fd); return 1; }
  samps_per_frame   = AFgetchannels(infile, AF_DEFAULT_TRACK);
  R                 = (int)AFgetrate(infile, AF_DEFAULT_TRACK);
  compression       = AFgetcompression(infile, AF_DEFAULT_TRACK);
  filefmt           = AFgetfilefmt(infile, &vers);   
  AFgetsampfmt(infile, AF_DEFAULT_TRACK, &samp_type, &bits_per_samp);
  framecnt          = AFgetframecnt(infile,  AF_DEFAULT_TRACK);
  if (bits_per_samp!=16) { AFclosefile(infile); return 3; }
  if (R!=Srate) { AFclosefile(infile); return 2; }
  if (num>-1) pt=num; else
    for (pt=0; pt<MAXLYD; pt++) if (sounds[pt].used==0) break;
  if (pt==MAXLYD) { AFclosefile(infile); return 1; }
  if ((sounds[pt].summary=(int *)
    calloc(framecnt/510,sizeof(int)))==NULL) {
      fprintf(stderr,"Unable to allocate memory for waveform summary!\n");
      audioexit(-1);
    }
  i=0;
  while (!eof) {
    eof=shiftin(&lmax,&rmax,&lmax2,&rmax2);
    if (samps_per_frame==1)
      sounds[pt].summary[i]=lmax;
    else if (samps_per_frame==2)
      sounds[pt].summary[i]=lmax+(rmax<<16);
    else if (samps_per_frame==4)
      sounds[pt].summary[i]=lmax/256+((rmax/256)<<8)+((lmax2/256)<<16)+
        ((rmax2/256)<<24);
    i++; if (i==framecnt/510) break;
  }
  AFclosefile(infile);

  sounds[pt].sumnum=i; sounds[pt].dur=(double)framecnt/R;
  sounds[pt].channels=samps_per_frame;
  sounds[pt].regstart=0; sounds[pt].regend=framecnt;
  sprintf(startstr,"%7.3f",start);
  sprintf(stopstr,"%7.3f",start+sounds[pt].dur);
  sprintf(trackstr,"%d",trk);
  sprintf(labelstr,"%d",pt);
  newlabel=labelstr;

  if (partial) XbaeMatrixSetCell(soundMatrix,pt,2,stopstr);
  else if (num_total==0) {
    XbaeMatrixSetCell(soundMatrix,0,0,filename);
    XbaeMatrixSetCell(soundMatrix,0,1,startstr);
    XbaeMatrixSetCell(soundMatrix,0,2,stopstr);
    XbaeMatrixSetCell(soundMatrix,0,3,trackstr);
    XtVaSetValues(soundMatrix,XmNrowLabels,&newlabel,NULL);
  } else {
    newrow[0][0]=filename; newrow[0][1]=startstr; newrow[0][2]=stopstr;
    newrow[0][3]=trackstr;
    XbaeMatrixAddRows(soundMatrix,num_total,&newrow[0][0],&newlabel,NULL,1);
  }

  if (partial) return 0;

  sounds[pt].used=1; sounds[pt].muted=0;
  sounds[pt].grouped=0; sounds[pt].groupdiff=0.; sounds[pt].groupdifftrack=0;
  strcpy(sounds[pt].filename,filename);
  sounds[pt].track=trk;
  sounds[pt].startsec=start;
  sounds[pt].color=red; sounds[pt].loop=loop; sounds[pt].lock=lock;
  sounds[pt].posinmatrix=num_total;
  num_total++;
  MakeSoundWidgets(pt);
  RedrawTrack(trk);
  return 0;
}

int newevent(name, trk, start, num)
int name, trk, num;
double start;
{
  int pt=-1;
  char startstr[20], trackstr[20];
  String newrow[1][4];
  String newlabel[1]={"      "};

  if (num>-1) pt=num; else
    for (pt=0; pt<MAXLYD; pt++) if (events[pt].used==0) break;
  if (pt==-1) return 1;
  newenum=pt;
  events[pt].used=1; events[pt].muted=0;
  events[pt].grouped=0; events[pt].name=name;
  events[pt].groupdiff=0.; events[pt].groupdifftrack=0;
  events[pt].track=trk;
  events[pt].startsec=start;
  events[pt].color=blue;
  sprintf(events[pt].value,"%s",Edefaults[name]);
  sprintf(startstr,"%6.2f",start);
  sprintf(trackstr,"%d",trk);
  sprintf(newlabel[0],"%d",pt);
  if (events_total==0) {
    XbaeMatrixSetCell(eventMatrix,0,0,Enames[name]);
    XbaeMatrixSetCell(eventMatrix,0,1,startstr);
    XbaeMatrixSetCell(eventMatrix,0,2,Edefaults[name]);
    XbaeMatrixSetCell(eventMatrix,0,3,trackstr);
    XtVaSetValues(eventMatrix,XmNrowLabels,newlabel,NULL);
  } else {
    newrow[0][0]=Enames[name]; newrow[0][1]=startstr;
    newrow[0][2]=Edefaults[name];
    newrow[0][3]=trackstr;
    XbaeMatrixAddRows(eventMatrix,events_total,&newrow[0][0],newlabel,NULL,1);
  }
  events[pt].posinmatrix=events_total;
  events_total++;
  MakeEventWidgets(pt);
  RedrawTrack(trk);
  return 0;
}

void RestoreWin(w, client, call)
Widget w;
XtPointer client, call;
{
  XCopyArea(theDisplay, bitmap, XtWindow(sketchpad),
    DefaultGCOfScreen(XtScreen(sketchpad)), 0, 0, 1000, theheight, 0, 0);
  if (realtiming)
    XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,xold,30,xold,740);
}

void RedrawTrack(pt)
{
  int i, xs, xe;

  if (pt==0) return;
  XSetForeground(theDisplay, bitGC, backcolor.pixel);
  XFillRectangle(theDisplay, bitmap, showGC, 0, pt*70-34, 1000, 69);
  if (pt==10) DrawMix(); else
  for (i=0; i<MAXLYD; i++)
    if ((sounds[i].used) && (sounds[i].track==pt)) DrawSound(i);
  for (i=0; i<MAXLYD; i++)
    if ((events[i].used) && (events[i].track==pt)) DrawEvent(i);
  if ((tracks[pt].vol) || (sett.showall)) DrawVol(pt);
  if ((tracks[pt].pan) || (sett.showall)) DrawPan(pt);
  if ((tracks[pt].aux) || (sett.showall)) DrawAux(pt);
  if (sett.showgrid) DrawGrid();
  if (marked) {
    xs=(int)(1000.*(mstart-leftsec)/sec_pr_scr);
    xe=(int)(1000.*(mstop-leftsec)/sec_pr_scr);
    XSetForeground(theDisplay, bitGC, markcolor.pixel^backcolor.pixel);
    XSetFunction(theDisplay,bitGC,GXxor);
    XFillRectangle(theDisplay, bitmap, bitGC, xs, pt*70-34, xe-xs, 69);
    XSetFunction(theDisplay,bitGC,GXcopy);
  }
  if (regmarked) {
    if (sounds[regnum].track==pt) {
      xs=(int)(1000.*(regmstart-leftsec)/sec_pr_scr);
      xe=(int)(1000.*(regmstop-leftsec)/sec_pr_scr);
      XSetForeground(theDisplay, bitGC, markcolor.pixel^backcolor.pixel);
      XSetFunction(theDisplay,bitGC,GXxor);
      XFillRectangle(theDisplay, bitmap, bitGC, xs, pt*70-34, xe-xs, 69);
      XSetFunction(theDisplay,bitGC,GXcopy);
    }
  }
  RestoreWin(NULL,NULL,NULL);
}

void RedrawWin(void)
{
  int i, xs, xe, track;

  SetWatchCursor(topLevel);
  XSetForeground(theDisplay, showGC, backcolor.pixel);
  XFillRectangle(theDisplay, bitmap, showGC, 0, 0, 1000, theheight);
  for (i=0; i<MAXLYD; i++) if (sounds[i].used) DrawSound(i);
  for (i=0; i<MAXLYD; i++) if (events[i].used) DrawEvent(i);
  DrawScale(); DrawMix();
  for (i=1; i<11; i++) if ((tracks[i].vol) || (sett.showall)) DrawVol(i);
  for (i=1; i<11; i++) if ((tracks[i].pan) || (sett.showall)) DrawPan(i);
  for (i=1; i<11; i++) if ((tracks[i].aux) || (sett.showall)) DrawAux(i);
  if (marked) {
    xs=(int)(1000.*(mstart-leftsec)/sec_pr_scr);
    xe=(int)(1000.*(mstop-leftsec)/sec_pr_scr);
    XSetForeground(theDisplay, bitGC, markcolor.pixel^backcolor.pixel);
    XSetFunction(theDisplay,bitGC,GXxor);
    XFillRectangle(theDisplay, bitmap, bitGC, xs, 35, xe-xs, 700);
    XSetFunction(theDisplay,bitGC,GXcopy);
  }
  if (regmarked) {
    track=sounds[regnum].track;
    xs=(int)(1000.*(regmstart-leftsec)/sec_pr_scr);
    xe=(int)(1000.*(regmstop-leftsec)/sec_pr_scr);
    XSetForeground(theDisplay, bitGC, markcolor.pixel^backcolor.pixel);
    XSetFunction(theDisplay,bitGC,GXxor);
    XFillRectangle(theDisplay, bitmap, bitGC, xs, track*70-34, xe-xs, 69);
    XSetFunction(theDisplay,bitGC,GXcopy);
  }
  RestoreWin(NULL,NULL,NULL);
  sprintf(streng, "%.3f", leftsec);
  XtVaSetValues(trackT, XmNvalue, streng, NULL);
  ResetCursor(topLevel);
}

void SnaptoGrid(w, client, call)
Widget w;
XtPointer client, call;
{
  int i;
  double beat;

  for (i=0; i<MAXLYD; i++) if ((sounds[i].used)&&(sounds[i].lock==0)) {
    if (sett.bpmscale) {
      beat=sett.bpm*sounds[i].startsec/60.;
      beat=floor(beat*sett.quant+0.5)/sett.quant;
      sounds[i].startsec=60.*beat/sett.bpm;
    } else {
      sounds[i].startsec=floor(sounds[i].startsec*sett.quant+0.5)/sett.quant;
    }
    sprintf(streng, "%7.3f", sounds[i].startsec);
    XtVaSetValues(sounds[i].startText,XmNvalue,streng,NULL);
    XbaeMatrixSetCell(soundMatrix,sounds[i].posinmatrix,1,streng);
    sprintf(streng, "%7.3f", sounds[i].startsec+sounds[i].dur);
    XtVaSetValues(sounds[i].stopText,XmNvalue,streng,NULL);
    XbaeMatrixSetCell(soundMatrix,sounds[i].posinmatrix,2,streng);
  }
  RedrawWin();
}

void Cancel(w, client, call)
Widget w;
XtPointer client, call;
{
  Widget dialog=(Widget) client;
  XtUnmanageChild(dialog);
}

void Slock(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;

  sounds[pt].lock=(int)XmToggleButtonGetState(w);
}

void Smute(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;

  if (sounds[pt].muted) {
    sounds[pt].muted=0;
    XtVaSetValues(w,XmNlabelPixmap,nomutepixmap,NULL);
  } else {
    sounds[pt].muted=1;
    XtVaSetValues(w,XmNlabelPixmap,mutepixmap,NULL);
  }
}

void Sedit(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;

  sprintf(streng, "%s %s &", app_data.soundeditor, sounds[pt].filename);
  system(streng);
}

void Sprocess(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;

  sprintf(streng,"soundprocess %s &", sounds[pt].filename);
  system(streng);
}

void Srefresh(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;
  int i, rcode;

  SetWatchCursor(w);
  for (i=0; i<MAXLYD; i++) {
    if ((sounds[i].used)&&(strcmp(sounds[i].filename,sounds[pt].filename)==0)) {
      free(sounds[i].summary);
      rcode=newsound(sounds[i].filename,
        sounds[i].startsec, sounds[i].track, sounds[i].loop, sounds[i].lock,i,1);
      if (rcode==1) openWarn(sounds[i].filename);
      if (rcode==2) XtManageChild(srateWarning);
      if (rcode==3) XtManageChild(bitsWarning);
      RedrawTrack(sounds[i].track);
    }
  }
  ResetCursor(w);
}

void Splay(w, client, call)
Widget w;
XtPointer client, call;
{
  int sampsread, chnls, srate, t=1, numfram, i;
  int pt=*(int *)client;
  ALport audioport;
  ALconfig config;
  long pvbuf[8];
  short int frames[4096];
  XmString text;
#ifdef LINUX  
  long tempBufOut [4] = {0,0,0,0};
#endif
  

  if (playing>-1) {
    text=XmStringCreateSimple("Play");
    XtVaSetValues(sounds[playing].playB,XmNlabelString,text,NULL);
    playing=-1; XtFree(text);
    return;
  }
  text=XmStringCreateSimple("Stop");
  XtVaSetValues(sounds[pt].playB,XmNlabelString,text,NULL); XtFree(text);
  sounds[pt].filehandle2=
    AFopenfile(sounds[pt].filename,"r",sounds[pt].filesetup2);
  if (sounds[pt].filehandle2==NULL) {
    openWarn(sounds[pt].filename);
    return;
  }

  playing=pt; chnls=AFgetchannels(sounds[pt].filehandle2, AF_DEFAULT_TRACK);
  srate=(int)AFgetrate(sounds[pt].filehandle2, AF_DEFAULT_TRACK);
  pvbuf[0] = AL_OUTPUT_RATE;  pvbuf[1] = srate;
  pvbuf[2] = AL_INPUT_RATE;   pvbuf[3] = srate;
  pvbuf[4] = AL_CHANNEL_MODE;
  if (chnls==1) pvbuf[5]=AL_MONO;
    else if (chnls==4) pvbuf[5]=AL_4CHANNEL; else pvbuf[5]=AL_STEREO;
  ALsetparams(AL_DEFAULT_DEVICE, pvbuf, 6);
  config=ALnewconfig();
  ALsetqueuesize(config, (int)(srate*sett.buffsize*chnls));
  ALsetchannels(config, chnls);
  audioport=ALopenport("Mix Play", "w", config);

  do {
    sampsread=AFreadframes(sounds[pt].filehandle2, AF_DEFAULT_TRACK,
      frames, 1024);
    numfram=sampsread*chnls;
    if (sett.nchnls>1) for (i=0; i<numfram; i++) frames[i]=frames[i]>>1;
    ALwritesamps(audioport, frames, numfram);
    if (XtAppPending(app_context) && ((t++)%3==0))
      XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
  } while ((playing>-1) && (sampsread==1024));

  while (ALgetfilled(audioport)) {
#ifdef LINUX
    ALwritesamps(audioport,tempBufOut,chnls);
#endif
  }  
  ALcloseport(audioport);
  if (sounds[pt].wmade) {
    text=XmStringCreateSimple("Play");
    XtVaSetValues(sounds[pt].playB,XmNlabelString,text,NULL);
    XtFree(text);
  }
  playing=-1;
}

void Desensgrp(void)
{
  XtVaSetValues(grpDelete,XmNsensitive,False,NULL);
  XtVaSetValues(grpDuplicate,XmNsensitive,False,NULL);
  XtVaSetValues(grpMute,XmNsensitive,False,NULL);
  XtVaSetValues(grpUngroupall,XmNsensitive,False,NULL);
}

void Sensgrp(void)
{
  XtVaSetValues(grpDelete,XmNsensitive,True,NULL);
  XtVaSetValues(grpDuplicate,XmNsensitive,True,NULL);
  XtVaSetValues(grpMute,XmNsensitive,True,NULL);
  XtVaSetValues(grpUngroupall,XmNsensitive,True,NULL);
}

void deletesound(pt)
int pt;
{
  String empty="";
  int i;

  sounds[pt].used=0;
  sounds[pt].wmade=0;
  if (sounds[pt].grouped) {
    numgrouped--;
    if (numgrouped==0) Desensgrp();
  }
  sounds[pt].grouped=0;
  XtDestroyWidget(sounds[pt].bulletin);
  free(sounds[pt].summary);
  XbaeMatrixSelectCell(soundMatrix,0,0);
  XbaeMatrixDeselectCell(soundMatrix,0,0);
  if (num_total>1) XbaeMatrixDeleteRows(soundMatrix,sounds[pt].posinmatrix,1);
  else {
    XbaeMatrixSetCell(soundMatrix,0,0,empty);
    XbaeMatrixSetCell(soundMatrix,0,1,empty);
    XbaeMatrixSetCell(soundMatrix,0,2,empty);
    XbaeMatrixSetCell(soundMatrix,0,3,empty);
    XtVaSetValues(soundMatrix,XmNrowLabels,rowlabels,NULL);
  }
  for (i=0; i<MAXLYD; i++)
    if (sounds[i].posinmatrix>sounds[pt].posinmatrix)
      sounds[i].posinmatrix--;
  num_total--;
  if (playing==pt) playing=-1;
}

void Sdelete(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;
  String empty="";

  if ((realtiming) || (saving)) return;
  backup();
  deletesound(pt);
  RedrawTrack(sounds[pt].track);
}

void Sduplicate(w, client, call)
Widget w;
XtPointer client, call;
{
  int r;

  int pt=*(int *)client;
  SetWatchCursor(sounds[pt].bulletin);
  backup();
  XtUnmanageChild(sounds[pt].bulletin);
  r=newsound(sounds[pt].filename,sounds[pt].startsec+1,sounds[pt].track,
    sounds[pt].loop,sounds[pt].lock,-1,0);
  ResetCursor(sounds[pt].bulletin);
  if (r==2) XtManageChild(srateWarning);
  if (r==3) XtManageChild(bitsWarning);
}

void SstartTime(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;
  char *valstring=XmTextGetString(sounds[pt].startText);
  double tid=atof(valstring);

  if (grabnum>-1) return;
  if (tid<0.) tid=0.;
  if (tid+sounds[pt].dur>MAXLEN) tid=MAXLEN-sounds[pt].dur;
  sounds[pt].startsec=tid;

  sprintf(streng, "%7.3f", tid);
  XbaeMatrixSetCell(soundMatrix,sounds[pt].posinmatrix,1,streng);
  sprintf(streng, "%7.3f", sounds[pt].startsec+sounds[pt].dur);
  XtVaSetValues(sounds[pt].stopText,XmNvalue,streng,NULL);
  XbaeMatrixSetCell(soundMatrix,sounds[pt].posinmatrix,2,streng);
  RedrawTrack(sounds[pt].track); 
}

void STrack(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;
  char *valstring=XmTextGetString(sounds[pt].trackText);
  int track=atoi(valstring), tb;

  if (grabnum>-1) return;
  if (track<1) track=1;
  if (track>9) track=9;
  tb=sounds[pt].track;
  sounds[pt].track=track;
  RedrawTrack(tb); RedrawTrack(track); 

  sprintf(streng, "%d", track);
  XbaeMatrixSetCell(soundMatrix,sounds[pt].posinmatrix,3,streng);
}

void SLoop(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;
  char *valstring=XmTextGetString(sounds[pt].loopText);
  int loop=atoi(valstring);

  backup();
  if (loop<1) loop=1;
  if (loop>999) loop=999;
  sounds[pt].loop=loop;
  RedrawTrack(sounds[pt].track);
}

void ProcessChosen(w, client, call)
Widget w;
XtPointer client, call;
{
  int n=(int)client;
  int pane, snd;
  char comm[200];
  
  pane=n&31; snd=n>>5;
  sprintf(comm, processStrings[pane], sounds[snd].filename);
  strcat(comm, " &");
  system(comm);
}

void SPostMenu(w, client, event)
Widget w;
XtPointer client;
XEvent *event;
{
  Widget popup = (Widget)client;
  XmMenuPosition(popup, (XButtonPressedEvent *)event);
  XtManageChild(popup);
}

void MakeSoundWidgets(pt)
int pt;
{
  XmString text;

  if (sounds[pt].wmade) return;
  sounds[pt].wmade=1;
  sounds[pt].bulletin=XmCreateBulletinBoardDialog(mainWindow,"trackBB",NULL,0);
  sprintf(streng, "#%d:  %s", pt, sounds[pt].filename);
  text=XmStringCreateSimple(streng);
  XtVaSetValues(sounds[pt].bulletin,XmNdialogTitle,text,XmNautoUnmanage,False,
    XmNwidth,500,XmNheight,250,NULL);
  sounds[pt].muteB=XtVaCreateManagedWidget("muteB",xmPushButtonWidgetClass,
    sounds[pt].bulletin,XmNlabelType,XmPIXMAP,XmNlabelPixmap,nomutepixmap,
    XmNx,10,XmNy,20,XmNwidth,120,NULL);
  XtAddCallback(sounds[pt].muteB,XmNactivateCallback,Smute,&sounds[pt].dummy);

  XtFree(text); text=XmStringCreateSimple("Play");
  sounds[pt].playB=XtVaCreateManagedWidget("playB",xmPushButtonWidgetClass,
    sounds[pt].bulletin,XmNx,10,XmNy,60,XmNwidth,120,XmNlabelString,text,NULL);
  XtVaSetValues(sounds[pt].playB,XmNrecomputeSize,False,NULL);
  XtAddCallback(sounds[pt].playB,XmNactivateCallback,Splay,&sounds[pt].dummy);

  XtFree(text); text=XmStringCreateSimple("Delete");
  sounds[pt].deleteB=XtVaCreateManagedWidget("deleteB",xmPushButtonWidgetClass,
    sounds[pt].bulletin,XmNx,10,XmNy,100,XmNwidth,120,XmNlabelString,text,NULL);
  XtAddCallback(sounds[pt].deleteB,XmNactivateCallback,Sdelete,
    &sounds[pt].dummy);

  XtFree(text); text=XmStringCreateSimple("Duplicate");
  sounds[pt].duplicateB=XtVaCreateManagedWidget("duplicateB",xmPushButtonWidgetClass,
    sounds[pt].bulletin,XmNx,10,XmNy,140,XmNwidth,120,XmNlabelString,text,NULL);
  XtAddCallback(sounds[pt].duplicateB,XmNactivateCallback,Sduplicate,
    &sounds[pt].dummy);

  XtFree(text); text=XmStringCreateSimple("Edit");
  sounds[pt].editB=XtVaCreateManagedWidget("editB",xmPushButtonWidgetClass,
    sounds[pt].bulletin,XmNx,150,XmNy,20,XmNwidth,120,XmNlabelString,text,NULL);
  XtAddCallback(sounds[pt].editB,XmNactivateCallback,Sedit,&sounds[pt].dummy);



  XtFree(text); text=XmStringCreateSimple("Process");
  sounds[pt].processB=XtVaCreateManagedWidget("processB",xmPushButtonWidgetClass,
    sounds[pt].bulletin,XmNx,150,XmNy,60,XmNwidth,120,XmNlabelString,text,NULL);
  XtFree(text);

  sounds[pt].processM=XmCreatePopupMenu(sounds[pt].processB,"processMenu",
    NULL,0);
  XtVaSetValues(sounds[pt].processM, XmNmenuPost, "<Btn1Down>", NULL);
  XtAddEventHandler(sounds[pt].processB, ButtonPressMask, False, SPostMenu,
    sounds[pt].processM);

  {
    FILE *fil;
    char mlabel[100];
    int i;

    fil=fopen(app_data.processmenu_file, "r");
    if (fil!=NULL) {
      for (i=0; i<30; i++) {
        fgets(mlabel, 99, fil); mlabel[strlen(mlabel)-1]='\0';
        if (feof(fil)) break;
        fgets(processStrings[i], 99, fil);
        processStrings[i][strlen(processStrings[i])-1]='\0';
        if (feof(fil)) break;
        text=XmStringCreateSimple(mlabel);
        sounds[pt].processMB[i]=XtVaCreateManagedWidget(
          "processMB",xmPushButtonWidgetClass,sounds[pt].processM,
          XmNlabelString, text,NULL);
        XtFree(text);
        XtAddCallback(sounds[pt].processMB[i], XmNactivateCallback,
          ProcessChosen, (XtPointer)((pt<<5)+i));
      }
      fclose(fil);
    }
  }

/*  XtVaSetValues(sounds[pt].processB, XmNsensitive, False, NULL); */



  text=XmStringCreateSimple("Refresh");
  sounds[pt].refreshB=XtVaCreateManagedWidget("refreshB",xmPushButtonWidgetClass,
    sounds[pt].bulletin,XmNx,150,XmNy,100,XmNwidth,120,XmNlabelString,text,NULL);
  XtAddCallback(sounds[pt].refreshB,XmNactivateCallback,Srefresh,&sounds[pt].dummy);

  XtFree(text); text=XmStringCreateSimple("Lock");
  sounds[pt].lockToggle=XtVaCreateManagedWidget("lockToggle",xmToggleButtonWidgetClass,
    sounds[pt].bulletin,XmNx,180,XmNy,148,XmNlabelString,text,NULL);
  XtAddCallback(sounds[pt].lockToggle,XmNvalueChangedCallback,Slock,&sounds[pt].dummy);

  sprintf(streng, "%7.3f", sounds[pt].startsec);
  sounds[pt].startText=XtVaCreateManagedWidget("startText",
    xmTextFieldWidgetClass,sounds[pt].bulletin,XmNwidth,100,XmNx,390,XmNy,20,
    XmNvalue,streng,NULL);
  XtAddCallback(sounds[pt].startText,XmNvalueChangedCallback,SstartTime,
    &sounds[pt].dummy);
  XtFree(text); text=XmStringCreateLocalized("Start:");
  sounds[pt].startLabel=XtVaCreateManagedWidget("startLabel",xmLabelWidgetClass,
    sounds[pt].bulletin,XmNlabelString,text,XmNx,300,XmNy,24,NULL);

  sprintf(streng, "%7.3f", sounds[pt].startsec+sounds[pt].dur);
  sounds[pt].stopText=XtVaCreateManagedWidget("stopText",
    xmTextFieldWidgetClass,sounds[pt].bulletin,XmNwidth,100,XmNx,390,XmNy,60,
    XmNvalue,streng,XmNeditable,False,NULL);
  XtFree(text); text=XmStringCreateLocalized("Stop:");
  sounds[pt].stopLabel=XtVaCreateManagedWidget("stopLabel",xmLabelWidgetClass,
    sounds[pt].bulletin,XmNlabelString,text,XmNx,300,XmNy,64,NULL);

  sprintf(streng, "%d", sounds[pt].track);
  sounds[pt].trackText=XtVaCreateManagedWidget("trackText",
    xmTextFieldWidgetClass,sounds[pt].bulletin,XmNwidth,100,XmNx,390,XmNy,100,
    XmNvalue,streng,NULL);
  XtAddCallback(sounds[pt].trackText,XmNvalueChangedCallback,STrack,
    &sounds[pt].dummy);
  XtFree(text); text=XmStringCreateLocalized("Track:");
  sounds[pt].trackLabel=XtVaCreateManagedWidget("trackLabel",xmLabelWidgetClass,
    sounds[pt].bulletin,XmNlabelString,text,XmNx,300,XmNy,104,NULL);

  sprintf(streng, "%d", sounds[pt].loop);
  sounds[pt].loopText=XtVaCreateManagedWidget("loopText",
    xmTextFieldWidgetClass,sounds[pt].bulletin,XmNwidth,100,XmNx,390,XmNy,140,
    XmNvalue,streng,NULL);
  XtAddCallback(sounds[pt].loopText,XmNvalueChangedCallback,SLoop,
    &sounds[pt].dummy);
  XtFree(text); text=XmStringCreateLocalized("Loop:");
  sounds[pt].loopLabel=XtVaCreateManagedWidget("loopLabel",xmLabelWidgetClass,
    sounds[pt].bulletin,XmNlabelString,text,XmNx,300,XmNy,144,NULL);

  XtFree(text); text=XmStringCreateSimple("Dismiss");
  sounds[pt].cancelB=XtVaCreateManagedWidget("cancelB",xmPushButtonWidgetClass,
    sounds[pt].bulletin,XmNx,200,XmNy,195,XmNwidth,120,XmNlabelString,text,NULL);
  XtAddCallback(sounds[pt].cancelB,XmNactivateCallback,Cancel,sounds[pt].bulletin);
  XtVaSetValues(sounds[pt].bulletin,XmNdefaultButton,sounds[pt].cancelB,NULL);

  XtFree(text);

}

void ManageSoundWidgets(pt)
int pt;
{
  XtManageChild(sounds[pt].bulletin);
}

void EVmute(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;

  if (events[pt].muted) {
    events[pt].muted=0;
    XtVaSetValues(w,XmNlabelPixmap,nomutepixmap,NULL);
  } else {
    events[pt].muted=1;
    XtVaSetValues(w,XmNlabelPixmap,mutepixmap,NULL);
  }
}

void deleteevent(pt)
int pt;
{
  String empty="";
  int i;

  events[pt].used=0;
  events[pt].wmade=0;
  if (events[pt].grouped) {
    numgrouped--;
    if (numgrouped==0) Desensgrp();
  }
  events[pt].grouped=0;
  XtDestroyWidget(events[pt].bulletin);
  XbaeMatrixSelectCell(eventMatrix,0,0);
  XbaeMatrixDeselectCell(eventMatrix,0,0);
  if (events_total>1) XbaeMatrixDeleteRows(eventMatrix,events[pt].posinmatrix,1);
  else {
    XbaeMatrixSetCell(eventMatrix,0,0,empty);
    XbaeMatrixSetCell(eventMatrix,0,1,empty);
    XbaeMatrixSetCell(eventMatrix,0,2,empty);
    XbaeMatrixSetCell(eventMatrix,0,3,empty);
    XtVaSetValues(eventMatrix,XmNrowLabels,rowlabels2,NULL);
  }
  for (i=0; i<MAXLYD; i++)
    if (events[i].posinmatrix>events[pt].posinmatrix)
      events[i].posinmatrix--;
  events_total--;
}

void EVdelete(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;

  backup();
  deleteevent(pt);
  RedrawTrack(events[pt].track);
}

void EVduplicate(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;
  backup();
  newevent(events[pt].name,events[pt].track,events[pt].startsec+1.,-1);
}

void EVstartTime(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;
  char *valstring=XmTextGetString(events[pt].startText);
  double tid=atof(valstring);

  if (evgrabnum>-1) return;
  if (tid<0.) tid=0.;
  if (tid>MAXLEN) tid=MAXLEN;
  events[pt].startsec=tid;

  sprintf(streng, "%6.2f", tid);
  XbaeMatrixSetCell(eventMatrix,events[pt].posinmatrix,1,streng);
  sprintf(streng, "%6.2f", events[pt].startsec+sounds[pt].dur);
  RedrawTrack(events[pt].track); 
}

void EVTrack(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;
  char *valstring=XmTextGetString(events[pt].trackText);
  int track=atoi(valstring), tb;

  if (evgrabnum>-1) return;
  if (track<1) track=1;
  if (track>9) track=9;
  tb=events[pt].track;
  events[pt].track=track;
  RedrawTrack(tb); RedrawTrack(track); 

  sprintf(streng, "%d", track);
  XbaeMatrixSetCell(eventMatrix,events[pt].posinmatrix,3,streng);
}

void EVvalue(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt=*(int *)client;
  char *valstring=XmTextGetString(events[pt].valueText);

  backup();
  sprintf(events[pt].value, "%s", valstring);
  XbaeMatrixSetCell(eventMatrix,events[pt].posinmatrix,2,valstring);
}

void MakeEventWidgets(pt)
int pt;
{
  XmString text;
  int name=events[pt].name;

  if (events[pt].wmade) return;
  events[pt].wmade=1;

  events[pt].bulletin=XmCreateBulletinBoardDialog(mainWindow,"evBB",NULL,0);
  sprintf(streng,"Variable %d: %s",pt,Enames[name]);
  text=XmStringCreateSimple(streng);
  XtVaSetValues(events[pt].bulletin,XmNdialogTitle,text,XmNautoUnmanage,False,
    XmNwidth,500,XmNheight,250,NULL);
  events[pt].muteB=XtVaCreateManagedWidget("muteB",xmPushButtonWidgetClass,
    events[pt].bulletin,XmNlabelType,XmPIXMAP,XmNlabelPixmap,nomutepixmap,
    XmNx,10,XmNy,20,XmNwidth,120,NULL);
  XtAddCallback(events[pt].muteB,XmNactivateCallback,EVmute,&events[pt].dummy);

  XtFree(text); text=XmStringCreateSimple("Delete");
  events[pt].deleteB=XtVaCreateManagedWidget("deleteB",xmPushButtonWidgetClass,
    events[pt].bulletin,XmNx,10,XmNy,100,XmNwidth,120,XmNlabelString,text,NULL);
  XtAddCallback(events[pt].deleteB,XmNactivateCallback,EVdelete,
    &events[pt].dummy);

  XtFree(text); text=XmStringCreateSimple("Duplicate");
  events[pt].duplicateB=XtVaCreateManagedWidget("duplicateB",xmPushButtonWidgetClass,
    events[pt].bulletin,XmNx,10,XmNy,140,XmNwidth,120,XmNlabelString,text,NULL);
  XtAddCallback(events[pt].duplicateB,XmNactivateCallback,EVduplicate,
    &events[pt].dummy);

  sprintf(streng, "%6.2f", events[pt].startsec);
  events[pt].startText=XtVaCreateManagedWidget("startText",
    xmTextFieldWidgetClass,events[pt].bulletin,XmNwidth,100,XmNx,390,XmNy,20,
    XmNvalue,streng,NULL);
  XtAddCallback(events[pt].startText,XmNvalueChangedCallback,EVstartTime,
    &events[pt].dummy);
  XtFree(text); text=XmStringCreateLocalized("Start:");
  events[pt].startLabel=XtVaCreateManagedWidget("startLabel",xmLabelWidgetClass,
    events[pt].bulletin,XmNlabelString,text,XmNx,300,XmNy,24,NULL);

  events[pt].valueText=XtVaCreateManagedWidget("valueText",
    xmTextFieldWidgetClass,events[pt].bulletin,XmNwidth,100,XmNx,390,XmNy,60,
    XmNvalue,events[pt].value,NULL);
  XtAddCallback(events[pt].valueText,XmNvalueChangedCallback,EVvalue,
    &events[pt].dummy);
  XtFree(text); text=XmStringCreateLocalized("Value:");
  events[pt].valueLabel=XtVaCreateManagedWidget("valueLabel",xmLabelWidgetClass,
    events[pt].bulletin,XmNlabelString,text,XmNx,300,XmNy,64,NULL);

  sprintf(streng, "%d", events[pt].track);
  events[pt].trackText=XtVaCreateManagedWidget("trackText",
    xmTextFieldWidgetClass,events[pt].bulletin,XmNwidth,100,XmNx,390,XmNy,100,
    XmNvalue,streng,NULL);
  XtAddCallback(events[pt].trackText,XmNvalueChangedCallback,EVTrack,
    &events[pt].dummy);
  XtFree(text); text=XmStringCreateLocalized("Track:");
  events[pt].trackLabel=XtVaCreateManagedWidget("trackLabel",xmLabelWidgetClass,
    events[pt].bulletin,XmNlabelString,text,XmNx,300,XmNy,104,NULL);

  XtFree(text); text=XmStringCreateSimple("Dismiss");
  events[pt].cancelB=XtVaCreateManagedWidget("cancelB",xmPushButtonWidgetClass,
    events[pt].bulletin,XmNx,200,XmNy,190,XmNwidth,120,XmNlabelString,text,NULL);
  XtAddCallback(events[pt].cancelB,XmNactivateCallback,Cancel,events[pt].bulletin);
  XtVaSetValues(events[pt].bulletin,XmNdefaultButton,events[pt].cancelB,NULL);

  XtFree(text);
}

void ManageEventWidgets(pt)
int pt;
{
  XtManageChild(events[pt].bulletin);
}

void Hslider(w, client, call)
Widget w;
XtPointer client, call;
{
  XmScrollBarCallbackStruct *calldata=(XmScrollBarCallbackStruct *) call;
  leftsec=(calldata->value)/10.;

  RedrawWin();
}

void Zoomin(w, client, call)
Widget w;
XtPointer client, call;
{
  if (sec_pr_scr>15./128.) {
    sec_pr_scr/=2.;
    XtVaSetValues(horiscroll,XmNsliderSize,(int)ceil(sec_pr_scr*10.),NULL);
    RedrawWin();
  }
  if (sec_pr_scr<60.) XtVaSetValues(horiscroll,XmNincrement,1,NULL);
}

void Zoomout(w, client, call)
Widget w;
XtPointer client, call;
{
  if (sec_pr_scr<MAXLEN) {
    sec_pr_scr*=2;
    if (leftsec+sec_pr_scr>MAXLEN) {
      leftsec=MAXLEN-sec_pr_scr;
      XtVaSetValues(horiscroll,XmNvalue,(int)(leftsec*10.),NULL);
    }
    XtVaSetValues(horiscroll,XmNsliderSize,(int)ceil(sec_pr_scr*10.),NULL);
    RedrawWin();
  }
  if (sec_pr_scr>=60.) XtVaSetValues(horiscroll,XmNincrement,10,NULL);
}

void Mute(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;

  if (tracks[i].muted) {
    tracks[i].muted=0;
    XtVaSetValues(w,XmNlabelPixmap,nomutepixmap,NULL);
  } else {
    tracks[i].muted=1;
    XtVaSetValues(w,XmNlabelPixmap,mutepixmap,NULL);
  }
}

void TrackZoom(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;

  if (i!=zoomtrack) zoomtrack=i;
    else zoomtrack=0;
  RedrawWin();
}

void VolScale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  double val;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;
  XmString text;

  sprintf(streng,"%d",calldata->value);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(w,XmNtitleString,text,NULL); XtFree(text);
  val=pow(10.,(calldata->value-60.)/24.)*32767.; if (val<105.) val=0.;
  tracks[i].volval=val;
}

void ZoomScale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  double val;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;

  tracks[i].zoomval=calldata->value*0.25;
  RedrawTrack(i);
}

void VolB(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;

  if (tracks[i].vol) {
    tracks[i].vol=0;
    XtVaSetValues(w,XmNlabelPixmap,novolpixmap,NULL);
  } else {
    tracks[i].vol=1;
    XtVaSetValues(w,XmNlabelPixmap,volpixmap,NULL);
    if (tracks[i].pan) {
      tracks[i].pan=0;
      XtVaSetValues(tracks[i].panB,XmNlabelPixmap,nopanpixmap,NULL);
    }
    if (tracks[i].aux) {
      tracks[i].aux=0;
      XtVaSetValues(tracks[i].auxB,XmNlabelPixmap,noauxpixmap,NULL);
    }
  }
  RedrawTrack(i);
}

void PanB(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;

  if (tracks[i].pan) {
    tracks[i].pan=0;
    if (i==10) XtVaSetValues(w,XmNlabelPixmap,noparpixmap,NULL);
      else XtVaSetValues(w,XmNlabelPixmap,nopanpixmap,NULL);
  } else {
    tracks[i].pan=1;
    if (i==10) XtVaSetValues(w,XmNlabelPixmap,parpixmap,NULL);
      else XtVaSetValues(w,XmNlabelPixmap,panpixmap,NULL);
    if (tracks[i].vol) {
      tracks[i].vol=0;
      XtVaSetValues(tracks[i].volB,XmNlabelPixmap,novolpixmap,NULL);
    }
    if (tracks[i].aux) {
      tracks[i].aux=0;
      XtVaSetValues(tracks[i].auxB,XmNlabelPixmap,noauxpixmap,NULL);
    }
  }
  RedrawTrack(i);
}

void AuxB(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;

  if (tracks[i].aux) {
    tracks[i].aux=0;
    XtVaSetValues(w,XmNlabelPixmap,noauxpixmap,NULL);
  } else {
    tracks[i].aux=1;
    XtVaSetValues(w,XmNlabelPixmap,auxpixmap,NULL);
    if (tracks[i].vol) {
      tracks[i].vol=0;
      XtVaSetValues(tracks[i].volB,XmNlabelPixmap,novolpixmap,NULL);
    }
    if (tracks[i].pan) {
      tracks[i].pan=0;
      XtVaSetValues(tracks[i].panB,XmNlabelPixmap,nopanpixmap,NULL);
    }
  }
  RedrawTrack(i);
}

void FbackB(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;

  if (tracks[i].pair==1) {
    tracks[i].pair=2;
    XtVaSetValues(w,XmNlabelPixmap,backpixmap,NULL);
  } else {
    tracks[i].pair=1;
    XtVaSetValues(w,XmNlabelPixmap,frontpixmap,NULL);
  }
}

void computegraph(int track, int type)
{
  int i, j, x, y, xold=-999999;
  double start, incr;
  short int val;

  XSetForeground(theDisplay, bitGC, gray.pixel);
  if (type==1) for (i=1; i<tracks[track].vnumsquare; i++) {
    start=(double)tracks[track].vypos[i];
    incr=((double)tracks[track].vypos[i+1]-start)/
      ((double)tracks[track].vxpos[i+1]-(double)tracks[track].vxpos[i]);
    for (j=tracks[track].vxpos[i]; j<=tracks[track].vxpos[i+1]; j++) {
      if (sett.linvol) val=start*546.1;
        else { val=exp(start*0.095941)*103.62; val&=(-(val>105)); }
      tracks[track].fullvol[j]=val;
      if (sett.showlin) {
        x=blk_to_pix(j);
        if ((x>0) && (x<1000) && (x!=xold) && (sett.linvol==0)) {
          y=track*70+30-tracks[track].fullvol[j]/546;
          XDrawLine(theDisplay,bitmap,bitGC,x,y,x,y);
          xold=x;
        }
      }
      start+=incr;
    }
    if (sett.showlin) RestoreWin(NULL,NULL,NULL);
  }
  else if (type==2) for (i=1; i<tracks[track].pnumsquare; i++) {
    start=(double)tracks[track].pypos[i];
    incr=546.1*((double)tracks[track].pypos[i+1]-start)/
      ((double)tracks[track].pxpos[i+1]-(double)tracks[track].pxpos[i]);
    start*=546.1;
    for (j=tracks[track].pxpos[i]; j<=tracks[track].pxpos[i+1]; j++) {
      tracks[track].fullpan[j]=(short int)start;
      start+=incr;
    }
  }
  else if (type==3) for (i=1; i<tracks[track].anumsquare; i++) {
    start=(double)tracks[track].aypos[i];
    incr=((double)tracks[track].aypos[i+1]-start)/
      ((double)tracks[track].axpos[i+1]-(double)tracks[track].axpos[i]);
    for (j=tracks[track].axpos[i]; j<=tracks[track].axpos[i+1]; j++) {
      if (sett.linvol) val=start*546.1;
        else { val=exp(start*0.095941)*103.62; val&=(-(val>105)); }
      tracks[track].fullaux[j]=val;
      if (sett.showlin) {
        x=blk_to_pix(j);
        if ((x>0) && (x<1000) && (x!=xold) && (sett.linvol==0)) {
           y=track*70+30-tracks[track].fullaux[j]/546;
           XDrawLine(theDisplay,bitmap,bitGC,x,y,x,y);
           xold=x;
        }
      }
      start+=incr;
    }
    if (sett.showlin) RestoreWin(NULL,NULL,NULL);
  }
}


void VolClick(x, y, val)
int x, y, val;
{
  int blk=pix_to_blk(x);
  int which=0, addpos, i, pt=seltrack;
  double time;
  XmString st;
  
  for (i=1; i<=tracks[pt].vnumsquare; i++)
    if ((abs(x-blk_to_pix(tracks[pt].vxpos[i]))<5) &&
        (abs(val-tracks[pt].vypos[i])<5) ) {
        which=i; break;
      }
  seltype=1;
  if (which>0) { selnum=which; RedrawWin(); return; }
  if (tracks[pt].vnumsquare==MAXNODE-1) return;
  for (i=1; i<=tracks[pt].vnumsquare; i++)
    if (blk<tracks[pt].vxpos[i]) { addpos=i; break; }
  tracks[pt].vnumsquare++;
  for (i=tracks[pt].vnumsquare; i>addpos; i--) {
    tracks[pt].vxpos[i]=tracks[pt].vxpos[i-1];
    tracks[pt].vypos[i]=tracks[pt].vypos[i-1];
  }
  tracks[pt].vxpos[addpos]=blk; tracks[pt].vypos[addpos]=val;
  selnum=addpos; RedrawWin(); dirtytrack=pt; dirtytype=1;
  time=(blk*1024./Srate);
  sprintf(streng,"Vol: %6.2fs  %2d",time,val);
  st=XmStringCreateLocalized(streng);
  XtVaSetValues(posL,XmNlabelString,st,NULL); XtFree(st);
}

void PanClick(x, y, val)
int x, y, val;
{
  int blk=pix_to_blk(x);
  int which=0, addpos, i, pt=seltrack;
  double time;
  XmString st;
  
  for (i=1; i<=tracks[pt].pnumsquare; i++)
    if ((abs(x-blk_to_pix(tracks[pt].pxpos[i]))<5) &&
        (abs(val-tracks[pt].pypos[i])<5) ) {
        which=i; break;
      }
  seltype=2;
  if (which>0) { selnum=which; RedrawWin(); return; }
  if (tracks[pt].pnumsquare==MAXNODE-1) return;
  for (i=1; i<=tracks[pt].pnumsquare; i++)
    if (blk<tracks[pt].pxpos[i]) { addpos=i; break; }
  tracks[pt].pnumsquare++;
  for (i=tracks[pt].pnumsquare; i>addpos; i--) {
    tracks[pt].pxpos[i]=tracks[pt].pxpos[i-1];
    tracks[pt].pypos[i]=tracks[pt].pypos[i-1];
  }
  tracks[pt].pxpos[addpos]=blk; tracks[pt].pypos[addpos]=val;
  selnum=addpos; RedrawWin(); dirtytrack=pt; dirtytype=2;
  time=(blk*1024./Srate);
  sprintf(streng,"Pan: %6.2fs  %2d",time,val);
  st=XmStringCreateLocalized(streng);
  XtVaSetValues(posL,XmNlabelString,st,NULL); XtFree(st);
}

void AuxClick(x, y, val)
int x, y, val;
{
  int blk=pix_to_blk(x);
  int which=0, addpos, i, pt=seltrack;
  double time;
  XmString st;

  for (i=1; i<=tracks[pt].anumsquare; i++)
    if ((abs(x-blk_to_pix(tracks[pt].axpos[i]))<5) &&
        (abs(val-tracks[pt].aypos[i])<5) ) {
        which=i; break;
      }
  seltype=3;
  if (which>0) { selnum=which; RedrawWin(); return; }
  if (tracks[pt].anumsquare==MAXNODE-1) return;
  for (i=1; i<=tracks[pt].anumsquare; i++)
    if (blk<tracks[pt].axpos[i]) { addpos=i; break; }
  tracks[pt].anumsquare++;
  for (i=tracks[pt].anumsquare; i>addpos; i--) {
    tracks[pt].axpos[i]=tracks[pt].axpos[i-1];
    tracks[pt].aypos[i]=tracks[pt].aypos[i-1];
  }
  tracks[pt].axpos[addpos]=blk; tracks[pt].aypos[addpos]=val;
  selnum=addpos; RedrawWin(); dirtytrack=pt; dirtytype=3;
  time=(blk*1024./Srate);
  sprintf(streng,"Aux: %6.2fs  %2d",time,val);
  st=XmStringCreateLocalized(streng);
  XtVaSetValues(posL,XmNlabelString,st,NULL); XtFree(st);
}

static void GraphClick(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, blk, val, strack;

  XButtonEvent *motion=(XButtonEvent *)event;
  x=motion->x; y=motion->y; xm=x; ym=y;
  if ((x<0) || (x>1000) || (y<35) || (y>735)) {
    selnum=0; RedrawTrack(seltrack); return;
  }
  strack=(y-35)/70+1;
  if (zoomtrack) strack=zoomtrack;
  blk=pix_to_blk(x);
  val=strack*70+30-y;
  if (val<0) val=0; if (val>60) val=60;
  if ((tracks[strack].vol==0) && (tracks[strack].pan==0) &&
    (tracks[strack].aux==0)) {
      selnum=0; RedrawTrack(seltrack); return;
  }
  backup();
  seltrack=strack;
  if (tracks[strack].vol) VolClick(x, y, val);
  else if (tracks[strack].pan) PanClick(x, y, val);
  else if (tracks[strack].aux) AuxClick(x, y, val);
  makelist(seltrack,seltype);
}


void VolDrag(x, y, val)
int x, y, val;
{
  int blk=pix_to_blk(x), pt=seltrack;
  double time;
  XmString st;

  if ((selnum==0) || (seltype!=1)) return;
  tracks[pt].vypos[selnum]=val;
  if (selnum==tracks[pt].vnumsquare-1) tracks[pt].vypos[selnum+1]=val;
  if ((selnum>1) && (selnum<tracks[pt].vnumsquare)) {
    if (blk<=tracks[pt].vxpos[selnum-1]) blk=tracks[pt].vxpos[selnum-1]+1;
    if (blk>=tracks[pt].vxpos[selnum+1]) blk=tracks[pt].vxpos[selnum+1]-1;
    tracks[pt].vxpos[selnum]=blk;
  }

  RedrawTrack(pt); dirtytrack=pt; dirtytype=1;
  makelist(pt,1);
  time=(blk*1024./Srate);
  sprintf(streng,"Vol: %6.2fs  %2d",time,val);
  st=XmStringCreateLocalized(streng);
  XtVaSetValues(posL,XmNlabelString,st,NULL); XtFree(st);
}

void PanDrag(x, y, val)
int x, y, val;
{
  int blk=pix_to_blk(x), pt=seltrack;
  double time;
  XmString st;

  if ((selnum==0) || (seltype!=2)) return;
  tracks[pt].pypos[selnum]=val;
  if (selnum==tracks[pt].pnumsquare-1) tracks[pt].pypos[selnum+1]=val;
  if ((selnum>1) && (selnum<tracks[pt].pnumsquare)) {
    if (blk<=tracks[pt].pxpos[selnum-1]) blk=tracks[pt].pxpos[selnum-1]+1;
    if (blk>=tracks[pt].pxpos[selnum+1]) blk=tracks[pt].pxpos[selnum+1]-1;
    tracks[pt].pxpos[selnum]=blk;
  }

  RedrawTrack(pt); dirtytrack=pt; dirtytype=2;
  makelist(pt,2);
  time=(blk*1024./Srate);
  sprintf(streng,"Pan: %6.2fs  %2d",time,val);
  st=XmStringCreateLocalized(streng);
  XtVaSetValues(posL,XmNlabelString,st,NULL); XtFree(st);
}

void AuxDrag(x, y, val)
int x, y, val;
{
  int blk=pix_to_blk(x), pt=seltrack;
  double time;
  XmString st;

  if ((selnum==0) || (seltype!=3)) return;
  tracks[pt].aypos[selnum]=val;
  if (selnum==tracks[pt].anumsquare-1) tracks[pt].aypos[selnum+1]=val;
  if ((selnum>1) && (selnum<tracks[pt].anumsquare)) {
    if (blk<=tracks[pt].axpos[selnum-1]) blk=tracks[pt].axpos[selnum-1]+1;
    if (blk>=tracks[pt].axpos[selnum+1]) blk=tracks[pt].axpos[selnum+1]-1;
    tracks[pt].axpos[selnum]=blk;
  }

  RedrawTrack(pt); dirtytrack=pt; dirtytype=3;
  makelist(pt,3);
  time=(blk*1024./Srate);
  sprintf(streng,"Aux: %6.2fs  %2d",time,val);
  st=XmStringCreateLocalized(streng);
  XtVaSetValues(posL,XmNlabelString,st,NULL); XtFree(st);
}

static void GraphDrag(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, blk, val;
  char str[10];

  XButtonEvent *motion=(XButtonEvent *)event;
  x=motion->x; y=motion->y;
  if (x<0) x=0; if (x>1000) x=1000; if (y<35) y=35; if (y>735) y=735;
  xm=x; ym=y;
  val=seltrack*70+30-y;
  if (val<0) val=0; if (val>60) val=60;
  if (tracks[seltrack].vol) VolDrag(x, y, val);
  if (tracks[seltrack].pan) PanDrag(x, y, val);
  if (tracks[seltrack].aux) AuxDrag(x, y, val);

}

static void GraphDrop(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  XmString st;

  if (dirtytrack) computegraph(dirtytrack,dirtytype); dirtytrack=0;
  sprintf(streng,"               "); st=XmStringCreateLocalized(streng);
  XtVaSetValues(posL,XmNlabelString,st,NULL); XtFree(st);
}

void VolDelete(void)
{
  int i, pt=seltrack;

  if ((selnum==1) || (selnum==tracks[pt].vnumsquare)) return;
  for (i=selnum; i<tracks[pt].vnumsquare; i++) {
    tracks[pt].vxpos[i]=tracks[pt].vxpos[i+1];
    tracks[pt].vypos[i]=tracks[pt].vypos[i+1];
  }
  tracks[pt].vnumsquare--; selnum--;
  RedrawTrack(pt); dirtytrack=pt; dirtytype=1;
  computegraph(dirtytrack,dirtytype); dirtytrack=0;
  makelist(pt,1);
}

void PanDelete(void)
{
  int i, pt=seltrack;

  if ((selnum==1) || (selnum==tracks[pt].pnumsquare)) return;
  for (i=selnum; i<tracks[pt].pnumsquare; i++) {
    tracks[pt].pxpos[i]=tracks[pt].pxpos[i+1];
    tracks[pt].pypos[i]=tracks[pt].pypos[i+1];
  }
  tracks[pt].pnumsquare--; selnum--;
  RedrawTrack(pt); dirtytrack=pt; dirtytype=2;
  computegraph(dirtytrack,dirtytype); dirtytrack=0;
  makelist(pt,2);
}

void AuxDelete(void)
{
  int i, pt=seltrack;

  if ((selnum==1) || (selnum==tracks[pt].anumsquare)) return;
  for (i=selnum; i<tracks[pt].anumsquare; i++) {
    tracks[pt].axpos[i]=tracks[pt].axpos[i+1];
    tracks[pt].aypos[i]=tracks[pt].aypos[i+1];
  }
  tracks[pt].anumsquare--; selnum--;
  RedrawTrack(pt); dirtytrack=pt; dirtytype=3;
  computegraph(dirtytrack,dirtytype); dirtytrack=0;
  makelist(pt,3);
}

static void GraphDelete(w, client, call)
Widget w;
XtPointer client, call;
{
  if (selnum==0) return;
  backup();
  if (seltype==1) VolDelete();
  if (seltype==2) PanDelete();
  if (seltype==3) AuxDelete();
}

static void DrawingSelect(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, track, i, pt=-1;
  double tid;

  XButtonEvent *motion=(XButtonEvent *)event;
  x=motion->x; y=motion->y;
  if ((x<0) || (x>1000) || (y<35) || (y>665)) return;
  track=(y-35)/70+1;
  if (zoomtrack) track=zoomtrack;
  tid=(x/1000.)*sec_pr_scr+leftsec;

  for (i=0; i<MAXLYD; i++)
    if ((events[i].used) && (track==events[i].track) && (tid>events[i].startsec)
      && (tid<events[i].startsec+sec_pr_scr/(1000./20.))) { pt=i; break; }
  if (pt>-1) {
    ManageEventWidgets(pt); return;
  }
  for (i=0; i<MAXLYD; i++)
    if ((sounds[i].used) && (track==sounds[i].track) && (tid>sounds[i].startsec)
      && (tid<sounds[i].startsec+sounds[i].loop*sounds[i].dur))
        { pt=i; break; }
  if (pt==-1) return;
  ManageSoundWidgets(pt);
}


static void DrawingDrag(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, track, track2, i, any=0;
  double tid, tid2;
  XEvent evnt, lastevnt;
  XmString st;

  XMotionEvent *motion=(XMotionEvent *)event;
  x=motion->x; y=motion->y;
  if (x<0) x=0; if (x>1000) x=1000;
  tid=(x/1000.)*sec_pr_scr+leftsec;
  if (realtiming || saving) return;
  if (marking) {
    XSetForeground(theDisplay, bitGC, markcolor.pixel^backcolor.pixel);
    XSetFunction(theDisplay,bitGC,GXxor);
    if (x>xold)
      XFillRectangle(theDisplay, bitmap, bitGC, xold, 35, x-xold, 700);
    else XFillRectangle(theDisplay, bitmap, bitGC, x, 35, xold-x, 700);
    XSetFunction(theDisplay,bitGC,GXcopy);
    RestoreWin(NULL,NULL,NULL);
    xold=x; mstop=tid;
    sprintf(streng,"Time: %6.2fs",tid);
    st=XmStringCreateLocalized(streng);
    XtVaSetValues(posL,XmNlabelString,st,NULL); XtFree(st);
    return;
  }
  if ((grabnum==-1) && (evgrabnum==-1)) return;
  if (y<35) y=35; if (y>664) y=664;
  track=(y-35)/70+1;
  if (zoomtrack) track=zoomtrack;
  tid-=graboffset;
  if (sett.autoquant) {
    double beat;
    if (sett.bpmscale) {
      beat=floor(sett.bpm*tid*sett.quant/60.+0.5)/sett.quant;
      tid=60.*beat/sett.bpm;
    } else tid=floor(tid*sett.quant+0.5)/sett.quant;
  }
  if (grabnum>-1) for (i=0; i<MAXLYD; i++) {
    if (((i==grabnum) || ((sounds[grabnum].grouped) && (sounds[i].grouped)))
     && (sounds[i].lock==0)) {
      tid2=tid+sounds[i].groupdiff;
      if (tid2<0.) tid2=0.;
      if (tid2+sounds[i].dur>MAXLEN) tid2=MAXLEN-sounds[i].dur;
      track2=track+sounds[i].groupdifftrack;
      if (track2<1) track2=1; if (track2>9) track2=9;
      XSetFunction(theDisplay, bitGC, GXxor);
      DrawSound(i); 
      sounds[i].track=track2;
      sounds[i].startsec=tid2;
      DrawSound(i);
      XSetFunction(theDisplay, bitGC, GXcopy);
      sprintf(streng, "%7.3f", tid2);
      XtVaSetValues(sounds[i].startText,XmNvalue,streng,NULL);
      XbaeMatrixSetCell(soundMatrix,sounds[i].posinmatrix,1,streng);
      sprintf(streng, "%7.3f", sounds[i].startsec+sounds[i].dur);
      XtVaSetValues(sounds[i].stopText,XmNvalue,streng,NULL);
      XbaeMatrixSetCell(soundMatrix,sounds[i].posinmatrix,2,streng);
      sprintf(streng, "%d", track2);
      XtVaSetValues(sounds[i].trackText,XmNvalue,streng,NULL);
      XbaeMatrixSetCell(soundMatrix,sounds[i].posinmatrix,3,streng);
    }
  } else for (i=0; i<MAXLYD; i++) {
    if ((i==evgrabnum) || ((events[evgrabnum].grouped) && (events[i].grouped))) {
      tid2=tid+events[i].groupdiff;
      if (tid2<0.) tid2=0.;
      if (tid2>MAXLEN) tid2=MAXLEN;
      track2=track+events[i].groupdifftrack;
      if (track2<1) track2=1; if (track2>9) track2=9;
      XSetFunction(theDisplay, bitGC, GXxor);
      DrawEvent(i); 
      events[i].track=track2;
      events[i].startsec=tid2;
      DrawEvent(i);
      XSetFunction(theDisplay, bitGC, GXcopy);
      sprintf(streng, "%6.2f", tid2);
      XtVaSetValues(events[i].startText,XmNvalue,streng,NULL);
      XbaeMatrixSetCell(eventMatrix,events[i].posinmatrix,1,streng);
      sprintf(streng, "%d", track2);
      XtVaSetValues(events[i].trackText,XmNvalue,streng,NULL);
      XbaeMatrixSetCell(eventMatrix,sounds[i].posinmatrix,3,streng);
    }
  }
  RestoreWin(NULL,NULL,NULL);
  while (XtAppPending(app_context)) {
      XtAppNextEvent(app_context, &evnt);
      if (evnt.type==MotionNotify) {
	any=1; memcpy(&lastevnt,&evnt,sizeof(XEvent));
      } else XtDispatchEvent(&evnt);
  }
  if (any) XtDispatchEvent(&lastevnt);
}

static void DrawingGrab(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, track, i;
  double tid;
  XButtonEvent *motion=(XButtonEvent *)event;

  grabnum=evgrabnum=-1;
  if (realtiming || saving) return;
  x=motion->x; y=motion->y;
  if ((x<0) || (x>1000) || (y>735)) return;
  tid=(x/1000.)*sec_pr_scr+leftsec;
  if ((y<35) || (y>665)) {
    if (marked) { marked=0; RedrawWin(); }
    marking=1; mstart=tid; xold=x; return;
  }
  marking=0;
  track=(y-35)/70+1;
  if (zoomtrack) track=zoomtrack;
  for (i=0; i<MAXLYD; i++)
    if ((events[i].used) && (track==events[i].track) && (tid>events[i].startsec)
      && (tid<events[i].startsec+(sec_pr_scr/(1000./20.)))) { evgrabnum=i; break; }
  if (evgrabnum>-1) {
    backup();
    graboffset=tid-events[evgrabnum].startsec;
    SetGrabCursor(topLevel);
    events[evgrabnum].nodraw=1;
    if (events[evgrabnum].grouped) for (i=0; i<MAXLYD; i++) {
      if (events[i].grouped) {
        events[i].nodraw=1;
        events[i].groupdiff=events[i].startsec-events[evgrabnum].startsec;
        events[i].groupdifftrack=events[i].track-events[evgrabnum].track;
      }
    }
    RedrawWin();
    for (i=0; i<MAXLYD; i++) events[i].nodraw=0;
    XSetFunction(theDisplay, bitGC, GXxor);
    if (events[evgrabnum].grouped) for (i=0; i<MAXLYD; i++) {
      if (events[i].grouped) DrawEvent(i);
    } else DrawEvent(evgrabnum);
  } else {
    for (i=0; i<MAXLYD; i++)
      if ((sounds[i].used) && (track==sounds[i].track)
        && (tid>sounds[i].startsec)
        && (tid<sounds[i].startsec+sounds[i].loop*sounds[i].dur))
          { grabnum=i; break; }
    if (grabnum==-1) {
      if (marked) { marked=0; RedrawWin(); }
      marking=1; mstart=tid; xold=x; return;
    }
    if (sounds[grabnum].lock) { grabnum=-1; return; }
    backup();
    graboffset=tid-sounds[grabnum].startsec;
    SetGrabCursor(topLevel);
    sounds[grabnum].nodraw=1;
    if (sounds[grabnum].grouped) for (i=0; i<MAXLYD; i++) {
      if (sounds[i].grouped) {
        sounds[i].nodraw=1;
        sounds[i].groupdiff=sounds[i].startsec-sounds[grabnum].startsec;
        sounds[i].groupdifftrack=sounds[i].track-sounds[grabnum].track;
      }
    }
    RedrawWin();
    for (i=0; i<MAXLYD; i++) sounds[i].nodraw=0;
    XSetFunction(theDisplay, bitGC, GXxor);
    if (sounds[grabnum].grouped) for (i=0; i<MAXLYD; i++) {
      if (sounds[i].grouped) DrawSound(i);
    } else DrawSound(grabnum);
  }
  RestoreWin(NULL,NULL,NULL);
  XSetFunction(theDisplay, bitGC, GXcopy);
}

static void DrawingDrop(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  XButtonEvent *motion=(XButtonEvent *)event;
  int x=motion->x;
  double tid;
  XmString st;

  if (x<0) x=0; if (x>1000) x=1000;
  tid=(x/1000.)*sec_pr_scr+leftsec;
  if (marking) {
    marking=0; mstop=tid;
    if (mstop<mstart) { mstop=mstart; mstart=tid; }
    if (mstart!=mstop) marked=1; else marked=0;
    st=XmStringCreateLocalized("             ");
    XtVaSetValues(posL,XmNlabelString,st,NULL); XtFree(st);
    return;
  }
  if ((grabnum==-1) && (evgrabnum==-1)) return;
  RedrawWin();
  ResetCursor(topLevel);
  grabnum=-1; evgrabnum=-1;
}

static void RegionDrag(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, track, i;
  double tid, tid2;
  XEvent evnt, lastevnt;
  XmString st;

  XMotionEvent *motion=(XMotionEvent *)event;
  x=motion->x; y=motion->y;
  if (x<0) x=0; if (x>1000) x=1000;
  tid=(x/1000.)*sec_pr_scr+leftsec;
  if (tid<sounds[regnum].startsec) {
    tid=sounds[regnum].startsec;
    x=1000.*(tid-leftsec)/sec_pr_scr;
  }
  if (tid>sounds[regnum].startsec+sounds[regnum].dur) {
    tid=sounds[regnum].startsec+sounds[regnum].dur;
    x=1000.*(tid-leftsec)/sec_pr_scr+1;
  }
  if (realtiming || saving) return;
  if (regmarking) {
    track=sounds[regnum].track;
    XSetForeground(theDisplay, bitGC, markcolor.pixel^backcolor.pixel);
    XSetFunction(theDisplay,bitGC,GXxor);
    if (x>xold)
      XFillRectangle(theDisplay, bitmap, bitGC, xold, track*70-34, x-xold, 69);
    else XFillRectangle(theDisplay, bitmap, bitGC, x, track*70-34, xold-x, 69);
    XSetFunction(theDisplay,bitGC,GXcopy);
    RestoreWin(NULL,NULL,NULL);
    xold=x; regmstop=tid;
    sprintf(streng,"Time: %6.2fs",tid);
    st=XmStringCreateLocalized(streng);
    XtVaSetValues(posL,XmNlabelString,st,NULL); XtFree(st);
  }
}

static void RegionGrab(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, track, i;
  double tid;
  XButtonEvent *motion=(XButtonEvent *)event;

  regnum=-1;
  if (realtiming || saving) return;
  x=motion->x; y=motion->y;
  if ((x<0) || (x>1000) || (y>735)) return;
  tid=(x/1000.)*sec_pr_scr+leftsec;
  if ((y<35) || (y>665)) {
    if (regmarked) { regmarked=0; RedrawWin(); } return;
  }
  regmarking=0;
  track=(y-35)/70+1;
  if (zoomtrack) track=zoomtrack;

  for (i=0; i<MAXLYD; i++)
    if ((sounds[i].used) && (track==sounds[i].track)
      && (tid>sounds[i].startsec)
      && (tid<sounds[i].startsec+sounds[i].loop*sounds[i].dur))
        { regnum=i; break; }
  if (regnum>-1) {
    if (regmarked) { regmarked=0; RedrawWin(); }
    regmarking=1; regmstart=tid; xold=x; return;
    regoffset=tid-sounds[grabnum].startsec;
  }
}

static void RegionDrop(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  XButtonEvent *motion=(XButtonEvent *)event;
  int x=motion->x;
  double tid;
  XmString st;

  if (x<0) x=0; if (x>1000) x=1000;
  tid=(x/1000.)*sec_pr_scr+leftsec;
  if (regmarking) {
    regmarking=0; regmstop=tid;
    if (regmstop<regmstart) { regmstop=regmstart; regmstart=tid; }
    if (regmstart!=regmstop) regmarked=1; else regmarked=0;
    st=XmStringCreateLocalized("             ");
    XtVaSetValues(posL,XmNlabelString,st,NULL); XtFree(st);
    return;
  }
  if (regnum==-1) return;
  RedrawWin();
  regnum=-1;
}

static void Group(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  int x, y, track, i, pt=-1;
  double tid;

  XButtonEvent *motion=(XButtonEvent *)event;
  x=motion->x; y=motion->y;
  if ((x<0) || (x>1000) || (y<35) || (y>665)) return;
  track=(y-35)/70+1;
  if (zoomtrack) track=zoomtrack;
  tid=(x/1000.)*sec_pr_scr+leftsec;
  for (i=0; i<MAXLYD; i++)
    if ((events[i].used) && (track==events[i].track) && (tid>events[i].startsec)
      && (tid<events[i].startsec+sec_pr_scr/(1000./20))) { pt=i; break; }
  if (pt>-1) {
    if (events[pt].grouped) {
      events[pt].grouped=0;
      events[pt].groupdiff=0.;
      events[pt].groupdifftrack=0;
      XbaeMatrixDeselectCell(eventMatrix,events[pt].posinmatrix,0);
      numgrouped--;
      if (numgrouped==0) Desensgrp();
    } else {
      events[pt].grouped=1;
      XbaeMatrixSelectCell(eventMatrix,events[pt].posinmatrix,0);
      numgrouped++;
      Sensgrp();
    }
    RedrawTrack(events[pt].track);
    return;
  }
  for (i=0; i<MAXLYD; i++)
    if ((sounds[i].used) && (track==sounds[i].track) && (tid>sounds[i].startsec)
      && (tid<sounds[i].startsec+sounds[i].dur)) { pt=i; break; }
  if (pt==-1) return;
  if (sounds[pt].grouped) {
    sounds[pt].grouped=0;
    sounds[pt].groupdiff=0.;
    sounds[pt].groupdifftrack=0;
    XbaeMatrixDeselectCell(soundMatrix,sounds[pt].posinmatrix,0);
    numgrouped--;
    if (numgrouped==0) Desensgrp();
  } else {
    sounds[pt].grouped=1;
    XbaeMatrixSelectCell(soundMatrix,sounds[pt].posinmatrix,0);
    numgrouped++;
    Sensgrp();
  }
  RedrawTrack(sounds[pt].track);
}


void ShowHelp(w, client, call)
Widget w;
XtPointer client, call;
{
  Widget dialog=(Widget) client;
  XtManageChild(dialog);
}

void newsamp(R)
int R;
{
  int i, j;
  if (Srate!=R) {
    for (i=1; i<11; i++) {
      for (j=2; j<=tracks[i].vnumsquare; j++)
        tracks[i].vxpos[j]=(int)(tracks[i].vxpos[j]*(double)R/Srate);
      tracks[i].vxpos[tracks[i].vnumsquare]=(int)(MAXLEN*R/1024.);
      computegraph(i,1);
      for (j=2; j<=tracks[i].pnumsquare; j++)
        tracks[i].pxpos[j]=(int)(tracks[i].pxpos[j]*(double)R/Srate);
      tracks[i].pxpos[tracks[i].pnumsquare]=(int)(MAXLEN*R/1024.);
      computegraph(i,2);
      for (j=2; j<=tracks[i].anumsquare; j++)
        tracks[i].axpos[j]=(int)(tracks[i].axpos[j]*(double)R/Srate);
      tracks[i].axpos[tracks[i].anumsquare]=(int)(MAXLEN*R/1024.);
      computegraph(i,3);
    }
  }
  Srate=R; RedrawWin();
}

void Initvalues(int fully)
{
  int i, j;
  String empty="";
  XmString text;

  XbaeMatrixSelectCell(soundMatrix,0,0);
  XbaeMatrixDeselectCell(soundMatrix,0,0);
  for (i=0; i<MAXLYD; i++) {
    if (sounds[i].used) {
      free(sounds[i].summary);
      if (num_total>1)
/*        XbaeMatrixDeleteRows(soundMatrix,sounds[i].posinmatrix,1); */
      XbaeMatrixDeleteRows(soundMatrix,1,1);
      num_total--;
/*      for (j=0; j<MAXLYD; j++)
        if (sounds[j].posinmatrix>sounds[i].posinmatrix)
          sounds[j].posinmatrix--; */
    }
    sounds[i].used=0;
    sounds[i].muted=0;
    if (sounds[i].wmade) XtDestroyWidget(sounds[i].bulletin);
    sounds[i].wmade=0;
    sounds[i].nodraw=0;
    sounds[i].dummy=i;
    sounds[i].grouped=0;
    sounds[i].loop=0;
  }
  XbaeMatrixSelectCell(eventMatrix,0,0);
  XbaeMatrixDeselectCell(eventMatrix,0,0);
  for (i=0; i<MAXLYD; i++) {
    if (events[i].used) {
      if (events_total>1)
/*        XbaeMatrixDeleteRows(eventMatrix,events[i].posinmatrix,1); */
        XbaeMatrixDeleteRows(eventMatrix,1,1);
      events_total--;
/*      for (j=0; j<MAXLYD; j++)
        if (events[j].posinmatrix>events[i].posinmatrix)
          events[j].posinmatrix--; */
    }
    events[i].used=0;
    events[i].muted=0;
    if (events[i].wmade) XtDestroyWidget(events[i].bulletin);
    events[i].wmade=0;
    events[i].nodraw=0;
    events[i].dummy=i;
    events[i].grouped=0;
  }
  numgrouped=0; Desensgrp();
  XbaeMatrixSetCell(soundMatrix,0,0,empty);
  XbaeMatrixSetCell(soundMatrix,0,1,empty);
  XbaeMatrixSetCell(soundMatrix,0,2,empty);
  XbaeMatrixSetCell(soundMatrix,0,3,empty);
  XtVaSetValues(soundMatrix,XmNrowLabels,rowlabels,NULL);
  XbaeMatrixSetCell(eventMatrix,0,0,empty);
  XbaeMatrixSetCell(eventMatrix,0,1,empty);
  XbaeMatrixSetCell(eventMatrix,0,2,empty);
  XbaeMatrixSetCell(eventMatrix,0,3,empty);
  XtVaSetValues(eventMatrix,XmNrowLabels,rowlabels2,NULL);
  for (i=1; i<11; i++) {
    tracks[i].muted=0;
    tracks[i].pan=0;
    tracks[i].vol=0;
    tracks[i].aux=0;
    tracks[i].pair=1;
    XtVaSetValues(tracks[i].volB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
      novolpixmap,NULL);
    XtVaSetValues(tracks[i].auxB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
      noauxpixmap,NULL);
    if (i<10) {
      XtVaSetValues(tracks[i].muteB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
        nomutepixmap,NULL);
      XtVaSetValues(tracks[i].panB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
        nopanpixmap,NULL);
      XtVaSetValues(tracks[i].fbackB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
        frontpixmap,XmNsensitive,False,NULL);
    }
    tracks[i].volval=32767;
    XtVaSetValues(tracks[i].volScale,XmNvalue,60,NULL);
    text=XmStringCreateLocalized("60");
    XtVaSetValues(tracks[i].volScale,XmNtitleString,text,NULL); XtFree(text);
    tracks[i].zoomval=1.;
    XtVaSetValues(tracks[i].zoomScale,XmNvalue,4,NULL);
    tracks[i].vnumsquare=2;
    tracks[i].pnumsquare=2;
    tracks[i].anumsquare=2;
    tracks[i].vxpos[1]=0;
    tracks[i].vypos[1]=60;
    tracks[i].vxpos[2]=(int)(MAXLEN*Srate/1024.);
    tracks[i].vypos[2]=60;
    tracks[i].pxpos[1]=0;
    tracks[i].pypos[1]=30;
    tracks[i].pxpos[2]=(int)(MAXLEN*Srate/1024.); 
    tracks[i].pypos[2]=30;
    tracks[i].axpos[1]=0;
    tracks[i].aypos[1]=0;
    tracks[i].axpos[2]=(int)(MAXLEN*Srate/1024.);
    tracks[i].aypos[2]=0;
    for (j=0; j<GRAPHSAMPLES; j++) {
      tracks[i].fullvol[j]=32767;
      tracks[i].fullpan[j]=16384;
      tracks[i].fullaux[j]=0;
    }
    makelist(i,1); makelist(i,2); makelist(i,3);
  }
  XtVaSetValues(tracks[10].panB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
    noparpixmap,NULL);
  master.sumnum=0; num_total=events_total=0;

  sett.showlin=sett.shownum=sett.showgrid=sett.showall=sett.scroll=0;
  sett.smallfont=0;
  sett.linvol=sett.fastdraw=sett.monitor=sett.recstereo=sett.bpmscale=0;
  sett.bpm=120; sett.bpb=4; sett.quant=1; sett.autoquant=0;
  sett.buffsize=0.5;
  if (fully) {
    XmToggleButtonSetState(nchnls2,True,False);
    XmToggleButtonSetState(nchnls1,False,False);
    XmToggleButtonSetState(nchnls4,False,False);
    sett.nchnls=2;
  }
  XmToggleButtonSetState(sovervSize,False,False);
  XtVaSetValues(soundMatrix,XmNfontList,largefontlist,NULL);
  XmToggleButtonSetState(displinToggle,False,False);
  XmToggleButtonSetState(dispallToggle,False,False);
  XmToggleButtonSetState(dispgridToggle,False,False);
  XmToggleButtonSetState(dispnumToggle,False,False);
  XmToggleButtonSetState(dispscrollToggle,False,False);
  XmToggleButtonSetState(dispfastdrawToggle,False,False);
  XmToggleButtonSetState(monitorToggle,False,False);
  XmToggleButtonSetState(recstereoToggle,False,False);
  XmToggleButtonSetState(linvolToggle,False,False);
  XmToggleButtonSetState(machineAutoQuant,False,False);
  XmToggleButtonSetState(timeToggleSec,True,False);
  XmToggleButtonSetState(timeToggleBeats,False,False);

  strcpy(recordname,"Record file name not set");
  selnum=seltrack=dirtytrack=0;
  mstart=0., mstop=MAXLEN; marked=0; zoomtrack=0;
  sett.RM1=500.; sett.ECtime=0.5; sett.ECfeed=0.2;
  sett.CHspeed=.2; sett.CHdepth=.05; sett.HAinter=4.8;
  sett.FLTcenter=500.; sett.FLTQ=10.; sett.filtertype=LOWPASS;
  sett.RVtime=5.; sett.CMA=-0.8;
  XtVaSetValues(effects1RMfreq,XmNvalue,500,NULL);
  XtVaSetValues(eff1ECtime,XmNvalue,500,NULL);
  XtVaSetValues(eff1ECfeed,XmNvalue,200,NULL);
  XtVaSetValues(eff1CHspeed,XmNvalue,200,NULL);
  XtVaSetValues(eff1CHdepth,XmNvalue,200,NULL);
  XtVaSetValues(eff1HAinter,XmNvalue,700,NULL);
  XtVaSetValues(eff1FLTcenter,XmNvalue,500,NULL);
  XmToggleButtonSetState(eff1FLTcenterT,False,False);
  XtVaSetValues(eff1FLTQ,XmNvalue,95,NULL);
  XtVaSetValues(eff2CMA,XmNvalue,100,NULL);
  XtVaSetValues(eff2RVtime,XmNvalue,500,NULL);
  sett.ringset=sett.echoset=sett.harmonizerset=sett.chorusset=sett.filterset=0;
  sett.reverbset=sett.compresset=sett.octaveset=0;
  sett.filtertype=LOWPASS;
  sett.centerT=0;
  XtVaSetValues(eff1RadEcho,XmNset,False,NULL);
  XtVaSetValues(eff1RadHarm,XmNset,False,NULL);
  XtVaSetValues(eff1RadRing,XmNset,False,NULL);
  XtVaSetValues(eff1RadChor,XmNset,False,NULL);
  XtVaSetValues(eff1RadFilter,XmNset,False,NULL);
  XtVaSetValues(eff1FLTlowpass,XmNset,True,NULL);
  XtVaSetValues(eff1FLThipass,XmNset,False,NULL);
  XtVaSetValues(eff1FLTbandpass,XmNset,False,NULL);
  XtVaSetValues(eff1FLTbandstop,XmNset,False,NULL);
  XtVaSetValues(eff2RadReverb,XmNset,False,NULL);
  XtVaSetValues(eff2RadCompress,XmNset,False,NULL);
  XtVaSetValues(eff2RadOctave,XmNset,False,NULL);
  RedrawWin();
}

void Refresh(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, rcode;

  SetWatchCursor(w);
  for (i=0; i<MAXLYD; i++) {
    if (sounds[i].used) {
      free(sounds[i].summary);
      rcode=newsound(sounds[i].filename,
        sounds[i].startsec, sounds[i].track, sounds[i].loop, sounds[i].lock,i,1);
      if (rcode==1) openWarn(sounds[i].filename);
      if (rcode==2) XtManageChild(srateWarning);
      if (rcode==3) XtManageChild(bitsWarning);
      RedrawTrack(sounds[i].track);
    }
  }
  ResetCursor(w);
}

int loadm(filename, w, init, offset)
char *filename; Widget w; int init; double offset;
{
  FILE *fil; DIR *dirp;
  char comm[500], p1c[150], p4c[50];
  int p1i, p2i, p3i, p4i, num, i, rcode, name;
  double p1f, p2f, p3f, p4f, val;
  XmString text;

  if (dirp=opendir(filename)) {
    closedir(dirp);
    return 1;
  }
  fil=fopen(filename, "r");
  if (fil==NULL) {
    openWarn(filename);
    return 1;
  }
  SetWatchCursor(w);
  if (init==0) backup();
  if ((XmToggleButtonGetState(loadmixToggle)==0) || init) Initvalues(1);
  while (!feof(fil)) {
    fscanf(fil, "%s", comm);
    if (feof(fil)) break;
    if (strcasecmp(comm,"showlinear")==0) fscanf(fil,"%d",&sett.showlin);
    else if (strcasecmp(comm,"shownumbers")==0) fscanf(fil,"%d",&sett.shownum);
    else if (strcasecmp(comm,"autoscroll")==0) fscanf(fil,"%d",&sett.scroll);
    else if (strcasecmp(comm,"fastdraw")==0) fscanf(fil,"%d",&sett.fastdraw);
    else if (strcasecmp(comm,"linearvol")==0) fscanf(fil,"%d",&sett.linvol);
    else if (strcasecmp(comm,"channels")==0) fscanf(fil,"%d",&sett.nchnls);
    else if (strcasecmp(comm,"monitor")==0) fscanf(fil,"%d",&sett.monitor);
    else if (strcasecmp(comm,"recstereo")==0) fscanf(fil,"%d",&sett.recstereo);
    else if (strcasecmp(comm,"bpmscale")==0) fscanf(fil,"%d",&sett.bpmscale);
    else if (strcasecmp(comm,"autoquant")==0) fscanf(fil,"%d",&sett.autoquant);
    else if (strcasecmp(comm,"bpm")==0) {
      fscanf(fil,"%d",&sett.bpm);
      if (sett.bpm<1) sett.bpm=1;
      if (sett.bpm>3840) sett.bpm=3840;
      sprintf(streng, "%d", sett.bpm);
      XtVaSetValues(timeBPMtext,XmNvalue,streng,NULL);
    }
    else if (strcasecmp(comm,"bpbar")==0) {
      fscanf(fil,"%d",&sett.bpb);
      if (sett.bpb<1) sett.bpb=1;
      if (sett.bpm>65536) sett.bpb=65536;
      sprintf(streng, "%d", sett.bpb);
      XtVaSetValues(timeBPBtext,XmNvalue,streng,NULL);
    }
    else if (strcasecmp(comm,"buffersize")==0) {
      fscanf(fil,"%lf",&sett.buffsize);
      if (sett.buffsize<0.1) sett.buffsize=0.1;
      if (sett.buffsize>2.) sett.buffsize=2.;
      sprintf(streng, "%6.2f", sett.buffsize);
      XtVaSetValues(buffText,XmNvalue,streng,NULL);
    }
    else if (strcasecmp(comm,"showgrid")==0) fscanf(fil,"%d",&sett.showgrid);
    else if (strcasecmp(comm,"smallfont")==0) fscanf(fil,"%d",&sett.smallfont);
    else if (strcasecmp(comm,"showallgraphs")==0) fscanf(fil,"%d",&sett.showall);
    else if (strcasecmp(comm,"samplerate")==0) {
      fscanf(fil,"%d",&Srate);
      if (Srate==8000) XmToggleButtonSetState(srate8,True,True);
      else if (Srate==11025) XmToggleButtonSetState(srate11025,True,True);
      else if (Srate==16000) XmToggleButtonSetState(srate16,True,True);
      else if (Srate==22050) XmToggleButtonSetState(srate2205,True,True);
      else if (Srate==32000) XmToggleButtonSetState(srate32,True,True);
      else if (Srate==44100) XmToggleButtonSetState(srate441,True,True);
      else if (Srate==48000) XmToggleButtonSetState(srate48,True,True);
      else { printf("Unsupported sampling rate %d\n",Srate); Srate=44100; }
    }
    else if (strcasecmp(comm,"quant")==0) {
      fscanf(fil,"%d",&sett.quant);
      if (sett.quant==1) XmToggleButtonSetState(machine1,True,True);
      else if (sett.quant==2) XmToggleButtonSetState(machine1d2,True,True);
      else if (sett.quant==3) XmToggleButtonSetState(machine1d3,True,True);
      else if (sett.quant==4) XmToggleButtonSetState(machine1d4,True,True);
      else if (sett.quant==8) XmToggleButtonSetState(machine1d8,True,True);
      else if (sett.quant==16) XmToggleButtonSetState(machine1d16,True,True);
      else { printf("Unsupported quantization %d\n",sett.quant); sett.quant=1; }
    }
    else if (strcasecmp(comm,"reverb")==0) fscanf(fil,"%d",&sett.reverbset);
    else if (strcasecmp(comm,"compressor")==0) fscanf(fil,"%d",&sett.compresset);
    else if (strcasecmp(comm,"octave")==0) fscanf(fil,"%d",&sett.octaveset);
    else if (strcasecmp(comm,"echo")==0) fscanf(fil,"%d",&sett.echoset);
    else if (strcasecmp(comm,"chorus")==0) fscanf(fil,"%d",&sett.chorusset);
    else if (strcasecmp(comm,"harmonizer")==0) fscanf(fil,"%d",&sett.harmonizerset);
    else if (strcasecmp(comm,"ring")==0) fscanf(fil,"%d",&sett.ringset);
    else if (strcasecmp(comm,"filter")==0) fscanf(fil,"%d",&sett.filterset);
    else if (strcasecmp(comm,"filter_freq_par")==0) fscanf(fil,"%d",&sett.centerT);
    else if (strcasecmp(comm,"filtertype")==0) {
      fscanf(fil,"%s",p1c);
      if (strcasecmp(p1c,"highpass")==0) {
        sett.filtertype=HIGHPASS;
        XtVaSetValues(eff1FLThipass,XmNset,True,NULL);
        XtVaSetValues(eff1FLTlowpass,XmNset,False,NULL);
        XtVaSetValues(eff1FLTbandpass,XmNset,False,NULL);
        XtVaSetValues(eff1FLTbandstop,XmNset,False,NULL);
      } else if (strcasecmp(p1c,"lowpass")==0) {
        sett.filtertype=LOWPASS;
        XtVaSetValues(eff1FLTlowpass,XmNset,True,NULL);
        XtVaSetValues(eff1FLThipass,XmNset,False,NULL);
        XtVaSetValues(eff1FLTbandpass,XmNset,False,NULL);
        XtVaSetValues(eff1FLTbandstop,XmNset,False,NULL);
      } else if (strcasecmp(p1c,"bandpass")==0) {
        sett.filtertype=BANDPASS;
        XtVaSetValues(eff1FLTbandpass,XmNset,True,NULL);
        XtVaSetValues(eff1FLTlowpass,XmNset,False,NULL);
        XtVaSetValues(eff1FLThipass,XmNset,False,NULL);
        XtVaSetValues(eff1FLTbandstop,XmNset,False,NULL);
      } else if (strcasecmp(p1c,"bandstop")==0) {
        sett.filtertype=BANDSTOP;
        XtVaSetValues(eff1FLTbandstop,XmNset,True,NULL);
        XtVaSetValues(eff1FLTlowpass,XmNset,False,NULL);
        XtVaSetValues(eff1FLThipass,XmNset,False,NULL);
        XtVaSetValues(eff1FLTbandpass,XmNset,False,NULL);
      }
    }
    else if (strcasecmp(comm,"echo_time")==0) fscanf(fil,"%lf",&sett.ECtime);
    else if (strcasecmp(comm,"echo_feedback")==0) fscanf(fil,"%lf",&sett.ECfeed);
    else if (strcasecmp(comm,"chorus_speed")==0) fscanf(fil,"%lf",&sett.CHspeed);
    else if (strcasecmp(comm,"chorus_depth")==0) fscanf(fil,"%lf",&sett.CHdepth);
    else if (strcasecmp(comm,"harmonizer_interval")==0)
      fscanf(fil,"%lf",&sett.HAinter);
    else if (strcasecmp(comm,"ring_frequency")==0) fscanf(fil,"%lf",&sett.RM1);
    else if (strcasecmp(comm,"filter_freq")==0) fscanf(fil,"%lf",&sett.FLTcenter);
    else if (strcasecmp(comm,"filter_Q")==0) fscanf(fil,"%lf",&sett.FLTQ);
    else if (strcasecmp(comm,"reverb_time")==0) fscanf(fil,"%lf",&sett.RVtime);
    else if (strcasecmp(comm,"compressor_A")==0) fscanf(fil,"%lf",&sett.CMA);
    else if ((strcasecmp(comm,"sound")==0)||(strcasecmp(comm,"sound_lock")==0)) {
      fscanf(fil,"%s %lf %d %d", p1c, &p2f, &p3i, &p4i);
      if (strcasecmp(comm,"sound_lock")==0)
        rcode=newsound(p1c, p2f+offset, p3i, p4i, 1, -1, 0);
      else rcode=newsound(p1c, p2f+offset, p3i, p4i, 0, -1, 0);
      if (rcode==1) openWarn(p1c); else
      if (rcode==2) XtManageChild(srateWarning);
      if (rcode==3) XtManageChild(bitsWarning);
    }
    else if (strcasecmp(comm,"variable")==0) {
      fscanf(fil,"%s %d %lf %s", p1c, &p2i, &p3f, p4c);
      name=-1;
      for (i=1; i<21; i++) if (strcmp(Enames[i],p1c)==0) name=i;
      if (name==-1) { printf("Illegal variable %s in mixfile\n",p1c); break; }
      newevent(name, p2i, p3f+offset, -1);
      XtVaSetValues(events[newenum].valueText,XmNvalue,p4c,NULL);
      XbaeMatrixSetCell(eventMatrix,events[newenum].posinmatrix,2,p4c);
    }
    else if (strcasecmp(comm,"volume")==0) {
      fscanf(fil,"%d %d", &p1i, &p2i);
      if (p2i<0) p2i=0;
      if (p2i>66) p2i=66;
      XtVaSetValues(tracks[p1i].volScale, XmNvalue, p2i, NULL);
      val=pow(10.,(p2i-60.)/24.)*32767.; if (val<105.) val=0.;
      tracks[p1i].volval=val;
      sprintf(streng,"%d",p2i);
      text=XmStringCreateLocalized(streng);
      XtVaSetValues(tracks[p1i].volScale,XmNtitleString,text,NULL);
      XtFree(text);
    }
    else if (strcasecmp(comm,"quadpos")==0) {
      fscanf(fil,"%d %d", &p1i, &p2i);
      if (p2i<1) p2i=1;
      if (p2i>2) p2i=2;
      if (p2i==2) {
        XtVaSetValues(tracks[p1i].fbackB,XmNlabelPixmap,backpixmap,NULL);
      } else {
        XtVaSetValues(tracks[p1i].fbackB,XmNlabelPixmap,frontpixmap,NULL);
      }
      tracks[p1i].pair=p2i;
    }
    else if (strcasecmp(comm,"volumegraph")==0) {
      fscanf(fil,"%d",&num);
      for (i=1; i<MAXNODE; i++) {
        fscanf(fil,"%s",p1c);
        if (strcasecmp(p1c,"end")==0) break;
        p1f=atof(p1c)+offset;
        if (p1f<0.) p1f=0.; if (p1f>MAXLEN) p1f=MAXLEN;
        fscanf(fil,"%d", &p2i);
        tracks[num].vxpos[i]=p1f*Srate/1024.;
        tracks[num].vypos[i]=p2i;
      }
      tracks[num].vnumsquare=i-1;
      tracks[num].vxpos[i-1]=MAXLEN*Srate/1024.;
      computegraph(num,1); makelist(num,1);
    }
    else if ((strcasecmp(comm,"pangraph")==0) || (strcasecmp(comm,"pargraph")==0)) {
      fscanf(fil,"%d",&num);
      for (i=1; i<MAXNODE; i++) {
        fscanf(fil,"%s",p1c);
        if (strcasecmp(p1c,"end")==0) break;
        p1f=atof(p1c)+offset;
        if (p1f<0.) p1f=0.; if (p1f>MAXLEN) p1f=MAXLEN;
        fscanf(fil,"%d", &p2i);
        tracks[num].pxpos[i]=p1f*Srate/1024.;
        tracks[num].pypos[i]=p2i;
      }
      tracks[num].pnumsquare=i-1;
      tracks[num].pxpos[i-1]=MAXLEN*Srate/1024.;
      computegraph(num,2); makelist(num,2);
    }
    else if (strcasecmp(comm,"auxgraph")==0) {
      fscanf(fil,"%d",&num);
      for (i=1; i<MAXNODE; i++) {
        fscanf(fil,"%s",p1c);
        if (strcasecmp(p1c,"end")==0) break;
        p1f=atof(p1c)+offset;
        if (p1f<0.) p1f=0.; if (p1f>MAXLEN) p1f=MAXLEN;
        fscanf(fil,"%d", &p2i);
        tracks[num].axpos[i]=p1f*Srate/1024.;
        tracks[num].aypos[i]=p2i;
      }
      tracks[num].anumsquare=i-1;
      tracks[num].axpos[i-1]=MAXLEN*Srate/1024.;
      computegraph(num,3); makelist(num,3);
    }
    else if (comm[0]=='\\') {
      p1c[0]='*';
      while ((!feof(fil)) && (p1c[0]!='\\')) fscanf(fil,"%c",p1c); 
    }
    else { printf("Illegal command %s in mixfile\n",comm); break; }
  }
  fclose(fil);

  if (sett.showlin) XmToggleButtonSetState(displinToggle,True,False);
    else XmToggleButtonSetState(displinToggle,False,False);
  if (sett.showgrid) XmToggleButtonSetState(dispgridToggle,True,False);
    else XmToggleButtonSetState(dispgridToggle,False,False);
  if (sett.smallfont) {
    XmToggleButtonSetState(sovervSize,True,False);
    XtVaSetValues(soundMatrix,XmNfontList,smallfontlist,NULL);
  } else {
    XmToggleButtonSetState(sovervSize,False,False);
    XtVaSetValues(soundMatrix,XmNfontList,largefontlist,NULL);
  }
  if (sett.showall) XmToggleButtonSetState(dispallToggle,True,False);
    else XmToggleButtonSetState(dispallToggle,False,False);
  if (sett.shownum) XmToggleButtonSetState(dispnumToggle,True,False);
    else XmToggleButtonSetState(dispnumToggle,False,False);
  if (sett.scroll) XmToggleButtonSetState(dispscrollToggle,True,False);
    else XmToggleButtonSetState(dispscrollToggle,False,False);
  if (sett.fastdraw) XmToggleButtonSetState(dispfastdrawToggle,True,False);
    else XmToggleButtonSetState(dispfastdrawToggle,False,False);
  if (sett.linvol) XmToggleButtonSetState(linvolToggle,True,False);
    else XmToggleButtonSetState(linvolToggle,False,False);
  if (sett.monitor) XmToggleButtonSetState(monitorToggle,True,False);
    else XmToggleButtonSetState(monitorToggle,False,False);
  if (sett.recstereo) XmToggleButtonSetState(recstereoToggle,True,False);
    else XmToggleButtonSetState(recstereoToggle,False,False);
  if (sett.autoquant) XmToggleButtonSetState(machineAutoQuant,True,False);
    else XmToggleButtonSetState(machineAutoQuant,False,False);
  XmToggleButtonSetState(timeToggleBeats,False,False);
  XmToggleButtonSetState(timeToggleSec,False,False);
  if (sett.bpmscale) XmToggleButtonSetState(timeToggleBeats,True,False);
    else XmToggleButtonSetState(timeToggleBeats,True,False);
  XmToggleButtonSetState(nchnls1,False,False);
  XmToggleButtonSetState(nchnls2,False,False);
  XmToggleButtonSetState(nchnls4,False,False);
  if (sett.nchnls==1) XmToggleButtonSetState(nchnls1,True,False);
    else if (sett.nchnls==2) XmToggleButtonSetState(nchnls2,True,False);
      else if (sett.nchnls==4) {
          XmToggleButtonSetState(nchnls4,True,False);
          for (i=1; i<10; i++)
            XtVaSetValues(tracks[i].fbackB,XmNsensitive,True,NULL);
        } else { printf("Unsupported channel number %d\n",sett.nchnls);
          sett.nchnls=2;
        }
  if (sett.ringset) XmToggleButtonSetState(eff1RadRing,True,False);
    else XmToggleButtonSetState(eff1RadRing,False,False);
  if (sett.chorusset) XmToggleButtonSetState(eff1RadChor,True,False);
    else XmToggleButtonSetState(eff1RadChor,False,False);
  if (sett.echoset) XmToggleButtonSetState(eff1RadEcho,True,False);
    else XmToggleButtonSetState(eff1RadEcho,False,False);
  if (sett.harmonizerset) XmToggleButtonSetState(eff1RadHarm,True,False);
    else XmToggleButtonSetState(eff1RadHarm,False,False);
  if (sett.filterset) XmToggleButtonSetState(eff1RadFilter,True,False);
    else XmToggleButtonSetState(eff1RadFilter,False,False);
  if (sett.centerT) XmToggleButtonSetState(eff1FLTcenterT,True,False);
    else XmToggleButtonSetState(eff1FLTcenterT,False,False);
  if (sett.reverbset) XmToggleButtonSetState(eff2RadReverb,True,False);
    else XmToggleButtonSetState(eff2RadReverb,False,False);
  if (sett.compresset) XmToggleButtonSetState(eff2RadCompress,True,False);
    else XmToggleButtonSetState(eff2RadCompress,False,False);
  if (sett.octaveset) XmToggleButtonSetState(eff2RadOctave,True,False);
    else XmToggleButtonSetState(eff2RadOctave,False,False);
  XtVaSetValues(effects1RMfreq,XmNvalue,(int)(sett.RM1),NULL);
  XtVaSetValues(eff1ECtime,XmNvalue,(int)(sett.ECtime*1000.),NULL);
  XtVaSetValues(eff1ECfeed,XmNvalue,(int)(sett.ECfeed*1000.),NULL);
  XtVaSetValues(eff1CHspeed,XmNvalue,(int)(sett.CHspeed*1000.),NULL);
  XtVaSetValues(eff1CHdepth,XmNvalue,(int)(sett.CHdepth*1000.),NULL);
  XtVaSetValues(eff1FLTcenter,XmNvalue,(int)(sett.FLTcenter),NULL);
  XtVaSetValues(eff1FLTQ,XmNvalue,(int)((sett.FLTQ-.5)*10.),NULL);
  XtVaSetValues(eff1HAinter,XmNvalue,(int)((sett.HAinter+12.)*41.6667),NULL);
  XtVaSetValues(eff2RVtime,XmNvalue,(int)(sett.RVtime*100.),NULL);
  XtVaSetValues(eff2CMA,XmNvalue,(int)((sett.CMA+1.)*500.),NULL);

  sprintf(streng,"%d Hz",(int)(sett.RM1));
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(effects1RMfreq,XmNtitleString,text,NULL); XtFree(text);

  sprintf(streng,"%5.3f s",sett.ECtime);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(eff1ECtime,XmNtitleString,text,NULL); XtFree(text);

  sprintf(streng,"%5.3f",sett.ECfeed);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(eff1ECfeed,XmNtitleString,text,NULL); XtFree(text);

  sprintf(streng,"%5.3f Hz",sett.CHspeed);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(eff1CHspeed,XmNtitleString,text,NULL); XtFree(text);

  sprintf(streng,"%5.3fs",sett.CHdepth);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(eff1CHdepth,XmNtitleString,text,NULL); XtFree(text);

  sprintf(streng,"%d Hz",(int)(sett.FLTcenter));
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(eff1FLTcenter,XmNtitleString,text,NULL); XtFree(text);

  sprintf(streng,"%5.2f",sett.FLTQ);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(eff1FLTQ,XmNtitleString,text,NULL); XtFree(text);

  sprintf(streng,"%5.2f",sett.HAinter);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(eff1HAinter,XmNtitleString,text,NULL); XtFree(text);

  sprintf(streng,"%5.3f s",sett.RVtime);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(eff2RVtime,XmNtitleString,text,NULL); XtFree(text);

  sprintf(streng,"%5.3f",sett.CMA);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(eff2CMA,XmNtitleString,text,NULL); XtFree(text);

  RedrawWin();
  ResetCursor(w);
  strcpy(comm,"mix: ");
  strcat(comm,filename);
  XtVaSetValues(topLevel,XmNtitle,comm,NULL);
  return 0;
}


void Loadmix(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(loadmixBox);
}

void LoadmixOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  float tim;

  tim=atof(XmTextGetString(loadmixtext));
  if (tim<0.) tim=0.; if (tim>MAXLEN) tim=MAXLEN;

  XmStringGetLtoR(calldata->value, charset, &filename);
  if (loadm(filename,w,0,tim)==0)
    XtUnmanageChild(loadmixBox);
  XtFree(filename);
}

void Savemix(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(savemixBox);
}

void Recordb(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(recordBox);
}

void RecordOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

  XmStringGetLtoR(calldata->value, charset, &filename);
  strcpy(recordname,filename);
  XtUnmanageChild(recordBox); XtFree(filename);
}

void shellsort(ptar, startar)
int ptar[]; double startar[];
{
  int i, j, inc, v2;
  double v1;

  for (i=0; i<MAXLYD; i++) startar[i]=sounds[i].startsec;
  inc=1;
  do { inc*=3; inc++; } while (inc<=MAXLYD);
  do {
    inc/=3;
    for (i=inc+1; i<=MAXLYD; i++) {
      v1=startar[i-1]; v2=ptar[i-1];
      j=i;
      while (startar[j-inc-1]>v1) {
        startar[j-1]=startar[j-inc-1]; ptar[j-1]=ptar[j-inc-1];
        j-=inc;
        if (j<=inc) break;
      }
      startar[j-1]=v1; ptar[j-1]=v2;
    }
  } while (inc>1);
}

int savem(filename)
char *filename;
{
  FILE *fil;
  int i, j, val, sw, pt;
  int ptar[MAXLYD]; double startar[MAXLYD];

  fil=fopen(filename, "w");
  if (fil==NULL) return 1;
  for (i=0; i<MAXLYD; i++) ptar[i]=i;
  if (XmToggleButtonGetState(savemixToggle)) shellsort(ptar,startar);

  fprintf(fil,"samplerate %d\n",Srate);
  fprintf(fil,"channels %d\n",sett.nchnls);
  fprintf(fil,"buffersize %6.2f\n",sett.buffsize);
  fprintf(fil,"linearvol %d\n",sett.linvol);
  fprintf(fil,"monitor %d\n",sett.monitor);
  fprintf(fil,"recstereo %d\n",sett.recstereo);
  fprintf(fil,"bpmscale %d\n",sett.bpmscale);
  fprintf(fil,"bpm %d\n",sett.bpm);
  fprintf(fil,"bpbar %d\n",sett.bpb);
  fprintf(fil,"quant %d\n",sett.quant);
  fprintf(fil,"autoquant %d\n",sett.autoquant);
  fprintf(fil,"showlinear %d\n",sett.showlin);
  fprintf(fil,"shownumbers %d\n",sett.shownum);
  fprintf(fil,"showgrid %d\n",sett.showgrid);
  fprintf(fil,"smallfont %d\n",sett.smallfont);
  fprintf(fil,"showallgraphs %d\n",sett.showall);
  fprintf(fil,"autoscroll %d\n",sett.scroll);
  fprintf(fil,"fastdraw %d\n\n",sett.fastdraw);
  fprintf(fil,"echo %d\n",sett.echoset);
  fprintf(fil,"chorus %d\n",sett.chorusset);
  fprintf(fil,"harmonizer %d\n",sett.harmonizerset);
  fprintf(fil,"ring %d\n",sett.ringset);
  fprintf(fil,"filter %d\n",sett.filterset);
  fprintf(fil,"filtertype ");
  if (sett.filtertype==HIGHPASS) fprintf(fil,"highpass\n");
  else if (sett.filtertype==LOWPASS) fprintf(fil,"lowpass\n");
  else if (sett.filtertype==BANDPASS) fprintf(fil,"bandpass\n");
  else if (sett.filtertype==BANDSTOP) fprintf(fil,"bandstop\n");
  fprintf(fil,"echo_time %.3f\n",sett.ECtime);
  fprintf(fil,"echo_feedback %.3f\n",sett.ECfeed);
  fprintf(fil,"chorus_speed %.3f\n",sett.CHspeed);
  fprintf(fil,"chorus_depth %.3f\n",sett.CHdepth);
  fprintf(fil,"harmonizer_interval %.3f\n",sett.HAinter);
  fprintf(fil,"ring_frequency %.3f\n",sett.RM1);
  fprintf(fil,"filter_freq %.3f\n",sett.FLTcenter);
  fprintf(fil,"filter_freq_par %d\n",sett.centerT);
  fprintf(fil,"filter_Q %.3f\n",sett.FLTQ);
  fprintf(fil,"reverb %d\n",sett.reverbset);
  fprintf(fil,"reverb_time %.3f\n",sett.RVtime);
  fprintf(fil,"compressor %d\n",sett.compresset);
  fprintf(fil,"compressor_A %.3f\n",sett.CMA);
  fprintf(fil,"octave %d\n\n",sett.octaveset);
  for (i=0; i<MAXLYD; i++) {
    pt=ptar[i];
    if (sounds[pt].used) {
      if (sounds[pt].lock) fprintf(fil, "sound_lock ");
        else fprintf(fil, "sound ");
      fprintf(fil,"%s\t%7.3f\t%d\t%d\n",
        sounds[pt].filename, sounds[pt].startsec, sounds[pt].track,
        sounds[pt].loop);
    }
  }
  fprintf(fil,"\n");
  for (i=0; i<MAXLYD; i++)
    if (events[i].used) fprintf(fil,"variable %s\t%d\t%6.2f\t%s\n",
      Enames[events[i].name], events[i].track, events[i].startsec,
      events[i].value);
  fprintf(fil,"\n");
  for (i=1; i<11; i++) {
    XmScaleGetValue(tracks[i].volScale, &val);
    fprintf(fil,"volume %d %d\n", i, val);
    if (i<10) fprintf(fil,"quadpos %d %d\n", i, tracks[i].pair);
  }
  fprintf(fil,"\n");
  for (i=1; i<11; i++) {
    fprintf(fil,"volumegraph %d\n",i);
    for (j=1; j<tracks[i].vnumsquare; j++)
      fprintf(fil,"  %6.2f %d\n",
        tracks[i].vxpos[j]*1024./Srate, tracks[i].vypos[j]);
    fprintf(fil,"  %6.2f %d\n", MAXLEN, tracks[i].vypos[j]);
    fprintf(fil, "end\n\n");
  }
  for (i=1; i<11; i++) {
    if (i==10) fprintf(fil,"pargraph %d\n",i); else fprintf(fil,"pangraph %d\n",i);
    for (j=1; j<tracks[i].pnumsquare; j++)
      fprintf(fil,"  %6.2f %d\n",
        tracks[i].pxpos[j]*1024./Srate, tracks[i].pypos[j]);
    fprintf(fil,"  %6.2f %d\n", MAXLEN, tracks[i].pypos[j]);
    fprintf(fil, "end\n\n");
  }
  for (i=1; i<11; i++) {
    fprintf(fil,"auxgraph %d\n",i);
    for (j=1; j<tracks[i].anumsquare; j++)
      fprintf(fil,"  %6.2f %d\n",
        tracks[i].axpos[j]*1024./Srate, tracks[i].aypos[j]);
    fprintf(fil,"  %6.2f %d\n", MAXLEN, tracks[i].aypos[j]);
    fprintf(fil, "end\n\n");
  }

  fclose(fil);
  return 0;
}

void SavemixOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  char *filename, comm[200];
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

  SetWatchCursor(w);
  XmStringGetLtoR(calldata->value, charset, &filename);

  if (savem(filename)) {
    ResetCursor(w);
    openWarn(filename);
    XtFree(filename);
    return;
  }
  XtUnmanageChild(savemixBox);

  ResetCursor(w);
  strcpy(comm,"mix: ");
  strcat(comm,filename);
  XtVaSetValues(topLevel,XmNtitle,comm,NULL);
  XtFree(filename);
}


void Load(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(loadFileBox);
}

void LoadOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call;
  char *filename;
  int rcode, track;
  float tid;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

  SetWatchCursor(w);
  XmStringGetLtoR(calldata->value, charset, &filename);
  tid=atof(XmTextGetString(trackT));
  if ((tid<0.) || (tid>=MAXLEN)) tid=leftsec; 
  for (track=1; track<10; track++)
    if (XmToggleButtonGadgetGetState(trackToggles[track])) break;
  rcode=newsound(filename, tid, track, 1, 0, -1, 0);
  if (rcode==1) openWarn(filename);
  if (rcode==2) XtManageChild(srateWarning);
  if (rcode==3) XtManageChild(bitsWarning);
  XtFree(filename);
  ResetCursor(w);
}

void LoadInfo(w, client, call)
Widget w;
XtPointer client, call;
{
  char *text, str[100];
  int rcode, R, fd, vers;
  DIR *dirp;
  char *filename=XmTextGetString(w);

  if (dirp=opendir(filename)) {
    closedir(dirp);
    return;
  }
  fd=open(filename,O_RDONLY);
  if (fd==-1) {
    text=XmStringCreateSimple("Can't open file");
    XtVaSetValues(infoL1,XmNlabelString,text,NULL); XtFree(text);
    text=XmStringCreateSimple(" ");
    XtVaSetValues(infoL2,XmNlabelString,text,NULL); XtFree(text);
    text=XmStringCreateSimple(" ");
    XtVaSetValues(infoL3,XmNlabelString,text,NULL); XtFree(text);
    XtFree(filename);
    return;
  }
  close(fd); 
  //infile=AFopenfd(fd, "r", in_AFsetup);
  infile=AFopenfile(filename, "rb", in_AFsetup);
  if (infile==NULL) {
    text=XmStringCreateSimple("Not a soundfile");
    XtVaSetValues(infoL1,XmNlabelString,text,NULL); XtFree(text);
    text=XmStringCreateSimple(" ");
    XtVaSetValues(infoL2,XmNlabelString,text,NULL); XtFree(text);
    text=XmStringCreateSimple(" ");
    XtVaSetValues(infoL3,XmNlabelString,text,NULL); XtFree(text);
    XtFree(filename);
    return;
  }

  samps_per_frame   = AFgetchannels(infile, AF_DEFAULT_TRACK);
  R                 = (int)AFgetrate(infile, AF_DEFAULT_TRACK);
  compression       = AFgetcompression(infile, AF_DEFAULT_TRACK);
  filefmt           = AFgetfilefmt(infile, &vers);   
  AFgetsampfmt(infile, AF_DEFAULT_TRACK, &samp_type, &bits_per_samp);
  framecnt          = AFgetframecnt(infile,  AF_DEFAULT_TRACK);

  strcpy(str,"File Format:  ");
  if (filefmt==AF_FILE_AIFFC) strcat(str,"AIFC");
    else if (filefmt==AF_FILE_AIFF) strcat(str,"AIFF");
      else strcat(str,"Unknown format");
  if (samps_per_frame==1) strcat(str,", Mono.");
    else if (samps_per_frame==2) strcat(str,", Stereo.");
      else if (samps_per_frame==4) strcat(str,", Quad.");
  text=XmStringCreateSimple(str);
  XtVaSetValues(infoL1,XmNlabelString,text,NULL); XtFree(text);
  sprintf(str,"Size:  \t%d frames, %.2f seconds.",framecnt,(double)framecnt/R);
  text=XmStringCreateSimple(str);
  XtVaSetValues(infoL2,XmNlabelString,text,NULL); XtFree(text);
  sprintf(str,"Sampling rate:  \t%d Hz.",R);
  text=XmStringCreateSimple(str);
  XtVaSetValues(infoL3,XmNlabelString,text,NULL); XtFree(text);
  AFclosefile(infile);
  XtFree(filename);
}

void LoadPlay(w, client, call)
Widget w;
XtPointer client, call;
{
  int sampsread, chnls, srate, t=1, numfram, i;
  Widget textw=(Widget)client;
  ALport audioport;
  ALconfig config;
  long pvbuf[8];
  short int frames[4096];
  XmString text;
  char *filename=XmTextGetString(textw);
#ifdef LINUX  
  long tempBufOut [4] = {0,0,0,0};
#endif
  

  if (ldplaying) {
    text=XmStringCreateSimple("Play");
    XtVaSetValues(lPlay,XmNlabelString,text,NULL);
    ldplaying=0; XtFree(text); XtFree(filename);
    return;
  }
  lPlayFilehandle=
    AFopenfile(filename,"r",lPlayFilesetup);
  if (lPlayFilehandle==NULL) {
    openWarn(filename); XtFree(filename);
    return;
  }
  XtFree(filename);
  text=XmStringCreateSimple("Stop");
  XtVaSetValues(lPlay,XmNlabelString,text,NULL); XtFree(text);

  ldplaying=1; chnls=AFgetchannels(lPlayFilehandle, AF_DEFAULT_TRACK);
  srate=(int)AFgetrate(lPlayFilehandle, AF_DEFAULT_TRACK);
  pvbuf[0] = AL_OUTPUT_RATE;  pvbuf[1] = srate;
  pvbuf[2] = AL_INPUT_RATE;   pvbuf[3] = srate;
  pvbuf[4] = AL_CHANNEL_MODE;
  if (chnls==1) pvbuf[5]=AL_MONO;
    else if (chnls==4) pvbuf[5]=AL_4CHANNEL; else pvbuf[5]=AL_STEREO;
  ALsetparams(AL_DEFAULT_DEVICE, pvbuf, 6);
  config=ALnewconfig();
  ALsetqueuesize(config, (int)(srate*sett.buffsize*chnls));
  ALsetchannels(config, chnls);
  audioport=ALopenport("Mix Play", "w", config);

  do {
    sampsread=AFreadframes(lPlayFilehandle, AF_DEFAULT_TRACK,
      frames, 1024);
    numfram=sampsread*chnls;
    if (sett.nchnls>1) for (i=0; i<numfram; i++) frames[i]=frames[i]>>1;
    ALwritesamps(audioport, frames, numfram);
    if (XtAppPending(app_context) && ((t++)%3==0))
      XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
  } while (ldplaying && (sampsread==1024));

  while (ALgetfilled(audioport)) {
#ifdef LINUX
    ALwritesamps(audioport,tempBufOut,chnls);
#endif 
  }  
  ALcloseport(audioport);
  text=XmStringCreateSimple("Play");
  XtVaSetValues(lPlay,XmNlabelString,text,NULL); XtFree(text);
  ldplaying=0;
}

void Save(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(saveFileBox);
}

void regist(pt)
int pt;
{
  if (sounds[pt].mixstatus!=WAITING) return;
  if ((sounds[pt].muted) || (tracks[sounds[pt].track].muted) ||
    ((marked) && (sounds[pt].startsec+sounds[pt].dur<mstart)) ||
    ((zoomtrack) && (zoomtrack!=sounds[pt].track))) {
    sounds[pt].mixstatus=FINISHED; num_mixed++;
    return;
  }
  sounds[pt].filehandle=AFopenfile(sounds[pt].filename,"r",sounds[pt].filesetup);
  if (sounds[pt].filehandle==NULL) {
    openWarn(sounds[pt].filename);
    return;
  }
  if ((marked) && (sounds[pt].startsec<mstart)
   && (sounds[pt].loopcount==sounds[pt].loop)) {
    AFseekframe(sounds[pt].filehandle, AF_DEFAULT_TRACK,
      (int)((mstart-sounds[pt].startsec)*Srate));
    sounds[pt].finestart=0;
  }
  sounds[pt].mixstatus=MIXING;
}

void unregist(pt)
int pt;
{
  AFclosefile(sounds[pt].filehandle);
  sounds[pt].mixstatus=FINISHED; num_mixed++;
}

double timecode2time(buf)
int buf[];
{
  double sec;
  int SMPTEtype=(buf[7]>>1)&3;
  int frame, second, minute, hour;

  frame=buf[0]+(buf[1]<<4);
  second=buf[2]+(buf[3]<<4);
  minute=buf[4]+(buf[5]<<4);
  hour=buf[6]+((buf[7]&1)<<4);
  sec=hour*3600+minute*60+second;
  if (SMPTEtype==0) sec+=frame/24.;
  else if (SMPTEtype==1) sec+=frame/25.;
  else sec+=frame/30.;
  return sec;
}

/*
void time2timecode(sec, buf)
double sec;
int buf[];
{
  int SMPTEtype=(buf[7]>>1)&3;
  int frame, second, minute, hour;
  double secspasthour;

  hour=(int)(sec/3600.);
  secspasthour=sec-hour*3600;
  minute=(int)(secspasthour/60.);
  second=(int)(secspasthour-minute*60.);
  frame=(int)(secspasthour-second);

  buf[0]=fmod(sec,24)


  frame=buf[0]+(buf[1]<<4);
  second=buf[2]+(buf[3]<<4);
  minute=buf[4]+(buf[5]<<4);
  hour=buf[6]+((buf[7]&1)<<4);
  sec=hour*3600+minute*60+second;
  if (SMPTEtype==0) sec+=frame/24.;
  else if (SMPTEtype==1) sec+=frame/25.;
  else sec+=frame/30.;
}
*/

void eregist(pt, catchup)
int pt, catchup;
{
  char *value;
  int name;
#ifdef SGI
  MDevent mevent;
#endif
  struct pollfd fds;
  int quarterbuf[8], status, byte1, byte2;

  if (events[pt].mixstatus!=WAITING) return;

  if ((events[pt].muted) || (tracks[events[pt].track].muted) ||
    ((marked) && (events[pt].startsec<mstart) && (catchup==0)) ||
    ((zoomtrack) && (zoomtrack!=events[pt].track))) {
    events[pt].mixstatus=FINISHED; return;
  }

  events[pt].mixstatus=MIXING;

  value=events[pt].value; name=events[pt].name;
  if (name==Echorus) sett2.chorusset=atoi(value);
  else if (name==Eecho) sett2.echoset=atoi(value);
  else if (name==Eharmonizer) sett2.harmonizerset=atoi(value);
  else if (name==Ering) sett2.ringset=atoi(value);
  else if (name==Efilter) sett2.filterset=atoi(value);
  else if (name==Ereverb) sett2.reverbset=atoi(value);
  else if (name==Ecompress) sett2.compresset=atoi(value);
  else if (name==Eoctave) sett2.octaveset=atoi(value);
  else if (name==Efiltertype) {
    if (strcmp("lowpass",value)==0) sett2.filtertype=LOWPASS;
    else if (strcmp("highpass",value)==0) sett2.filtertype=HIGHPASS;
    else if (strcmp("bandpass",value)==0) sett2.filtertype=BANDPASS;
    else if (strcmp("bandstop",value)==0) sett2.filtertype=BANDSTOP;
  }
  else if (name==Eecho_time) sett2.ECtime=atof(value);
  else if (name==Eecho_feedback) sett2.ECfeed=atof(value);
  else if (name==Echorus_speed) sett2.CHspeed=atof(value);
  else if (name==Echorus_depth) sett2.CHdepth=atof(value);
  else if (name==Eharmonizer_interval) sett2.HAinter=atof(value);
  else if (name==Ering_frequency) sett2.RM1=atof(value);
  else if (name==Efilter_freq) sett2.FLTcenter=atof(value);
  else if (name==Efilter_Q) sett2.FLTQ=atof(value);
  else if (name==Ereverb_time) sett2.RVtime=atof(value);
  else if (name==Ecompress_A) sett2.CMA=atof(value);
  else if (name==Equadpos) tracks[events[pt].track].pair=atoi(value);
  else if ((name==Ecsound) && (catchup==0)) {
    sprintf(streng, "csound -go devaudio %s.orc %s.sco &", value, value);
    system(streng);
  }
  else if ((name==Esync) && (havemidi) && (catchup==0))
   while ((MIDItime<atof(events[pt].value)) && (realtiming)) {
/*
     fds.fd = mdGetFd(MIDIport);
     fds.events=POLLIN;
     poll(&fds, 1, 0);
     if (fds.revents==POLLIN) {
       if (mdReceive(MIDIport, &mevent, 1)>0) {
         if (mevent.msg[0]==MD_SYSEX) mdFree(mevent.sysexmsg); else {
           status=mevent.msg[0];
           byte1=mevent.msg[1];        
           byte2=mevent.msg[2];
           if ((status==241) || (byte2==241)) {
             quarterbuf[byte1>>4]=byte1&15;
             quartercount++;
             if (quartercount==8) {
               MIDItime=timecode2time(quarterbuf); quartercount=0;
             }
           }
         }
       }
     }

    if (XtAppPending(app_context))
      XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
*/
  }
}

void MakeHarm(void)
{
  int i, j;
  double f;

  for (i=0; i<4096; i++) {
    j=(i>=2048 ? 4096-i : i);
    if (j<512) f=0.;
    else if (j>=2048-512) f=1.;
    else f=(j-512)/1024.;
    harm[i]=cos(.5*PI*(1.-f));
  }

  for (i=0; i<4096; i++) sintab[i]=(int)(32767.*sin(PI*i/2048.));
}

void do_effect(int t)
{
  int i, del, utpek, diff, rv1, rv2, rv3, rv4, rv5, feed, v1, v2;
  float fact, fdel, frac, amp, point, fact2, val, vf1, vf2;
  int is, l, b, h, f1, q1;
  int samp1, samp2, samp3, samp4;

  v1=tracks[10].fullpan[t]; v2=tracks[10].fullpan[t+1]-v1;
  if (sett2.ringset) {
    fact=4096.*sett2.RM1/Srate;
    for (i=0; i<1024; i++) {
      auxbuff[i]=(auxbuff[i]*sintab[(int)(phase)&4095])>>15;
      phase+=fact;
    }
    if (phase>=4096.) phase-=4096.;
  }
  if (sett2.octaveset) {
    for (i=0; i<1024; i++) {
      if (auxbuff[i]>0) auxbuff[i]=20000; else auxbuff[i]=-20000;
    }
  }
  if (sett2.chorusset) {
    fact=2.0f*PI*sett2.CHspeed/Srate; fact2=sett2.CHdepth*(double)Srate/2.0f;
    for (i=0; i<1024; i++) {
      fdel=(sinf(phase2)+1.0f)*fact2+1.0f;
      phase2+=fact;
      del=(int)(fdel);
      frac=1.0f-fdel+(float)del;
      delay2[delpt2]=auxbuff[i];
      utpek=(delpt2-del+1); utpek+=(48000&(-(utpek<0))); diff=delay2[utpek];
      utpek=(delpt2-del);   utpek+=(48000&(-(utpek<0))); diff-=delay2[utpek];
      auxbuff[i]=(int)(delay2[utpek]+frac*diff);
      delpt2++; delpt2-=(48000&(-(delpt2==48000)));
    }
    if (phase2>=2.0f*PI) phase2-=2.f*PI;
  }
  if (sett2.harmonizerset) {
    float v; int va;
    fact=expf(sett2.HAinter*log(2.)/12.);
    fact=fabs((1.-fact)/0.065)/Srate; fact2=Srate*.065;
    for (i=0; i<1024; i++) {
      v=fact*phase3; point=v-floorf(v);
      if (sett2.HAinter>0.) point=1.0f-point;
      fdel=fact2*point+2.0f;
      del=(int)(fdel);
      frac=1.0f-fdel+(float)del;
      delay3[delpt3]=auxbuff[i];
      utpek=(delpt3-del);   if (utpek<0) utpek+=48000; va=delay3[utpek];
      utpek=(delpt3-del+1); if (utpek<0) utpek+=48000; diff=delay3[utpek]-va;
      val=harm[(int)(point*4096)]*(va+frac*diff);
      v+=.5f; point=v-floorf(v); phase3++;
      if (sett2.HAinter>0.) point=1.0f-point;
      fdel=fact2*point+2.0f;
      del=(int)(fdel);
      frac=1.0f-fdel+(float)del;
      utpek=(delpt3-del);   if (utpek<0) utpek+=48000; va=delay3[utpek];
      utpek=(delpt3-del+1); if (utpek<0) utpek+=48000; diff=delay3[utpek]-va;
      val+=harm[(int)(point*4096)]*(va+frac*diff);
      auxbuff[i]=(int)val;
      delpt3++; if (delpt3==48000) delpt3=0;
    }
  }
  if (sett2.echoset) {
    del=(int)(sett2.ECtime*Srate); feed=(int)(16383.*sett2.ECfeed);
    for (i=0; i<1024; i++) {
      delay[delpt]=auxbuff[i];
      utpek=(delpt-del); utpek+=(48000&(-(utpek<0)));
      delay[delpt]+=(delay[utpek]*feed)>>14;
      auxbuff[i]=delay[utpek];
      delpt++; delpt-=(48000&(-(delpt==48000)));
    }
  }
  if (sett2.filterset) {
    q1=4096./sett2.FLTQ;
    if (sett2.centerT) f1=1000.*PI*v1/Srate; /* ??? Var 500 ??? */
      else f1=8192.*PI*sett2.FLTcenter/Srate;
    if (sett2.filtertype==HIGHPASS)
      for (i=0; i<1024; i++) {
        is=auxbuff[i];
        l=d2+((f1*d1)>>12); h=is-l-((q1*d1)>>12); b=((f1*h)>>12)+d1; d1=b; d2=l;
        auxbuff[i]=h;
      }
    else if (sett2.filtertype==LOWPASS)
      for (i=0; i<1024; i++) {
        is=auxbuff[i];
        l=d2+((f1*d1)>>12); h=is-l-((q1*d1)>>12); b=((f1*h)>>12)+d1; d1=b; d2=l;
        auxbuff[i]=l;
      }
    else if (sett2.filtertype==BANDPASS)
      for (i=0; i<1024; i++) {
        is=auxbuff[i];
        l=d2+((f1*d1)>>12); h=is-l-((q1*d1)>>12); b=((f1*h)>>12)+d1; d1=b; d2=l;
        auxbuff[i]=b;
      }
    else if (sett2.filtertype==BANDSTOP)
      for (i=0; i<1024; i++) {
        is=auxbuff[i];
        l=d2+((f1*d1)>>12); h=is-l-((q1*d1)>>12); b=((f1*h)>>12)+d1; d1=b; d2=l;
        auxbuff[i]=h+l;
      }
  }
  if (sett2.reverbset) {
    int val;
    rv1=(int)(sett2.RVtime*0.0496*Srate);
    rv2=(int)(sett2.RVtime*0.03465*Srate);
    rv3=(int)(sett2.RVtime*0.02418*Srate);
    rv4=(int)(sett2.RVtime*0.01785*Srate);
    rv5=(int)(sett2.RVtime*0.01098*Srate);
    for (i=0; i<1024; i++) {
      utpek=(rvpt1-rv1); utpek+=(20000&(-(utpek<0)));
      val=((rvdel1[utpek]*12288)>>14)+auxbuff[i];
      rvdel1[rvpt1]=val;
      rvpt1++; rvpt1-=(20000&(-(rvpt1==20000)));
      samp1=rvdel1[utpek]-((val*12288)>>14);

      utpek=(rvpt2-rv2); utpek+=(20000&(-(utpek<0)));
      val=((rvdel2[utpek]*11796)>>14) + samp1;
      rvdel2[rvpt2]=val;
      rvpt2++; rvpt2-=(20000&(-(rvpt2==20000)));
      samp2=rvdel2[utpek]-((val*11796) >> 14);

      utpek=(rvpt3-rv3); utpek+=(20000&(-(utpek<0)));
      val=((rvdel3[utpek]*11321) >> 14) + samp2;
      rvdel3[rvpt3]=val;
      rvpt3++; rvpt3-=(20000&(-(rvpt3==20000)));
      samp3=rvdel3[utpek] - ((val*11321) >> 14);

      utpek=(rvpt4-rv4); utpek+=(20000&(-(utpek<0)));
      val=((rvdel4[utpek]*10633) >> 14) + samp3;
      rvdel4[rvpt4]=val;
      rvpt4++; rvpt4-=(20000&(-(rvpt4==20000)));
      samp4=rvdel4[utpek]-((val*10633) >> 14);

      utpek=(rvpt5-rv5); utpek+=(20000&(-(utpek<0)));
      val=((rvdel5[utpek]*10846) >> 14) + samp4;
      rvdel5[rvpt5]=val;
      rvpt5++; rvpt5-=(20000&(-(rvpt5==20000)));
      auxbuff[i]=rvdel5[utpek]-((val*10846) >> 14);
    }
  }
  if (sett2.compresset) {
    float o, is;
    for (i=0; i<1024; i++) {
      is=(float)auxbuff[i]/32768.0f;
      o=0.01f*cdel; cdel=cdel-o+fabsf(is);
      is*=powf(o,sett2.CMA);
      auxbuff[i]=(int)(is*32767.0f);
    }
  }
}

void sensitive(void) {
  XtVaSetValues(fileButton,XmNsensitive,True,NULL);
  XtVaSetValues(meditButton,XmNsensitive,True,NULL);
  XtVaSetValues(settingsButton,XmNsensitive,True,NULL);
  XtVaSetValues(effectsButton,XmNsensitive,True,NULL);
  XtVaSetValues(toolsButton,XmNsensitive,True,NULL);
  XtVaSetValues(graphButton,XmNsensitive,True,NULL);
  XtVaSetValues(grpButton,XmNsensitive,True,NULL);
  XtVaSetValues(eventButton,XmNsensitive,True,NULL);
  XtVaSetValues(regionButton,XmNsensitive,True,NULL);
  XtVaSetValues(helpButton,XmNsensitive,True,NULL);
  XtVaSetValues(horiscroll,XmNsensitive,True,NULL);
}

void unsensitive(void) {
  XtVaSetValues(fileButton,XmNsensitive,False,NULL);
  XtVaSetValues(meditButton,XmNsensitive,False,NULL);
  XtVaSetValues(settingsButton,XmNsensitive,False,NULL);
  XtVaSetValues(effectsButton,XmNsensitive,False,NULL);
  XtVaSetValues(toolsButton,XmNsensitive,False,NULL);
  XtVaSetValues(graphButton,XmNsensitive,False,NULL);
  XtVaSetValues(grpButton,XmNsensitive,False,NULL);
  XtVaSetValues(eventButton,XmNsensitive,False,NULL);
  XtVaSetValues(regionButton,XmNsensitive,False,NULL);
  XtVaSetValues(helpButton,XmNsensitive,False,NULL);
  XtVaSetValues(horiscroll,XmNsensitive,False,NULL);
}

void SaveOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  int i, j, sampsread, lmax, rmax, lmax2, rmax2, sample, volval, volval2, t=0,
    lvolval, lvolval2, rvolval, rvolval2, track, auxval, auxval2, *pek,
    val, clipped=0, acc, acc2, js;
  double time=0., blocklen=1024./Srate, timeceil;

  if (saving) return;
  saving=1;
  SetWatchCursor(w);
  unsensitive();
  memcpy(&sett2, &sett, sizeof(struct settings));

  XmStringGetLtoR(calldata->value, charset, &filename);

  for (i=0; i<MAXLYD; i++) if (sounds[i].used) {
    if (strcmp(sounds[i].filename,filename)==0) {
      ResetCursor(w);
      XtManageChild(writeWarning);
      XtFree(filename);
      saving=0;
      sensitive();
      return;
    }
  }

  out_AFsetup=AFnewfilesetup();
  AFinitchannels(out_AFsetup, AF_DEFAULT_TRACK, sett.nchnls);
  AFinitrate(out_AFsetup, AF_DEFAULT_TRACK, Srate);
  AFinitcompression(out_AFsetup, AF_DEFAULT_TRACK, compression);
  AFinitfilefmt(out_AFsetup, filefmt);
  AFinitsampfmt(out_AFsetup, AF_DEFAULT_TRACK, samp_type, bits_per_samp);
  outfile=AFopenfile(filename, "w", out_AFsetup);

  if (outfile==NULL) {
    ResetCursor(w);
    XtManageChild(writeWarning);
    XtFree(filename); AFfreefilesetup(out_AFsetup);
    saving=0;
    sensitive();
    return;
  }
  XtVaSetValues(progressScale, XmNvalue, 0, NULL);
  XtManageChild(progressDialog);
  while (XtAppPending(app_context)) {
     XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
  }  

  num_mixed=0; phase=phase2=phase3=0; delpt=delpt2=delpt3=0;
  rvpt1=rvpt2=rvpt3=rvpt4=rvpt5=0;
  for (i=0; i<MAXLYD; i++) if (sounds[i].used) {
    sounds[i].mixstatus=WAITING; sounds[i].loopcount=sounds[i].loop;
    sounds[i].finestart=(int)(sounds[i].startsec*Srate)%1024;
  }
  for (i=0; i<MAXLYD; i++) events[i].mixstatus=WAITING;
  for (i=0; i<48000; i++) delay[i]=delay2[i]=delay3[i]=0;
  for (i=0; i<20000; i++) rvdel1[i]=rvdel2[i]=rvdel3[i]=rvdel4[i]=rvdel5[i]=0;
  d1=d2=0; cdel=0.;
  if (marked) time=mstart;
  master.sumnum=(int)(time*Srate/1024.);
  t=master.sumnum; timeceil=time+blocklen;
  for (i=0; i<MAXLYD; i++)
    if ((timeceil>sounds[i].startsec) && (sounds[i].used)) regist(i);
  for (i=0; i<MAXLYD; i++)
    if ((time>=events[i].startsec) && (events[i].used)) eregist(i,1);
  do {
    if (sett.nchnls==1) for (i=0; i<1024; i++) outbuff[i]=0;
      else if (sett.nchnls==2) for (i=0; i<2048; i++) outbuff[i]=0;
        else for (i=0; i<4096; i++) outbuff[i]=0;
    for (i=0; i<1024; i++) auxbuff[i]=0;
    for (i=0; i<MAXLYD; i++) if (sounds[i].mixstatus==MIXING) {
      sampsread=mixin(i); track=sounds[i].track;

      volval=(tracks[track].volval*tracks[track].fullvol[t])>>15;
      val=((tracks[track].volval*tracks[track].fullvol[t+1])>>15);
      volval2=val-volval;
      auxval=tracks[track].fullaux[t]; auxval2=tracks[track].fullaux[t+1]-auxval;
      lvolval=(tracks[track].fullpan[t]*volval)>>15;
      lvolval2=((tracks[track].fullpan[t+1]*val)>>15)-lvolval;
      rvolval=((32767-tracks[track].fullpan[t])*volval)>>15;
      rvolval2=(((32767-tracks[track].fullpan[t+1])*val)>>15)-rvolval;

      if (sounds[i].channels==1) {
        for (j=0; j<sampsread; j++)
          auxbuff[j]+=(int)(framebuff[j]*(auxval+((j*auxval2)>>10)))>>15;
        if (sett.nchnls==1) for (j=0; j<sampsread; j++) {
          outbuff[j]+=(framebuff[j]*(volval+((j*volval2)>>10)))>>15;
        } else if (sett.nchnls==2) for (j=0; j<sampsread; j++) {
          pek=outbuff+j+j;
          *pek    +=(framebuff[j]*(lvolval+((j*lvolval2)>>10)))>>15;
          *(pek+1)+=(framebuff[j]*(rvolval+((j*rvolval2)>>10)))>>15;
        } else for (j=0; j<sampsread; j++) {
          if (tracks[track].pair==2) {
           outbuff[j<<2]+=(framebuff[j]*(lvolval+((j*lvolval2)>>10)))>>15;
           outbuff[(j<<2)+1]+=(framebuff[j]*(rvolval+((j*rvolval2)>>10)))>>15;
          } else {
           outbuff[(j<<2)+2]+=(framebuff[j]*(lvolval+((j*lvolval2)>>10)))>>15;
           outbuff[(j<<2)+3]+=(framebuff[j]*(rvolval+((j*rvolval2)>>10)))>>15;
          }
        }
      } else if (sounds[i].channels==2) {
        for (j=0; j<sampsread; j++)
          auxbuff[j]+=(int)((framebuff[j+j]+framebuff[j+j+1])*
            (auxval+((j*auxval2)>>10)))>>16;
        if (sett.nchnls==1) for (j=0; j<sampsread; j++) {
          outbuff[j]+=((framebuff[j+j]+framebuff[j+j+1])*
            (volval+((j*volval2)>>10)))>>16;
        } else if (sett.nchnls==2) for (j=0; j<sampsread; j++) {
          pek=outbuff+j+j;
          *pek    +=(framebuff[j+j]*(lvolval+((j*lvolval2)>>10)))>>15;
          *(pek+1)+=(framebuff[j+j+1]*(rvolval+((j*rvolval2)>>10)))>>15;
        } else for (j=0; j<sampsread; j++) {
          js=j<<2;
          if (tracks[track].pair==2) {
            outbuff[js]+=(framebuff[j+j]*(lvolval+((j*lvolval2)>>10)))>>15;
            outbuff[js+1]+=(framebuff[j+j+1]*(rvolval+((j*rvolval2)>>10)))>>15;
          } else {
            outbuff[js+2]+=(framebuff[j+j]*(lvolval+((j*lvolval2)>>10)))>>15;
            outbuff[js+3]+=(framebuff[j+j+1]*(rvolval+((j*rvolval2)>>10)))>>15;
          }
        }
      } else {
        acc=0;
        for (j=0; j<sampsread; j++) {
          js=j<<2;
          auxbuff[j]+=(int)((framebuff[js]+framebuff[js+1]
            +framebuff[js+2]+framebuff[js+3])*(auxval+(acc>>10)))>>17;
          acc+=auxval2;
        }
        acc=0; acc2=0;
        if (sett.nchnls==1) for (j=0; j<sampsread; j++) {
          pek=outbuff+j; js=j<<2;
          val=((framebuff[js]+framebuff[js+1]+framebuff[js+2]
            +framebuff[js+3])*(volval+(acc>>10)))>>17;
            *pek+=val; acc+=volval2;
        } else if (sett.nchnls==2) for (j=0; j<sampsread; j++) {
          pek=outbuff+j+j; js=j<<2;
          *pek    +=((framebuff[js]+framebuff[js+1])*(lvolval+(acc>>10)))>>16;
          *(pek+1)+=((framebuff[js+2]+framebuff[js+3])*(rvolval+(acc2>>10)))>>16;
          acc+=lvolval2; acc2+=rvolval2;
        } else for (j=0; j<sampsread; j++) {
          js=j<<2;
          outbuff[js]+=(framebuff[js]*(lvolval+(acc>>10)))>>15;
          outbuff[js+1]+=(framebuff[js+1]*(lvolval+(acc>>10)))>>15;
          outbuff[js+2]+=(framebuff[js+2]*(rvolval+(acc2>>10)))>>15;
          outbuff[js+3]+=(framebuff[js+3]*(rvolval+(acc2>>10)))>>15;
          acc+=lvolval2; acc2+=rvolval2;
        }
      }
      if (sampsread<1024) {
        unregist(i);
        XtVaSetValues(progressScale, XmNvalue, 100*num_mixed/num_total, NULL);
        XmUpdateDisplay(topLevel);
      }
    }
    while (XtAppPending(app_context))
       XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
    if (!XtIsManaged(progressDialog)) {
      ResetCursor(w);
      AFclosefile(outfile); AFfreefilesetup(out_AFsetup);
      RedrawWin();
      XtManageChild(writeWarning);
      XtUnmanageChild(progressDialog);
      XtFree(filename);
      sensitive();
      saving=0;
      return;
    }
    do_effect(t);
    volval=tracks[10].fullaux[t]>>1; volval2=(tracks[10].fullaux[t+1]>>1)-volval;
    acc=0;
    for (i=0; i<1024; i++) {
      auxbuff[i]=(auxbuff[i]*(volval+(acc>>10)))>>14;
      acc+=volval2;
    }
    volval=(tracks[10].volval*tracks[10].fullvol[t])>>15;
    volval2=((tracks[10].volval*tracks[10].fullvol[t+1])>>15)-volval;
    acc=0;
    if (sett.nchnls==1)
      for (i=0; i<1024; i++) {
        sample=((volval+(acc>>10))*(outbuff[i]+auxbuff[i]))>>15;
        mixbuff[i]=(short int)sample;
        if ((sample>32767) || (sample<-32768)) clipped=1;
        acc+=volval2;
      }
    else if (sett.nchnls==2)
      for (i=0; i<2048; i++) {
        sample=((volval+((i*volval2)>>11))*(outbuff[i]+auxbuff[i>>1]))>>15;
        mixbuff[i]=(short int)sample;
        if ((sample>32767) || (sample<-32768)) clipped=1;
      }
    else
      for (i=0; i<4096; i++) {
        sample=((volval+((i*volval2)>>12))*(outbuff[i]+auxbuff[i>>2]))>>15;
        mixbuff[i]=(short int)sample;
        if ((sample>32767) || (sample<-32768)) clipped=1;
      }
    if (clipped) {
      ResetCursor(w);
      AFclosefile(outfile); AFfreefilesetup(out_AFsetup);
      RedrawWin();
      XtManageChild(clipWarning);
      XtUnmanageChild(progressDialog);
      XtFree(filename);
      saving=0;
      sensitive();
      return;
    }
    if (AFwriteframes(outfile, AF_DEFAULT_TRACK, mixbuff, 1024)<1024) {
      ResetCursor(w);
      AFclosefile(outfile); AFfreefilesetup(out_AFsetup);
      RedrawWin();
      XtManageChild(writeWarning);
      XtUnmanageChild(progressDialog);
      XtFree(filename);
      saving=0;
      sensitive();
      return;
    }
    lmax=0; rmax=0; lmax2=0; rmax2=0;
    if (sett.nchnls==1) for (j=0; j<1024; j++) {
      sample=mixbuff[j];
      if (abs(sample)>lmax) lmax=abs(sample);
    } else if (sett.nchnls==2) for (j=0; j<1024; j++) {
      sample=mixbuff[j+j];
      if (abs(sample)>lmax) lmax=abs(sample);
      sample=mixbuff[j+j+1];
      if (abs(sample)>rmax) rmax=abs(sample);
    } else for (j=0; j<1024; j++) {
      sample=mixbuff[j<<2];
      if (abs(sample)>lmax) lmax=abs(sample);
      sample=mixbuff[(j<<2)+1];
      if (abs(sample)>rmax) rmax=abs(sample);
      sample=mixbuff[(j<<2)+2];
      if (abs(sample)>lmax2) lmax2=abs(sample);
      sample=mixbuff[(j<<2)+3];
      if (abs(sample)>rmax2) rmax2=abs(sample);
    }
    master.summary[master.sumnum][0]=(lmax>>8)+(rmax&0xff00);
    master.summary[master.sumnum++][1]=(lmax2>>8)+(rmax2&0xff00);
    time+=blocklen; timeceil+=blocklen;
    for (i=0; i<MAXLYD; i++)
      if ((timeceil>sounds[i].startsec) && (sounds[i].used)) regist(i);
    for (i=0; i<MAXLYD; i++)
      if ((time>=events[i].startsec) && (events[i].used)) eregist(i,0);
    t++;
  } while ((num_mixed<num_total) && ((marked==0) || (time<mstop)));

  AFclosefile(outfile); AFfreefilesetup(out_AFsetup);
  for (i=0; i<MAXLYD; i++) if (sounds[i].mixstatus==MIXING) unregist(i);

  XtFree(filename);
  fileDirty = FALSE;
  RedrawWin();
  XtUnmanageChild(saveFileBox);
  XtUnmanageChild(progressDialog);
  ResetCursor(w);
  sensitive();
  saving=0;
}

void Realtime(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, j, sampsread, lmax, rmax, sample, x, volval, volval2,
    fill, buffwarning=0, t=0, lvolval, lvolval2, rvolval, rvolval2,
    lmax2, rmax2, track, chnls, auxval, auxval2, outsamp, *pek, val, js;
  int newsock, sel, rcode, acc, acc2, nports;
  double time=0., blocklen=1024./Srate, timeceil;
  ALport audioport, recordport;
  ALconfig config;
  unsigned int pbuf[8];
  long pvbuf[8];
  char buf[50], *filename;
#ifdef LINUX  
  long tempBufOut [4] = {0,0,0,0};
#endif
  
  if (realtiming || saving) return;
  strcpy(buf,"play");
  for (i=0; i<MAXSENDTO; i++) if (sendto_connected[i])
    write(sendto_sockets[i], buf, strlen(buf)+1);

  memcpy(&sett2, &sett, sizeof(struct settings));
  if (sett.nchnls==4) chnls=4; else chnls=2;

  if (recording) {
    if (strcmp(recordname,"Record file name not set")==0) {
      XtManageChild(writeWarning);
      XtVaSetValues(recordB,XmNlabelPixmap,norecordpixmap,NULL);
      return;
    }
    for (i=0; i<MAXLYD; i++) {
      if ((sounds[i].used)&&(strcmp(sounds[i].filename,recordname)==0)) {
        sounds[i].used=0;
        sounds[i].wmade=0;
        XtDestroyWidget(sounds[i].bulletin);
        num_total--;
      }
    }
  }

  realtiming=1; xold=-999999;
  XtVaSetValues(playB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,playpixmap,NULL);
  unsensitive();
  XSetFunction(theDisplay, markerGC, GXxor);
  XSetForeground(theDisplay, markerGC, backcolor.pixel);

  pvbuf[0] = AL_OUTPUT_RATE;  pvbuf[1] = Srate;
  pvbuf[2] = AL_INPUT_RATE;   pvbuf[3] = Srate;
  pvbuf[4] = AL_CHANNEL_MODE;
  if (chnls==4) pvbuf[5]=AL_4CHANNEL; else pvbuf[5]=AL_STEREO;
  ALsetparams(AL_DEFAULT_DEVICE, pvbuf, 6);
  config=ALnewconfig();
  ALsetqueuesize(config, (int)(Srate*sett.buffsize*chnls));
  ALsetchannels(config, chnls);
  audioport=ALopenport("Realtime Mix", "w", config);
  if (recording) {
    ALsetchannels(config, sett.recstereo?2:1);
    recordport=ALopenport("Mix record", "r", config);
    out_AFsetup=AFnewfilesetup();
    AFinitchannels(out_AFsetup, AF_DEFAULT_TRACK, sett.recstereo?2:1);
    AFinitrate(out_AFsetup, AF_DEFAULT_TRACK, Srate);
    AFinitcompression(out_AFsetup, AF_DEFAULT_TRACK, compression);
    AFinitfilefmt(out_AFsetup, filefmt);
    AFinitsampfmt(out_AFsetup, AF_DEFAULT_TRACK, samp_type, bits_per_samp);
    outfile=AFopenfile(recordname, "w", out_AFsetup);
    if (outfile==NULL) {
      XtManageChild(writeWarning); AFfreefilesetup(out_AFsetup);
      return;
    }
  }

#ifdef SGI
  nports=mdInit();
  MIDIport=mdOpenInPort(0);

  if ((nports<=0) || (MIDIport==NULL)) havemidi=0; else
  havemidi=1;
#else
  havemidi=0;
#endif

  quartercount=0; MIDItime=-1.;

  num_mixed=0; phase=phase2=phase3=0; delpt=delpt2=delpt3=0;
  rvpt1=rvpt2=rvpt3=rvpt4=rvpt5=0;
  for (i=0; i<MAXLYD; i++) if (sounds[i].used) {
    sounds[i].mixstatus=WAITING; sounds[i].loopcount=sounds[i].loop;
    sounds[i].finestart=(int)(sounds[i].startsec*Srate)%1024;
  }
  for (i=0; i<MAXLYD; i++) events[i].mixstatus=WAITING;
  for (i=0; i<48000; i++) delay[i]=delay2[i]=delay3[i]=0;
  d1=d2=0; cdel=0.;
  for (i=0; i<20000; i++) rvdel1[i]=rvdel2[i]=rvdel3[i]=rvdel4[i]=rvdel5[i]=0;
  if (marked) time=mstart;
  master.sumnum=(int)(time*Srate/1024.);
  t=master.sumnum; timeceil=time+blocklen;
  for (i=0; i<MAXLYD; i++)
    if ((timeceil>sounds[i].startsec) && (sounds[i].used)) regist(i);
  for (i=0; i<MAXLYD; i++)
    if ((time>=events[i].startsec) && (events[i].used)) eregist(i,1);
  do {
    if (chnls==4) i=4095; else i=2047;
    do outbuff[i]=0; while (i--);
    i=1023; do auxbuff[i]=0; while (i--);
    for (i=0; i<MAXLYD; i++) if (sounds[i].mixstatus==MIXING) {
      sampsread=mixin(i); track=sounds[i].track;
      volval=(tracks[track].volval*tracks[track].fullvol[t])>>15;
      val=((tracks[track].volval*tracks[track].fullvol[t+1])>>15);
      volval2=val-volval;
      auxval=tracks[track].fullaux[t]; auxval2=tracks[track].fullaux[t+1]-auxval;
      lvolval=(tracks[track].fullpan[t]*volval)>>15;
      lvolval2=((tracks[track].fullpan[t+1]*val)>>15)-lvolval;
      rvolval=((32767-tracks[track].fullpan[t])*volval)>>15;
      rvolval2=(((32767-tracks[track].fullpan[t+1])*val)>>15)-rvolval;
      if (sounds[i].channels==1) {
        acc=0;
        for (j=0; j<sampsread; j++) {
          auxbuff[j]+=(int)(framebuff[j]*(auxval+(acc>>10)))>>15;
          acc+=auxval2;
        }
        acc=0; acc2=0;
        if (sett.nchnls==1) for (j=0; j<sampsread; j++) {
          pek=outbuff+j+j;
          val=(framebuff[j]*(volval+(acc>>10)))>>15;
          *pek+=val; *(pek+1)+=val; acc+=volval2;
        } else if (sett.nchnls==2) for (j=0; j<sampsread; j++) {
          pek=outbuff+j+j;
          *pek    +=(framebuff[j]*(lvolval+(acc>>10)))>>15;
          *(pek+1)+=(framebuff[j]*(rvolval+(acc2>>10)))>>15;
          acc+=lvolval2; acc2+=rvolval2;
        } else {
          if (tracks[track].pair==2) for (j=0; j<sampsread; j++) {
            *(outbuff+(j<<2))+=(*(framebuff+j)*(lvolval+(acc>>10)))>>15;
            *(outbuff+(j<<2)+1)+=(*(framebuff+j)*(rvolval+(acc2>>10)))>>15;
            acc+=lvolval2; acc2+=rvolval2;
          } else for (j=0; j<sampsread; j++) {
            *(outbuff+(j<<2)+2)+=(*(framebuff+j)*(lvolval+(acc>>10)))>>15;
            *(outbuff+(j<<2)+3)+=(*(framebuff+j)*(rvolval+(acc2>>10)))>>15;
            acc+=lvolval2; acc2+=rvolval2;
          }
        }
      } else if (sounds[i].channels==2) {
        acc=0;
        for (j=0; j<sampsread; j++) {
          auxbuff[j]+=(int)((framebuff[j+j]
            +framebuff[j+j+1])*(auxval+(acc>>10)))>>16;
          acc+=auxval2;
        }
        acc=0; acc2=0;
        if (sett.nchnls==1) for (j=0; j<sampsread; j++) {
          pek=outbuff+j+j;
          val=((framebuff[j+j]+framebuff[j+j+1])*(volval+(acc>>10)))>>16;
            *pek+=val; *(pek+1)+=val; acc+=volval2;
        } else if (sett.nchnls==2) for (j=0; j<sampsread; j++) {
          pek=outbuff+j+j;
          *pek    +=(framebuff[j+j]*(lvolval+(acc>>10)))>>15;
          *(pek+1)+=(framebuff[j+j+1]*(rvolval+(acc2>>10)))>>15;
          acc+=lvolval2; acc2+=rvolval2;
        } else {
          if (tracks[track].pair==2) for (j=0; j<sampsread; j++) {
            *(outbuff+(j<<2))+=(*(framebuff+j+j)*(lvolval+(acc>>10)))>>15;
            *(outbuff+(j<<2)+1)+=(*(framebuff+j+j+1)*(rvolval+(acc2>>10)))>>15;
            acc+=lvolval2; acc2+=rvolval2;
          } else for (j=0; j<sampsread; j++) {
           *(outbuff+(j<<2)+2)+=(*(framebuff+j+j)*(lvolval+(acc>>10)))>>15;
           *(outbuff+(j<<2)+3)+=(*(framebuff+j+j+1)*(rvolval+(acc2>>10)))>>15;
            acc+=lvolval2; acc2+=rvolval2;
          }
        }
      } else {
        acc=0;
        for (j=0; j<sampsread; j++) {
          js=j<<2;
          *(auxbuff+j)+=(int)((*(framebuff+js)+*(framebuff+js+1)
            +*(framebuff+js+2)+*(framebuff+js+3))
            *(auxval+(acc>>10)))>>17;
          acc+=auxval2;
        }
        acc=0; acc2=0;
        if (sett.nchnls==1) for (j=0; j<sampsread; j++) {
          pek=outbuff+j+j; js=j<<2;
          val=((*(framebuff+js)+*(framebuff+js+1)+*(framebuff+js+2)
            +*(framebuff+js+3))*(volval+(acc>>10)))>>17;
            *pek+=val; *(pek+1)+=val; acc+=volval2;
        } else if (sett.nchnls==2) for (j=0; j<sampsread; j++) {
          pek=outbuff+j+j; js=j<<2;
          *pek    +=((*(framebuff+js)+*(framebuff+js+1))
            *(lvolval+(acc>>10)))>>16;
          *(pek+1)+=((*(framebuff+js+2)+*(framebuff+js+3))
            *(rvolval+(acc2>>10)))>>16;
          acc+=lvolval2; acc2+=rvolval2;
        } else for (j=0; j<sampsread; j++) {
          js=j<<2;
          *(outbuff+js)+=(*(framebuff+js)*(lvolval+(acc>>10)))>>15;
          *(outbuff+js+1)+=(*(framebuff+js+1)*(lvolval+(acc>>10)))>>15;
          *(outbuff+js+2)+=(*(framebuff+js+2)*(rvolval+(acc2>>10)))>>15;
          *(outbuff+js+3)+=(*(framebuff+js+3)*(rvolval+(acc2>>10)))>>15;
          acc+=lvolval2; acc2+=rvolval2;
        }
      }
      if ((sampsread<1024) || (sounds[i].muted) ||
        (tracks[sounds[i].track].muted)) unregist(i);
    }
    if (recording) {
      ALreadsamps(recordport, recordbuff, 1024);
      if (sett.monitor) {
        if (sett.recstereo) 
          for (i=0; i<1024; i++) auxbuff[i] += (int)(recordbuff[i+i]);
        else for (i=0; i<1024; i++) auxbuff[i] += (int)(recordbuff[i]);
      }
      AFwriteframes(outfile, AF_DEFAULT_TRACK, recordbuff, 1024);
    }
    do_effect(t);
    volval=tracks[10].fullaux[t]>>1; volval2=(tracks[10].fullaux[t+1]>>1)-volval;
    acc=0;
    for (i=0; i<1024; i++) {
      auxbuff[i]=(auxbuff[i]*(volval+(acc>>10)))>>14;
      acc+=volval2;
    }
    volval=(tracks[10].volval*tracks[10].fullvol[t])>>15;
    volval2=((tracks[10].volval*tracks[10].fullvol[t+1])>>15)-volval;
    if (chnls==4) for (i=0; i<4096; i++) {
      outsamp=((volval+((i*volval2)>>12))*(outbuff[i]+auxbuff[i>>2]))>>15;
/*      if (outsamp>32767) outsamp=32767; else if (outsamp<-32767) outsamp=-32767;*/
      mixbuff[i]=(short int)outsamp;
    }
    else for (i=0; i<2048; i++) {
      outsamp=((volval+((i*volval2)>>11))*(outbuff[i]+auxbuff[i>>1]))>>15;
/*      if (outsamp>32767) outsamp=32767; else if (outsamp<-32767) outsamp=-32767; */
      mixbuff[i]=(short int)outsamp;
    }
    if ((fasting==0) || (t%10==0)) {
      if (chnls==4) ALwritesamps(audioport, mixbuff, 4096);
        else ALwritesamps(audioport, mixbuff, 2048);
    }
    lmax=0; rmax=0; lmax2=0; rmax2=0;
    if (sett.nchnls==1) for (j=0; j<1024; j++) {
      sample=mixbuff[j+j];
      if (sample>lmax) lmax=sample;
    } else if (sett.nchnls==2) for (j=0; j<1024; j++) {
      sample=mixbuff[j+j];
      if (sample>lmax) lmax=sample;
      sample=mixbuff[j+j+1];
      if (sample>rmax) rmax=sample;
    } else for (j=0; j<1024; j++) {
      sample=mixbuff[j<<2];
      if (sample>lmax) lmax=sample;
      sample=mixbuff[(j<<2)+1];
      if (sample>rmax) rmax=sample;
      sample=mixbuff[(j<<2)+2];
      if (sample>lmax2) lmax2=sample;
      sample=mixbuff[(j<<2)+3];
      if (sample>rmax2) rmax2=sample;
    }
    master.summary[master.sumnum][0]=(lmax>>8)+(rmax&0xff00);
    master.summary[master.sumnum++][1]=(lmax2>>8)+(rmax2&0xff00);
    fill=ALgetfilled(audioport);
#ifdef LINUX
    //ALwritesamps(audioport,tempBufOut,chnls);
#endif 
    x=(int)((time-leftsec-(double)fill/Srate/chnls)*1000./sec_pr_scr);
    if (x!=xold) {
      if (x<-10000) x=-10000; if (x>10000) x=10000;
      XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,xold,30,xold,735);
      XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,x,30,x,735);
      xold=x;
    }
    if (buffwarning==0) {
      if (fill<3000) {
        buffwarning=1;
        XtVaSetValues(stopB,XmNlabelPixmap,stoppixmap,NULL);
      } 
    } else if (fill>3000) {
      buffwarning=0;
      XtVaSetValues(stopB,XmNlabelPixmap,nostoppixmap,NULL);
    }
    XFlush(theDisplay);
    if (sett.scroll) {
      if ((x>=1000) || (x<0)) {
        leftsec=floor(time);
        if (leftsec+sec_pr_scr>MAXLEN) leftsec=MAXLEN-sec_pr_scr;
        RedrawWin();
        XtVaSetValues(horiscroll,XmNvalue,(int)(leftsec*10.),NULL);
      }
    }
    time+=blocklen; timeceil+=blocklen;
    for (i=0; i<MAXLYD; i++)
      if ((timeceil>sounds[i].startsec) && (sounds[i].used)) regist(i);
    for (i=0; i<MAXLYD; i++)
      if ((time>=events[i].startsec) && (events[i].used)) eregist(i,0);
    if (XtAppPending(app_context) && ((fill>(int)(sett.buffsize*Srate/2.))
      || (t%10==0))) {
      XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
    }
    FD_ZERO(&client_sockets_set);
    for (i=0; i<MAXCLIENTS; i++) { newsock=client_sockets[i];
      if (newsock>=0) FD_SET(newsock, &client_sockets_set);
    }
    timeout.tv_sec=0; timeout.tv_usec=0;
    sel=select(FD_SETSIZE, &client_sockets_set, NULL, NULL, &timeout);
    if (sel>0) for (i=0; i<MAXCLIENTS; i++) {
      newsock=client_sockets[i];
      if (newsock>=0) {
        if (FD_ISSET(newsock, &client_sockets_set)) readcontrol(i);
      }
    }
    t++;
  } while (((num_mixed<num_total) || (recording)) && realtiming &&
    ((marked==0) || (time<mstop)));

  while (fill=ALgetfilled(audioport)) {
#ifdef LINUX
    ALwritesamps(audioport,tempBufOut,chnls);
#endif 
    x=(int)((time-leftsec-(double)fill/Srate/chnls)*1000./sec_pr_scr);
    if (x!=xold) {
      XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,xold,30,xold,735);
      XDrawLine(theDisplay,XtWindow(sketchpad),markerGC,x,30,x,735);
      xold=x; XFlush(theDisplay);
    }
  }
  ALcloseport(audioport);
  if (recording) {
    ALcloseport(recordport);
    AFclosefile(outfile); AFfreefilesetup(out_AFsetup);
    SetWatchCursor(topLevel);
    rcode=newsound(recordname, marked?mstart:0., 1, 1, 0, -1, 0);
    if (rcode==1) openWarn(recordname);
    if (rcode==2) XtManageChild(srateWarning);
    if (rcode==3) XtManageChild(bitsWarning);
    ResetCursor(topLevel);
  }
#ifdef SGI
  if (havemidi) mdClosePort(MIDIport);
#endif
  for (i=0; i<MAXLYD; i++) if (sounds[i].mixstatus==MIXING) unregist(i);
  realtiming=0; fasting=0; recording=0;

  XtVaSetValues(playB,XmNlabelPixmap,noplaypixmap,NULL);
  XtVaSetValues(stopB,XmNlabelPixmap,nostoppixmap,NULL);
  XtVaSetValues(fastB,XmNlabelPixmap,nofastpixmap,NULL);
  XtVaSetValues(recordB,XmNlabelPixmap,norecordpixmap,NULL);
  sensitive();
  RedrawWin();
}

void StopRealT(w, client, call)
Widget w;
XtPointer client, call;
{
  int i;
  char buf[20];

  realtiming=0;
  strcpy(buf,"stop");
  for (i=0; i<MAXSENDTO; i++) if (sendto_connected[i])
    write(sendto_sockets[i], buf, strlen(buf)+1);
}

void Fast(w, client, call)
Widget w;
XtPointer client, call;
{
  if (fasting) {
    fasting=0;
    XtVaSetValues(fastB,XmNlabelPixmap,nofastpixmap,NULL);
  } else {
    fasting=1;
    XtVaSetValues(fastB,XmNlabelPixmap,fastpixmap,NULL);
    if (realtiming==0) Realtime(NULL,NULL,NULL);
  }
}

void Record(w, client, call)
Widget w;
XtPointer client, call;
{
  if (recording) {
    recording=0;
    XtVaSetValues(recordB,XmNlabelPixmap,norecordpixmap,NULL);
  } else if ((realtiming==0) && (saving==0)) {
    recording=1;
    XtVaSetValues(recordB,XmNlabelPixmap,recordpixmap,NULL);
    Realtime(NULL,NULL,NULL);
  }
}

void New(w, client, call)
Widget w;
XtPointer client, call;
{
  backup();
  Initvalues(0);
}

void Quit(w, client, call)
Widget w;
XtPointer client, call;
{
  int i;

  for (i=0; i<MAXCLIENTS; i++)
    if (client_sockets[i]>=0) close(client_sockets[i]);
  close(server_socket);
  system("rm -f mix.backup");
  audioexit(0);
}

void Undo(w, client, call)
Widget w;
XtPointer client, call;
{
  loadm("mix.backup",topLevel,1,0);
}

void Cut(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, j, trk, strt;
  double tid;

  if (!marked) return;
  SetWatchCursor(w);
  backup();
  for (i=0; i<MAXLYD; i++) { pasteSounds[i].used=0; pasteEvents[i].used=0; }

  for (i=0; i<MAXLYD; i++) {
    if ((sounds[i].used) && (sounds[i].startsec>=mstart) &&
      (sounds[i].startsec<mstop)) {
        pasteSounds[i].used=1;
        pasteSounds[i].startsec=sounds[i].startsec-mstart;
        pasteSounds[i].track=sounds[i].track;
        pasteSounds[i].loop=sounds[i].loop;
        strcpy(pasteSounds[i].filename, sounds[i].filename);
        deletesound(i);
      }
    if ((events[i].used) && (events[i].startsec>=mstart) &&
      (events[i].startsec<mstop)) {
        pasteEvents[i].used=1;
        pasteEvents[i].startsec=events[i].startsec;
        pasteEvents[i].track=events[i].track;
        pasteEvents[i].name=events[i].name;
        strcpy(pasteEvents[i].value, events[i].value);
        deleteevent(i);
      }
  }

  strt=mstart*Srate/1024;
  for (trk=1; trk<11; trk++) {
    pasteTracks[trk].vnumsquare=0;
    pasteTracks[trk].pnumsquare=0;
    pasteTracks[trk].anumsquare=0;
    for (i=tracks[trk].vnumsquare-1; i>1; i--) {
      tid=tracks[trk].vxpos[i]*1024./Srate;
      if ((tid>=mstart) && (tid<mstop)) {
        pasteTracks[trk].vxpos[pasteTracks[trk].vnumsquare]=
          tracks[trk].vxpos[i]-strt;
        pasteTracks[trk].vypos[pasteTracks[trk].vnumsquare++]=tracks[trk].vypos[i];
        for (j=i; j<tracks[trk].vnumsquare; j++) {
          tracks[trk].vxpos[j]=tracks[trk].vxpos[j+1];
          tracks[trk].vypos[j]=tracks[trk].vypos[j+1];
        }
        tracks[trk].vnumsquare--;
      }
    }
    for (i=tracks[trk].pnumsquare-1; i>1; i--) {
      tid=tracks[trk].pxpos[i]*1024./Srate;
      if ((tid>=mstart) && (tid<mstop)) {
        pasteTracks[trk].pxpos[pasteTracks[trk].pnumsquare]=
          tracks[trk].pxpos[i]-strt;
        pasteTracks[trk].pypos[pasteTracks[trk].pnumsquare++]=tracks[trk].pypos[i];
        for (j=i; j<tracks[trk].pnumsquare; j++) {
          tracks[trk].pxpos[j]=tracks[trk].pxpos[j+1];
          tracks[trk].pypos[j]=tracks[trk].pypos[j+1];
        }
        tracks[trk].pnumsquare--;
      }
    }
    for (i=tracks[trk].anumsquare-1; i>1; i--) {
      tid=tracks[trk].axpos[i]*1024./Srate;
      if ((tid>=mstart) && (tid<mstop)) {
        pasteTracks[trk].axpos[pasteTracks[trk].anumsquare]=
          tracks[trk].axpos[i]-strt;
        pasteTracks[trk].aypos[pasteTracks[trk].anumsquare++]=tracks[trk].aypos[i];
        for (j=i; j<tracks[trk].anumsquare; j++) {
          tracks[trk].axpos[j]=tracks[trk].axpos[j+1];
          tracks[trk].aypos[j]=tracks[trk].aypos[j+1];
        }
        tracks[trk].anumsquare--;
      }
    }
  }

  for (i=0; i<MAXLYD; i++) {
    if ((sounds[i].used) && (sounds[i].startsec>=mstart))
      sounds[i].startsec-=(mstop-mstart);
    if ((events[i].used) && (events[i].startsec>=mstart))
      events[i].startsec-=(mstop-mstart);
  }

  for (trk=1; trk<11; trk++) {
    for (i=2; i<tracks[trk].vnumsquare; i++) {
      tid=tracks[trk].vxpos[i]*1024./Srate;
      if (tid>=mstart) tracks[trk].vxpos[i]-=(mstop-mstart)*Srate/1024.;
    }
    computegraph(trk, 1);
    makelist(trk, 1);
    for (i=2; i<tracks[trk].pnumsquare; i++) {
      tid=tracks[trk].pxpos[i]*1024./Srate;
      if (tid>=mstart) tracks[trk].pxpos[i]-=(mstop-mstart)*Srate/1024.;
    }
    computegraph(trk, 2);
    makelist(trk, 2);
    for (i=2; i<tracks[trk].anumsquare; i++) {
      tid=tracks[trk].axpos[i]*1024./Srate;
      if (tid>=mstart) tracks[trk].axpos[i]-=(mstop-mstart)*Srate/1024.;
    }
    computegraph(trk, 3);
    makelist(trk, 3);
  }

  pasteOk=1; pastelen=mstop-mstart; selnum=0;
  XtVaSetValues(paste, XmNsensitive, True, NULL);
  XtVaSetValues(insert, XmNsensitive, True, NULL);
  RedrawWin();
  ResetCursor(w);
}

void Copy(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, j, trk, strt;
  double tid;

  if (!marked) return;
  SetWatchCursor(w);
  for (i=0; i<MAXLYD; i++) { pasteSounds[i].used=0; pasteEvents[i].used=0; }

  for (i=0; i<MAXLYD; i++) {
    if ((sounds[i].used) && (sounds[i].startsec>=mstart) &&
      (sounds[i].startsec<mstop)) {
        pasteSounds[i].used=1;
        pasteSounds[i].startsec=sounds[i].startsec-mstart;
        pasteSounds[i].track=sounds[i].track;
        pasteSounds[i].loop=sounds[i].loop;
        strcpy(pasteSounds[i].filename, sounds[i].filename);
      }
    if ((events[i].used) && (events[i].startsec>=mstart) &&
      (events[i].startsec<mstop)) {
        pasteEvents[i].used=1;
        pasteEvents[i].startsec=events[i].startsec;
        pasteEvents[i].track=events[i].track;
        pasteEvents[i].name=events[i].name;
        strcpy(pasteEvents[i].value, events[i].value);
      }
  }

  strt=mstart*Srate/1024;
  for (trk=1; trk<11; trk++) {
    pasteTracks[trk].vnumsquare=0;
    pasteTracks[trk].pnumsquare=0;
    pasteTracks[trk].anumsquare=0;
    for (i=tracks[trk].vnumsquare-1; i>1; i--) {
      tid=tracks[trk].vxpos[i]*1024./Srate;
      if ((tid>=mstart) && (tid<mstop)) {
        pasteTracks[trk].vxpos[pasteTracks[trk].vnumsquare]=
          tracks[trk].vxpos[i]-strt;
        pasteTracks[trk].vypos[pasteTracks[trk].vnumsquare++]=tracks[trk].vypos[i];
      }
    }
    for (i=tracks[trk].pnumsquare-1; i>1; i--) {
      tid=tracks[trk].pxpos[i]*1024./Srate;
      if ((tid>=mstart) && (tid<mstop)) {
        pasteTracks[trk].pxpos[pasteTracks[trk].pnumsquare]=
          tracks[trk].pxpos[i]-strt;
        pasteTracks[trk].pypos[pasteTracks[trk].pnumsquare++]=tracks[trk].pypos[i];
      }
    }
    for (i=tracks[trk].anumsquare-1; i>1; i--) {
      tid=tracks[trk].axpos[i]*1024./Srate;
      if ((tid>=mstart) && (tid<mstop)) {
        pasteTracks[trk].axpos[pasteTracks[trk].anumsquare]=
          tracks[trk].axpos[i]-strt;
        pasteTracks[trk].aypos[pasteTracks[trk].anumsquare++]=tracks[trk].aypos[i];
      }
    }
  }

  pasteOk=1; pastelen=mstop-mstart; selnum=0;
  XtVaSetValues(paste, XmNsensitive, True, NULL);
  XtVaSetValues(insert, XmNsensitive, True, NULL);
  RedrawWin();
  ResetCursor(w);
}

void Paste(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, j, trk, strt, addpos, blk;
  double tid;

  SetWatchCursor(w);
  backup();

  for (i=0; i<MAXLYD; i++) {
    if (pasteSounds[i].used) {
      newsound(pasteSounds[i].filename, pasteSounds[i].startsec+mstart,
        pasteSounds[i].track, pasteSounds[i].loop, 0, -1, 0);
      }
    if (pasteEvents[i].used) {
      newevent(pasteEvents[i].name, pasteEvents[i].track,
      pasteEvents[i].startsec+mstart, -1);
      strcpy(events[i].value, pasteEvents[i].value);
      XtVaSetValues(events[newenum].valueText,XmNvalue,events[i].value,NULL);
      XbaeMatrixSetCell(eventMatrix,events[newenum].posinmatrix,2,events[i].value);
    }
  }

  strt=mstart*Srate/1024;
  for (trk=1; trk<11; trk++) {
    for (i=0; i<pasteTracks[trk].vnumsquare; i++) {
      blk=pasteTracks[trk].vxpos[i]+strt;
      if ((tracks[trk].vnumsquare<MAXNODE-1) && (blk<GRAPHSAMPLES)) {
        for (j=1; j<=tracks[trk].vnumsquare; j++)
          if (blk<tracks[trk].vxpos[j]) { addpos=j; break; }
        tracks[trk].vnumsquare++;
        for (j=tracks[trk].vnumsquare; j>addpos; j--) {
          tracks[trk].vxpos[j]=tracks[trk].vxpos[j-1];
          tracks[trk].vypos[j]=tracks[trk].vypos[j-1];
        }
        tracks[trk].vxpos[addpos]=blk;
        tracks[trk].vypos[addpos]=pasteTracks[trk].vypos[i];
      }
    }
    computegraph(trk, 1);
    makelist(trk, 1);

    for (i=0; i<pasteTracks[trk].pnumsquare; i++) {
      blk=pasteTracks[trk].pxpos[i]+strt;
      if ((tracks[trk].pnumsquare<MAXNODE-1) && (blk<GRAPHSAMPLES)) {
        for (j=1; j<=tracks[trk].pnumsquare; j++)
          if (blk<tracks[trk].pxpos[j]) { addpos=j; break; }
        tracks[trk].pnumsquare++;
        for (j=tracks[trk].pnumsquare; j>addpos; j--) {
          tracks[trk].pxpos[j]=tracks[trk].pxpos[j-1];
          tracks[trk].pypos[j]=tracks[trk].pypos[j-1];
        }
        tracks[trk].pxpos[addpos]=blk;
        tracks[trk].pypos[addpos]=pasteTracks[trk].pypos[i];
      }
    }
    computegraph(trk, 2);
    makelist(trk, 2);

    for (i=0; i<pasteTracks[trk].anumsquare; i++) {
      blk=pasteTracks[trk].axpos[i]+strt;
      if ((tracks[trk].anumsquare<MAXNODE-1) && (blk<GRAPHSAMPLES)) {
        for (j=1; j<=tracks[trk].anumsquare; j++)
          if (blk<tracks[trk].axpos[j]) { addpos=j; break; }
        tracks[trk].anumsquare++;
        for (j=tracks[trk].anumsquare; j>addpos; j--) {
          tracks[trk].axpos[j]=tracks[trk].axpos[j-1];
          tracks[trk].aypos[j]=tracks[trk].aypos[j-1];
        }
        tracks[trk].axpos[addpos]=blk;
        tracks[trk].aypos[addpos]=pasteTracks[trk].aypos[i];
      }
    }
    computegraph(trk, 3);
    makelist(trk, 3);
  }

  selnum=0;
  RedrawWin();
  ResetCursor(w);
}

void Insert(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, j, trk, strt, addpos, blk;
  double tid;

  SetWatchCursor(w);
  backup();
  strt=mstart*Srate/1024;
  blk=(int)(pastelen*Srate/1024.);

  for (i=0; i<MAXLYD; i++) {
    if ((sounds[i].used) && (sounds[i].startsec>=mstart) &&
     (sounds[i].startsec+pastelen<MAXLEN)) sounds[i].startsec+=pastelen;
    if ((events[i].used) && (events[i].startsec>=mstart))
      events[i].startsec+=pastelen;
  }

  for (trk=1; trk<11; trk++) {
    for (i=2; i<tracks[trk].vnumsquare; i++) {
      tid=tracks[trk].vxpos[i]*1024./Srate;
      if ((tid>=mstart) && (tracks[trk].vxpos[i]+blk<GRAPHSAMPLES))
        tracks[trk].vxpos[i]+=blk;
    }
    for (i=2; i<tracks[trk].pnumsquare; i++) {
      tid=tracks[trk].pxpos[i]*1024./Srate;
      if ((tid>=mstart) && (tracks[trk].pxpos[i]+blk<GRAPHSAMPLES))
        tracks[trk].pxpos[i]+=blk;
    }
    for (i=2; i<tracks[trk].anumsquare; i++) {
      tid=tracks[trk].axpos[i]*1024./Srate;
      if ((tid>=mstart) && (tracks[trk].axpos[i]+blk<GRAPHSAMPLES))
        tracks[trk].axpos[i]+=blk;
    }
  }

  for (i=0; i<MAXLYD; i++) {
    if (pasteSounds[i].used) {
      newsound(pasteSounds[i].filename, pasteSounds[i].startsec+mstart,
        pasteSounds[i].track, pasteSounds[i].loop, 0, -1, 0);
      }
    if (pasteEvents[i].used) {
      newevent(pasteEvents[i].name, pasteEvents[i].track,
      pasteEvents[i].startsec+mstart, -1);
      strcpy(events[i].value, pasteEvents[i].value);
      XtVaSetValues(events[newenum].valueText,XmNvalue,events[i].value,NULL);
      XbaeMatrixSetCell(eventMatrix,events[newenum].posinmatrix,2,events[i].value);
    }
  }

  for (trk=1; trk<11; trk++) {
    for (i=0; i<pasteTracks[trk].vnumsquare; i++) {
      blk=pasteTracks[trk].vxpos[i]+strt;
      if ((tracks[trk].vnumsquare<MAXNODE-1) && (blk<GRAPHSAMPLES)) {
        for (j=1; j<=tracks[trk].vnumsquare; j++)
          if (blk<tracks[trk].vxpos[j]) { addpos=j; break; }
        tracks[trk].vnumsquare++;
        for (j=tracks[trk].vnumsquare; j>addpos; j--) {
          tracks[trk].vxpos[j]=tracks[trk].vxpos[j-1];
          tracks[trk].vypos[j]=tracks[trk].vypos[j-1];
        }
        tracks[trk].vxpos[addpos]=blk;
        tracks[trk].vypos[addpos]=pasteTracks[trk].vypos[i];
      }
    }
    computegraph(trk, 1);
    makelist(trk, 1);

    for (i=0; i<pasteTracks[trk].pnumsquare; i++) {
      blk=pasteTracks[trk].pxpos[i]+strt;
      if ((tracks[trk].pnumsquare<MAXNODE-1) && (blk<GRAPHSAMPLES)) {
        for (j=1; j<=tracks[trk].pnumsquare; j++)
          if (blk<tracks[trk].pxpos[j]) { addpos=j; break; }
        tracks[trk].pnumsquare++;
        for (j=tracks[trk].pnumsquare; j>addpos; j--) {
          tracks[trk].pxpos[j]=tracks[trk].pxpos[j-1];
          tracks[trk].pypos[j]=tracks[trk].pypos[j-1];
        }
        tracks[trk].pxpos[addpos]=blk;
        tracks[trk].pypos[addpos]=pasteTracks[trk].pypos[i];
      }
    }
    computegraph(trk, 2);
    makelist(trk, 2);

    for (i=0; i<pasteTracks[trk].anumsquare; i++) {
      blk=pasteTracks[trk].axpos[i]+strt;
      if ((tracks[trk].anumsquare<MAXNODE-1) && (blk<GRAPHSAMPLES)) {
        for (j=1; j<=tracks[trk].anumsquare; j++)
          if (blk<tracks[trk].axpos[j]) { addpos=j; break; }
        tracks[trk].anumsquare++;
        for (j=tracks[trk].anumsquare; j>addpos; j--) {
          tracks[trk].axpos[j]=tracks[trk].axpos[j-1];
          tracks[trk].aypos[j]=tracks[trk].aypos[j-1];
        }
        tracks[trk].axpos[addpos]=blk;
        tracks[trk].aypos[addpos]=pasteTracks[trk].aypos[i];
      }
    }
    computegraph(trk, 3);
    makelist(trk, 3);
  }

  selnum=0;
  RedrawWin();
  ResetCursor(w);
}

void Play(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(playFileBox);
}

void PlayOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

  XmStringGetLtoR(calldata->value, charset, &filename);
  strcpy(streng,"sfplay ");
  strcat(streng, filename);
  strcat(streng," &");
  system(streng);

  XtFree(filename);
}

void DisplayB(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(displayBB);
}

void DispGrid(w, client, call)
Widget w;
XtPointer client, call;
{
  sett.showgrid=(int)XmToggleButtonGetState(w);
  if (sett.showgrid) { DrawGrid(); RestoreWin(NULL,NULL,NULL); }
    else RedrawWin();
}

void DispAll(w, client, call)
Widget w;
XtPointer client, call;
{
  sett.showall=(int)XmToggleButtonGetState(w);
  RedrawWin();
}

void DispLin(w, client, call)
Widget w;
XtPointer client, call;
{
  sett.showlin=(int)XmToggleButtonGetState(w);
}

void DispNum(w, client, call)
Widget w;
XtPointer client, call;
{
  sett.shownum=(int)XmToggleButtonGetState(w);
  RedrawWin();
}

void DispScroll(w, client, call)
Widget w;
XtPointer client, call;
{
  sett.scroll=(int)XmToggleButtonGetState(w);
}

void DispFastdraw(w, client, call)
Widget w;
XtPointer client, call;
{
  sett.fastdraw=(int)XmToggleButtonGetState(w);
  RedrawWin();
}

void White(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) { backcolor=white; RedrawWin(); }
}

void Blue(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) { backcolor=skyblue; RedrawWin(); }
}

void Monitor(w, client, call)
Widget w;
XtPointer client, call;
{
  sett.monitor=(int)XmToggleButtonGetState(w);
}

void Stereorec(w, client, call)
Widget w;
XtPointer client, call;
{
  sett.recstereo=(int)XmToggleButtonGetState(w);
}

void TimeB(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(timeBB);
}

void TimeToggle(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(timeToggleSec)) sett.bpmscale=0;
  else if (XmToggleButtonGetState(timeToggleBeats)) sett.bpmscale=1;
  RedrawWin();
}

void BPMtext(w, client, call)
Widget w;
XtPointer client, call;
{
  sett.bpm=atoi(XmTextGetString(timeBPMtext));
  RedrawWin();
}

void BPBtext(w, client, call)
Widget w;
XtPointer client, call;
{
  sett.bpb=atoi(XmTextGetString(timeBPBtext));
  RedrawWin();
}

void Audio(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(audioBB);
}

void SRate(w, client, call)
Widget w;
XtPointer client, call;
{
  int R;
  SetWatchCursor(topLevel);
  if (XmToggleButtonGetState(srate8)) R=8000;
  else if (XmToggleButtonGetState(srate11025)) R=11025;
  else if (XmToggleButtonGetState(srate16)) R=16000;
  else if (XmToggleButtonGetState(srate2205)) R=22050;
  else if (XmToggleButtonGetState(srate32)) R=32000;
  else if (XmToggleButtonGetState(srate441)) R=44100;
  else if (XmToggleButtonGetState(srate48)) R=48000;
  newsamp(R);
  ResetCursor(topLevel);
}

void Buffval(w, client, call)
Widget w;
XtPointer client, call;
{
  sett.buffsize=atof(XmTextGetString(buffText));
}

void AudioLinvol(w, client, call)
Widget w;
XtPointer client, call;
{
  int i;
  sett.linvol=(int)XmToggleButtonGetState(w);
  for (i=1; i<11; i++) { computegraph(i,1); computegraph(i,3); }
}

void Nchnls1(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) sett.nchnls=1;
}

void Nchnls2(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) sett.nchnls=2;
}

void Nchnls4(w, client, call)
Widget w;
XtPointer client, call;
{
  int i;

  if (XmToggleButtonGetState(w)) {
    sett.nchnls=4;
    for (i=1; i<10; i++) XtVaSetValues(tracks[i].fbackB,XmNsensitive,True,NULL); 
  } else
    for (i=1; i<10; i++) XtVaSetValues(tracks[i].fbackB,XmNsensitive,False,NULL); 
}

void SendtoB(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(sendtoBB);
}

void AcceptConnect(w, client, call)
Widget w;
XtPointer client, call;
{
  int newsock, i, sel;
  int me_sockaddr_len=sizeof(me_sockaddr);
  struct hostent *callinginfo;

  if (XmToggleButtonGetState(w)) accepting=1; else accepting=0;
  while (accepting) {
    if (XtAppPending(app_context))
      XtAppProcessEvent(app_context,XtIMXEvent|XtIMTimer|XtIMAlternateInput);
    if ((newsock=accept(server_socket, (struct sockaddr *)&me_sockaddr,
     &me_sockaddr_len))>=0) {
      callinginfo=gethostbyaddr(&(me_sockaddr.sin_addr.s_addr),
        sizeof(struct in_addr), AF_INET);
      for (i=0; i<MAXCLIENTS; i++) if (client_sockets[i]==-1) break;
      if (i==MAXCLIENTS) printf("Error: Max number of controlling clients exceeded\n");
      else {
        connectWarn(callinginfo->h_name);
        client_sockets[i]=newsock;
      }
    }
    FD_ZERO(&client_sockets_set);
    for (i=0; i<MAXCLIENTS; i++) { newsock=client_sockets[i];
      if (newsock>=0) FD_SET(newsock, &client_sockets_set);
    }
    timeout.tv_sec=0; timeout.tv_usec=0;
    sel=select(FD_SETSIZE, &client_sockets_set, NULL, NULL, &timeout);
    if (sel>0) for (i=0; i<MAXCLIENTS; i++) {
      newsock=client_sockets[i];
      if (newsock>=0) {
        if (FD_ISSET(newsock, &client_sockets_set)) readcontrol(i);
      }
    }
  }
}

void SendtoConnect(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  struct hostent *nameinfo;
  char *txt, buf[10];

  if (XmToggleButtonGetState(sendtoToggle[i])) {
    txt=XmTextGetString(sendtoText[i]);
    if (strlen(txt)==0) {
      slavefailWarn("**Not given **");
      XmToggleButtonSetState(sendtoToggle[i],False,False);
      return;
    }
    nameinfo=gethostbyname(txt); XtFree(txt);
    bcopy((char *)nameinfo->h_addr,
      (char *)&sendto_addresses[i].sin_addr, nameinfo->h_length);
    sendto_addresses[i].sin_family=AF_INET;
    sendto_addresses[i].sin_port=htons(PORTNUM);
    sendto_sockets[i]=socket(AF_INET, SOCK_STREAM, 0);
    if (connect(sendto_sockets[i], (struct sockaddr *)&sendto_addresses[i],
      sizeof(sendto_addresses[i]))==-1) {
        slavefailWarn(nameinfo->h_name);
        close(sendto_sockets[i]);
        XmToggleButtonSetState(sendtoToggle[i],False,False);
      } else sendto_connected[i]=1;
  } else {
    strcpy(buf,"close");
    write(sendto_sockets[i], buf, strlen(buf)+1);
    close(sendto_sockets[i]);
    sendto_connected[i]=0;
  }
}

void Soverv(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(sovervBB);
}

void Eoverv(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(eovervBB);
}

void Effects1(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(effects1BB);
}

void Effects2(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(effects2BB);
}

void Eff1FLTRad(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(eff1FLTlowpass)) sett.filtertype=LOWPASS;
  else if (XmToggleButtonGetState(eff1FLThipass)) sett.filtertype=HIGHPASS;
  else if (XmToggleButtonGetState(eff1FLTbandpass)) sett.filtertype=BANDPASS;
  else if (XmToggleButtonGetState(eff1FLTbandstop)) sett.filtertype=BANDSTOP;
  sett2.filtertype=sett.filtertype;
}

void Eff1Echo(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) sett.echoset=1;
    else sett.echoset=0;
  sett2.echoset=sett.echoset;
}

void Eff1Harm(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) sett.harmonizerset=1;
    else sett.harmonizerset=0;
  sett2.harmonizerset=sett.harmonizerset;
}

void Eff1Ring(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) sett.ringset=1;
    else sett.ringset=0;
  sett2.ringset=sett.ringset;
}

void Eff1Chorus(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) sett.chorusset=1;
    else sett.chorusset=0;
  sett2.chorusset=sett.chorusset;
}

void Eff1Filter(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) sett.filterset=1;
    else sett.filterset=0;
  sett2.filterset=sett.filterset;
}

void Eff2Reverb(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) sett.reverbset=1;
    else sett.reverbset=0;
  sett2.reverbset=sett.reverbset;
}

void Eff2Compress(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) sett.compresset=1;
    else sett.compresset=0;
  sett2.compresset=sett.compresset;
}

void Eff2Octave(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) sett.octaveset=1;
    else sett.octaveset=0;
  sett2.octaveset=sett.octaveset;
}


void RM1Scale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;
  XmString text;

  sprintf(streng,"%d Hz",calldata->value);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(w,XmNtitleString,text,NULL);
  sett.RM1=(double)(calldata->value);
  sett2.RM1=sett.RM1;
  XtFree(text);
}

void ECtimeScale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;
  XmString text;

  sprintf(streng,"%5.3f s",(calldata->value)/1000.);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(w,XmNtitleString,text,NULL);
  sett.ECtime=(double)(calldata->value)/1000.;
  sett2.ECtime=sett.ECtime;
  XtFree(text);
}

void ECfeedScale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;
  XmString text;

  sprintf(streng,"%5.3f",(calldata->value)/1000.);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(w,XmNtitleString,text,NULL);
  sett.ECfeed=(double)(calldata->value)/1000.;
  sett2.ECfeed=sett.ECfeed;
  XtFree(text);
}

void CHspeedScale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;
  XmString text;

  sprintf(streng,"%5.3f Hz",(calldata->value)/1000.);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(w,XmNtitleString,text,NULL);
  sett.CHspeed=(double)(calldata->value)/1000.;
  sett2.CHspeed=sett.CHspeed;
  XtFree(text);
}

void CHdepthScale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;
  XmString text;

  sprintf(streng,".%04ds",(int)((calldata->value)));
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(w,XmNtitleString,text,NULL);
  sett.CHdepth=(double)(calldata->value)/10000.;
  sett2.CHdepth=sett.CHdepth;
  XtFree(text);
}

void HAinterScale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;
  XmString text;

  sprintf(streng,"%5.2f",(calldata->value)/41.666667-12.);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(w,XmNtitleString,text,NULL);
  sett.HAinter=(double)(calldata->value)/41.666667-12.;
  sett2.HAinter=sett.HAinter;
  XtFree(text);
}

void FLTcenterScale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;
  XmString text;

  sprintf(streng,"%d Hz",calldata->value);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(w,XmNtitleString,text,NULL);
  sett.FLTcenter=(double)(calldata->value);
  sett2.FLTcenter=sett.FLTcenter;
  XtFree(text);
}

void FLTcenterToggle(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(w)) sett.centerT=1;
    else sett.centerT=0;
  sett2.centerT=sett.centerT;
}

void FLTQScale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;
  XmString text;

  sprintf(streng,"%5.2f",(calldata->value)/10.+.5);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(w,XmNtitleString,text,NULL);
  sett.FLTQ=(double)(calldata->value)/10.+.5;
  sett2.FLTQ=sett.FLTQ;
  XtFree(text);
}

void RVtimeScale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;
  XmString text;

  sprintf(streng,"%5.3f s",(calldata->value)/100.);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(w,XmNtitleString,text,NULL);
  sett.RVtime=(double)(calldata->value)/100.;
  sett2.RVtime=sett.RVtime;
  XtFree(text);
}

void CMAScale(w, client, call)
Widget w;
XtPointer client, call;
{
  int i=*(int *)client;
  XmScaleCallbackStruct *calldata=(XmScaleCallbackStruct *) call;
  XmString text;

  sprintf(streng,"%5.3f",(calldata->value)/500.-1.);
  text=XmStringCreateLocalized(streng);
  XtVaSetValues(w,XmNtitleString,text,NULL);
  sett.CMA=(double)(calldata->value)/500.-1.;
  sett2.CMA=sett.CMA;
  XtFree(text);
}

void To_machine(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(machineBB);
}

void QuantSet(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(machine1)) sett.quant=1;
  else if (XmToggleButtonGetState(machine1d2)) sett.quant=2;
  else if (XmToggleButtonGetState(machine1d3)) sett.quant=3;
  else if (XmToggleButtonGetState(machine1d4)) sett.quant=4;
  else if (XmToggleButtonGetState(machine1d8)) sett.quant=8;
  else if (XmToggleButtonGetState(machine1d16)) sett.quant=16;
}

void AutoQuant(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(machineAutoQuant)) sett.autoquant=1;
  else sett.autoquant=0;
}

void TempoApply(w, client, call)
Widget w;
XtPointer client, call;
{
  int i, t;
  double tempo;

  tempo=atof(XmTextGetString(machineTimeText));
  if (tempo<0.01) tempo=0.01;
  if (tempo>100.) tempo=100.;

  for (i=0; i<MAXLYD; i++) if (sounds[i].used) {
    sounds[i].startsec*=tempo;
    sprintf(streng, "%7.3f", sounds[i].startsec);
    XtVaSetValues(sounds[i].startText,XmNvalue,streng,NULL);
    XbaeMatrixSetCell(soundMatrix,sounds[i].posinmatrix,1,streng);
    sprintf(streng, "%7.3f", sounds[i].startsec+sounds[i].dur);
    XtVaSetValues(sounds[i].stopText,XmNvalue,streng,NULL);
    XbaeMatrixSetCell(soundMatrix,sounds[i].posinmatrix,2,streng);
  }
  for (i=0; i<MAXLYD; i++) if (events[i].used) {
    events[i].startsec*=tempo;
    sprintf(streng, "%7.3f", events[i].startsec);
    XtVaSetValues(events[i].startText,XmNvalue,streng,NULL);
    XbaeMatrixSetCell(eventMatrix,events[i].posinmatrix,1,streng);
  }

  for (t=1; t<11; t++) {
    for (i=2; i<tracks[t].vnumsquare; i++)
      tracks[t].vxpos[i]*=tempo;
    for (i=2; i<tracks[t].anumsquare; i++)
      tracks[t].axpos[i]*=tempo;
    for (i=2; i<tracks[t].pnumsquare; i++)
      tracks[t].pxpos[i]*=tempo;
    computegraph(t,1);
    computegraph(t,2);
    computegraph(t,3);
    makelist(t,1);
    makelist(t,2);
    makelist(t,3);
  }

  RedrawWin();
}

void To_srate(w, client, call)
Widget w;
XtPointer client, call;
{
  system("soundfiler &");
}

void To_edit(w, client, call)
Widget w;
XtPointer client, call;
{
  system("soundeditor &");
}

void ShowList(w, client, call)
Widget w;
XtPointer client, call;
{
  if (seltrack==0) return;
  if (seltype==1) XtManageChild(tracks[seltrack].volSel);
  else if (seltype==2) XtManageChild(tracks[seltrack].panSel);
  else if (seltype==3) XtManageChild(tracks[seltrack].auxSel);
}

void NewRegion(w, client, call)
Widget w;
XtPointer client, call;
{
  if (regmarked==0) return;
  XtManageChild(newRegionBB);
}

void NewregionOk(w, client, call)
Widget w;
XtPointer client, call;
{
  int track;
  float tid;

  SetWatchCursor(w);
  tid=atof(XmTextGetString(trackT));
  if ((tid<0.) || (tid>=MAXLEN)) tid=leftsec; 
  for (track=1; track<10; track++)
    if (XmToggleButtonGadgetGetState(newregToggles[track])) break;
  ResetCursor(w);
}

void MatrixSelect(w, client, call)
Widget w;
XtPointer client, call;
{
  Boolean set;
  int pt;
  String *spt;

  XbaeMatrixSelectCellCallbackStruct *data=
    (XbaeMatrixSelectCellCallbackStruct *)call;
  XtVaGetValues(soundMatrix,XmNrowLabels,&spt,NULL);
  pt=atoi(*(spt+data->row));
  set=**(data->selected_cells+data->row);
  if (set) {
    XbaeMatrixDeselectCell(soundMatrix,data->row,0);
    sounds[pt].grouped=0;
    sounds[pt].groupdiff=0.;
    sounds[pt].groupdifftrack=0;
    numgrouped--;
    if (numgrouped==0) Desensgrp();
  } else {
    XbaeMatrixSelectCell(soundMatrix,data->row,0);
    sounds[pt].grouped=1;
    numgrouped++;
    Sensgrp();
  }
  RedrawTrack(sounds[pt].track);
}

void MatrixEnter(w, client, call)
Widget w;
XtPointer client, call;
{
  XbaeMatrixEnterCellCallbackStruct *data=
    (XbaeMatrixEnterCellCallbackStruct *)call;

  if (data->column==2) (data->doit)=False; 
}

void MatrixLeave(w, client, call)
Widget w;
XtPointer client, call;
{
  XbaeMatrixLeaveCellCallbackStruct *data=
    (XbaeMatrixLeaveCellCallbackStruct *)call;
  int track, pt, tb;
  double start, stop;
  String *spt;

  if (num_total==0) return;
  XtVaGetValues(soundMatrix,XmNrowLabels,&spt,NULL);
  pt=atoi(*(spt+data->row));
  if (data->column==3) {
    track=atoi(data->value);
    if ((track<1) || (track>9)) (data->doit)=False;
    else {
      sprintf(streng, "%d", track);
      XtVaSetValues(sounds[pt].trackText,XmNvalue,streng,NULL);
    }
  } else if (data->column==1) {
    start=atof(data->value);
    if ((start<0.) || (start+sounds[pt].dur>MAXLEN)) (data->doit)=False;
    else {
      sprintf(streng, "%7.3f", start);
      XtVaSetValues(sounds[pt].startText,XmNvalue,streng,NULL);
      sprintf(streng, "%7.3f", sounds[pt].startsec+sounds[pt].dur);
      XtVaSetValues(sounds[pt].stopText,XmNvalue,streng,NULL);
    }
  }
}

void SovervSize(w, client, call)
Widget w;
XtPointer client, call;
{
  if (XmToggleButtonGetState(sovervSize)) {
     sett.smallfont=1;
     XtVaSetValues(soundMatrix,XmNfontList,smallfontlist,NULL);
  } else {
    sett.smallfont=0;
    XtVaSetValues(soundMatrix,XmNfontList,largefontlist,NULL);
  }
}

void EMatrixSelect(w, client, call)
Widget w;
XtPointer client, call;
{
  Boolean set;
  int pt;
  String *spt;
  XbaeMatrixSelectCellCallbackStruct *data=
    (XbaeMatrixSelectCellCallbackStruct *)call;

  if (events_total==0) return;
  XtVaGetValues(eventMatrix,XmNrowLabels,&spt,NULL);
  pt=atoi(*(spt+data->row));
  set=**(data->selected_cells+data->row);
  if (set) {
    XbaeMatrixDeselectCell(eventMatrix,data->row,0);
    events[pt].grouped=0;
    events[pt].groupdiff=0.;
    events[pt].groupdifftrack=0;
    numgrouped--;
    if (numgrouped==0) Desensgrp();
  } else {
    XbaeMatrixSelectCell(eventMatrix,data->row,0);
    events[pt].grouped=1;
    numgrouped++;
    Sensgrp();
  }
  RedrawTrack(events[pt].track);
}

void EMatrixEnter(w, client, call)
Widget w;
XtPointer client, call;
{
  XbaeMatrixEnterCellCallbackStruct *data=
    (XbaeMatrixEnterCellCallbackStruct *)call;

/*  if (data->column==2) (data->doit)=False; */
}

void EMatrixLeave(w, client, call)
Widget w;
XtPointer client, call;
{
  XbaeMatrixLeaveCellCallbackStruct *data=
    (XbaeMatrixLeaveCellCallbackStruct *)call;
  int track, pt, tb;
  double start, stop;
  String *spt;

  if (events_total==0) return;
  XtVaGetValues(eventMatrix,XmNrowLabels,&spt,NULL);
  pt=atoi(*(spt+data->row));
  if (data->column==3) {
    track=atoi(data->value);
    if ((track<1) || (track>9)) (data->doit)=False;
    else {
      sprintf(streng, "%d", track);
      XtVaSetValues(events[pt].trackText,XmNvalue,streng,NULL);
    }
  } else if (data->column==1) {
    start=atof(data->value);
    if ((start<0.) || (start>MAXLEN)) (data->doit)=False;
    else {
      sprintf(streng, "%6.2f", start);
      XtVaSetValues(events[pt].startText,XmNvalue,streng,NULL);
    }
  } else if (data->column==2) {
    sprintf(events[pt].value, "%s", data->value);
    XtVaSetValues(events[pt].valueText,XmNvalue,data->value,NULL);
  }
}

void To_print(w, client, call)
Widget w;
XtPointer client, call;
{
  XtManageChild(printFileBox);
}

void PrintVol(pt,fil)
int pt; FILE *fil;
{
 int i;
 double x1, x2;

 fprintf(fil,"gsave 0.5 setgray\n");

 for (i=1; i<tracks[pt].vnumsquare; i++) {
   x1=53.*blk_to_pix(tracks[pt].vxpos[i])/100.+50.;
   x2=53.*blk_to_pix(tracks[pt].vxpos[i+1])/100.+50.;
   fprintf(fil,"newpath %7.2f %d moveto %7.2f %d lineto stroke\n",
     x1,800-pt*70-30+tracks[pt].vypos[i],x2,800-pt*70-30+tracks[pt].vypos[i+1]);
 }
 fprintf(fil,"grestore\n");
}

void PrintPan(pt,fil)
int pt; FILE *fil;
{
 int i;
 double x1, x2;

 fprintf(fil,"gsave 0.5 setgray\n");

 for (i=1; i<tracks[pt].pnumsquare; i++) {
   x1=53.*blk_to_pix(tracks[pt].pxpos[i])/100.+50.;
   x2=53.*blk_to_pix(tracks[pt].pxpos[i+1])/100.+50.;
   fprintf(fil,"newpath %7.2f %d moveto %7.2f %d lineto stroke\n",
     x1,800-pt*70-30+tracks[pt].pypos[i],x2,800-pt*70-30+tracks[pt].pypos[i+1]);
 }
 fprintf(fil,"grestore\n");
}

void PrintGrid(fil)
FILE *fil;
{
  int i, sec;
  double x;

  fprintf(fil,"gsave 0.5 setgray [1 1] 0 setdash newpath\n");
  for (i=1; i<(int)sec_pr_scr; i++) {
    x=i*530./sec_pr_scr; sec=i+leftsec;
    if (((sec%10==0) && (sec_pr_scr<MAXLEN)) || (sec%60==0))
      fprintf(fil,"%7.2f 65 moveto 0 705 rlineto\n",x+50.);
  }
  fprintf(fil,"stroke grestore\n");
}

void PrintScale(fil)
FILE *fil;
{
  int i, sec;
  double x;
  char tall[10];

  fprintf(fil,"/Helvetica findfont 10 scalefont setfont\n");
  fprintf(fil,"newpath 50 785 moveto 530 0 rlineto stroke\n");
  for (i=1; i<(int)sec_pr_scr; i++) {
    x=i*530./sec_pr_scr; sec=i+leftsec;
    fprintf(fil,"newpath ");
    if (sec%10==0) fprintf(fil,"%7.2f 775 moveto 0 10 rlineto",x+50);
    if (sec%60==0) fprintf(fil,"%7.2f 770 moveto 0 15 rlineto",x+50);
    else if (sec_pr_scr<MAXLEN) fprintf(fil,"%7.2f 780 moveto 0 5 rlineto",x+50);
    fprintf(fil," stroke\n");

    if ((sec_pr_scr<240.) && (sec%10==0))
      fprintf(fil,"%7.2f 787 moveto (%2d:%02d) show\n",x+41,sec/60,sec%60);
    else if (sec%60==0)
      fprintf(fil,"%7.2f 787 moveto (%2d) show\n",x+45,sec/60);
  }
  fprintf(fil,"gsave [1 1] 0 setdash newpath\n");
  for (i=1; i<10; i++)
    fprintf(fil,"50 %d moveto 530 0 rlineto\n",800-i*70-35);
  fprintf(fil,"stroke grestore\n");
  if (sett.showgrid) PrintGrid(fil);
}

void PrintSound(pt,fil)
int pt; FILE *fil;
{
  int i, vert, heightl, heightr;
  double sec, scale, scale2, x;
  char tall[10];

  if (sounds[pt].nodraw) return;

  scale=1024./Srate; scale2=530./sec_pr_scr;
  vert=sounds[pt].track*70;
  fprintf(fil,"newpath\n");
  if (sounds[pt].channels==1)
    for (i=0; i<sounds[pt].sumnum; i++) {
      sec=sounds[pt].startsec+i*scale;
      x=(sec-leftsec)*scale2;
      if (x>530.) break;
      heightl=(int)(*(sounds[pt].summary+i)&65535)+1;
      fprintf(fil,"0 %d %7.2f %d l\n",1*heightl,x+50,800-vert-heightl);
    }
    else
    for (i=0; i<sounds[pt].sumnum; i++) {
      sec=sounds[pt].startsec+i*scale;
      x=(sec-leftsec)*scale2;
      if (x>530.) break;
      if (x>=0.) {
        heightl=(int)((*(sounds[pt].summary+i)&65535)>>10)+1;
        heightr=(int)(*(sounds[pt].summary+i)>>18)+1;
        fprintf(fil,"0 %d %7.2f %d l\n",2*heightl,x+50,800-vert+17-heightl);
        fprintf(fil,"0 %d %7.2f %d l\n",2*heightr,x+50,800-vert-17-heightr);
      }
    }
  fprintf(fil,"stroke\n");
  if (sett.shownum) {
    x=(sounds[pt].startsec-leftsec)*scale2;
    fprintf(fil,"gsave 0.5 setgray\n");
    fprintf(fil,"%d %d moveto (#%d) show grestore\n",(int)x+50,770-vert,pt);
  }
}

void PrintMix(fil)
FILE *fil;
{
  int i, heightl, heightr;
  double scale, scale2, x;
  char tall[10];

  scale=1024./Srate; scale2=530./sec_pr_scr;
  fprintf(fil,"newpath\n");
  for (i=0; i<master.sumnum; i++) {
    x=(i*scale-leftsec)*scale2;
    if (x>530.) break;
    if (x>=0.) {
      heightl=(int)(master.summary[i][0]&255)>>2;
      heightr=(int)(master.summary[i][0]>>8);
      fprintf(fil,"0 %d %7.2f %d l\n",2*heightl,x+50,800-700+17-heightl);
      fprintf(fil,"0 %d %7.2f %d l\n",2*heightr,x+50,800-700-17-heightr);
    }
  }
  fprintf(fil,"stroke\n");
}

void printOk(w, client, call)
Widget w;
XtPointer client, call;
{
  XmFileSelectionBoxCallbackStruct
    *calldata=(XmFileSelectionBoxCallbackStruct*) call; 
  char *filename;
  FILE *fil;
  int i;
  static XmStringCharSet charset =
                           (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

  XmStringGetLtoR(calldata->value, charset, &filename);

  XtUnmanageChild(printFileBox);
  fil=fopen(filename,"w");
  if (fil==NULL) {
    XtManageChild(writeWarning);
    XtFree(filename);
    return;
  }
  fprintf(fil,"%%!PS-Adobe-3.0\n");
  fprintf(fil,"%%%%Creator: NoTAM MIX\n");
  fprintf(fil,"%%%%Title: Soundfile mix\n");
  fprintf(fil,"%%%%BoundingBox: 0 0 600 900\n");
  fprintf(fil,"%%%%EndComments\n");
  fprintf(fil,"/l { moveto rlineto } def\n");
  fprintf(fil,"%%%%Page: 1 1\n");
  fprintf(fil,"1 0.5 scale\n");
  fprintf(fil,"0 200 translate\n");

  PrintScale(fil);
  for (i=0; i<MAXLYD; i++) if (sounds[i].used) PrintSound(i,fil);
  PrintMix(fil);
  for (i=1; i<11; i++) if ((tracks[i].vol) || (sett.showall)) PrintVol(i,fil);
  for (i=1; i<11; i++) if ((tracks[i].pan) || (sett.showall)) PrintPan(i,fil);

  fprintf(fil,"showpage\n");
  fprintf(fil,"%%%%EOF\n");
  fclose(fil);
  XtFree(filename);
}

void InventorVol(pt,fil)
int pt; FILE *fil;
{
  int i;
  double x1, x2, y1, y2, z1, z2, height, theta;

  for (i=1; i<tracks[pt].vnumsquare; i++) {
   fprintf(fil,"  Separator {\n");
   fprintf(fil,"    Material {\n");
   fprintf(fil,"      diffuseColor [ .3 .3 .7 ]\n");
   fprintf(fil,"      specularColor [ .5 .5 1 ]\n");
   fprintf(fil,"    }\n");
   x1=tracks[pt].vxpos[i]*1024./Srate;
   x2=tracks[pt].vxpos[i+1]*1024./Srate;
   y1=pt*10.+2.;
   y2=pt*10.+2.;
   z1=tracks[pt].vypos[i]/2.;
   z2=tracks[pt].vypos[i+1]/2.;
   theta=-PI/2.+atan2(z2-z1, x2-x1);
   height=sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1)+(z2-z1)*(z2-z1));
   fprintf(fil,"    Transform { translation %f %f %f rotation 0 0 1 %f }\n",
     (x1+x2)/2., (z1+z2)/2., (y1+y2)/2., theta);
   theta=atan2(y2-y1, x2-x1);
/*   fprintf(fil,"    Rotation { rotation 0 1 0 %f }\n", theta); */
   fprintf(fil,"    Cylinder { height %f radius .5 }\n", height);
   fprintf(fil,"  }\n");
 }
}


void InventorGrid(fil)
FILE *fil;
{
  int i, sec;
  double x;

  fprintf(fil,"gsave 0.5 setgray [1 1] 0 setdash newpath\n");
  for (i=1; i<(int)sec_pr_scr; i++) {
    x=i*530./sec_pr_scr; sec=i+leftsec;
    if (((sec%10==0) && (sec_pr_scr<MAXLEN)) || (sec%60==0))
      fprintf(fil,"%7.2f 65 moveto 0 705 rlineto\n",x+50.);
  }
  fprintf(fil,"stroke grestore\n");
}

void InventorScale(fil)
FILE *fil;
{
  int i, sec;
  double x;
  char tall[10];

  fprintf(fil,"  Separator {\n");
  fprintf(fil,"    Material {\n");
  fprintf(fil,"      diffuseColor [ .7 .7 0 ]\n");
  fprintf(fil,"      specularColor [ 1 1 0 ]\n");
  fprintf(fil,"    }\n");
  fprintf(fil,"    Translation { translation %f 0 -1 }\n", MAXLEN/2.);
  fprintf(fil,"    Cube { height .5 depth .5 width %f }\n", MAXLEN);
  fprintf(fil,"  }\n");
/*  for (i=1; i<(int)sec_pr_scr; i++) {
    x=i*530./sec_pr_scr; sec=i+leftsec;
    fprintf(fil,"newpath ");
    if (sec%10==0) fprintf(fil,"%7.2f 775 moveto 0 10 rlineto",x+50);
    if (sec%60==0) fprintf(fil,"%7.2f 770 moveto 0 15 rlineto",x+50);
    else if (sec_pr_scr<MAXLEN) fprintf(fil,"%7.2f 780 moveto 0 5 rlineto",x+50);
    fprintf(fil," stroke\n");

    if ((sec_pr_scr<240.) && (sec%10==0))
      fprintf(fil,"%7.2f 787 moveto (%2d:%02d) show\n",x+41,sec/60,sec%60);
    else if (sec%60==0)
      fprintf(fil,"%7.2f 787 moveto (%2d) show\n",x+45,sec/60);
  }
  fprintf(fil,"gsave [1 1] 0 setdash newpath\n"); */
  for (i=1; i<9; i++) {
    fprintf(fil,"  Separator {\n");
    fprintf(fil,"    Material {\n");
    fprintf(fil,"      diffuseColor [ .7 .0 0 ]\n");
    fprintf(fil,"      specularColor [ 1 .0 0 ]\n");
    fprintf(fil,"    }\n");
    fprintf(fil,"    Translation { translation %f 0 %f }\n", MAXLEN/2.,
      i*10.+5.);
    fprintf(fil,"    Cube { height .5 depth .5 width %f }\n", MAXLEN);
    fprintf(fil,"  }\n");   
  }
/*  if (sett.showgrid) InventorGrid(fil); */
}

void InventorSound(pt,fil)
int pt; FILE *fil;
{
  int i, vert, heightl, heightr;
  double sec, scale, scale2, x;
  char tall[10];

  scale=1024./Srate; scale2=530./sec_pr_scr;
  vert=sounds[pt].track*10;
  fprintf(fil,"  Separator {\n");
  fprintf(fil,"    Material {\n");
  fprintf(fil,"      diffuseColor [ %f 0.7 %f ]\n",
    sounds[pt].track/9., 1.-sounds[pt].track/9.);
  fprintf(fil,"      specularColor [ %f 1 %f ]\n",
    sounds[pt].track/9., 1.-sounds[pt].track/9.);
  fprintf(fil,"    }\n");
  fprintf(fil,"    Translation { translation %f 5 %f }\n",
    sounds[pt].startsec+sounds[pt].dur/2., (float)vert);
  fprintf(fil,"    Cube { height 10 depth 2 width %f }\n",
    sounds[pt].dur);
/*  if (sounds[pt].channels==1)
    for (i=0; i<sounds[pt].sumnum; i++) {
      sec=sounds[pt].startsec+i*scale;
      x=(sec-leftsec)*scale2;
      if (x>530.) break;
      heightl=(int)(*(sounds[pt].summary+i)&127)+1;
      fprintf(fil,"0 %d %7.2f %d l\n",2*heightl,x+50,800-vert-heightl);
    }
    else
    for (i=0; i<sounds[pt].sumnum; i++) {
      sec=sounds[pt].startsec+i*scale;
      x=(sec-leftsec)*scale2;
      if (x>530.) break;
      if (x>=0.) {
        heightl=(int)((*(sounds[pt].summary+i)&127)>>1)+1;
        heightr=(int)(*(sounds[pt].summary+i)>>8)+1;
        fprintf(fil,"0 %d %7.2f %d l\n",2*heightl,x+50,800-vert+17-heightl);
        fprintf(fil,"0 %d %7.2f %d l\n",2*heightr,x+50,800-vert-17-heightr);
      }
    }
  if (sett.shownum) {
    x=(sounds[pt].startsec-leftsec)*scale2;
    fprintf(fil,"gsave 0.5 setgray\n");
    fprintf(fil,"%d %d moveto (#%d) show grestore\n",(int)x+50,770-vert,pt);
  } */
  fprintf(fil, "  }\n");
}

void To_Inventor(w, client, call)
Widget w;
XtPointer client, call;
{
  FILE *fil;
  int i;

  fil=fopen("mix.iv", "w");
  if (fil==NULL) {
    XtManageChild(writeWarning);
    return;
  }
  fprintf(fil,"#Inventor V2.0 ascii\n");
  fprintf(fil,"Separator {\n");

  InventorScale(fil);
  for (i=0; i<MAXLYD; i++) if (sounds[i].used) InventorSound(i,fil);
  for (i=1; i<11; i++) if ((tracks[i].vol) || (sett.showall)) InventorVol(i,fil);
/*  for (i=1; i<11; i++) if ((tracks[i].pan) || (sett.showall)) InventorPan(i,fil);
*/
  fprintf(fil,"}\n");
  fclose(fil);
  system("ivview mix.iv&");
}

void Convert(w, client, call)
Widget w;
XtPointer client, call;
{
  sprintf(streng,"mv %s %s.bak",newsnd_name,newsnd_name);
  system(streng);
  sprintf(streng,"sfconvert %s.bak %s rate %d",newsnd_name,newsnd_name,Srate);
  system(streng);
  newsound(newsnd_name,newsnd_start,newsnd_trk,newsnd_lop,newsnd_loc,newsnd_num,0);
}

void Showcase(w, client, call)
Widget w;
XtPointer client, call;
{
  system("showcase -v /usr/lib/HelpCards/mix.help &");
}

void Hyper(w, client, call)
Widget w;
XtPointer client, call;
{
  printf("%s %s\n",app_data.www_browser,app_data.www_file);
  sprintf(streng,"%s %s &",app_data.www_browser,app_data.www_file);
  system(streng);
}

void backup(void)
{
  if (savem("mix.backup")) printf("Couldn't open backup file\n"); 
}

static void Keydown(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  KeySym keysym;
  char key_string[1];
  int key_string_len;

  key_string_len=XLookupString((XKeyEvent *) event, key_string,
    sizeof key_string, &keysym, (XComposeStatus *) NULL);
  if ((keysym==XK_BackSpace) || (keysym==XK_Delete)) {
    if (selnum==0) return;
    if (seltype==1) VolDelete();
    if (seltype==2) PanDelete();
    if (seltype==3) AuxDelete();
  } else if (keysym==XK_p) Realtime(NULL,NULL,NULL);
  else if (keysym==XK_s) StopRealT(NULL,NULL,NULL);
  else if (keysym==XK_f) Fast(NULL,NULL,NULL);
  else if (keysym==XK_plus) Zoomin(NULL,NULL,NULL);
  else if (keysym==XK_minus) Zoomout(NULL,NULL,NULL);
}


void GrpGroupall(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt;

  for (pt=0; pt<MAXLYD; pt++)
    if ((sounds[pt].grouped==0) && (sounds[pt].used)) {
      sounds[pt].grouped=1;
      XbaeMatrixSelectCell(soundMatrix,sounds[pt].posinmatrix,0);
      numgrouped++;
    }
  for (pt=0; pt<MAXLYD; pt++)
    if ((events[pt].grouped==0) && (events[pt].used)) {
      events[pt].grouped=1;
      XbaeMatrixSelectCell(eventMatrix,events[pt].posinmatrix,0);
      numgrouped++;
    } 
  Sensgrp();
  RedrawWin();
}

void GrpUngroupall(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt;

  for (pt=0; pt<MAXLYD; pt++)
    if ((sounds[pt].grouped) && (sounds[pt].used)) {
      sounds[pt].grouped=0;
      sounds[pt].groupdiff=0.;
      sounds[pt].groupdifftrack=0;
      XbaeMatrixDeselectCell(soundMatrix,sounds[pt].posinmatrix,0);
      numgrouped--;
    }
  for (pt=0; pt<MAXLYD; pt++)
    if ((events[pt].grouped) && (events[pt].used)) {
      events[pt].grouped=0;
      events[pt].groupdiff=0.;
      events[pt].groupdifftrack=0;
      XbaeMatrixDeselectCell(eventMatrix,events[pt].posinmatrix,0);
      numgrouped--;
    } 
  Desensgrp();
  RedrawWin();
}

void GrpMute(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt;

  for (pt=0; pt<MAXLYD; pt++)
   if ((sounds[pt].grouped) && (sounds[pt].used)) {
    if (sounds[pt].muted) {
      sounds[pt].muted=0;
      XtVaSetValues(sounds[pt].muteB,XmNlabelPixmap,nomutepixmap,NULL);
    } else {
      sounds[pt].muted=1;
      XtVaSetValues(sounds[pt].muteB,XmNlabelPixmap,mutepixmap,NULL);
    }
  }
  for (pt=0; pt<MAXLYD; pt++)
   if ((events[pt].grouped) && (events[pt].used)) {
    if (events[pt].muted) {
      events[pt].muted=0;
      XtVaSetValues(events[pt].muteB,XmNlabelPixmap,nomutepixmap,NULL);
    } else {
      events[pt].muted=1;
      XtVaSetValues(sounds[pt].muteB,XmNlabelPixmap,mutepixmap,NULL);
    }
  }
}

void GrpDelete(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt;

  backup();
  for (pt=0; pt<MAXLYD; pt++)
    if ((sounds[pt].grouped) && (sounds[pt].used)) deletesound(pt);
  for (pt=0; pt<MAXLYD; pt++)
    if ((events[pt].grouped) && (events[pt].used)) deleteevent(pt);
  RedrawWin();
}

void GrpDuplicate(w, client, call)
Widget w;
XtPointer client, call;
{
  int pt;

  SetWatchCursor(topLevel);
  for (pt=0; pt<MAXLYD; pt++)
    if ((sounds[pt].grouped) && (sounds[pt].used))
      newsound(sounds[pt].filename,sounds[pt].startsec+1,sounds[pt].track,
        sounds[pt].loop, sounds[pt].lock, -1, 0);
  for (pt=0; pt<MAXLYD; pt++)
    if ((events[pt].grouped) && (events[pt].used))
      newevent(events[pt].name,events[pt].track,events[pt].startsec+1,-1);
  RedrawWin();
  ResetCursor(topLevel);
}

void Variables(w, client, call)
Widget w;
XtPointer client, call;
{
  int name;

  if (w==e_echo) name=Eecho;
  else if (w==e_chorus) name=Echorus;
  else if (w==e_harmonizer) name=Eharmonizer;
  else if (w==e_ring) name=Ering;
  else if (w==e_filter) name=Efilter;
  else if (w==e_filtertype) name=Efiltertype;
  else if (w==e_echo_time) name=Eecho_time;
  else if (w==e_echo_feedback) name=Eecho_feedback;
  else if (w==e_chorus_speed) name=Echorus_speed;
  else if (w==e_chorus_depth) name=Echorus_depth;
  else if (w==e_harmonizer_interval) name=Eharmonizer_interval;
  else if (w==e_ring_frequency) name=Ering_frequency;
  else if (w==e_filter_freq) name=Efilter_freq;
  else if (w==e_filter_Q) name=Efilter_Q;
  else if (w==e_reverb) name=Ereverb;
  else if (w==e_compress) name=Ecompress;
  else if (w==e_octave) name=Eoctave;
  else if (w==e_reverb_time) name=Ereverb_time;
  else if (w==e_compress_A) name=Ecompress_A;
  else if (w==e_quadpos) name=Equadpos;
  else if (w==e_sync) name=Esync;
  else if (w==e_sync) name=Esync;
  else if (w==e_csound) name=Ecsound;

  newevent(name,1,leftsec,-1);
}

void readcontrol(sockpt)
int sockpt;
{
  char message[200], c='*';
  int p=0;
  int sock=client_sockets[sockpt];

  while (c) {
    if (read(sock, &c, 1)!=1) return;
    message[p++]=c;
    if (p==200) { printf("Socket message too long\n"); break; }
  }
  if (strcasecmp("play",message)==0) Realtime(NULL,NULL,NULL);
  if (strcasecmp("stop",message)==0) realtiming=0;
  if (strcasecmp("close",message)==0) { close(sock); client_sockets[sockpt]=-1; }
}

main(argc, argv)
int argc;
char **argv;
{
  XmString text;
  Pixmap pixelmap, bitmap2;
  int defdepth;
  static XtActionsRec actions[]={
    {"DrawingSelect", DrawingSelect},
    {"Group",       Group},
    {"GraphClick",  GraphClick},
    {"GraphDrag",   GraphDrag},
    {"GraphDrop",   GraphDrop},
    {"DrawingDrag", DrawingDrag},
    {"DrawingGrab", DrawingGrab},
    {"DrawingDrop", DrawingDrop},
    {"RegionDrag",  RegionDrag},
    {"RegionGrab",  RegionGrab},
    {"RegionDrop",  RegionDrop},
    {"Keydown",     Keydown},
  };

  int i, j;

  unsigned char columnalignments[4]={
    XmALIGNMENT_END,  XmALIGNMENT_BEGINNING, XmALIGNMENT_BEGINNING,
    XmALIGNMENT_CENTER
  };
  short widths[4] = { 30, 7, 7, 5 };
  short widths2[4] = { 25, 7, 8, 5 };
  String columnlabels[4]={
    "File name","Start","Stop","Track"
  };
  String columnlabels2[4]={
    "Variable name","Start","Value","Track"
  };

  #ifdef LINUX
  initialiseAudio();
  #endif

  theheight=735; sett.buffsize=0.5;
  if ((framebuff =(short int *)calloc(4096,sizeof(short int)))==NULL) {
    fprintf(stderr,"Unable to allocate frame buffer!\n");
    audioexit(-1); }
  if ((recordbuff =(short int *)calloc(4096,sizeof(short int)))==NULL) {
    fprintf(stderr,"Unable to allocate record buffer!\n");
    audioexit(-1); }
  if ((mixbuff =(short int *)calloc(4096,sizeof(short int)))==NULL) {
    fprintf(stderr,"Unable to allocate mixing buffer!\n");
    audioexit(-1); }
  if ((outbuff =(int *)calloc(4096,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate out buffer!\n");
    audioexit(-1); }
  if ((auxbuff =(int *)calloc(4096,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate aux buffer!\n");
    audioexit(-1); }
  if ((delay =(int *)calloc(48000,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate delay buffer!\n");
    audioexit(-1); }
  if ((delay2 =(int *)calloc(48000,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate delay buffer!\n");
    audioexit(-1); }
  if ((delay3 =(int *)calloc(48000,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate delay buffer!\n");
    audioexit(-1); }
  if ((rvdel1 =(int *)calloc(20000,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate reverb buffer!\n");
    audioexit(-1); }
  if ((rvdel2 =(int *)calloc(20000,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate reverb buffer!\n");
    audioexit(-1); }
  if ((rvdel3 =(int *)calloc(20000,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate reverb buffer!\n");
    audioexit(-1); }
  if ((rvdel4 =(int *)calloc(20000,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate reverb buffer!\n");
    audioexit(-1); }
  if ((rvdel5 =(int *)calloc(20000,sizeof(int)))==NULL) {
    fprintf(stderr,"Unable to allocate reverb buffer!\n");
    audioexit(-1); }

  printf("Starting Mix ."); fflush(stdout);

  topLevel=XtVaAppInitialize(
    &app_context,"Mix",NULL,0,&argc,argv,NULL,NULL);
  XtVaGetApplicationResources(topLevel, &app_data, resources,
    XtNumber(resources), NULL);
  dw = XmGetXmDisplay(XtDisplay(topLevel));
  XtVaSetValues(dw, XmNdragInitiatorProtocolStyle, XmDRAG_NONE, NULL);

  mainWindow=XtVaCreateManagedWidget(
    "mainWindow",xmMainWindowWidgetClass,topLevel, NULL);

  menuBar=XmCreateMenuBar(mainWindow,"menuBar",NULL,0);
  XtManageChild(menuBar);

  frame=XtVaCreateManagedWidget(
    "frame",xmFrameWidgetClass,mainWindow,
    XmNwidth, 1000, NULL);

  XmMainWindowSetAreas(mainWindow, menuBar, NULL, NULL, NULL, frame);
  XtVaSetValues(mainWindow,XmNheight,theheight+59, NULL);

  largeBB = XtVaCreateManagedWidget(
    "largeBB", xmBulletinBoardWidgetClass, frame, NULL);

   sketchpad = XtVaCreateManagedWidget(
     "sketchpad",xmDrawingAreaWidgetClass, largeBB, XmNheight,theheight, NULL);

   theDisplay = XtDisplay(sketchpad);
   theWindow = XtWindow(sketchpad);
   screen = DefaultScreen(theDisplay);
   theRoot = RootWindowOfScreen(XtScreen(sketchpad));
   theCmap = DefaultColormap( theDisplay, DefaultScreen(theDisplay) );
   bitmap = XCreatePixmap(XtDisplay(sketchpad),theRoot,1000,theheight,
     DefaultDepthOfScreen(XtScreen(sketchpad)));
   bitGC = XCreateGC( theDisplay, bitmap, 0, 0 );
   showGC = XCreateGC( theDisplay, theRoot, 0, 0 );
   markerGC = XCreateGC( theDisplay, theRoot, 0, 0 );

   XParseColor(theDisplay,theCmap,"red",&red);
   XAllocColor( theDisplay, theCmap,&red);
   XParseColor(theDisplay,theCmap,"blue",&blue);
   XAllocColor( theDisplay, theCmap,&blue);
   XParseColor(theDisplay,theCmap,"yellow",&yellow);
   XAllocColor( theDisplay, theCmap,&yellow);
   XParseColor(theDisplay,theCmap,"black",&black);
   XAllocColor( theDisplay, theCmap,&black);
   XParseColor(theDisplay,theCmap,"green",&green);
   XAllocColor( theDisplay, theCmap,&green);
   XParseColor(theDisplay,theCmap,"white",&white);
   XAllocColor( theDisplay, theCmap,&white);
   XParseColor(theDisplay,theCmap,"brown",&brown);
   XAllocColor( theDisplay, theCmap,&brown);
   XParseColor(theDisplay,theCmap,"gray75",&gray);
   XAllocColor( theDisplay, theCmap,&gray);
   XParseColor(theDisplay,theCmap,"skyblue",&skyblue);
   XAllocColor( theDisplay, theCmap,&skyblue);
   XParseColor(theDisplay,theCmap,"violet",&purple);
   XAllocColor( theDisplay, theCmap,&purple);
   XParseColor(theDisplay,theCmap,"gray10",&markcolor);
   XAllocColor( theDisplay, theCmap,&markcolor);

   backcolor=white;

   XtAppAddActions(app_context, actions, XtNumber(actions));
   XtAddCallback(sketchpad,XmNexposeCallback,RestoreWin,0);


  printf("."); fflush(stdout);
  horiscroll=XtVaCreateManagedWidget(
    "horiscroll",xmScrollBarWidgetClass, largeBB, XmNincrement,10,
    XmNorientation, XmHORIZONTAL, XmNx, 350, XmNy, 750,
    XmNmaximum, (int)(MAXLEN*10.), XmNwidth, 500, XmNheight, 20,
    XmNsliderSize, 2400, NULL);
  XtAddCallback(horiscroll, XmNdragCallback, Hslider, 0);
  XtAddCallback(horiscroll, XmNvalueChangedCallback, Hslider, 0);


  zoomBB=XtVaCreateManagedWidget("zoomBB", xmBulletinBoardWidgetClass,
    largeBB, NULL);
  zoominB=XtVaCreateManagedWidget("zoominB",xmPushButtonWidgetClass,
    zoomBB,NULL);
  XtAddCallback(zoominB,XmNactivateCallback,Zoomin,NULL);
  zoomoutB=XtVaCreateManagedWidget("zoomoutB",xmPushButtonWidgetClass,
    zoomBB,NULL);
  XtAddCallback(zoomoutB,XmNactivateCallback,Zoomout,NULL);
  text=XmStringCreateLocalized(" ");
  posL=XtVaCreateManagedWidget("posL",xmLabelWidgetClass,
    zoomBB,XmNlabelString,text,NULL);

  playBB=XtVaCreateManagedWidget("playBB", xmBulletinBoardWidgetClass,
    largeBB, NULL);
  playB=XtVaCreateManagedWidget("playB",xmPushButtonWidgetClass,playBB,NULL);
  XtAddCallback(playB,XmNactivateCallback,Realtime,NULL);
  fastB=XtVaCreateManagedWidget("fastB",xmPushButtonWidgetClass,playBB,NULL);
  XtAddCallback(fastB,XmNactivateCallback,Fast,NULL);
  stopB=XtVaCreateManagedWidget("stopB",xmPushButtonWidgetClass,playBB,NULL);
  XtAddCallback(stopB,XmNactivateCallback,StopRealT,NULL);
  recordB=XtVaCreateManagedWidget("recordB",xmPushButtonWidgetClass,playBB,NULL);
  XtAddCallback(recordB,XmNactivateCallback,Record,NULL);


  /* FILE MENU */

  fileButton=XtVaCreateManagedWidget(
    "fileButton",xmCascadeButtonWidgetClass,menuBar,NULL);

  fileMenu=XmCreatePulldownMenu(menuBar,"fileMenu",NULL,0);
  XtVaSetValues(fileButton,XmNsubMenuId,fileMenu,NULL);

  load=XtVaCreateManagedWidget("load",xmPushButtonWidgetClass,fileMenu,
    XmNmnemonic,XStringToKeysym("A"),XmNaccelerator,"Alt<Key>A",
    XmNacceleratorText,XmStringCreateLocalized("Alt+A"),NULL);

  loadFileBox=XmCreateFileSelectionDialog(topLevel,"loadFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(loadFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  infoRowCol=XtVaCreateManagedWidget("infoRowCol",xmRowColumnWidgetClass,
    loadFileBox,NULL);
  text=XmStringCreateLocalized(" ");
  infoL1=XtVaCreateManagedWidget("infoL1",xmLabelWidgetClass,
    infoRowCol,XmNlabelString,text,NULL);
  infoL2=XtVaCreateManagedWidget("infoL2",xmLabelWidgetClass,
    infoRowCol,XmNlabelString,text,NULL);
  infoL3=XtVaCreateManagedWidget("infoL3",xmLabelWidgetClass,
    infoRowCol,XmNlabelString,text,NULL);
  trackS=XtVaCreateManagedWidget("trackS",xmSeparatorWidgetClass,
    infoRowCol,NULL);
  trackL=XtVaCreateManagedWidget("trackL",xmLabelWidgetClass,infoRowCol,NULL);
  trackR=XtVaCreateManagedWidget("trackR",xmRowColumnWidgetClass,infoRowCol,NULL);
  for (i=1; i<10; i++) {
    sprintf(streng,"%d",i);
    text=XmStringCreateLocalized(streng);
    trackToggles[i]=XtVaCreateManagedWidget("trackTog",
      xmToggleButtonGadgetClass, trackR, XmNlabelString, text, NULL);
    XtFree(text);
  }
  XmToggleButtonSetState(trackToggles[1],True,False);
  trackL2=XtVaCreateManagedWidget("trackL2",xmLabelWidgetClass,infoRowCol,NULL);
  trackT=XtVaCreateManagedWidget("trackT",xmTextFieldWidgetClass,infoRowCol,NULL);

  text=XmStringCreateLocalized("Play");
  lPlay=XtVaCreateManagedWidget("lPlay",xmPushButtonWidgetClass,
    loadFileBox,XmNlabelString,text,NULL);

  loadtextchild=XmFileSelectionBoxGetChild(loadFileBox,XmDIALOG_TEXT);
  XtAddCallback(loadtextchild,XmNvalueChangedCallback,LoadInfo,NULL);
  XtAddCallback(lPlay,XmNactivateCallback,LoadPlay,loadtextchild);

  XtAddCallback(loadFileBox,XmNcancelCallback,Cancel,loadFileBox);
  XtAddCallback(loadFileBox,XmNokCallback,LoadOk,NULL);
  XtAddCallback(load,XmNactivateCallback,Load,NULL);

  printf("."); fflush(stdout);

  loadmix=XtVaCreateManagedWidget(
    "loadmix",xmPushButtonWidgetClass,fileMenu,
    XmNmnemonic,XStringToKeysym("L"),XmNaccelerator,"Alt<Key>L",
    XmNacceleratorText,XmStringCreateLocalized("Alt+L"),NULL);
  loadmixBox=XmCreateFileSelectionDialog(topLevel,"loadmixBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(loadmixBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmFileSelectionBoxGetChild(loadmixBox,XmDIALOG_CANCEL_BUTTON);
  text=XmStringCreateLocalized("Dismiss");
  XtVaSetValues(temp, XmNlabelString, text, NULL);
  loadmRowCol=XtVaCreateManagedWidget("loadmRowCol",xmRowColumnWidgetClass,
    loadmixBox,NULL);
  loadmixToggle=XtVaCreateManagedWidget("loadmixToggle",
    xmToggleButtonWidgetClass,loadmRowCol,NULL);
  loadmixlabel=XtVaCreateManagedWidget("loadmixlabel",xmLabelWidgetClass,
    loadmRowCol,NULL);
  loadmixtext=XtVaCreateManagedWidget("loadmixtext",xmTextFieldWidgetClass,
    loadmRowCol,NULL);

  XtAddCallback(loadmixBox,XmNcancelCallback,Cancel,loadmixBox);
  XtAddCallback(loadmixBox,XmNokCallback,LoadmixOk,NULL);
  XtAddCallback(loadmix,XmNactivateCallback,Loadmix,NULL);


  savemix=XtVaCreateManagedWidget(
    "savemix",xmPushButtonWidgetClass,fileMenu,
    XmNmnemonic,XStringToKeysym("S"),XmNaccelerator,"Alt<Key>S",
    XmNacceleratorText,XmStringCreateLocalized("Alt+S"),NULL);
  savemixBox=XmCreateFileSelectionDialog(topLevel,"savemixBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(savemixBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  savemixToggle=XtVaCreateManagedWidget("savemixToggle",
    xmToggleButtonWidgetClass,savemixBox,NULL);

  XtAddCallback(savemixBox,XmNcancelCallback,Cancel,savemixBox);
  XtAddCallback(savemixBox,XmNokCallback,SavemixOk,NULL);
  XtAddCallback(savemix,XmNactivateCallback,Savemix,NULL);


  refresh=XtVaCreateManagedWidget(
    "refresh",xmPushButtonWidgetClass,fileMenu,
    XmNmnemonic,XStringToKeysym("R"),XmNaccelerator,"Alt<Key>R",
    XmNacceleratorText,XmStringCreateLocalized("Alt+R"),NULL);
  XtAddCallback(refresh,XmNactivateCallback,Refresh,NULL);


  save=XtVaCreateManagedWidget("save",xmPushButtonWidgetClass,fileMenu,
    XmNmnemonic,XStringToKeysym("M"),XmNaccelerator,"Alt<Key>M",
    XmNacceleratorText,XmStringCreateLocalized("Alt+M"),NULL);

  saveFileBox=XmCreateFileSelectionDialog(topLevel,"saveFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(saveFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  XtAddCallback(saveFileBox,XmNcancelCallback,Cancel,saveFileBox);
  XtAddCallback(saveFileBox,XmNokCallback,SaveOk,NULL);
  XtAddCallback(save,XmNactivateCallback,Save,NULL);

  new=XtVaCreateManagedWidget("new",xmPushButtonWidgetClass,fileMenu,
    XmNmnemonic,XStringToKeysym("N"),XmNaccelerator,"Alt<Key>N",
    XmNacceleratorText,XmStringCreateLocalized("Alt+N"),NULL);
  XtAddCallback(new,XmNactivateCallback,New,0);

  quit=XtVaCreateManagedWidget(
    "quit",xmPushButtonWidgetClass,fileMenu,
    XmNmnemonic,XStringToKeysym("Q"),XmNaccelerator,"Alt<Key>Q",
    XmNacceleratorText,XmStringCreateLocalized("Alt+Q"),NULL);
  XtAddCallback(quit,XmNactivateCallback,Quit,0);

  fileWarning=XmCreateWarningDialog(mainWindow,"fileWarning",NULL,0);
  temp=XmMessageBoxGetChild(fileWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(fileWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);

  bitsWarning=XmCreateWarningDialog(mainWindow,"bitsWarning",NULL,0);
  temp=XmMessageBoxGetChild(bitsWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(bitsWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);

  connectWarning=XmCreateWarningDialog(mainWindow,"connectWarning",NULL,0);
  temp=XmMessageBoxGetChild(connectWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(connectWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);

  slavefailWarning=XmCreateWarningDialog(mainWindow,"slavefailWarning",NULL,0);
  temp=XmMessageBoxGetChild(slavefailWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(slavefailWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);

  writeWarning=XmCreateWarningDialog(mainWindow,"writeWarning",NULL,0);
  temp=XmMessageBoxGetChild(writeWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(writeWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);

  srateWarning=XmCreateWarningDialog(mainWindow,"srateWarning",NULL,0);
  temp=XmMessageBoxGetChild(srateWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  XtAddCallback(srateWarning,XmNokCallback,Convert,NULL);

  clipWarning=XmCreateWarningDialog(mainWindow,"clipWarning",NULL,0);
  temp=XmMessageBoxGetChild(clipWarning,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(clipWarning,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);

  progressDialog=XmCreateWorkingDialog(mainWindow,"progressDialog",NULL,0);
  temp=XmMessageBoxGetChild(progressDialog,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(progressDialog,XmDIALOG_OK_BUTTON);
  XtUnmanageChild(temp);
  XtAddCallback(progressDialog,XmNcancelCallback,Cancel,progressDialog);
  progressScale=XtVaCreateManagedWidget("progressScale",
    xmScaleWidgetClass, progressDialog, NULL);


  /* MENU EDIT MENU */

  text=XmStringCreateLocalized("Edit");
  meditButton=XtVaCreateManagedWidget(
    "meditButton",xmCascadeButtonWidgetClass,menuBar,
    XmNlabelString,text,NULL);

  meditMenu=XmCreatePulldownMenu(menuBar,"meditMenu",NULL,0);
  XtVaSetValues(meditButton,XmNsubMenuId,meditMenu,NULL);

  text=XmStringCreateLocalized("Undo");
  undo=XtVaCreateManagedWidget(
    "undo",xmPushButtonWidgetClass,meditMenu,XmNlabelString,text,
    XmNaccelerator,"Alt<Key>B",
    XmNacceleratorText,XmStringCreateLocalized("Alt+B"),NULL);
  XtAddCallback(undo,XmNactivateCallback,Undo,NULL);

  text=XmStringCreateLocalized("Cut");
  cut=XtVaCreateManagedWidget(
    "cut",xmPushButtonWidgetClass,meditMenu,XmNlabelString,text,
    XmNaccelerator,"Ctrl<Key>X",
    XmNacceleratorText,XmStringCreateLocalized("Ctrl+X"),NULL);
  XtAddCallback(cut,XmNactivateCallback,Cut,NULL);

  text=XmStringCreateLocalized("Copy");
  copy=XtVaCreateManagedWidget(
    "copy",xmPushButtonWidgetClass,meditMenu,XmNlabelString,text,
    XmNaccelerator,"Ctrl<Key>C",
    XmNacceleratorText,XmStringCreateLocalized("Ctrl+C"),NULL);
  XtAddCallback(copy,XmNactivateCallback,Copy,NULL);

  text=XmStringCreateLocalized("Paste");
  paste=XtVaCreateManagedWidget(
    "paste",xmPushButtonWidgetClass,meditMenu,XmNlabelString,text,
    XmNaccelerator,"Ctrl<Key>V",
    XmNacceleratorText,XmStringCreateLocalized("Ctrl+V"),NULL);
  XtAddCallback(paste,XmNactivateCallback,Paste,NULL);
  XtVaSetValues(paste,XmNsensitive,False,NULL);

  text=XmStringCreateLocalized("Insert");
  insert=XtVaCreateManagedWidget(
    "insert",xmPushButtonWidgetClass,meditMenu,XmNlabelString,text,
    XmNaccelerator,"Ctrl<Key>I",
    XmNacceleratorText,XmStringCreateLocalized("Ctrl+I"),NULL);
  XtAddCallback(insert,XmNactivateCallback,Insert,NULL);
  XtVaSetValues(insert,XmNsensitive,False,NULL);

  printf("."); fflush(stdout);

  /* SETTINGS MENU */

  settingsButton=XtVaCreateManagedWidget(
    "settingsButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  settingsMenu=XmCreatePulldownMenu(menuBar,"settingsMenu",NULL,0);
  XtVaSetValues(settingsButton,XmNsubMenuId,settingsMenu,NULL);

  displayButton=XtVaCreateManagedWidget(
    "displayB",xmPushButtonWidgetClass,settingsMenu,
    XmNmnemonic,XStringToKeysym("D"),XmNaccelerator,"Alt<Key>D",
    XmNacceleratorText,XmStringCreateLocalized("Alt+D"),NULL);
  XtAddCallback(displayButton,XmNactivateCallback,DisplayB,NULL);

  displayBB=XmCreateBulletinBoardDialog(topLevel,"displayBB",NULL,0);
  displayOkButton=XtVaCreateManagedWidget("displayOk",xmPushButtonWidgetClass,
    displayBB,NULL);

  XtVaSetValues(displayBB,XmNdefaultButton,displayOkButton,NULL);

  dispgridToggle=XtVaCreateManagedWidget("dispgridToggle",
    xmToggleButtonWidgetClass,displayBB,NULL);
  XtAddCallback(dispgridToggle, XmNvalueChangedCallback, DispGrid, NULL);
  displinToggle=XtVaCreateManagedWidget("displinToggle",
    xmToggleButtonWidgetClass,displayBB,NULL);
  XtAddCallback(displinToggle, XmNvalueChangedCallback, DispLin, NULL);
  dispnumToggle=XtVaCreateManagedWidget("dispnumToggle",
    xmToggleButtonWidgetClass,displayBB,NULL);
  XtAddCallback(dispnumToggle, XmNvalueChangedCallback, DispNum, NULL);
  dispscrollToggle=XtVaCreateManagedWidget("dispscrollToggle",
    xmToggleButtonWidgetClass,displayBB,NULL);
  XtAddCallback(dispscrollToggle, XmNvalueChangedCallback, DispScroll, NULL);
  dispfastdrawToggle=XtVaCreateManagedWidget("dispfastdrawToggle",
    xmToggleButtonWidgetClass,displayBB,NULL);
  XtAddCallback(dispfastdrawToggle, XmNvalueChangedCallback, DispFastdraw, NULL);
  dispallToggle=XtVaCreateManagedWidget("dispallToggle",
    xmToggleButtonWidgetClass,displayBB,NULL);
  XtAddCallback(dispallToggle, XmNvalueChangedCallback, DispAll, NULL);

  colorRadio=XtVaCreateManagedWidget("colorRadio",
    xmRowColumnWidgetClass,displayBB,NULL);
  colorToggleWhite=XtVaCreateManagedWidget("colorToggleWhite",
    xmToggleButtonWidgetClass,colorRadio,NULL);
  XtAddCallback(colorToggleWhite, XmNvalueChangedCallback, White, NULL);
  colorToggleBlue=XtVaCreateManagedWidget("colorToggleBlue",
    xmToggleButtonWidgetClass,colorRadio,NULL);
  XtAddCallback(colorToggleBlue, XmNvalueChangedCallback, Blue, NULL);


  audioButton=XtVaCreateManagedWidget(
    "audioB",xmPushButtonWidgetClass,settingsMenu,
    XmNmnemonic,XStringToKeysym("u"),XmNaccelerator,"Alt<Key>U",
    XmNacceleratorText,XmStringCreateLocalized("Alt+U"),NULL);
  XtAddCallback(audioButton,XmNactivateCallback,Audio,NULL);
  audioBB=XmCreateBulletinBoardDialog(topLevel,"audioBB",NULL,0);

  srateLabel=XtVaCreateManagedWidget("srateLabel",xmLabelWidgetClass,
    audioBB,NULL);
  srateRadio=XtVaCreateManagedWidget("srateRadio",
    xmRowColumnWidgetClass,audioBB,NULL);
  srate8=XtVaCreateManagedWidget("srate8",
    xmToggleButtonWidgetClass,srateRadio,NULL);
  XtAddCallback(srate8,XmNvalueChangedCallback,SRate,NULL);
  srate11025=XtVaCreateManagedWidget("srate11025",
    xmToggleButtonWidgetClass,srateRadio,NULL);
  XtAddCallback(srate11025,XmNvalueChangedCallback,SRate,NULL);
  srate16=XtVaCreateManagedWidget("srate16",
    xmToggleButtonWidgetClass,srateRadio,NULL);
  XtAddCallback(srate16,XmNvalueChangedCallback,SRate,NULL);
  srate2205=XtVaCreateManagedWidget("srate2205",
    xmToggleButtonWidgetClass,srateRadio,NULL);
  XtAddCallback(srate2205,XmNvalueChangedCallback,SRate,NULL);
  srate32=XtVaCreateManagedWidget("srate32",
    xmToggleButtonWidgetClass,srateRadio,NULL);
  XtAddCallback(srate32,XmNvalueChangedCallback,SRate,NULL);
  srate441=XtVaCreateManagedWidget("srate441",
    xmToggleButtonWidgetClass,srateRadio,NULL);
  XtAddCallback(srate441,XmNvalueChangedCallback,SRate,NULL);
  srate48=XtVaCreateManagedWidget("srate48",
    xmToggleButtonWidgetClass,srateRadio,NULL);
  XtAddCallback(srate48,XmNvalueChangedCallback,SRate,NULL);

  nchnlsLabel=XtVaCreateManagedWidget("nchnlsLabel",xmLabelWidgetClass,
    audioBB,NULL);
  nchnlsRadio=XtVaCreateManagedWidget("nchnlsRadio",
    xmRowColumnWidgetClass,audioBB,NULL);
  nchnls1=XtVaCreateManagedWidget("nchnls1",
    xmToggleButtonWidgetClass,nchnlsRadio,NULL);
  XtAddCallback(nchnls1,XmNvalueChangedCallback,Nchnls1,NULL);
  nchnls2=XtVaCreateManagedWidget("nchnls2",
    xmToggleButtonWidgetClass,nchnlsRadio,NULL);
  XtAddCallback(nchnls2,XmNvalueChangedCallback,Nchnls2,NULL);
  nchnls4=XtVaCreateManagedWidget("nchnls4",
    xmToggleButtonWidgetClass,nchnlsRadio,NULL);
  XtAddCallback(nchnls4,XmNvalueChangedCallback,Nchnls4,NULL);

  sprintf(streng, "%6.2f", sett.buffsize);
  buffText=XtVaCreateManagedWidget("buffText",xmTextFieldWidgetClass,audioBB,
    XmNvalue,streng,NULL);
  buffLabel=XtVaCreateManagedWidget("buffLabel",xmLabelWidgetClass,audioBB,NULL);
  XtAddCallback(buffText,XmNvalueChangedCallback,Buffval,NULL);

  linvolToggle=XtVaCreateManagedWidget("linvolToggle",
    xmToggleButtonWidgetClass,audioBB,NULL);
  XtAddCallback(linvolToggle, XmNvalueChangedCallback, AudioLinvol, NULL);

  audioOkButton=XtVaCreateManagedWidget("audioOk",xmPushButtonWidgetClass,
    audioBB,NULL);
  XtVaSetValues(audioBB,XmNdefaultButton,audioOkButton,NULL);


  timeButton=XtVaCreateManagedWidget(
    "timeB",xmPushButtonWidgetClass,settingsMenu,
    XmNmnemonic,XStringToKeysym("T"),XmNaccelerator,"Alt<Key>T",
    XmNacceleratorText,XmStringCreateLocalized("Alt+T"),NULL);
  XtAddCallback(timeButton,XmNactivateCallback,TimeB,NULL);
  timeBB=XmCreateBulletinBoardDialog(topLevel,"timeBB",NULL,0);

  timeRadio=XtVaCreateManagedWidget("timeRadio",
    xmRowColumnWidgetClass,timeBB,NULL);
  timeToggleSec=XtVaCreateManagedWidget("timeSec",
    xmToggleButtonWidgetClass,timeRadio,NULL);
  XtAddCallback(timeToggleSec,XmNvalueChangedCallback,TimeToggle,NULL);
  timeToggleBeats=XtVaCreateManagedWidget("timeBeats",
    xmToggleButtonWidgetClass,timeRadio,NULL);
  XtAddCallback(timeToggleBeats,XmNvalueChangedCallback,TimeToggle,NULL);

  sprintf(streng, "%d", 120);
  timeBPMtext=XtVaCreateManagedWidget("timeBPMtext",xmTextFieldWidgetClass,
    timeBB,XmNvalue,streng,NULL);
  timeBPMlabel=XtVaCreateManagedWidget("timeBPMlabel",xmLabelWidgetClass,
    timeBB,NULL);
  XtAddCallback(timeBPMtext,XmNvalueChangedCallback,BPMtext,NULL);

  sprintf(streng, "%d", 4);
  timeBPBtext=XtVaCreateManagedWidget("timeBPBtext",xmTextFieldWidgetClass,
    timeBB,XmNvalue,streng,NULL);
  timeBPBlabel=XtVaCreateManagedWidget("timeBPBlabel",xmLabelWidgetClass,
    timeBB,NULL);
  XtAddCallback(timeBPBtext,XmNvalueChangedCallback,BPBtext,NULL);

  timeOkButton=XtVaCreateManagedWidget("timeOk",xmPushButtonWidgetClass,
    timeBB,NULL);
  XtVaSetValues(timeBB,XmNdefaultButton,timeOkButton,NULL);


  recordButton=XtVaCreateManagedWidget(
    "recordButton",xmPushButtonWidgetClass,settingsMenu,
    XmNmnemonic,XStringToKeysym("o"),XmNaccelerator,"Alt<Key>F",
    XmNacceleratorText,XmStringCreateLocalized("Alt+F"),NULL);
  recordBox=XmCreateFileSelectionDialog(topLevel,"recordBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(recordBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  recRowCol=XtVaCreateManagedWidget("recRowCol",xmRowColumnWidgetClass,
    recordBox,NULL);
  monitorToggle=XtVaCreateManagedWidget("monitorToggle",
    xmToggleButtonWidgetClass,recRowCol,NULL);
  XtAddCallback(monitorToggle, XmNvalueChangedCallback, Monitor, NULL);
  recstereoToggle=XtVaCreateManagedWidget("recstereoToggle",
    xmToggleButtonWidgetClass,recRowCol,NULL);
  XtAddCallback(recstereoToggle, XmNvalueChangedCallback, Stereorec, NULL);

  XtAddCallback(recordBox,XmNcancelCallback,Cancel,recordBox);
  XtAddCallback(recordBox,XmNokCallback,RecordOk,NULL);
  XtAddCallback(recordButton,XmNactivateCallback,Recordb,NULL);

  sendtoButton=XtVaCreateManagedWidget(
    "sendtoB",xmPushButtonWidgetClass,settingsMenu,
    XmNmnemonic,XStringToKeysym("I"),XmNaccelerator,"Alt<Key>I",
    XmNacceleratorText,XmStringCreateLocalized("Alt+I"),NULL);
  XtAddCallback(sendtoButton,XmNactivateCallback,SendtoB,NULL);

  sendtoBB=XmCreateBulletinBoardDialog(topLevel,"sendtoBB",NULL,0);
  sendtoOkButton=XtVaCreateManagedWidget("sendtoOk",xmPushButtonWidgetClass,
    sendtoBB,NULL);
  XtVaSetValues(sendtoBB,XmNdefaultButton,sendtoOkButton,NULL);
  sendtoAccept=XtVaCreateManagedWidget("sendtoAccept",xmToggleButtonWidgetClass,
    sendtoBB,NULL);
  XtAddCallback(sendtoAccept, XmNvalueChangedCallback, AcceptConnect, NULL);

  sendtoTogLabel=XtVaCreateManagedWidget("sendtoLabel1",
    xmLabelWidgetClass,sendtoBB,NULL);
  sendtoTextLabel=XtVaCreateManagedWidget("sendtoLabel2",
    xmLabelWidgetClass,sendtoBB,NULL);

  for (i=0; i<MAXSENDTO; i++) {
    sendto_dummy[i]=i; sendto_connected[i]=0;
    sendtoToggle[i]=XtVaCreateManagedWidget("sendtoToggle",
      xmToggleButtonWidgetClass,sendtoBB,NULL);
    XtVaSetValues(sendtoToggle[i],XmNx,20,XmNy,i*40+45,NULL);
    XtAddCallback(sendtoToggle[i],XmNvalueChangedCallback,SendtoConnect,
      &sendto_dummy[i]);
    sendtoText[i]=XtVaCreateManagedWidget("sendtoText",
      xmTextFieldWidgetClass,sendtoBB,NULL);
    XtVaSetValues(sendtoText[i],XmNx,60,XmNy,i*40+40,NULL);
  }


  /* EFFECTS 1 */

  effectsButton=XtVaCreateManagedWidget(
    "effectsButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  effectsMenu=XmCreatePulldownMenu(menuBar,"effectsMenu",NULL,0);
  XtVaSetValues(effectsButton,XmNsubMenuId,effectsMenu,NULL);

  effects1=XtVaCreateManagedWidget(
    "effects1",xmPushButtonWidgetClass,effectsMenu,
    XmNmnemonic,XStringToKeysym("1"),XmNaccelerator,"Alt<Key>1",
    XmNacceleratorText,XmStringCreateLocalized("Alt+1"),NULL);
  XtAddCallback(effects1,XmNactivateCallback,Effects1,NULL);

  effects1BB=XmCreateBulletinBoardDialog(topLevel,"effects1BB",NULL,0);
  effects1OkButton=XtVaCreateManagedWidget("effects1Ok",xmPushButtonWidgetClass,
    effects1BB,NULL);
  XtVaSetValues(effects1BB,XmNdefaultButton,effects1OkButton,NULL);

  eff1RadLabel=XtVaCreateManagedWidget("eff1RadLabel",xmLabelWidgetClass,
    effects1BB,NULL);
  eff1RadEcho=XtVaCreateManagedWidget("Echo",
    xmToggleButtonWidgetClass,effects1BB,NULL);
  XtAddCallback(eff1RadEcho,XmNvalueChangedCallback,Eff1Echo,NULL);
  eff1RadChor=XtVaCreateManagedWidget("Chorus",
    xmToggleButtonWidgetClass,effects1BB,NULL);
  XtAddCallback(eff1RadChor,XmNvalueChangedCallback,Eff1Chorus,NULL);
  eff1RadHarm=XtVaCreateManagedWidget("Harmonizer",
    xmToggleButtonWidgetClass,effects1BB,NULL);
  XtAddCallback(eff1RadHarm,XmNvalueChangedCallback,Eff1Harm,NULL);
  eff1RadRing=XtVaCreateManagedWidget("RingMod",
    xmToggleButtonWidgetClass,effects1BB,NULL);
  XtAddCallback(eff1RadRing,XmNvalueChangedCallback,Eff1Ring,NULL);
  eff1RadFilter=XtVaCreateManagedWidget("Filter",
    xmToggleButtonWidgetClass,effects1BB,NULL);
  XtAddCallback(eff1RadFilter,XmNvalueChangedCallback,Eff1Filter,NULL);

  eff1FLTtypeRadio=XtVaCreateManagedWidget("filterRadio",
    xmRowColumnWidgetClass,effects1BB,NULL);
  eff1FLTlowpass=XtVaCreateManagedWidget("Lowpass",
    xmToggleButtonWidgetClass,eff1FLTtypeRadio,NULL);
  XtAddCallback(eff1FLTlowpass,XmNvalueChangedCallback,Eff1FLTRad,NULL);  
  eff1FLThipass=XtVaCreateManagedWidget("Highpass",
    xmToggleButtonWidgetClass,eff1FLTtypeRadio,NULL);
  XtAddCallback(eff1FLThipass,XmNvalueChangedCallback,Eff1FLTRad,NULL);
  eff1FLTbandpass=XtVaCreateManagedWidget("Bandpass",
    xmToggleButtonWidgetClass,eff1FLTtypeRadio,NULL);
  XtAddCallback(eff1FLTbandpass,XmNvalueChangedCallback,Eff1FLTRad,NULL);
  eff1FLTbandstop=XtVaCreateManagedWidget("Bandstop",
    xmToggleButtonWidgetClass,eff1FLTtypeRadio,NULL);
  XtAddCallback(eff1FLTbandstop,XmNvalueChangedCallback,Eff1FLTRad,NULL);

  eff1ECtimeLabel=XtVaCreateManagedWidget("ECtimeLabel",xmLabelWidgetClass,
    effects1BB,NULL);
  eff1ECtime=XtVaCreateManagedWidget("timeScale",xmScaleWidgetClass,
    effects1BB,NULL);
  XtAddCallback(eff1ECtime,XmNvalueChangedCallback,ECtimeScale,NULL);
  XtAddCallback(eff1ECtime,XmNdragCallback,ECtimeScale,NULL);

  eff1ECfeedLabel=XtVaCreateManagedWidget("ECfeedLabel",xmLabelWidgetClass,
    effects1BB,NULL);
  eff1ECfeed=XtVaCreateManagedWidget("feedScale",xmScaleWidgetClass,
    effects1BB,NULL);
  XtAddCallback(eff1ECfeed,XmNvalueChangedCallback,ECfeedScale,NULL);
  XtAddCallback(eff1ECfeed,XmNdragCallback,ECfeedScale,NULL);

  eff1CHspeedLabel=XtVaCreateManagedWidget("CHspeedLabel",xmLabelWidgetClass,
    effects1BB,NULL);
  eff1CHspeed=XtVaCreateManagedWidget("speedScale",xmScaleWidgetClass,
    effects1BB,NULL);
  XtAddCallback(eff1CHspeed,XmNvalueChangedCallback,CHspeedScale,NULL);
  XtAddCallback(eff1CHspeed,XmNdragCallback,CHspeedScale,NULL);

  eff1CHdepthLabel=XtVaCreateManagedWidget("CHdepthLabel",xmLabelWidgetClass,
    effects1BB,NULL);
  eff1CHdepth=XtVaCreateManagedWidget("depthScale",xmScaleWidgetClass,
    effects1BB,NULL);
  XtAddCallback(eff1CHdepth,XmNvalueChangedCallback,CHdepthScale,NULL);
  XtAddCallback(eff1CHdepth,XmNdragCallback,CHdepthScale,NULL);

  eff1HAinterLabel=XtVaCreateManagedWidget("HAinterLabel",xmLabelWidgetClass,
    effects1BB,NULL);
  eff1HAinter=XtVaCreateManagedWidget("interScale",xmScaleWidgetClass,
    effects1BB,NULL);
  XtAddCallback(eff1HAinter,XmNvalueChangedCallback,HAinterScale,NULL);
  XtAddCallback(eff1HAinter,XmNdragCallback,HAinterScale,NULL);

  eff1RMfreqLabel=XtVaCreateManagedWidget("RMfreqLabel",xmLabelWidgetClass,
    effects1BB,NULL);
  effects1RMfreq=XtVaCreateManagedWidget("frqScale",xmScaleWidgetClass,
    effects1BB,NULL);
  XtAddCallback(effects1RMfreq,XmNvalueChangedCallback,RM1Scale,NULL);
  XtAddCallback(effects1RMfreq,XmNdragCallback,RM1Scale,NULL);

  eff1FLTcenterLabel=XtVaCreateManagedWidget("FLTcenterLabel",xmLabelWidgetClass,
    effects1BB,NULL);
  eff1FLTcenter=XtVaCreateManagedWidget("filtScale",xmScaleWidgetClass,
    effects1BB,NULL);
  XtAddCallback(eff1FLTcenter,XmNvalueChangedCallback,FLTcenterScale,NULL);
  XtAddCallback(eff1FLTcenter,XmNdragCallback,FLTcenterScale,NULL);
  eff1FLTcenterT=XtVaCreateManagedWidget("centerToggle",xmToggleButtonWidgetClass,
    effects1BB,NULL);
  XtAddCallback(eff1FLTcenterT,XmNvalueChangedCallback,FLTcenterToggle,NULL);

  text=XmStringCreateLocalized("Filter Q");
  eff1FLTQLabel=XtVaCreateManagedWidget("FLTQLabel",xmLabelWidgetClass,
    effects1BB,NULL);
  eff1FLTQ=XtVaCreateManagedWidget("QScale",xmScaleWidgetClass,
    effects1BB,NULL);
  XtAddCallback(eff1FLTQ,XmNvalueChangedCallback,FLTQScale,NULL);
  XtAddCallback(eff1FLTQ,XmNdragCallback,FLTQScale,NULL);

  eff1sep1=XtVaCreateManagedWidget("effsep1",xmSeparatorWidgetClass,
    effects1BB,NULL);
  eff1sep2=XtVaCreateManagedWidget("effsep2",xmSeparatorWidgetClass,
    effects1BB,NULL);
  eff1sep3=XtVaCreateManagedWidget("effsep3",xmSeparatorWidgetClass,
    effects1BB,NULL);


  /* EFFECTS 2 */

  effects2=XtVaCreateManagedWidget(
    "effects2",xmPushButtonWidgetClass,effectsMenu,
    XmNmnemonic,XStringToKeysym("2"),XmNaccelerator,"Alt<Key>2",
    XmNacceleratorText,XmStringCreateLocalized("Alt+2"),NULL);
  XtAddCallback(effects2,XmNactivateCallback,Effects2,NULL);

  effects2BB=XmCreateBulletinBoardDialog(topLevel,"effects2BB",NULL,0);
  effects2OkButton=XtVaCreateManagedWidget("effects2Ok",xmPushButtonWidgetClass,
    effects2BB,NULL);
  XtVaSetValues(effects2BB,XmNdefaultButton,effects2OkButton,NULL);

  eff2RadLabel=XtVaCreateManagedWidget("eff2RadLabel",xmLabelWidgetClass,
    effects2BB,NULL);
  eff2RadReverb=XtVaCreateManagedWidget("Reverb",
    xmToggleButtonWidgetClass,effects2BB,NULL);
  XtAddCallback(eff2RadReverb,XmNvalueChangedCallback,Eff2Reverb,NULL);
  eff2RadCompress=XtVaCreateManagedWidget("Compress",
    xmToggleButtonWidgetClass,effects2BB,NULL);
  XtAddCallback(eff2RadCompress,XmNvalueChangedCallback,Eff2Compress,NULL);
  eff2RadOctave=XtVaCreateManagedWidget("Octave",
    xmToggleButtonWidgetClass,effects2BB,NULL);
  XtAddCallback(eff2RadOctave,XmNvalueChangedCallback,Eff2Octave,NULL);

  eff2RVtimeLabel=XtVaCreateManagedWidget("RVtimeLabel",xmLabelWidgetClass,
    effects2BB,NULL);
  eff2RVtime=XtVaCreateManagedWidget("RVtimeScale",xmScaleWidgetClass,
    effects2BB,NULL);
  XtAddCallback(eff2RVtime,XmNvalueChangedCallback,RVtimeScale,NULL);
  XtAddCallback(eff2RVtime,XmNdragCallback,RVtimeScale,NULL);

  eff2CMALabel=XtVaCreateManagedWidget("CMALabel",xmLabelWidgetClass,
    effects2BB,NULL);
  eff2CMA=XtVaCreateManagedWidget("CMAScale",xmScaleWidgetClass,
    effects2BB,NULL);
  XtAddCallback(eff2CMA,XmNvalueChangedCallback,CMAScale,NULL);
  XtAddCallback(eff2CMA,XmNdragCallback,CMAScale,NULL);

  eff2sep1=XtVaCreateManagedWidget("eff2sep1",xmSeparatorWidgetClass,
    effects2BB,NULL);
  eff2sep2=XtVaCreateManagedWidget("eff2sep2",xmSeparatorWidgetClass,
    effects2BB,NULL);
  eff2sep3=XtVaCreateManagedWidget("eff2sep3",xmSeparatorWidgetClass,
    effects2BB,NULL);


  /* TOOLS */

  toolsButton=XtVaCreateManagedWidget(
    "toolsButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  toolsMenu=XmCreatePulldownMenu(menuBar,"toolsMenu",NULL,0);
  XtVaSetValues(toolsButton,XmNsubMenuId,toolsMenu,NULL);

  to_machine=XtVaCreateManagedWidget(
    "to_machine",xmPushButtonWidgetClass,toolsMenu,
    XmNmnemonic,XStringToKeysym("h"),XmNaccelerator,"Alt<Key>H",
    XmNacceleratorText,XmStringCreateLocalized("Alt+H"),NULL);
  XtAddCallback(to_machine,XmNactivateCallback,To_machine,NULL);

  machineBB=XmCreateBulletinBoardDialog(topLevel,"machineBB",NULL,0);

  machineQuantLabel=XtVaCreateManagedWidget("quantLabel",xmLabelWidgetClass,
    machineBB,NULL);
  machineQuantRadio=XtVaCreateManagedWidget("quantRadio",
    xmRowColumnWidgetClass,machineBB,NULL);
  machine1=XtVaCreateManagedWidget("quant1",
    xmToggleButtonWidgetClass,machineQuantRadio,NULL);
  XtAddCallback(machine1,XmNvalueChangedCallback,QuantSet,NULL);
  machine1d2=XtVaCreateManagedWidget("quant1d2",
    xmToggleButtonWidgetClass,machineQuantRadio,NULL);
  XtAddCallback(machine1d2,XmNvalueChangedCallback,QuantSet,NULL);
  machine1d3=XtVaCreateManagedWidget("quant1d3",
    xmToggleButtonWidgetClass,machineQuantRadio,NULL);
  XtAddCallback(machine1d3,XmNvalueChangedCallback,QuantSet,NULL);
  machine1d4=XtVaCreateManagedWidget("quant1d4",
    xmToggleButtonWidgetClass,machineQuantRadio,NULL);
  XtAddCallback(machine1d4,XmNvalueChangedCallback,QuantSet,NULL);
  machine1d8=XtVaCreateManagedWidget("quant1d8",
    xmToggleButtonWidgetClass,machineQuantRadio,NULL);
  XtAddCallback(machine1d8,XmNvalueChangedCallback,QuantSet,NULL);
  machine1d16=XtVaCreateManagedWidget("quant1d16",
    xmToggleButtonWidgetClass,machineQuantRadio,NULL);
  XtAddCallback(machine1d16,XmNvalueChangedCallback,QuantSet,NULL);

  machineQuantNow=XtVaCreateManagedWidget("quantNow",xmPushButtonWidgetClass,
    machineBB,NULL);
  XtAddCallback(machineQuantNow,XmNactivateCallback,SnaptoGrid,NULL);

  machineAutoQuant=XtVaCreateManagedWidget("quantAuto",xmToggleButtonWidgetClass,
    machineBB,NULL);
  XtAddCallback(machineAutoQuant,XmNvalueChangedCallback,AutoQuant,NULL);

  machineTimeLabel=XtVaCreateManagedWidget("tempoLabel",xmLabelWidgetClass,
    machineBB,NULL);
  sprintf(streng, "1");
  machineTimeText=XtVaCreateManagedWidget("tempoText",xmTextFieldWidgetClass,
    machineBB,XmNvalue,streng,NULL);
  machineTimeApply=XtVaCreateManagedWidget("tempoOk",xmPushButtonWidgetClass,
    machineBB,NULL);
  XtAddCallback(machineTimeApply,XmNactivateCallback,TempoApply,NULL);

  machineOkButton=XtVaCreateManagedWidget("machineOk",xmPushButtonWidgetClass,
    machineBB,NULL);
  XtVaSetValues(machineBB,XmNdefaultButton,machineOkButton,NULL);

  machineSep=XtVaCreateManagedWidget("machsep",xmSeparatorWidgetClass,
    machineBB,NULL);

  to_srate=XtVaCreateManagedWidget(
    "to_srate",xmPushButtonWidgetClass,toolsMenu,
    XmNmnemonic,XStringToKeysym("C"),XmNaccelerator,"Alt<Key>C",
    XmNacceleratorText,XmStringCreateLocalized("Alt+C"),NULL);
  XtAddCallback(to_srate,XmNactivateCallback,To_srate,NULL);

  to_edit=XtVaCreateManagedWidget(
    "to_edit",xmPushButtonWidgetClass,toolsMenu,
    XmNmnemonic,XStringToKeysym("E"),XmNaccelerator,"Alt<Key>E",
    XmNacceleratorText,XmStringCreateLocalized("Alt+E"),NULL);
  XtAddCallback(to_edit,XmNactivateCallback,To_edit,NULL);

  to_print=XtVaCreateManagedWidget(
    "to_print",xmPushButtonWidgetClass,toolsMenu,
    XmNmnemonic,XStringToKeysym("W"),XmNaccelerator,"Alt<Key>W",
    XmNacceleratorText,XmStringCreateLocalized("Alt+W"),NULL);
  XtAddCallback(to_print,XmNactivateCallback,To_print,NULL);
  printFileBox=XmCreateFileSelectionDialog(topLevel,"printFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(printFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  XtAddCallback(printFileBox,XmNcancelCallback,Cancel,printFileBox);
  XtAddCallback(printFileBox,XmNokCallback,printOk,NULL);

  to_inventor=XtVaCreateManagedWidget(
    "to_inventor",xmPushButtonWidgetClass,toolsMenu,
    XmNmnemonic,XStringToKeysym("W"),XmNaccelerator,"Alt<Key>W",
    XmNacceleratorText,XmStringCreateLocalized("Alt+W"),NULL);
  XtAddCallback(to_inventor,XmNactivateCallback,To_Inventor,NULL);

  play=XtVaCreateManagedWidget(
    "play",xmPushButtonWidgetClass,toolsMenu,
    XmNmnemonic,XStringToKeysym("P"),XmNaccelerator,"Alt<Key>P",
    XmNacceleratorText,XmStringCreateLocalized("Alt+P"),NULL);
  playFileBox=XmCreateFileSelectionDialog(topLevel,"playFileBox",NULL,0);
  temp=XmFileSelectionBoxGetChild(playFileBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  XtAddCallback(playFileBox,XmNcancelCallback,Cancel,playFileBox);
  XtAddCallback(playFileBox,XmNokCallback,PlayOk,NULL);
  XtAddCallback(play,XmNactivateCallback,Play,NULL);

  soverv=XtVaCreateManagedWidget(
    "soverv",xmPushButtonWidgetClass,toolsMenu,
    XmNmnemonic,XStringToKeysym("o"),XmNaccelerator,"Alt<Key>O",
    XmNacceleratorText,XmStringCreateLocalized("Alt+O"),NULL);
  XtAddCallback(soverv,XmNactivateCallback,Soverv,NULL);
  sovervBB=XmCreateFormDialog(topLevel,"sovervBB",NULL,0);
  sovervOkButton=XtVaCreateManagedWidget("sovervOk",xmPushButtonWidgetClass,
    sovervBB,NULL);
  sovervSize=XtVaCreateManagedWidget("sovervSize",xmToggleButtonWidgetClass,
    sovervBB,NULL);
  XtVaSetValues(sovervBB,XmNdefaultButton,sovervOkButton,NULL);

  soundMatrix = XtVaCreateManagedWidget("soundMatrix",
    xbaeMatrixWidgetClass, sovervBB,
    XmNcolumnWidths,   widths,
    XmNcells,          NULL,
    XmNcolumnLabels,   columnlabels,
    XmNrowLabels,      rowlabels,
    XmNrowLabelWidth,  3,
    XmNcolumnAlignments,columnalignments,
    XmNbottomWidget,   sovervOkButton,
    NULL);
  thin10p=XmFontListEntryLoad(XtDisplay(topLevel),
    "-*-courier-medium-r-*--10-*", XmFONT_IS_FONT, "tag1");
  thin14p=XmFontListEntryLoad(XtDisplay(topLevel),
    "-*-courier-medium-r-*--14-*", XmFONT_IS_FONT, "tag2");
  smallfontlist=XmFontListAppendEntry(NULL, thin10p);
  largefontlist=XmFontListAppendEntry(NULL, thin14p);
  XtAddCallback(soundMatrix, XmNselectCellCallback, MatrixSelect, NULL);
  XtAddCallback(soundMatrix, XmNenterCellCallback,  MatrixEnter, NULL);
  XtAddCallback(soundMatrix, XmNleaveCellCallback,  MatrixLeave, NULL);
  XtAddCallback(sovervSize,  XmNvalueChangedCallback,  SovervSize, NULL);


  /* VARIABLES */

  eventButton=XtVaCreateManagedWidget(
    "eventButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  eventMenu=XmCreatePulldownMenu(menuBar,"eventMenu",NULL,0);
  XtVaSetValues(eventButton,XmNsubMenuId,eventMenu,NULL);

  e_echo=XtVaCreateManagedWidget(
    "e_echo",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_echo,XmNactivateCallback,Variables,NULL);
  e_chorus=XtVaCreateManagedWidget(
    "e_chorus",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_chorus,XmNactivateCallback,Variables,NULL);
  e_harmonizer=XtVaCreateManagedWidget(
    "e_harmonizer",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_harmonizer,XmNactivateCallback,Variables,NULL);
  e_ring=XtVaCreateManagedWidget(
    "e_ring",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_ring,XmNactivateCallback,Variables,NULL);
  e_filter=XtVaCreateManagedWidget(
    "e_filter",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_filter,XmNactivateCallback,Variables,NULL);
  e_filtertype=XtVaCreateManagedWidget(
    "e_filtertype",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_filtertype,XmNactivateCallback,Variables,NULL);
  e_reverb=XtVaCreateManagedWidget(
    "e_reverb",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_reverb,XmNactivateCallback,Variables,NULL);
  e_echo_time=XtVaCreateManagedWidget(
    "e_echo_time",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_echo_time,XmNactivateCallback,Variables,NULL);
  e_echo_feedback=XtVaCreateManagedWidget(
    "e_echo_feedback",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_echo_feedback,XmNactivateCallback,Variables,NULL);
  e_chorus_speed=XtVaCreateManagedWidget(
    "e_chorus_speed",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_chorus_speed,XmNactivateCallback,Variables,NULL);
  e_chorus_depth=XtVaCreateManagedWidget(
    "e_chorus_depth",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_chorus_depth,XmNactivateCallback,Variables,NULL);
  e_harmonizer_interval=XtVaCreateManagedWidget(
    "e_harmonizer_interval",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_harmonizer_interval,XmNactivateCallback,Variables,NULL);
  e_ring_frequency=XtVaCreateManagedWidget(
    "e_ring_frequency",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_ring_frequency,XmNactivateCallback,Variables,NULL);
  e_filter_freq=XtVaCreateManagedWidget(
    "e_filter_freq",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_filter_freq,XmNactivateCallback,Variables,NULL);
  e_filter_Q=XtVaCreateManagedWidget(
    "e_filter_Q",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_filter_Q,XmNactivateCallback,Variables,NULL);
  e_reverb_time=XtVaCreateManagedWidget(
    "e_reverb_time",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_reverb_time,XmNactivateCallback,Variables,NULL);
  e_quadpos=XtVaCreateManagedWidget(
    "e_quadpos",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_quadpos,XmNactivateCallback,Variables,NULL);
  e_sync=XtVaCreateManagedWidget(
    "e_sync",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_sync,XmNactivateCallback,Variables,NULL);
  e_csound=XtVaCreateManagedWidget(
    "e_csound",xmPushButtonWidgetClass,eventMenu,NULL);
  XtAddCallback(e_csound,XmNactivateCallback,Variables,NULL);

  eoverv=XtVaCreateManagedWidget(
    "eoverv",xmPushButtonWidgetClass,eventMenu,
    XmNmnemonic,XStringToKeysym("v"),XmNaccelerator,"Alt<Key>V",
    XmNacceleratorText,XmStringCreateLocalized("Alt+V"),NULL);
  XtAddCallback(eoverv,XmNactivateCallback,Eoverv,NULL);
  eovervBB=XmCreateFormDialog(topLevel,"eovervBB",NULL,0);
  eovervOkButton=XtVaCreateManagedWidget("eovervOk",xmPushButtonWidgetClass,
    eovervBB,NULL);
  XtVaSetValues(eovervBB,XmNdefaultButton,eovervOkButton,NULL);
  eventMatrix = XtVaCreateManagedWidget("eventMatrix",
    xbaeMatrixWidgetClass, eovervBB,
    XmNcolumnWidths,   widths2,
    XmNcells,          NULL,
    XmNcolumnLabels,   columnlabels2,
    XmNrowLabels,      rowlabels2,
    XmNrowLabelWidth,  3,
    XmNcolumnAlignments,columnalignments,
    XmNbottomWidget,   eovervOkButton,
    NULL);
  XtAddCallback(eventMatrix, XmNselectCellCallback, EMatrixSelect, NULL);
  XtAddCallback(eventMatrix, XmNenterCellCallback,  EMatrixEnter, NULL);
  XtAddCallback(eventMatrix, XmNleaveCellCallback,  EMatrixLeave, NULL);


  /* GRAPH */

  graphButton=XtVaCreateManagedWidget(
    "graphButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  graphMenu=XmCreatePulldownMenu(menuBar,"graphMenu",NULL,0);
  XtVaSetValues(graphButton,XmNsubMenuId,graphMenu,NULL);

  nodDelete=XtVaCreateManagedWidget(
    "nodDelete",xmPushButtonWidgetClass,graphMenu,
    XmNacceleratorText,XmStringCreateLocalized("Backspace"),NULL);
  XtAddCallback(nodDelete,XmNactivateCallback,GraphDelete,NULL);

  showList=XtVaCreateManagedWidget(
    "showList",xmPushButtonWidgetClass,graphMenu,NULL);
  XtAddCallback(showList,XmNactivateCallback,ShowList,NULL);


  /* GROUP */

  grpButton=XtVaCreateManagedWidget(
    "grpButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  grpMenu=XmCreatePulldownMenu(menuBar,"grpMenu",NULL,0);
  XtVaSetValues(grpButton,XmNsubMenuId,grpMenu,NULL);

  grpGroupall=XtVaCreateManagedWidget(
    "grpGroupall",xmPushButtonWidgetClass,grpMenu,
    XmNmnemonic,XStringToKeysym("a"),XmNaccelerator,"Alt Shift<Key>A",
    XmNacceleratorText,XmStringCreateLocalized("Alt+Shift+A"),NULL);
  XtAddCallback(grpGroupall,XmNactivateCallback,GrpGroupall,NULL);

  grpUngroupall=XtVaCreateManagedWidget(
    "grpUngroupall",xmPushButtonWidgetClass,grpMenu,
    XmNmnemonic,XStringToKeysym("U"),XmNaccelerator,"Alt Shift<Key>U",
    XmNacceleratorText,XmStringCreateLocalized("Alt+Shift+U"),NULL);
  XtAddCallback(grpUngroupall,XmNactivateCallback,GrpUngroupall,NULL);

  grpMute=XtVaCreateManagedWidget(
    "grpMute",xmPushButtonWidgetClass,grpMenu,
    XmNmnemonic,XStringToKeysym("M"),XmNaccelerator,"Alt Shift<Key>M",
    XmNacceleratorText,XmStringCreateLocalized("Alt+Shift+M"),NULL);
  XtAddCallback(grpMute,XmNactivateCallback,GrpMute,NULL);

  grpDelete=XtVaCreateManagedWidget(
    "grpDelete",xmPushButtonWidgetClass,grpMenu,
    XmNmnemonic,XStringToKeysym("D"),XmNaccelerator,"Alt Shift<Key>D",
    XmNacceleratorText,XmStringCreateLocalized("Alt+Shift+D"),NULL);
  XtAddCallback(grpDelete,XmNactivateCallback,GrpDelete,NULL);

  grpDuplicate=XtVaCreateManagedWidget(
    "grpDuplicate",xmPushButtonWidgetClass,grpMenu,
    XmNmnemonic,XStringToKeysym("l"),XmNaccelerator,"Alt Shift<Key>L",
    XmNacceleratorText,XmStringCreateLocalized("Alt+Shift+L"),NULL);
  XtAddCallback(grpDuplicate,XmNactivateCallback,GrpDuplicate,NULL);


  /* REGION */

  regionButton=XtVaCreateManagedWidget(
    "regionButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  regionMenu=XmCreatePulldownMenu(menuBar,"regionMenu",NULL,0);
  XtVaSetValues(regionButton,XmNsubMenuId,regionMenu,NULL);

  newRegion=XtVaCreateManagedWidget(
    "newRegion",xmPushButtonWidgetClass, regionMenu, NULL);
  XtAddCallback(newRegion,XmNactivateCallback, NewRegion,NULL);

  newRegionBB=XmCreateBulletinBoardDialog(topLevel,"newRegionBB",NULL,0);
  newregRowCol=XtVaCreateManagedWidget("newregRowCol",xmRowColumnWidgetClass,
    newRegionBB,NULL);
  text=XmStringCreateLocalized(" ");
  newreginfoL1=XtVaCreateManagedWidget("newreginfoL1",xmLabelWidgetClass,
    newregRowCol,XmNlabelString,text,NULL);
  newreginfoL2=XtVaCreateManagedWidget("newreginfoL2",xmLabelWidgetClass,
    newregRowCol,XmNlabelString,text,NULL);
  newreginfoL3=XtVaCreateManagedWidget("newreginfoL3",xmLabelWidgetClass,
    newregRowCol,XmNlabelString,text,NULL);
  newregS=XtVaCreateManagedWidget("newregS",xmSeparatorWidgetClass,
    newregRowCol,NULL);
  newregL=XtVaCreateManagedWidget("newregL",xmLabelWidgetClass,newregRowCol,NULL);
  newregR=XtVaCreateManagedWidget("newregR",xmRowColumnWidgetClass,newregRowCol,NULL);
  for (i=1; i<10; i++) {
    sprintf(streng,"%d",i);
    text=XmStringCreateLocalized(streng);
    newregToggles[i]=XtVaCreateManagedWidget("newregTog",
      xmToggleButtonGadgetClass, newregR, XmNlabelString, text, NULL);
    XtFree(text);
  }
  XmToggleButtonSetState(newregToggles[1],True,False);
  newregL2=XtVaCreateManagedWidget("newregL2",xmLabelWidgetClass,newregRowCol,NULL);
  trackT=XtVaCreateManagedWidget("newregT",xmTextFieldWidgetClass,newregRowCol,NULL);
  newregOkButton=XtVaCreateManagedWidget("newregOk",xmPushButtonWidgetClass,
    newregRowCol,XmNsensitive,False,NULL);
  XtVaSetValues(newRegionBB,XmNdefaultButton,newregOkButton,NULL);
  XtAddCallback(newregOkButton,XmNactivateCallback,NewregionOk,NULL);

  printf("."); fflush(stdout);


  /* CREATE TRACK THINGS */

  trackBB=XtVaCreateManagedWidget("trackBB",xmBulletinBoardWidgetClass,
    largeBB, NULL);
  for (i=1; i<11; i++) {
    tracks[i].dummy=i;
    sprintf(streng,"%d",i);
    if (i==10) strcpy(streng,"M");
    text=XmStringCreateLocalized(streng);
    tracks[i].button=XtVaCreateManagedWidget("trackB",xmPushButtonWidgetClass,
      trackBB,XmNlabelString,text,XmNwidth,30,XmNheight,31,
      XmNx,0,XmNy,i*70-32,NULL);
    XtAddCallback(tracks[i].button,XmNactivateCallback,TrackZoom,&tracks[i].dummy);

    if (i<10) {
      tracks[i].muteB=XtVaCreateManagedWidget("muteB",xmPushButtonGadgetClass,
        trackBB,XmNx,40,XmNy,i*70-32,NULL);
      XtAddCallback(tracks[i].muteB,XmNactivateCallback,Mute,&tracks[i].dummy);

      tracks[i].fbackB=XtVaCreateManagedWidget("fbackB",xmPushButtonGadgetClass,
        trackBB,XmNx,71,XmNy,i*70-32,NULL);
      XtAddCallback(tracks[i].fbackB,XmNactivateCallback,FbackB,&tracks[i].dummy);
    }

    tracks[i].volB=XtVaCreateManagedWidget("volB",xmPushButtonGadgetClass,
      trackBB,XmNx,40,XmNy,i*70+2,NULL);
    XtAddCallback(tracks[i].volB,XmNactivateCallback,VolB,&tracks[i].dummy);

    tracks[i].auxB=XtVaCreateManagedWidget("auxB",xmPushButtonGadgetClass,
      trackBB,XmNx,0,XmNy,i*70+2,NULL);
    XtAddCallback(tracks[i].auxB,XmNactivateCallback,AuxB,&tracks[i].dummy);

    tracks[i].panB=XtVaCreateManagedWidget("panB",xmPushButtonGadgetClass,
      trackBB,XmNx,71,XmNy,i*70+2,NULL);
    XtAddCallback(tracks[i].panB,XmNactivateCallback,PanB,&tracks[i].dummy);

    text=XmStringCreateLocalized("60");
    tracks[i].volScale=XtVaCreateManagedWidget("volScale",xmScaleWidgetClass,
      trackBB,XmNscaleHeight,65,XmNorientation,XmVERTICAL,
      XmNmaximum,66,XmNvalue,60,XmNx,113,XmNy,i*70-32,
      XmNtitleString,text,NULL);
    XtAddCallback(tracks[i].volScale,XmNdragCallback,VolScale,&tracks[i].dummy);
    XtAddCallback(tracks[i].volScale,XmNvalueChangedCallback,VolScale,&tracks[i].dummy);
    tracks[i].zoomScale=XtVaCreateManagedWidget("zoomScale",xmScaleWidgetClass,
      trackBB,XmNscaleHeight,65,XmNorientation,XmVERTICAL,XmNminimum,4,
      XmNmaximum,100,XmNvalue,4,XmNx,159,XmNy,i*70-32,NULL);
    XtAddCallback(tracks[i].zoomScale,XmNdragCallback,ZoomScale,&tracks[i].dummy);
    XtAddCallback(tracks[i].zoomScale,XmNvalueChangedCallback,ZoomScale,&tracks[i].dummy);

    sprintf(streng,"Volume #%d",i);
    text=XmStringCreateLocalized(streng);
    tracks[i].volSel=XmCreateSelectionDialog(topLevel,"volSel",NULL,0);
    XtVaSetValues(tracks[i].volSel,XmNdialogTitle,text,XmNmustMatch,True,NULL);
    text=XmStringCreateLocalized("Graph nodes");
    XtVaSetValues(tracks[i].volSel,XmNlistLabelString,text,NULL);
    temp=XmSelectionBoxGetChild(tracks[i].volSel,XmDIALOG_APPLY_BUTTON);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].volSel,XmDIALOG_CANCEL_BUTTON);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].volSel,XmDIALOG_HELP_BUTTON);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].volSel,XmDIALOG_TEXT);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].volSel,XmDIALOG_SELECTION_LABEL);
    XtUnmanageChild(temp);

    sprintf(streng,"Pan #%d",i);
    text=XmStringCreateLocalized(streng);
    tracks[i].panSel=XmCreateSelectionDialog(topLevel,"panSel",NULL,0);
    XtVaSetValues(tracks[i].panSel,XmNdialogTitle,text,XmNmustMatch,True,NULL);
    text=XmStringCreateLocalized("Graph nodes");
    XtVaSetValues(tracks[i].panSel,XmNlistLabelString,text,NULL);
    temp=XmSelectionBoxGetChild(tracks[i].panSel,XmDIALOG_APPLY_BUTTON);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].panSel,XmDIALOG_CANCEL_BUTTON);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].panSel,XmDIALOG_HELP_BUTTON);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].panSel,XmDIALOG_TEXT);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].panSel,XmDIALOG_SELECTION_LABEL);
    XtUnmanageChild(temp);

    sprintf(streng,"Aux #%d",i);
    text=XmStringCreateLocalized(streng);
    tracks[i].auxSel=XmCreateSelectionDialog(topLevel,"auxSel",NULL,0);
    XtVaSetValues(tracks[i].auxSel,XmNdialogTitle,text,XmNmustMatch,True,NULL);
    text=XmStringCreateLocalized("Graph nodes");
    XtVaSetValues(tracks[i].auxSel,XmNlistLabelString,text,NULL);
    temp=XmSelectionBoxGetChild(tracks[i].auxSel,XmDIALOG_APPLY_BUTTON);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].auxSel,XmDIALOG_CANCEL_BUTTON);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].auxSel,XmDIALOG_HELP_BUTTON);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].auxSel,XmDIALOG_TEXT);
    XtUnmanageChild(temp);
    temp=XmSelectionBoxGetChild(tracks[i].auxSel,XmDIALOG_SELECTION_LABEL);
    XtUnmanageChild(temp);

  }

  printf(".\n");

  helpButton=XtVaCreateManagedWidget(
    "helpButton",xmCascadeButtonWidgetClass,menuBar,NULL);
  XtVaSetValues(menuBar,XmNmenuHelpWidget,helpButton,NULL);

  helpMenu=XmCreatePulldownMenu(menuBar,"helpMenu",NULL,0);
  help=XtVaCreateManagedWidget(
    "help",xmPushButtonWidgetClass,helpMenu,NULL);
  hyperB=XtVaCreateManagedWidget(
    "hyperB",xmPushButtonWidgetClass,helpMenu,NULL);
  showcaseB=XtVaCreateManagedWidget(
    "showcaseB",xmPushButtonWidgetClass,helpMenu,
    XmNmnemonic,XStringToKeysym("H"),XmNaccelerator,"Alt<Key>H",
    XmNacceleratorText,XmStringCreateLocalized("Alt+H"),NULL);
  XtVaSetValues(helpButton,XmNsubMenuId,helpMenu,NULL);

  helpBox=XmCreateMessageDialog(topLevel,"helpBox",NULL,0);
  text=XmStringCreateLtoR(
    "MIX is a simple soundfile mixer.\n\n\
	     Version 0.57,   Oyvind Hammer, NoTAM 1999\n\n\
Thanks to Doug Cook, Henrik Klagges,\n\
Hans-Christian Holm.\n\n\
Please send your comments to oyvindha@notam.uio.no.",
     XmFONTLIST_DEFAULT_TAG);
  XtVaSetValues(helpBox,XmNmessageString,text,XmNwidth,500,NULL);

  temp=XmMessageBoxGetChild(helpBox,XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  temp=XmMessageBoxGetChild(helpBox,XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);
  XtAddCallback(help,XmNactivateCallback,ShowHelp,helpBox);
  XtAddCallback(showcaseB,XmNactivateCallback,Showcase,NULL);
  XtAddCallback(hyperB,XmNactivateCallback,Hyper,NULL);


  XtRealizeWidget(topLevel);
  defdepth=DefaultDepthOfScreen(XtScreen(topLevel));
  pixelmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),XtWindow(topLevel),
    zoomin_bits, 16, 16, black.pixel, yellow.pixel, defdepth);
  XtVaSetValues(zoominB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
    pixelmap,NULL);
  pixelmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),XtWindow(topLevel),
    zoomout_bits, 16, 16, black.pixel, yellow.pixel, defdepth);
  XtVaSetValues(zoomoutB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,pixelmap,NULL);

  nomutepixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),mute_bits, 19, 19, black.pixel, yellow.pixel,defdepth);
  mutepixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),mute_bits, 19, 19, black.pixel, red.pixel,defdepth);
  volpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),vol_bits, 19, 19, black.pixel, green.pixel, defdepth);
  novolpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),vol_bits, 19, 19, black.pixel, yellow.pixel, defdepth);
  panpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),pan_bits, 19, 19, black.pixel, blue.pixel, defdepth);
  nopanpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),pan_bits, 19, 19, black.pixel, yellow.pixel, defdepth);
  parpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),par_bits, 19, 19, black.pixel, blue.pixel, defdepth);
  noparpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),par_bits, 19, 19, black.pixel, yellow.pixel, defdepth);
  auxpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),aux_bits, 19, 19, black.pixel, purple.pixel, defdepth);
  noauxpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),aux_bits, 19, 19, black.pixel, yellow.pixel, defdepth);
  backpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),back_bits, 19, 19, black.pixel, yellow.pixel, defdepth);
  frontpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),front_bits, 19, 19, black.pixel, yellow.pixel, defdepth);
  varpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),var_bits, 20, 24, black.pixel, white.pixel, defdepth);
  varselpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),var_bits, 20, 24, black.pixel, green.pixel, defdepth);

  playpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),play_bits, 16, 16, black.pixel, red.pixel, defdepth);
  noplaypixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),play_bits, 16, 16, black.pixel, yellow.pixel, defdepth);
  fastpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),fast_bits, 16, 16, black.pixel, red.pixel, defdepth);
  nofastpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),fast_bits, 16, 16, black.pixel, yellow.pixel, defdepth);
  stoppixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),stop_bits, 16, 16, black.pixel, red.pixel, defdepth);
  nostoppixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),stop_bits, 16, 16, black.pixel, yellow.pixel, defdepth);
  XtVaSetValues(playB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
    noplaypixmap,NULL);
  XtVaSetValues(fastB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
    nofastpixmap,NULL);
  XtVaSetValues(stopB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
    nostoppixmap,NULL);
  recordpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),record_bits, 16, 16, black.pixel, red.pixel, defdepth);
  norecordpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),record_bits, 16, 16, black.pixel,yellow.pixel,defdepth);
  XtVaSetValues(recordB,XmNlabelType,XmPIXMAP,XmNlabelPixmap,
    norecordpixmap,NULL);

  mixiconpixmap=XCreatePixmapFromBitmapData(XtDisplay(topLevel),
    XtWindow(topLevel),mixicon_bits, 64, 64, white.pixel,black.pixel,defdepth);
  XtVaSetValues(topLevel,XmNiconPixmap,mixiconpixmap,NULL);
  XtVaSetValues(helpBox,XmNsymbolPixmap,mixiconpixmap,NULL);


  for (i=0; i<MAXLYD; i++) {
    sounds[i].used=0;
    sounds[i].wmade=0;
  }
  flush_all_underflows_to_zero();
  Initvalues(1);
  MakeHarm();

  if (argc>1) loadm(argv[1],topLevel,0,0);

  /* Set me up as a server, accepting controls. */

  server_socket=socket(AF_INET, SOCK_STREAM, 0);
  me_sockaddr.sin_family=AF_INET;
  me_sockaddr.sin_port=htons(PORTNUM);
  me_sockaddr.sin_addr.s_addr=htonl(INADDR_ANY);
  bind(server_socket, (struct sockaddr *)&me_sockaddr,sizeof(me_sockaddr));
  listen(server_socket, 5);
  fcntl(server_socket, F_SETFL, FNDELAY);
  for (i=0; i<MAXCLIENTS; i++) client_sockets[i]=-1;
  
  XtAppMainLoop(app_context);
  
  #ifdef LINUX
  finishAudio();
  #endif


}
